@@ -32,7 +32,7 @@
 (define tmhtml-images? #f)
 (define tmhtml-image-serial 0)
 (define tmhtml-image-cache (make-ahash-table))
-(define tmhtml-image-root-url (string->url "image"))
+(define tmhtml-image-root-url (unix->url "image"))
 (define tmhtml-image-root-string "image")
 
 (tm-define (tmhtml-initialize opts)
@@ -51,10 +51,10 @@
 	  (set! tmhtml-image-serial 0)
 	  (set! tmhtml-image-root-url (url-unglue current-save-target n))
 	  (set! tmhtml-image-root-string
-		(url->string (url-tail tmhtml-image-root-url))))
+		(url->unix (url-tail tmhtml-image-root-url))))
 	(begin
 	  (set! tmhtml-image-serial 0)
-	  (set! tmhtml-image-root-url (string->url "image"))
+	  (set! tmhtml-image-root-url (unix->url "image"))
 	  (set! tmhtml-image-root-string "image")))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -151,6 +151,16 @@
 	(mathml "math { font-family: cmr, times, verdana } "))
     (if tmhtml-mathml? (string-append html mathml) html)))
 
+(define (with-extract w var)
+  (cond ((and (pair? w) (== (car w) 'with)
+	      (pair? (cdr w)) (== (cadr w) var)
+	      (pair? (cddr w)))
+	 (tmhtml-force-string (caddr w)))
+	((and (pair? w) (== (car w) 'with)
+	      (pair? (cdr w)) (pair? (cddr w)))
+	 (with-extract `(with ,@(cdddr w)) var))
+	(else #f)))
+
 (define (tmhtml-file l)
   ;; This handler is special:
   ;; Since !file is a special node used only at the top of trees
@@ -161,26 +171,40 @@
 	 (tmpath (cadddr l))
 	 (title (tmhtml-find-title doc))
 	 (css `(h:style (@ (type "text/css")) ,(tmhtml-css-header)))
+	 (xhead '())
 	 (body (tmhtml doc)))
-    (set! title (cond ((not title) "No title")
-		      ((or (in? "tmdoc" styles) (in? "tmweb" styles))
-		       `(concat ,(tmhtml-force-string title)
-				" (FSF GNU project)"))
-		      (else (tmhtml-force-string title))))
-    (if (or (in? "tmdoc" styles) (in? "tmweb" styles) (in? "mmxdoc" styles))
-	(with ss (if (in? "mmxdoc" styles)
-		     "http://www.texmacs.org/css/mmxdoc.css"
-		     "http://www.texmacs.org/css/tmdoc.css")
-	  (set! css `(h:link (@ (rel "stylesheet")
-				(href ,ss)
-				(type "text/css"))))
-	  (set! body (tmhtml-tmdoc-post body))))
+    (set! title
+	  (cond ((with-extract doc "html-title")
+		 (with-extract doc "html-title"))
+		((not title) "No title")
+		((or (in? "tmdoc" styles) (in? "tmweb" styles))
+		 `(concat ,(tmhtml-force-string title)
+			  " (FSF GNU project)"))
+		(else (tmhtml-force-string title))))
+    (set! css
+	  (cond ((with-extract doc "html-css")
+		 `(h:link (@ (rel "stylesheet")
+			     (href ,(with-extract doc "html-css"))
+			     (type "text/css"))))
+		(else css)))
+    (if (with-extract doc "html-head-javascript-src")
+	(let* ((src (with-extract doc "html-head-javascript-src"))
+	       (script `(h:script (@ (language "javascript") (src ,src)))))
+	  (set! xhead (append xhead (list script)))))
+    (if (with-extract doc "html-head-javascript")
+	(let* ((code (with-extract doc "html-head-javascript"))
+	       (script `(h:script (@ (language "javascript")) ,code)))
+	  (set! xhead (append xhead (list script)))))
+    (if (or (in? "tmdoc" styles) (in? "tmweb" styles)
+            (in? "mmxdoc" styles) (in? "magix-web" styles))
+	(set! body (tmhtml-tmdoc-post body)))
     `(h:html
       (h:head
        (h:title ,@(tmhtml title))
        (h:meta (@ (name "generator")
 		  (content ,(string-append "TeXmacs " (texmacs-version)))))
-       ,css)
+       ,css
+       ,@xhead)
       (h:body ,@body))))
 
 (define (tmhtml-finalize-document top)
@@ -455,8 +479,9 @@
 
 (define (tmhtml-concat l)
   (set! l (tmhtml-glue-scripts l))
+  ;;(display* "l << " l "\n")
   (set! l (tmconcat-structure-tabs l))
-  ;; FIXME: tabs which are inside a 'with' are not treated correctly
+  ;;(display* "l >> " l "\n")
   (tmhtml-post-simplify-nodes
    (let ((l (tmhtml-list l)))
      (cond ((null? l) '())
@@ -747,9 +772,9 @@
 	(tmhtml arg))))
 
 (define (tmhtml-with-one var val arg)
-  (cond ((drd-ref tmhtml-with-cmd% (list var val)) =>
+  (cond ((logic-ref tmhtml-with-cmd% (list var val)) =>
 	 (lambda (w) (list (append w (tmhtml arg)))))
-	((drd-ref tmhtml-with-cmd% var) =>
+	((logic-ref tmhtml-with-cmd% var) =>
 	 (lambda (h) (h val arg)))
 	(else (tmhtml arg))))
 
@@ -761,7 +786,7 @@
 	((func? x 'tmlen 3)
 	 (string-append (tmhtml-force-string (caddr x)) "tmpt"))
 	;;(else (force-string x))))
-	(else (texmacs->verbatim (tm->tree x)))))
+	(else (texmacs->code x "utf-8"))))
 
 (define (tmhtml-with l)
   (cond ((null? l) '())
@@ -862,6 +887,7 @@
 
 (define (tmhtml-specific l)
   (cond ((== (car l) "html") (list (tmstring->string (force-string (cadr l)))))
+	((== (car l) "html*") (tmhtml (cadr l)))
 	((== (car l) "image") (tmhtml-png (cadr l)))
 	(else '())))
 
@@ -892,7 +918,7 @@
 (define (border-attr what x)
   (length-attr what x " solid"))
 
-(define (tmhtml-make-cell-attr x)
+(define (tmhtml-make-cell-attr x all)
   (cond ((== (car x) "cell-width") (length-attr "width" (cadr x)))
 	((== (car x) "cell-height") (length-attr "height" (cadr x)))
 	((== x '("cell-halign" "l")) "text-align: left")
@@ -912,12 +938,23 @@
 	((== (car x) "cell-rsep") (length-attr "padding-right" (cadr x)))
 	((== (car x) "cell-tsep") (length-attr "padding-top" (cadr x)))
 	((== (car x) "cell-bsep") (length-attr "padding-bottom" (cadr x)))
+	((== (car x) "cell-bsep") (length-attr "padding-bottom" (cadr x)))
+	((== x '("cell-block" "no")) "white-space: nowrap")
+	((== x '("cell-block" "yes")) #f)
+	((== x '("cell-block" "auto"))
+         (if (or (in? '("cell-hyphen" "t") all)
+                 (in? '("cell-hyphen" "c") all)
+                 (in? '("cell-hyphen" "b") all))
+             #f
+             "white-space: nowrap"))
 	(else #f)))
 
 (define (tmhtml-make-cell c cellf)
+  (if (not (tm-func? c 'cell 1)) (set! c `(cell ,c)))
   (ahash-with tmhtml-env :left-margin 0
-    `(h:td ,@(html-css-attrs (map* tmhtml-make-cell-attr cellf))
-	   ,@(tmhtml (cadr c)))))
+    (with make (lambda (attr) (tmhtml-make-cell-attr attr cellf))
+      `(h:td ,@(html-css-attrs (map* make cellf))
+             ,@(tmhtml (cadr c))))))
 
 (define (tmhtml-make-cells-bis l cellf)
   (if (null? l) l
@@ -1004,10 +1041,10 @@
 	((and (func? x 'label 1) (string? (cadr x))) `((id ,(cadr x))))
 	(else (append-map tmhtml-collect-labels (cdr x)))))
 
-(define (tmhtml-png-names)
+(define (tmhtml-image-names ext)
   (set! tmhtml-image-serial (+ tmhtml-image-serial 1))
   (let* ((postfix (string-append
-		   "-" (number->string tmhtml-image-serial) ".png"))
+		   "-" (number->string tmhtml-image-serial) "." ext))
 	 (name-url (url-glue tmhtml-image-root-url postfix))
 	 (name-string (string-append tmhtml-image-root-string postfix)))
     (values name-url name-string)))
@@ -1020,7 +1057,7 @@
 	 (l2 (if (null? l1) l1 (list (car l1)))))
     (with cached (ahash-ref tmhtml-image-cache x)
       (if (not cached)
-	  (receive (name-url name-string) (tmhtml-png-names)
+	  (receive (name-url name-string) (tmhtml-image-names "png")
 	    ;;(display* x " -> " name-url ", " name-string "\n")
 	    (let* ((extents (print-snippet name-url x))
 		   ;;(pixels (inexact->exact (/ (second extents) 2100)))
@@ -1038,10 +1075,12 @@
 
 (define (tmhtml-image-name name)
   ;; FIXME: we should replace ~, environment variables, etc.
-  (with u (url-relative current-save-target (string->url name))
-    (if (and (or (string-ends? name ".ps") (string-ends? name ".eps"))
+  (with u (url-relative current-save-target (unix->url name))
+    (if (and (or (string-ends? name ".ps")
+                 (string-ends? name ".eps")
+                 (string-ends? name ".pdf"))
 	     (url-exists? u))
-	(receive (name-url name-string) (tmhtml-png-names)
+	(receive (name-url name-string) (tmhtml-image-names "png")
 	  (system-2 "convert" u name-url)
 	  name-string)
 	name)))
@@ -1049,59 +1088,66 @@
 (define (tmhtml-image l)
   ;; FIXME: Should also test that width and height are not magnifications.
   ;; Currently, magnifications make tmlength->htmllength return #f.
-  (if (nstring? (first l))
-      (tmhtml-png (cons 'image l))
-      (let* ((s (tmhtml-image-name (cork->html (first l))))
-	     (w (tmlength->htmllength (second l) #f))
-	     (h (tmlength->htmllength (third l) #f)))
-	`((h:img (@ (src ,s)
-		    ,@(if w `((width ,w)) '())
-		    ,@(if h `((height ,h)) '())))))))
+  (cond ((and (func? (car l) 'tuple 2)
+              (func? (cadar l) 'raw-data 1)
+              (string? (cadr (cadar l)))
+              (string? (caddar l))
+              (not (in? (caddar l) '("ps" "eps" "pdf"))))
+	  (receive (name-url name-string) (tmhtml-image-names (caddar l))
+            (string-save (cadr (cadar l)) name-url)
+            (tmhtml-image (cons name-string (cdr l)))))
+        ((nstring? (first l))
+         (tmhtml-png (cons 'image l)))
+        (else
+          (let* ((s (tmhtml-image-name (cork->html (first l))))
+                 (w (tmlength->htmllength (second l) #f))
+                 (h (tmlength->htmllength (third l) #f)))
+            `((h:img (@ (src ,s)
+                        ,@(if w `((width ,w)) '())
+                        ,@(if h `((height ,h)) '()))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Standard markup
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (tmhtml-list-document list-doc)
-  ;; Convert a list-document to a list of <h:li> elements.
-  ;; WARNING: makes the xpath environment inconsistent
-  (define (item->li mark item)
-    (cond ((null? item) '(h:li))
-	  ((null? (cdr item)) `(h:li ,@(tmhtml (car item))))
-	  (else `(h:li ,@(tmhtml `(document ,@item))))))
-  (if (null? (cdr list-doc)) '((h:li))
-      (stm-list-map item->li
-		    (lambda (x) (== x '(item)))
-		    (cdr list-doc))))
+(define (transform-item-post l)
+  (if (not (tm-is? (car l) '!item))
+      `(document ,@l)
+      `(!item ,(cadar l) (document ,(caddar l) ,@(cdr l)))))
+
+(define (transform-items x)
+  (cond ((and (tm-is? x 'concat)
+              (nnull? (cdr x))
+              (tm-in? (cadr x) '(item item*)))
+         `(!item ,(cadr x) (concat ,@(cddr x))))
+        ((tm-is? x 'document)
+         (let* ((r  (map transform-items (cdr x)))
+                (p? (lambda (i) (tm-is? i '!item)))
+                (sr (list-scatter r p? #t))
+                (fr (list-filter sr nnull?)))
+           `(document ,@(map transform-item-post fr))))
+        (else x)))
 
 ;; TODO: when the first data of the list is a label, it must be used to set the
 ;; ID attribute of the resulting xhtml element. When that is done, remove the
 ;; warning comment from htmltm-handler.
 
+(define (tmhtml-post-item args)
+  (let* ((i (car args))
+         (r (tmhtml (cadr args))))
+    (if (or (tm-is? i 'item) (null? (cdr i)))
+        `((h:li ,@r))
+        `((h:dt ,@(tmhtml (cadr i)))
+          (h:dd ,@r)))))
+
 (define (tmhtml-itemize args)
-  `((h:ul ,@(tmhtml-list-document (car args)))))
+  `((h:ul ,@(tmhtml (transform-items (car args))))))
 
 (define (tmhtml-enumerate args)
-  `((h:ol ,@(tmhtml-list-document (car args)))))
-
-(define (tmhtml-desc-document desc-doc)
-  ;; WARNING: makes the xpath environment inconsistent
-  (define (item->dt-dd mark item)
-    (let ((html-item (if (null? (cdr item))
-			 (tmhtml (car item))
-			 (tmhtml `(document ,@item)))))
-      (append
-       (if mark (tmhtml mark) '())
-       (cond ((and (null? html-item) mark) '())
-	     ((null? html-item) '((h:dd)))
-	     (else `((h:dd ,@html-item)))))))
-  (if (null? (cdr desc-doc)) '((h:dd))
-      (apply append (stm-list-map item->dt-dd
-				  (lambda (x) (func? x 'item* 1))
-				  (cdr desc-doc)))))
+  `((h:ol ,@(tmhtml (transform-items (car args))))))
 
 (define (tmhtml-description args)
-  `((h:dl ,@(tmhtml-desc-document (car args)))))
+  `((h:dl ,@(tmhtml (transform-items (car args))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Verbatim
@@ -1159,6 +1205,41 @@
 		       (h:td (@ (align "right"))
 			     "(" ,@(tmhtml (cadr l)) ")")))))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Tags for customized html generation
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define (tmhtml-html-div l)
+  (list `(h:div (@ (class ,(tmhtml-force-string (car l))))
+		,@(tmhtml (cadr l)))))
+
+(define (tmhtml-html-style l)
+  (list `(h:div (@ (style ,(tmhtml-force-string (car l))))
+		,@(tmhtml (cadr l)))))
+
+(define (tmhtml-html-javascript l)
+  (list `(h:script (@ (language "javascript"))
+		   ,(tmhtml-force-string (car l)))))
+
+(define (tmhtml-html-javascript-src l)
+  (list `(h:script (@ (language "javascript")
+		      (src ,(tmhtml-force-string (car l)))))))
+
+(define (tmhtml-html-video l)
+  (let* ((dest (cork->html (force-string (car l))))
+         (mp4 (string-append dest ".mp4"))
+         (ogg (string-append dest ".ogg"))
+         (webm (string-append dest ".webm"))
+         (swf (string-append dest ".swf"))
+         (width (force-string (cadr l)))
+         (height (force-string (caddr l))))
+    (list `(h:video (@ (width ,width) (height ,height) (controls "controls"))
+             (h:source (@ (src ,mp4) (type "video/mp4")))
+             (h:source (@ (src ,ogg) (type "video/ogg")))
+             (h:source (@ (src ,webm) (type "video/webm")))
+             (h:object (@ (data ,mp4) (width ,width) (height ,height))
+               (h:embed (@ (src ,swf) (width ,width) (height ,height))))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Tmdoc tags
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1241,7 +1322,7 @@
   (append-map tmhtml l))
 
 (define (tmhtml-dispatch htable l)
-  (let ((x (drd-ref ,htable (car l))))
+  (let ((x (logic-ref ,htable (car l))))
     (cond ((not x) #f)
 	  ((procedure? x) (x (cdr l)))
 	  (else (tmhtml-post-simplify-element
@@ -1277,7 +1358,7 @@
 ;; Dispatching
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(drd-dispatcher tmhtml-primitives%
+(logic-dispatcher tmhtml-primitives%
   (document tmhtml-document)
   (para tmhtml-paragraph)
   (surround tmhtml-surround)
@@ -1384,7 +1465,7 @@
 
   (!file tmhtml-file))
 
-(drd-table tmhtml-stdmarkup%
+(logic-table tmhtml-stdmarkup%
   ;; special auxiliary tags
   (!left ,tmhtml-align-left)
   (!middle ,tmhtml-align-middle)
@@ -1406,6 +1487,7 @@
 	description-align description-long)
    ,tmhtml-description)
   (item* (h:dt))
+  (!item ,tmhtml-post-item)
   ;; Phrase elements
   (strong (h:strong))
   (em (h:em))
@@ -1431,6 +1513,12 @@
   (equation* ,tmhtml-equation*)
   (equation-lab ,tmhtml-equation-lab)
   (equations-base ,tmhtml-equation*)
+  ;; tags for customized html generation
+  (html-div ,tmhtml-html-div)
+  (html-style ,tmhtml-html-style)
+  (html-javascript ,tmhtml-html-javascript)
+  (html-javascript-src ,tmhtml-html-javascript-src)
+  (html-video ,tmhtml-html-video)
   ;; tmdoc tags
   (tmdoc-title ,tmhtml-tmdoc-title)
   (tmdoc-title* ,tmhtml-tmdoc-title*)
@@ -1444,7 +1532,7 @@
 ;;    (name (h:name)) ; not in HTML4
 ;;    (person (h:person)))) ; not in HTML4
 
-(drd-table tmhtml-with-cmd%
+(logic-table tmhtml-with-cmd%
   ("mode" ,tmhtml-with-mode)
   ("color" ,tmhtml-with-color)
   ("font-size" ,tmhtml-with-font-size)
@@ -1467,12 +1555,12 @@
   (("font-shape" "small-caps")
    (h:class (@ (style "font-variant: small-caps")))))
 
-(drd-table tmhtml-with-cmd% ; deprecated
+(logic-table tmhtml-with-cmd% ; deprecated
   (("par-mode" "left") (h:div (@ (align "left"))))
   (("par-mode" "justify") (h:div (@ (align "justify"))))
   (("par-mode" "center") (h:center)))
 
-(drd-table tmhtml-with-cmd% ; netscape4
+(logic-table tmhtml-with-cmd% ; netscape4
   (("par-columns" "1") (h:multicol (@ (cols "1"))))
   (("par-columns" "2") (h:multicol (@ (cols "2"))))
   (("par-columns" "3") (h:multicol (@ (cols "3"))))
@@ -1489,7 +1577,7 @@
 	     (style* (tmfile-extract x 'style))
 	     (style (if (list? style*) style* (list style*)))
 	     (lan (tmfile-init x "language"))
-	     (doc (list '!file body style lan (get-texmacs-path))))
+	     (doc (list '!file body style lan (url->string (get-texmacs-path)))))
 	(texmacs->html doc opts))
       (begin
 	(tmhtml-initialize opts)
