@@ -19,7 +19,7 @@
 ;; Extra TeXmacs symbols
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(drd-table latex-texmacs-symbol%
+(logic-table latex-texmacs-symbol%
   ;; arrows with limits
   (leftarrowlim "\\mathop{\\leftarrow}\\limits")
   (rightarrowlim "\\mathop{\\rightarrow}\\limits")
@@ -40,7 +40,7 @@
   (Longleftarrowlim "\\mathop{\\longleftarrow}\\limits")
   (Longrightarrowlim "\\mathop{\\longrightarrow}\\limits")
   (Longleftrightarrowlim "\\mathop{\\longleftrightarrow}\\limits")
-   
+
   ;; asymptotic relations by Joris
   (nasymp "\\not\\asymp")
   (asympasymp "{\\asymp\\!\\!\\!\\!\\!\\!-}")
@@ -68,8 +68,10 @@
   (mapmulti "{\\mbox{\\rotatebox[origin=c]{180}{$\\multimap$}}}")
   (mathcatalan "C")
   (mathd "\\mathrm{d}")
+  (mathD "\\mathrm{D}")
   (mathe "\\mathrm{e}")
   (matheuler "\\gamma")
+  (mathlambda "\\lambda")
   (mathi "\\mathrm{i}")
   (mathpi "\\pi")
   (Alpha "\\mathrm{A}")
@@ -87,16 +89,26 @@
   (Zeta "\\mathrm{Z}")
 
   ;; other extra symbols
+  (exterior "\\wedge")
   (Exists "\\exists")
   (bigintwl "\\int")
   (bigointwl "\\oint")
-  (bignone "")
   (asterisk "*")
-  (nonesep "")
+  (point ".")
+  (cdummy "\\cdot")
+  (comma "{,}")
+  (copyright "\\copyright")
+  (bignone "")
+  (nobracket "")
+  (nospace "")
+  (nocomma "")
+  (noplus "")
+  (nosymbol "")
   (nin "\\not\\in")
   (nni "\\not\\ni")
   (notni "\\not\\ni")
   (nequiv "\\not\\equiv")
+  (nleadsto "\\not\\leadsto")
   (dotamalg "\\mathaccent95{\\amalg}")
   (dottimes "\\mathaccent95{\\times}")
   (dotoplus "\\mathaccent95{\\oplus}")
@@ -129,7 +141,7 @@
   (transtype "\\,:\\!!>")
   (udots "{\\mathinner{\\mskip1mu\\raise1pt\\vbox{\\kern7pt\\hbox{.}}\\mskip2mu\\raise4pt\\hbox{.}\\mskip2mu\\raise7pt\\hbox{.}\\mskip1mu}}"))
 
-(drd-rules
+(logic-rules
   ((latex-texmacs-macro% 'x 'body) (latex-texmacs-symbol% 'x 'body))
   ((latex-texmacs-arity% 'x 0) (latex-texmacs-symbol% 'x 'body))
   ;;;
@@ -141,22 +153,29 @@
 ;; Extra TeXmacs macros
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(drd-table latex-texmacs-0%
+(logic-table latex-texmacs-0%
   (tmunsc "\\_")
   (emdash "---")
+  (tmhrule "\\noindent\\rule[0.3\\baselineskip]{\\textwidth}{0.4pt}")
   (tmat "\\symbol{\"40}")
   (tmbsl "\\ensuremath{\\backslash}")
   (tmdummy "$\\mbox{}$")
   (TeXmacs "T\\kern-.1667em\\lower.5ex\\hbox{E}\\kern-.125emX\\kern-.1em\\lower.5ex\\hbox{\\textsc{m\\kern-.05ema\\kern-.125emc\\kern-.05ems}}")
   (madebyTeXmacs (footnote (!recurse (withTeXmacstext))))
   (withTeXmacstext
-   (!append (!translate "This document has been produced using") " "
-	    (!recurse (TeXmacs)) " (" (!translate "see") " "
-	    (texttt "http://www.texmacs.org") ")"))
+    (!append (!translate "This document has been produced using the GNU") " "
+             (!group (!recurse (TeXmacs))) " " (!translate "text editor") " ("
+             (!translate "see") " "
+             (url "http://www.texmacs.org") ")"))
   (scheme "{\\sc Scheme}")
+  (tmsep  ", ")
+  (tmSep  "; ")
   (pari "{\\sc Pari}"))
 
-(drd-table latex-texmacs-1%
+(logic-table latex-texmacs-1%
+  (tmrsub (ensuremath (!append "_{" (textrm 1) "}")))
+  (tmrsup (textsuperscript 1))
+  (tmverbatim (!group (ttfamily) (!group 1)))
   (tmtextrm (!group (rmfamily) (!group 1)))
   (tmtextsf (!group (sffamily) (!group 1)))
   (tmtexttt (!group (ttfamily) (!group 1)))
@@ -171,6 +190,7 @@
   (tmstrong (textbf 1))
   (tmem (!group "\\em " 1 "\\/"))
   (tmtt (texttt 1))
+  (tmdate (today))
   (tmname (textsc 1))
   (tmsamp (textsf 1))
   (tmabbr 1)
@@ -183,30 +203,42 @@
   (tmdef 1)
   (dueto (textup (textbf (!append "(" 1 ") "))))
   (op 1)
-  (email (!group (textit (!translate "Email:")) " " (texttt 1)))
-  (homepage (!group (textit (!translate "Web:")) " "(texttt 1)))
-  (keywords (!group (textbf (!translate "Keywords:")) " " 1))
-  (AMSclass (!group (textbf (!translate "A.M.S. subject classification:"))
-		    " " 1)))
+  (tmrunningtitle (!append (!translate "Running title:") " " 1))
+  (tmrunningauthor (!append (!translate "Running author:") " " 1))
+  (tmaffiliation (thanks (!append (textit (!translate "Affiliation:")) " " 1)))
+  (tmemail (thanks (!append (textit (!translate "Email:")) " " (texttt 1))))
+  (tmhomepage (thanks (!append (textit (!translate "Web:")) " " (texttt 1))))
+  (tmsubtitle (thanks (!append (textit (!translate "Subtitle:")) " " 1)))
+  (tmacmhomepage (titlenote (!append (textit (!translate "Web:")) " " 1)))
+  (tmacmmisc (titlenote (!append (textit (!translate "Misc:")) " " 1)))
+  (tmieeeemail (!append (textit (!translate "Email:")) " " 1))
+  (tmnote (thanks (!append (textit (!translate "Note:")) " " 1)))
+  (tmmisc (thanks (!append (textit (!translate "Misc:")) " " 1))))
 
-(drd-table latex-texmacs-2%
-  (tmhlink (!group "\\color{blue} " 1))
-  (tmaction (!group "\\color{blue} " 1))
+(logic-table latex-texmacs-1*%
+  (tmcodeinline ((!option "") (!group (ttfamily) (!group 2)))))
+
+(logic-table latex-texmacs-2%
+  (tmhlink  (!group (!append (color "blue") 1)))
+  (tmaction (!group (!append (color "blue") 1)))
+  (ontop ("{\\genfrac{}{}{0pt}{}{#1}{#2}}"))
   (subindex (index (!append 1 "!" 2))))
 
-(drd-table latex-texmacs-3%
+(logic-table latex-texmacs-3%
   (subsubindex (index (!append 1 "!" 2 "!" 3)))
   (tmref 1)
   (glossaryentry (!append (item (!option (!append 1 (hfill)))) 2 (dotfill) 3)))
 
-(drd-table latex-texmacs-4%
+(logic-table latex-texmacs-4%
   (subsubsubindex (index (!append 1 "!" 2 "!" 3 "!" 4))))
 
-(drd-rules
+(logic-rules
   ((latex-texmacs-macro% 'x 'body) (latex-texmacs-0% 'x 'body))
   ((latex-texmacs-arity% 'x 0) (latex-texmacs-0% 'x 'body))
   ((latex-texmacs-macro% 'x 'body) (latex-texmacs-1% 'x 'body))
   ((latex-texmacs-arity% 'x 1) (latex-texmacs-1% 'x 'body))
+  ((latex-texmacs-macro% 'x 'body) (latex-texmacs-1*% 'x 'body))
+  ((latex-texmacs-arity% 'x 1) (latex-texmacs-1*% 'x 'body))
   ((latex-texmacs-macro% 'x 'body) (latex-texmacs-2% 'x 'body))
   ((latex-texmacs-arity% 'x 2) (latex-texmacs-2% 'x 'body))
   ((latex-texmacs-macro% 'x 'body) (latex-texmacs-3% 'x 'body))
@@ -216,15 +248,19 @@
   ;;;
   ((latex-texmacs% 'x) (latex-texmacs-0% 'x 'body))
   ((latex-texmacs% 'x) (latex-texmacs-1% 'x 'body))
+  ((latex-texmacs% 'x) (latex-texmacs-1*% 'x 'body))
   ((latex-texmacs% 'x) (latex-texmacs-2% 'x 'body))
   ((latex-texmacs% 'x) (latex-texmacs-3% 'x 'body))
-  ((latex-texmacs% 'x) (latex-texmacs-4% 'x 'body)))
+  ((latex-texmacs% 'x) (latex-texmacs-4% 'x 'body))
+
+  ((latex-texmacs-option% 'x #t) (latex-texmacs-1*% 'x 'body))
+  ((latex-optional-arg% 'x) (latex-texmacs-option% 'x 'body)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Extra TeXmacs environments
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(drd-table latex-texmacs-environment%
+(logic-table latex-texmacs-environment%
   ("proof"
    (!append (noindent) (textbf (!append (!translate "Proof") "\\ "))
 	    ---
@@ -233,6 +269,19 @@
    (!append (noindent) (textbf (!append 1 "\\ "))
 	    ---
 	    (hspace* (fill)) (!math (Box)) (medskip)))
+  ("left-aligned"
+   (!append (!begin "flushleft")
+	    ---
+            (!end "flushleft")))
+  ("tmcode"
+   ((!option "")
+    (!append (begin "alltt")
+	    ---
+             (end "alltt"))))
+  ("right-aligned"
+   (!append (!begin "flushright")
+	    ---
+            (!end "flushright")))
   ("tmparmod"
    ((!begin "list" "" (!append "\\setlength{\\topsep}{0pt}"
 			       "\\setlength{\\leftmargin}{" 1 "}"
@@ -256,9 +305,10 @@
 			       "\\setlength{\\leftmargin}{7em}"
 			       "\\small")) ---)))
 
-(drd-table latex-texmacs-env-arity%
+(logic-table latex-texmacs-env-arity%
   ("proof" 0)
   ("proof*" 1)
+  ("tmcode" 0)
   ("tmparmod" 3)
   ("tmparsep" 1)
   ("tmindent" 0)
@@ -267,23 +317,26 @@
   ("elsequation*" 0)
   ("theglossary" 1))
 
-(drd-group latex-texmacs-tag%
+(logic-table latex-texmacs-option%
+  ("tmcode" #t))
+
+(logic-group latex-texmacs-tag%
   begin-proof begin-proof* begin-tmparmod begin-tmparsep begin-tmindent
   begin-elsequation begin-elsequation* begin-theglossary)
 
-(drd-group latex-environment-0%
+(logic-group latex-environment-0%
   begin-proof begin-tmindent begin-elsequation begin-elsequation*)
 
-(drd-group latex-environment-1%
+(logic-group latex-environment-1%
   begin-proof* begin-theglossary)
 
-(drd-group latex-environment-3%
+(logic-group latex-environment-3%
   begin-tmparmod begin-tmparsep)
 
 (define-macro (latex-texmacs-itemize env lab)
   (with env-sym (string->symbol (string-append "begin-" env))
     `(begin
-       (drd-table latex-texmacs-environment%
+       (logic-table latex-texmacs-environment%
 	 (,env
 	  ((!begin "itemize")
 	   (!append "\\renewcommand{\\labelitemi}{" ,lab "}"
@@ -291,30 +344,30 @@
 		    "\\renewcommand{\\labelitemiii}{" ,lab "}"
 		    "\\renewcommand{\\labelitemiv}{" ,lab "}"
 		    ---))))
-       (drd-table latex-texmacs-env-arity% (,env 0))
+       (logic-table latex-texmacs-env-arity% (,env 0))
        ;;;
-       (drd-group latex-texmacs-tag% ,env-sym)
-       (drd-group latex-list% ,env-sym))))
+       (logic-group latex-texmacs-tag% ,env-sym)
+       (logic-group latex-list% ,env-sym))))
 
 (define-macro (latex-texmacs-enumerate env lab)
   (with env-sym (string->symbol (string-append "begin-" env))
     `(begin
-       (drd-table latex-texmacs-environment%
+       (logic-table latex-texmacs-environment%
 	 (,env ((!begin "enumerate" (!option ,lab)) ---)))
-       (drd-table latex-texmacs-env-arity% (,env 0))
+       (logic-table latex-texmacs-env-arity% (,env 0))
        ;;;
-       (drd-group latex-texmacs-tag% ,env-sym)
-       (drd-group latex-list% ,env-sym))))
+       (logic-group latex-texmacs-tag% ,env-sym)
+       (logic-group latex-list% ,env-sym))))
 
 (define-macro (latex-texmacs-description env)
   (with env-sym (string->symbol (string-append "begin-" env))
     `(begin
-       (drd-table latex-texmacs-environment%
+       (logic-table latex-texmacs-environment%
 	 (,env ((!begin "description") ---)))
-       (drd-table latex-texmacs-env-arity% (,env 0))
+       (logic-table latex-texmacs-env-arity% (,env 0))
        ;;;
-       (drd-group latex-texmacs-tag% ,env-sym)
-       (drd-group latex-list% ,env-sym))))
+       (logic-group latex-texmacs-tag% ,env-sym)
+       (logic-group latex-list% ,env-sym))))
 
 (latex-texmacs-itemize "itemizeminus" "$-$")
 (latex-texmacs-itemize "itemizedot" "$\\bullet$")
@@ -333,16 +386,58 @@
 ;; Extra preamble definitions which are needed to export certain macros
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(drd-table latex-texmacs-preamble%
+(logic-table latex-texmacs-preamble%
+  (tmkeywords (!append
+                (newcommand (tmkeywords)
+                            (!append (textbf (!translate "Keywords:")) " "))
+                "\n"))
+  (tmacm (!append
+           (newcommand (tmacm)
+                       (!append
+                         (textbf
+                           (!translate "A.C.M. subject classification:")) " "))
+           "\n"))
+  (tmarxiv (!append
+             (newcommand (tmarxiv)
+                         (!append
+                           (textbf
+                             (!translate "arXiv subject classification:")) " "))
+           "\n"))
+  (tmpacs (!append
+            (newcommand (tmpacs)
+                        (!append
+                          (textbf
+                            (!translate "P.A.C.S. subject classification:"))
+                          " ")) "\n"))
+  (tmmsc (!append
+           (newcommand (tmmsc)
+                       (!append
+                         (textbf
+                           (!translate "A.M.S. subject classification:")) " "))
+           "\n"))
+  (fmtext        (!append "\\newcommand{\\fmtext}[2][]{\\fntext[#1]{"
+                          (!translate "Misc:") " #2}}\n"))
+  (tdatetext     (!append "\\newcommand{\\tdatetext}[2][]{\\tnotetext[#1]{"
+                          (!translate "Date:") " #2}}\n"))
+  (tmisctext     (!append "\\newcommand{\\tmisctext}[2][]{\\tnotetext[#1]{"
+                          (!translate "Misc:") " #2}}\n"))
+  (tsubtitletext (!append "\\newcommand{\\tsubtitletext}[2][]{\\tnotetext[#1]{"
+                          (!translate "Subtitle:") " #2}}\n"))
+  (thankshomepage (!append "\\newcommand{\\thankshomepage}[2][]{\\thanks[#1]{"
+                          (!translate "URL:") " #2}}\n"))
+  (thanksemail    (!append "\\newcommand{\\thanksemail}[2][]{\\thanks[#1]{"
+                          (!translate "Email:") " #2}}\n"))
+  (thanksdate     (!append "\\newcommand{\\thanksdate}[2][]{\\thanks[#1]{"
+                           (!translate "Date:") " #2}}\n"))
+  (thanksamisc    (!append "\\newcommand{\\thanksamisc}[2][]{\\thanks[#1]{"
+                           (!translate "Misc:") " #2}}\n"))
+  (thanksmisc     (!append "\\newcommand{\\thanksmisc}[2][]{\\thanks[#1]{"
+                           (!translate "Misc:") " #2}}\n"))
+  (thankssubtitle (!append "\\newcommand{\\thankssubtitle}[2][]{\\thanks[#1]{"
+                           (!translate "Subtitle:") " #2}}\n"))
   (mho
    (!append
     "\\renewcommand{\\mho}{\\mbox{\\rotatebox[origin=c]{180}{$\\omega$}}}"))
-  (color
-   (!append
-    "\\definecolor{grey}{rgb}{0.75,0.75,0.75}\n"
-    "\\definecolor{orange}{rgb}{1.0,0.5,0.5}\n"
-    "\\definecolor{brown}{rgb}{0.5,0.25,0.0}\n"
-    "\\definecolor{pink}{rgb}{1.0,0.5,0.5}\n"))
   (tmfloat
    (!append
     "\\newcommand{\\tmfloatcontents}{}\n"
@@ -361,15 +456,52 @@
     "    \\end{center}\n"
     "  \\end{minipage}}\n")))
 
+;;(define-macro (latex-texmacs-long prim x l m r)
+;;  `(logic-table latex-texmacs-preamble%
+;;     (,(string->symbol (substring prim 1 (string-length prim)))
+;;      (!append
+;;       "\\def" ,prim "fill@{\\arrowfill@" ,l ,m ,r "}\n"
+;;       "\\providecommand{" ,prim "}[2][]{"
+;;       "\\ext@arrow 0099" ,prim "fill@{#1}{#2}}\n"))))
+
+(define-macro (latex-texmacs-long prim x l m r)
+  `(logic-table latex-texmacs-preamble%
+     (,(string->symbol (substring prim 1 (string-length prim)))
+      (!append
+       "\\providecommand{" ,prim "}[2][]{"
+       "\\mathop{" ,x "}\\limits_{#1}^{#2}}\n"))))
+
+(latex-texmacs-long "\\xminus" "-"
+                    "\\DOTSB\\relbar" "\\relbar" "\\DOTSB\\relbar")
+(latex-texmacs-long "\\xleftrightarrow" "\\longleftrightarrow"
+                    "\\leftarrow" "\\relbar" "\\rightarrow")
+(latex-texmacs-long "\\xmapsto" "\\longmapsto"
+                    "\\vdash" "\\relbar" "\\rightarrow")
+(latex-texmacs-long "\\xmapsfrom" "\\leftarrow\\!\\!\\dashv"
+                    "\\leftarrow" "\\relbar" "\\dashv")
+(latex-texmacs-long "\\xequal" "="
+                    "\\DOTSB\\Relbar" "\\Relbar" "\\DOTSB\\Relbar")
+(latex-texmacs-long "\\xLeftarrow" "\\Longleftarrow"
+                    "\\Leftarrow" "\\Relbar" "\\Relbar")
+(latex-texmacs-long "\\xRightarrow" "\\Longrightarrow"
+                    "\\Relbar" "\\Relbar" "\\Rightarrow")
+(latex-texmacs-long "\\xLeftrightarrow" "\\Longleftrightarrow"
+                    "\\Leftarrow" "\\Relbar" "\\Rightarrow")
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Plain style theorems
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (define-macro (latex-texmacs-thmenv prim name before after)
   (with env-sym (string->symbol (string-append "begin-" prim))
     `(begin
-       (drd-table latex-texmacs-env-preamble%
+       (logic-table latex-texmacs-env-preamble%
 	 (,prim (!append ,@before (newtheorem ,prim (!translate ,name))
-			 ,@after "\n")))
+			 ,@after "\n")
+		no-amsthm-package%))
        ;;;
-       (drd-group latex-texmacs-tag% ,env-sym)
-       (drd-group latex-environment-0% ,env-sym))))
+       (logic-group latex-texmacs-tag% ,env-sym)
+       (logic-group latex-environment-0% ,env-sym))))
 
 (define-macro (latex-texmacs-theorem prim name)
   `(latex-texmacs-thmenv ,prim ,name () ()))
@@ -395,23 +527,87 @@
 (latex-texmacs-remark "example" "Example")
 (latex-texmacs-remark "convention" "Convention")
 (latex-texmacs-remark "warning" "Warning")
+(latex-texmacs-remark "acknowledgments" "Acknowledgments")
+(latex-texmacs-remark "answer" "Answer")
+(latex-texmacs-remark "question" "Question")
 (latex-texmacs-exercise "exercise" "Exercise")
 (latex-texmacs-exercise "problem" "Problem")
+(latex-texmacs-exercise "solution" "Solution")
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; AMS style theorems
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define-macro (ams-texmacs-theorem abbr full)
+  `(begin
+     (logic-table latex-texmacs-env-preamble%
+       (,abbr (!append "\\theoremstyle{plain}\n"
+		       (newtheorem ,abbr (!translate ,full))
+		       "\n")
+	      amsthm-package%))))
+
+(define-macro (ams-texmacs-remark abbr full)
+  `(begin
+     (logic-table latex-texmacs-env-preamble%
+       (,abbr (!append "\\theoremstyle{remark}\n"
+		       (newtheorem ,abbr (!translate ,full))
+		       "\n")
+	      amsthm-package%))))
+
+(define-macro (ams-texmacs-indent-exercise abbr full)
+  `(begin
+     (logic-table latex-texmacs-env-preamble%
+		  (,abbr (!append "\\newtheoremstyle{indent-exercise}{3pt}{3pt}{\\small}"
+				  "{\\parindent}{\\bf\\small}{.}{.5em}{}{}\n"
+				  "\\theoremstyle{indent-exercise}\n"
+		       (newtheorem ,abbr (!translate ,full))
+		       "\n")
+	      amsthm-package%))))
+
+(define-macro (ams-texmacs-noindent-exercise abbr full)
+  `(begin
+     (logic-table latex-texmacs-env-preamble%
+		  (,abbr (!append "\\newtheoremstyle{noindent-exercise}{3pt}{3pt}{\\small}"
+				  "{}{\\bf\\small}{.}{.5em}{}{}\n"
+				  "\\theoremstyle{noindent-exercise}\n"
+		       (newtheorem ,abbr (!translate ,full))
+		       "\n")
+	      amsthm-package%))))
+
+(ams-texmacs-theorem "theorem" "Theorem")
+(ams-texmacs-theorem "proposition" "Proposition")
+(ams-texmacs-theorem "lemma" "Lemma")
+(ams-texmacs-theorem "corollary" "Corollary")
+(ams-texmacs-theorem "axiom" "Axiom")
+(ams-texmacs-theorem "definition" "Definition")
+(ams-texmacs-theorem "notation" "Notation")
+(ams-texmacs-theorem "conjecture" "Conjecture")
+(ams-texmacs-remark "remark" "Remark")
+(ams-texmacs-remark "note" "Note")
+(ams-texmacs-remark "example" "Example")
+(ams-texmacs-remark "convention" "Convention")
+(ams-texmacs-remark "acknowledgments" "Acknowledgments")
+(ams-texmacs-remark "warning" "Warning")
+(ams-texmacs-remark "answer" "Answer")
+(ams-texmacs-remark "question" "Question")
+(ams-texmacs-indent-exercise "exercise" "Exercise")
+(ams-texmacs-indent-exercise "problem" "Problem")
+(ams-texmacs-noindent-exercise "solution" "Solution")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Style-dependent extra macros
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(drd-table latex-texmacs-0%
+(logic-table latex-texmacs-0%
   (appendix "" letter-style%))
 
 (define-macro (latex-texmacs-section name inside . conds)
-  `(drd-table latex-texmacs-1%
+  `(logic-table latex-texmacs-1%
      (,name (!append (medskip) (bigskip) "\n\n" (noindent) (textbf ,inside))
 	    ,@conds)))
 
 (define-macro (latex-texmacs-paragraph name inside . conds)
-  `(drd-table latex-texmacs-1%
+  `(logic-table latex-texmacs-1%
      (,name (!append (smallskip) "\n\n" (noindent) (textbf ,inside))
 	    ,@conds)))
 
@@ -427,13 +623,13 @@
 ;; Deprecated extra macros
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(drd-table latex-texmacs-0%
+(logic-table latex-texmacs-0%
   (labeleqnum "\\addtocounter{equation}{-1}\\refstepcounter{equation}\\addtocounter{equation}{1})")
   (eqnumber (!append "\\hfill(\\theequation" (!recurse (labeleqnum)) ")"))
   (leqnumber (!append "(\\theequation" (!recurse (labeleqnum)) ")\\hfill"))
   (reqnumber (!append "\\hfill(\\theequation" (!recurse (labeleqnum)) ")")))
 
-(drd-table latex-texmacs-1%
+(logic-table latex-texmacs-1%
   (key (!append "\\fbox{\\rule[-2pt]{0pt}{9pt}" (texttt 1) "}"))
   (skey (!recurse (key (!append "shift-" 1))))
   (ckey (!recurse (key (!append "ctrl-" 1))))
@@ -441,270 +637,11 @@
   (mkey (!recurse (key (!append "meta-" 1))))
   (hkey (!recurse (key (!append "hyper-" 1)))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Language specific preambles
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-(drd-table latex-preamble-language-def%
-  ("bulgarian"
-   "\\usepackage[cp1251]{inputenc}\n\\usepackage[bulgarian]{babel}")
-  ("czech" "\\usepackage[czech]{babel}")
-  ("danish" "\\usepackage[danish]{babel}")
-  ("dutch" "\\usepackage[dutch]{babel}")
-  ("finnish" "\\usepackage[finnish]{babel}")
-  ("french" "\\usepackage[french]{babel}")
-  ("german" "\\usepackage[german]{babel}")
-  ("hungarian" "\\usepackage[hungarian]{babel}")
-  ("italian" "\\usepackage[italian]{babel}")
-  ("polish" "\\usepackage[polish]{babel}")
-  ("portuguese" "\\usepackage[portuges]{babel}")
-  ("romanian" "\\usepackage[romanian]{babel}")
-  ("russian" "\\usepackage[cp1251]{inputenc}\n\\usepackage[russian]{babel}")
-  ("slovene" "\\usepackage[slovene]{babel}")
-  ("spanish" "\\usepackage[spanish]{babel}")
-  ("swedish" "\\usepackage[swedish]{babel}")
-  ("ukrainian"
-   "\\usepackage[cp1251]{inputenc}\n\\usepackage[ukrainian]{babel}"))
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Catcode tables
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-(drd-table corkT1-to-latex-catcodes%
-  ("\x00" "\\`{ }")
-  ("\x01" "\\'{ }")
-  ("\x02" "\\^{ }")
-  ("\x03" "\\~{ }")
-  ("\x04" "\\\"{ }")
-  ("\x05" "\\H{ }")
-  ("\x06" "\\r{ }")
-  ("\x07" "\\v{ }")
-  ("\x08" "\\u{ }")
-  ("\x09" "\\b{ }")
-;  ("\x0A" "\\.{ }") ; newline may still be present in strings!!!
-  ("\x0B" "\\c{ }")
-  ("\x0C" "\\k{ }")
-  ("\x0D" ",")
-  ("\x0E" "{\\guilsinglleft}")
-  ("\x0F" "{\\guilsinglright}")
-  ("\x10" "``")
-  ("\x11" "''")
-  ("\x12" ",,")
-  ("\x13" "<<")
-  ("\x14" ">>")
-  ("\x15" "--")
-  ("\x16" "---")
-  ("\x17" "{}")
-  ("\x18" "{}")
-  ("\x19" "\\i")
-  ("\x1A" "\\j")
-  ("\x1B" "ff")
-  ("\x1C" "fi")
-  ("\x1D" "fl")
-  ("\x1E" "ffi")
-  ("\x1F" "ffl")
-  ("\x80" "\\u{A}")
-  ("\x81" "\\k{A}")
-  ("\x82" "\\'C")
-  ("\x83" "\\v{C}")
-  ("\x84" "\\v{D}")
-  ("\x85" "\\v{E}")
-  ("\x86" "\\k{E}")
-  ("\x87" "\\u{G}")
-  ("\x88" "\\'L")
-  ("\x89" "\\v{L}")
-  ("\x8A" "\\L")
-  ("\x8B" "\\'N")
-  ("\x8C" "\\v{N}")
-  ("\x8D" "{\\NG}")
-  ("\x8E" "\\H{O}")
-  ("\x8F" "\\'R")
-  ("\x90" "\\v{R}")
-  ("\x91" "\\'S")
-  ("\x92" "\\v{S}")
-  ("\x93" "\\c{S}")
-  ("\x94" "\\v{T}")
-  ("\x95" "\\c{T}")
-  ("\x96" "\\H{U}")
-  ("\x97" "\\r{U}")
-  ("\x98" "\\\"Y")
-  ("\x99" "\\'Z")
-  ("\x9A" "\\v{Z}")
-  ("\x9B" "\\.Z")
-  ("\x9C" "IJ")
-  ("\x9D" "\\.I")
-  ("\x9E" "{\\dj}")
-  ("\x9F" "{\\S}")
-  ("\xA0" "\\u{a}")
-  ("\xA1" "\\k{a}")
-  ("\xA2" "\\'c")
-  ("\xA3" "\\v{c}")
-  ("\xA4" "\\v{d}")
-  ("\xA5" "\\v{e}")
-  ("\xA6" "\\k{e}")
-  ("\xA7" "\\u{g}")
-  ("\xA8" "\\'l")
-  ("\xA9" "\\v{l}")
-  ("\xAA" "{\\l}")
-  ("\xAB" "\\'n")
-  ("\xAC" "\\v{n}")
-  ("\xAD" "{\\ng}")
-  ("\xAE" "\\H{o}")
-  ("\xAF" "\\'r")
-  ("\xB0" "\\v{r}")
-  ("\xB1" "\\'s")
-  ("\xB2" "\\v{s}")
-  ("\xB3" "\\c{s}")
-  ("\xB4" "\\v{t}")
-  ("\xB5" "\\c{t}")
-  ("\xB6" "\\H{u}")
-  ("\xB7" "\\r{u}")
-  ("\xB8" "\\\"y")
-  ("\xB9" "\\'z")
-  ("\xBA" "\\v{z}")
-  ("\xBB" "\\.z")
-  ("\xBC" "ij")
-  ("\xBD" "!`")
-  ("\xBE" "?`")
-  ("\xBF" "{\\pounds}")
-  ("\xC0" "\\`A")
-  ("\xC1" "\\'A")
-  ("\xC2" "\\^A")
-  ("\xC3" "\\~A")
-  ("\xC4" "\\\"A")
-  ("\xC5" "{\\AA}")
-  ("\xC6" "{\\AE}")
-  ("\xC7" "\\c{C}")
-  ("\xC8" "\\`E")
-  ("\xC9" "\\'E")
-  ("\xCA" "\\^E")
-  ("\xCB" "\\\"E")
-  ("\xCC" "\\`I")
-  ("\xCD" "\\'I")
-  ("\xCE" "\\^I")
-  ("\xCF" "\\\"I")
-  ("\xD0" "{\\DH}")
-  ("\xD1" "\\~N")
-  ("\xD2" "\\`O")
-  ("\xD3" "\\'O")
-  ("\xD4" "\\^O")
-  ("\xD5" "\\~O")
-  ("\xD6" "\\\"O")
-  ("\xD7" "{\\OE}")
-  ("\xD8" "{\\O}")
-  ("\xD9" "\\`U")
-  ("\xDA" "\\'U")
-  ("\xDB" "\\^U")
-  ("\xDC" "\\\"U")
-  ("\xDD" "\\'Y")
-  ("\xDE" "{\\TH}")
-  ("\xDF" "{\\SS}")
-  ("\xE0" "\\`a")
-  ("\xE1" "\\'a")
-  ("\xE2" "\\^a")
-  ("\xE3" "\\~a")
-  ("\xE4" "\\\"a")
-  ("\xE5" "{\\aa}")
-  ("\xE6" "{\\ae}")
-  ("\xE7" "\\c{c}")
-  ("\xE8" "\\`e")
-  ("\xE9" "\\'e")
-  ("\xEA" "\\^e")
-  ("\xEB" "\\\"e")
-  ("\xEC" "\\`{\\i}")
-  ("\xED" "\\'{\\i}")
-  ("\xEE" "\\^{\\i}")
-  ("\xEF" "\\\"{\\i}")
-  ("\xF0" "{\\dh}")
-  ("\xF1" "\\~n")
-  ("\xF2" "\\`o")
-  ("\xF3" "\\'o")
-  ("\xF4" "\\^o")
-  ("\xF5" "\\~o")
-  ("\xF6" "\\\"o")
-  ("\xF7" "{\\oe}")
-  ("\xF8" "{\\o}")
-  ("\xF9" "\\`u")
-  ("\xFA" "\\'u")
-  ("\xFB" "\\^u")
-  ("\xFC" "\\\"u")
-  ("\xFD" "\\'y")
-  ("\xFE" "{\\th}")
-  ("\xFF" "{\\ss}"))
-
-(drd-table cyrillic-catcodes%
-  ("" "\\CYRA")
-  ("" "\\cyra")
-  ("" "\\CYRB")
-  ("" "\\cyrb")
-  ("" "\\CYRV")
-  ("" "\\cyrv")
-  ("" "\\CYRG")
-  ("" "\\cyrg")
-  ("" "\\CYRD")
-  ("" "\\cyrd")
-  ("" "\\CYRE")
-  ("" "\\cyre")
-  ("" "\\CYRZH")
-  ("" "\\cyrzh")
-  ("" "\\CYRZ")
-  ("" "\\cyrz")
-  ("" "\\CYRI")
-  ("" "\\cyri")
-  ("" "\\CYRISHRT")
-  ("" "\\cyrishrt")
-  ("" "\\CYRK")
-  ("" "\\cyrk")
-  ("" "\\CYRL")
-  ("" "\\cyrl")
-  ("" "\\CYRM")
-  ("" "\\cyrm")
-  ("" "\\CYRN")
-  ("" "\\cyrn")
-  ("" "\\CYRO")
-  ("" "\\cyro")
-  ("" "\\CYRP")
-  ("" "\\cyrp")
-  ("" "\\CYRR")
-  ("" "\\cyrr")
-  ("" "\\CYRS")
-  ("" "\\cyrs")
-  ("" "\\CYRT")
-  ("" "\\cyrt")
-  ("" "\\CYRU")
-  ("" "\\cyru")
-  ("" "\\CYRF")
-  ("" "\\cyrf")
-  ("" "\\CYRH")
-  ("" "\\cyrh")
-  ("" "\\CYRC")
-  ("" "\\cyrc")
-  ("" "\\CYRCH")
-  ("" "\\cyrch")
-  ("" "\\CYRSH")
-  ("" "\\cyrsh")
-  ("" "\\CYRSHCH")
-  ("" "\\cyrshch")
-  ("" "\\CYRHRDSN")
-  ("" "\\cyrhrdsn")
-  ("" "\\CYRERY")
-  ("" "\\cyrery")
-  ("" "\\CYRSFTSN")
-  ("" "\\cyrsftsn")
-  ("" "\\CYREREV")
-  ("" "\\cyrerev")
-  ("" "\\CYRYU")
-  ("" "\\cyryu")
-  ("" "\\CYRYA")
-  ("" "\\cyrya")
-  ("" "\\CYRYO")
-  ("" "\\cyryo"))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Page size settings
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(drd-table latex-paper-type%
+(logic-table latex-paper-type%
   ("a0" "a0paper")
   ("a1" "a1paper")
   ("a2" "a2paper")
