@@ -12,15 +12,35 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (texmacs-module (generic document-menu)
-  (:use (generic document-edit)))
+  (:use (generic document-edit)
+        (generic generic-menu)
+        (texmacs menus file-menu)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Project menu
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (include-list base t)
+  (cond ((tree-is? t 'document)
+         (apply append (map (cut include-list base <>) (tree-children t))))
+        ((and (tree-is? t 'include) (tree-atomic? (tree-ref t 0)))
+         (list (url-relative base (tree->string (tree-ref t 0)))))
+        (else (list))))
+
+(tm-define (project-file-list)
+  (if (project-attached?)
+      (let* ((prj (project-get))
+             (t (buffer->tree prj)))
+        (include-list prj t))
+      (list)))
+
+(tm-define (project-menu)
+  (buffer-list-menu (project-file-list)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Submenus for the Document menu and the iconbars
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(menu-bind project-menu
-  (link project-buffer-menu))
-
 (menu-bind document-style-menu
   ("No style" (init-style "none"))
   ---
@@ -34,6 +54,7 @@
       ("Default" (init-default "info-flag"))
       ---
       ("None" (init-env "info-flag" "none"))
+      ("Minimal" (init-env "info-flag" "minimal"))
       ("Short" (init-env "info-flag" "short"))
       ("Detailed" (init-env "info-flag" "detailed"))
       ("Also on paper" (init-env "info-flag" "paper")))
@@ -98,7 +119,7 @@
        (set-output-language "chinese"))))
   ("Czech"
    (begin
-     (init-language <"czech")
+     (init-language "czech")
      (set-output-language "czech")))
   ("Danish"
    (begin
@@ -182,9 +203,9 @@
 
 (tm-menu (supported-scripts-menu)
   (let* ((dummy (lazy-plugin-force))
-         (l (list-sort supported-scripts-list string<=?)))
+         (l (scripts-list)))
     (for (name l)
-      ((eval (ahash-ref supported-scripts-table name))
+      ((eval (scripts-name name))
        (noop) ;; NOTE: inhibit segfault due to property searching?
        (init-env "prog-scripts" name)))))
 
@@ -257,6 +278,12 @@
       ("Quarto" (init-page-type "quarto"))
       ("Statement" (init-page-type "statement"))
       ("Tabloid" (init-page-type "tabloid")))
+  (-> "Beamer"
+      ("Widescreen 16:9" (init-page-type "16:9"))
+      ("Widescreen 8:5" (init-page-type "8:5"))
+      ;;("Widescreen 3:2" (init-page-type "3:2"))
+      ("Standard 4:3" (init-page-type "4:3"))
+      ("Standard 5:4" (init-page-type "5:4")))
   ---
   ("Other" (interactive init-page-size)))
 
@@ -276,6 +303,10 @@
 ;; The Document menu
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define (wait-update-current-buffer)
+  (system-wait "Updating current buffer, " "please wait")
+  (update-current-buffer))
+
 (menu-bind document-menu
   (-> "Style" (link document-style-menu))
   (if (detailed-menus?)
@@ -291,128 +322,136 @@
 	  (-> "Part" (link document-part-menu)))
       (-> "View" (link document-view-menu)))
   ---
-  (-> "Font"
-      (-> "Text font"
-	  ("Default" (init-default "font"))
-	  ---
-	  ("Concrete" (init-env "font" "concrete"))
-	  ("Cyrillic" (init-env "font" "cyrillic"))
-	  ("Pandora" (init-env "font" "pandora"))
-	  ("Roman" (init-env "font" "roman"))
-	  ---
-	  ("Bookman" (init-env "font" "bookman"))
-	  ("Courier" (init-env "font" "courier"))
-	  ("Helvetica" (init-env "font" "helvetica"))
-	  ("Palatino" (init-env "font" "palatino"))
-	  ("Times" (init-env "font" "times"))
-	  ---
-	  (if (font-exists-in-tt? "DejaVuSerif")
-	      ("Dejavu" (init-env "font" "dejavu")))
-	  ("Lucida" (init-env "font" "x-lucida"))
-	  (if (> (get-font-type) 0)
-	      (if (font-exists-in-tt? "luxirr")
-		  ("Luxi" (init-env "font" "luxi"))))
-	  (if (font-exists-in-tt? "STIXGeneral")
-	      ("Stix" (init-env "font" "stix")))
-	  ("Utopia" (init-env "font" "x-utopia"))
-	  (if (or (supports-chinese?)
-		  (supports-japanese?)
-		  (supports-korean?))
-	      ---
-	      (if (font-exists-in-tt? "Batang")
-		  ("Batang" (init-env "font" "batang")))
-	      (if (font-exists-in-tt? "fireflysung")
-		  ("Fireflysung" (init-env "font" "fireflysung")))
-	      (if (font-exists-in-tt? "AppleGothic")
-		  ("Gothic" (init-env "font" "apple-gothic")))
-	      (if (font-exists-in-tt? "Gulim")
-		  ("Gulim" (init-env "font" "gulim")))
-	      (if (font-exists-in-tt? "华文细黑")
-		  ("HeiTi" (init-env "font" "heiti")))
-	      (if (font-exists-in-tt? "ヒラギノ明朝 ProN W6")
-		  ("Hiragino Kaku" (init-env "font" "kaku")))
-	      (if (font-exists-in-tt? "ipam")
-		  ("Ipa" (init-env "font" "ipa")))
-	      (if (font-exists-in-tt? "ttf-japanese-gothic")
-		  ("Japanese" (init-env "font" "ttf-japanese")))
-	      (if (font-exists-in-tt? "kochi-mincho")
-		  ("Kochi" (init-env "font" "kochi")))
-	      (if (font-exists-in-tt? "儷黑 Pro")
-		  ("LiHei" (init-env "font" "lihei")))
-	      (if (font-exists-in-tt? "wqy-microhei")
-		  ("MicroHei" (init-env "font" "wqy-microhei")))
-	      (if (font-exists-in-tt? "mingliu")
-		  ("MingLiU" (init-env "font" "mingliu")))
-	      (if (font-exists-in-tt? "PMingLiU")
-		  ("MingLiU" (init-env "font" "pmingliu")))	      
-	      (if (font-exists-in-tt? "MS Gothic")
-		  ("MS Gothic" (init-env "font" "ms-gothic")))
-	      (if (font-exists-in-tt? "MS Mincho")
-		  ("MS Mincho" (init-env "font" "ms-mincho")))
-	      (if (font-exists-in-tt? "sazanami-gothic")
-		  ("Sazanami" (init-env "font" "sazanami")))
-	      (if (font-exists-in-tt? "simfang")
-		  ("SimFang" (init-env "font" "simfang")))
-	      (if (font-exists-in-tt? "simhei")
-		  ("SimHei" (init-env "font" "simhei")))
-	      (if (font-exists-in-tt? "simkai")
-		  ("SimKai" (init-env "font" "simkai")))
-	      (if (font-exists-in-tt? "simli")
-		  ("SimLi" (init-env "font" "simli")))
-	      (if (font-exists-in-tt? "simsun")
-		  ("SimSun" (init-env "font" "simsun")))
-	      (if (and (font-exists-in-tt? "SimSun")
-		       (not (font-exists-in-tt? "simsun")))
-		  ("SimSun" (init-env "font" "apple-simsun")))
-	      (if (font-exists-in-tt? "simyou")
-		  ("SimYou" (init-env "font" "simyou")))
-	      (if (font-exists-in-tt? "ukai")
-		  ("UKai" (init-env "font" "ukai")))
-	      (if (font-exists-in-tt? "unbatang")
-		  ("Unbatang" (init-env "font" "unbatang")))
-	      (if (font-exists-in-tt? "uming")
-		  ("UMing" (init-env "font" "uming")))
-	      (if (font-exists-in-tt? "wqy-zenhei")
-		  ("ZenHei" (init-env "font" "wqy-zenhei")))))
-      (-> "Mathematical font"
-	  ("Default" (init-default "math-font"))
-	  ---
-	  ("Adobe" (init-env "math-font" "adobe"))
-	  (if (font-exists-in-tt? "Apple Symbols")
-	      ("Apple symbols" (init-env "math-font" "math-apple")))
-	  (if (font-exists-in-tt? "Asana-Math")
-	      ("Asana" (init-env "math-font" "math-asana")))
-	  ("Concrete" (init-env "math-font" "concrete"))
-	  (if (font-exists-in-tt? "DejaVuSerif")
-	      ("Dejavu" (init-env "math-font" "math-dejavu")))
-	  ("Euler new roman" (init-env "math-font" "ENR"))
-	  (if (font-exists-in-tt? "LucidaGrande")
-	      ("Lucida" (init-env "math-font" "math-lucida")))
-	  ("Roman" (init-env "math-font" "roman"))
-	  (if (font-exists-in-tt? "STIXGeneral")
-	      ("Stix" (init-env "math-font" "math-stix"))))
-      (-> "Program font"
-	  ("Default" (init-default "prog-font"))
-	  ---
-	  ("Concrete" (init-env "prog-font" "concrete"))
-	  ("Pandora" (init-env "prog-font" "pandora"))
-	  ("Roman" (init-env "prog-font" "roman"))
-	  ("Times" (init-env "prog-font" "times")))
-      ---
-      (-> "Size"
-	  (link document-font-base-size-menu))
-      (-> "Dpi"
-	  ("Default" (init-default "dpi"))
-	  ---
-	  ("150" (init-env "dpi" "150"))
-	  ("200" (init-env "dpi" "200"))
-	  ("300" (init-env "dpi" "300"))
-	  ("400" (init-env "dpi" "400"))
-	  ("600" (init-env "dpi" "600"))
-	  ("800" (init-env "dpi" "800"))
-	  ("1200" (init-env "dpi" "1200"))
-	  ---
-	  ("Other" (init-interactive-env "dpi"))))
+  (if (new-fonts?)
+      ("Font" (interactive open-document-font-selector)))
+  (if (not (new-fonts?))
+      (-> "Font"
+          (-> "Text font"
+              ("Default" (init-default "font"))
+              ---
+              ("Concrete" (init-env "font" "concrete"))
+              (if (url-exists-in-tex? "pnr10.mf")
+                  ("Pandora" (init-env "font" "pandora")))
+              ("Roman" (init-env "font" "roman"))
+              (if (font-exists-in-tt? "STIX-Regular")
+                  ("Stix" (init-env "font" "stix")))
+              ---
+              ("Bookman" (init-env "font" "bookman"))
+              ("Courier" (init-env "font" "courier"))
+              ("Helvetica" (init-env "font" "helvetica"))
+              ("N.C. Schoolbook" (init-env "font" "new-century-schoolbook"))
+              ("Palatino" (init-env "font" "palatino"))
+              ("Times" (init-env "font" "times"))
+              ---
+              (if (font-exists-in-tt? "DejaVuSerif")
+                  ("Dejavu" (init-env "font" "dejavu")))
+              ("Lucida" (init-env "font" "x-lucida"))
+              (if (font-exists-in-tt? "luxirr")
+                  ("Luxi" (init-env "font" "luxi")))
+              ("Utopia" (init-env "font" "x-utopia"))
+              (if (or (supports-chinese?)
+                      (supports-japanese?)
+                      (supports-korean?))
+                  ---
+                  (if (font-exists-in-tt? "Batang")
+                      ("Batang" (init-env "font" "batang")))
+                  (if (font-exists-in-tt? "fireflysung")
+                      ("Fireflysung" (init-env "font" "fireflysung")))
+                  (if (font-exists-in-tt? "AppleGothic")
+                      ("Gothic" (init-env "font" "apple-gothic")))
+                  (if (font-exists-in-tt? "Gulim")
+                      ("Gulim" (init-env "font" "gulim")))
+                  (if (font-exists-in-tt? "华文细黑")
+                      ("HeiTi" (init-env "font" "heiti")))
+                  (if (font-exists-in-tt? "ヒラギノ明朝 ProN W6")
+                      ("Hiragino Kaku" (init-env "font" "kaku")))
+                  (if (font-exists-in-tt? "ipam")
+                      ("Ipa" (init-env "font" "ipa")))
+                  (if (font-exists-in-tt? "ttf-japanese-gothic")
+                      ("Japanese" (init-env "font" "ttf-japanese")))
+                  (if (font-exists-in-tt? "kochi-mincho")
+                      ("Kochi" (init-env "font" "kochi")))
+                  (if (font-exists-in-tt? "儷黑 Pro")
+                      ("LiHei" (init-env "font" "lihei")))
+                  (if (font-exists-in-tt? "wqy-microhei")
+                      ("MicroHei" (init-env "font" "wqy-microhei")))
+                  (if (font-exists-in-tt? "mingliu")
+                      ("MingLiU" (init-env "font" "mingliu")))
+                  (if (font-exists-in-tt? "PMingLiU")
+                      ("MingLiU" (init-env "font" "pmingliu")))	      
+                  (if (font-exists-in-tt? "MS Gothic")
+                      ("MS Gothic" (init-env "font" "ms-gothic")))
+                  (if (font-exists-in-tt? "MS Mincho")
+                      ("MS Mincho" (init-env "font" "ms-mincho")))
+                  (if (font-exists-in-tt? "sazanami-gothic")
+                      ("Sazanami" (init-env "font" "sazanami")))
+                  (if (font-exists-in-tt? "simfang")
+                      ("SimFang" (init-env "font" "simfang")))
+                  (if (font-exists-in-tt? "simhei")
+                      ("SimHei" (init-env "font" "simhei")))
+                  (if (font-exists-in-tt? "simkai")
+                      ("SimKai" (init-env "font" "simkai")))
+                  (if (font-exists-in-tt? "simli")
+                      ("SimLi" (init-env "font" "simli")))
+                  (if (font-exists-in-tt? "simsun")
+                      ("SimSun" (init-env "font" "simsun")))
+                  (if (and (font-exists-in-tt? "SimSun")
+                           (not (font-exists-in-tt? "simsun")))
+                      ("SimSun" (init-env "font" "apple-simsun")))
+                  (if (font-exists-in-tt? "simyou")
+                      ("SimYou" (init-env "font" "simyou")))
+                  (if (font-exists-in-tt? "ukai")
+                      ("UKai" (init-env "font" "ukai")))
+                  (if (font-exists-in-tt? "unbatang")
+                      ("Unbatang" (init-env "font" "unbatang")))
+                  (if (font-exists-in-tt? "uming")
+                      ("UMing" (init-env "font" "uming")))
+                  (if (font-exists-in-tt? "wqy-zenhei")
+                      ("ZenHei" (init-env "font" "wqy-zenhei")))))
+          (-> "Mathematical font"
+              ("Default" (init-default "math-font"))
+              ---
+              ("Adobe" (init-env "math-font" "adobe"))
+              (if (font-exists-in-tt? "Apple Symbols")
+                  ("Apple symbols" (init-env "math-font" "math-apple")))
+              (if (font-exists-in-tt? "Asana-Math")
+                  ("Asana" (init-env "math-font" "math-asana")))
+              ("Concrete" (init-env "math-font" "concrete"))
+              (if (font-exists-in-tt? "DejaVuSerif")
+                  ("Dejavu" (init-env "math-font" "math-dejavu")))
+              ("Euler new roman" (init-env "math-font" "ENR"))
+              (if (font-exists-in-tt? "LucidaGrande")
+                  ("Lucida" (init-env "math-font" "math-lucida")))
+              (if (font-exists-in-tt? "texgyrepagella-math")
+                  ("Pagella" (init-env "math-font" "math-pagella")))
+              ("Roman" (init-env "math-font" "roman"))
+              (if (font-exists-in-tt? "STIX-Regular")
+                  ("Stix" (init-env "math-font" "math-stix")))
+              (if (font-exists-in-tt? "texgyretermes-math")
+                  ("Termes" (init-env "math-font" "math-termes"))))
+          (-> "Program font"
+              ("Default" (init-default "prog-font"))
+              ---
+              ("Concrete" (init-env "prog-font" "concrete"))
+              (if (url-exists-in-tex? "pnr10.mf")
+                  ("Pandora" (init-env "prog-font" "pandora")))
+              ("Roman" (init-env "prog-font" "roman"))
+              ("Times" (init-env "prog-font" "times")))
+          ---
+          (-> "Size"
+              (link document-font-base-size-menu))
+          (-> "Dpi"
+              ("Default" (init-default "dpi"))
+              ---
+              ("150" (init-env "dpi" "150"))
+              ("200" (init-env "dpi" "200"))
+              ("300" (init-env "dpi" "300"))
+              ("400" (init-env "dpi" "400"))
+              ("600" (init-env "dpi" "600"))
+              ("800" (init-env "dpi" "800"))
+              ("1200" (init-env "dpi" "1200"))
+              ---
+              ("Other" (init-interactive-env "dpi")))))
   (-> "Magnification"
       ("Default" (init-default "magnification"))
       ---
@@ -431,12 +470,16 @@
 	  ---
 	  (pick-color (init-env "color" answer))
 	  ---
+          ("Palette" (interactive-color
+                      (lambda (col) (init-env "color" col)) '()))
 	  ("Other" (init-interactive-env "color")))
       (-> "Background"
 	  ("Default" (init-default "bg-color"))
 	  ---
-	  (pick-background (init-env-tree "bg-color" answer))
+	  (pick-background "" (init-env-tree "bg-color" answer))
 	  ---
+          ("Palette" (interactive-background
+                      (lambda (col) (init-env "bg-color" col)) '()))
 	  ("Other" (init-interactive-env "bg-color"))))
   (if (detailed-menus?)
       (-> "Language"
@@ -478,9 +521,9 @@
 	  ("Default" (init-default "par-mode"))
 	  ---
 	  ("Justified" (init-env "par-mode" "justify"))
-	  ("Left ragged" (init-env "par-mode" "left"))
+	  ("Left aligned" (init-env "par-mode" "left"))
 	  ("Centered" (init-env "par-mode" "center"))
-	  ("Right ragged" (init-env "par-mode" "right")))
+	  ("Right aligned" (init-env "par-mode" "right")))
       (-> "Hyphenation"
 	  ("Default" (init-default "par-hyphen"))
 	  ---
@@ -505,19 +548,16 @@
 	  ("3" (init-env "par-columns" "3"))))
   (-> "Page"
       (-> "Type"
-	  ("Default" (check "*" (test-default? "page-medium"))
-	   (init-default "page-medium")
-	   (notify-page-change))
+	  ("Default" (init-default-page-medium))
 	  ---
 	  ("Paper" (init-page-medium "paper"))
 	  ("Papyrus" (init-page-medium "papyrus"))
-	  ("Automatic" (init-page-medium "automatic")))
+	  ("Automatic" (init-page-medium "automatic"))
+	  ("Beamer" (init-page-medium "beamer")))
       (-> "Size"
 	  (link document-page-size-menu))
       (-> "Orientation"
-	  ("Default" (check "*" (test-default? "page-orientation"))
-	   (init-default "page-orientation")
-	   (notify-page-change))
+	  ("Default" (init-default-page-orientation))
 	  ---
 	  ("Portrait" (init-page-orientation "portrait"))
 	  ("Landscape" (init-page-orientation "landscape")))
@@ -565,10 +605,10 @@
 	      ("Other" (init-interactive-env "page-flexibility")))))
   ---
   (-> "Update"
-      ("All" (generate-all-aux) (update-buffer))
+      ("All" (generate-all-aux) (inclusions-gc) (wait-update-current-buffer))
       ---
-      ("Buffer" (update-buffer))
-      ("Bibliography" (generate-all-aux) (update-buffer))
+      ("Buffer" (wait-update-current-buffer))
+      ("Bibliography" (generate-all-aux) (wait-update-current-buffer))
 ;;    ("Bibliography" (generate-aux "bibliography"))
       ("Table of contents" (generate-aux "table-of-contents"))
       ("Index" (generate-aux "index"))
@@ -581,54 +621,66 @@
 ;; Document focus menus
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(tm-menu (standard-focus-menu t)
-  (:require (tree-is-buffer? t))
+(tm-menu (focus-document-extra-menu t))
+
+(tm-menu (focus-style-menu t)
   (group "Style")
-  (let* ((st* (tree->stree (get-style-tree)))
-         (st (if (== st* '(tuple)) '(no-style) (cdr st*))))
+  (let* ((st* (get-style-list))
+         (st (if (null? st*) (list "no style") st*)))
     (-> (eval (upcase-first (car st)))
         (link style-menu))
     (for (pack (cdr st))
       (-> (eval (upcase-first pack))
           ("Remove package" (init-remove-package pack)))))
   (-> "Add style package"
-      (link add-package-menu))
-  ---
+      (link add-package-menu)))
+
+(tm-menu (focus-document-menu t)
   (group "Document")
   (-> (eval (upcase-first (get-init "page-type")))
       (link document-page-size-menu))
   (-> (eval (string-append (get-init "font-base-size") " pt"))
-      (link document-font-base-size-menu))
+      (link document-font-base-size-menu)))
+
+(tm-menu (standard-focus-menu t)
+  (:require (tree-is-buffer? t))
+  (dynamic (focus-style-menu t))
+  ---
+  (dynamic (focus-document-menu t))
   (dynamic (focus-document-extra-menu t))
   ---
   ("Help" (focus-help)))
 
 (tm-menu (focus-document-extra-icons t))
 
-(tm-menu (standard-focus-icons t)
-  (:require (tree-is-buffer? t))
+(tm-menu (focus-style-icons t)
   (minibar
-   (let* ((st* (tree->stree (get-style-tree)))
-	  (st (if (== st* '(tuple)) '(no-style) (cdr st*))))
+   (let* ((st* (get-style-list))
+          (st (if (null? st*) (list "no style") st*)))
      (=> (balloon (eval (upcase-first (car st))) "Document style")
 	 (link style-menu))
      (for (pack (cdr st))
        (=> (eval pack)
 	   ("Remove package" (init-remove-package pack)))))
    (=> (balloon (icon "tm_add.xpm") "Add style package")
-       (link add-package-menu)))
+       (link add-package-menu))
+   (assuming (tree-is-buffer? t)
+     ((balloon (icon "tm_focus_help.xpm") "Describe tag")
+      (focus-help)))))
 
-  (glue #f #f 5 0)
+(tm-menu (focus-document-icons t)
   (minibar
    (=> (balloon (eval (upcase-first (get-init "page-type")))
 		"Paper size")
        (link document-page-size-menu))
    (=> (balloon (eval (string-append (get-init "font-base-size") "pt"))
 		"Font size")
-       (link document-font-base-size-menu)))
-  (glue #f #f 5 0)
-  (minibar
-   ((balloon (icon "tm_focus_help.xpm") "Describe tag")
-    (focus-help)))
-  (glue #f #f 5 0)   
+       (link document-font-base-size-menu))))
+
+(tm-menu (standard-focus-icons t)
+  (:require (tree-is-buffer? t))
+  (dynamic (focus-style-icons t))  
+  //
+  (dynamic (focus-document-icons t))  
+  //   
   (dynamic (focus-document-extra-icons t)))
