@@ -347,10 +347,8 @@ texmacs_to_tree (string s, string version) {
 * Conversion of TeXmacs strings to TeXmacs trees
 ******************************************************************************/
 
-static bool
-is_apply (tree t, string s, int n) {
-  return (L(t) == APPLY) && (N(t) == n+1) && (t[0] == s);
-}
+inline bool is_apply (tree t, string s, int n) {
+  return (L(t) == APPLY) && (N(t) == (n+1)) && (t[0] == s); }
 
 static bool
 is_expand (tree t, string s, int n) {
@@ -370,6 +368,7 @@ texmacs_document_to_tree (string s) {
     int n= arity (t);
 
     tree doc (DOCUMENT);
+    doc << compound ("TeXmacs", version);
     if (n<3) return error;
     else if (n<4)
       doc << compound ("body", t[2])
@@ -402,6 +401,12 @@ texmacs_document_to_tree (string s) {
 	is_apply (doc, "TeXmacs", 1))
       doc= tree (DOCUMENT, doc);
     if (!is_document (doc)) return error;
+    if (N(doc) == 0 || !is_compound (doc[0], "TeXmacs", 1)) {
+      tree d (DOCUMENT);
+      d << compound ("TeXmacs", version);
+      d << A(doc);
+      doc= d;
+    }
     return upgrade (doc, version);
   }
   return error;
@@ -431,6 +436,7 @@ extract (tree doc, string attr) {
 	return r;
       }
 
+  if (attr == "TeXmacs") return "";
   if (attr == "body") return tree (DOCUMENT, "");
   if (attr == "project") return "";
   if (attr == "style") return tree (TUPLE);
@@ -490,9 +496,13 @@ tree
 change_doc_attr (tree doc, string attr, tree val) {
   int i, n= arity (doc);
   tree r (doc, n);
+  bool done= false;
   for (i=0; i<n; i++)
-    if (is_compound (doc[i], attr, 1))
+    if (is_compound (doc[i], attr, 1)) {
       r[i]= tree (L(doc[i]), val);
+      done= true;
+    }
     else r[i]= doc[i];
+  if (!done) r << compound (attr, val);
   return r;
 }
