@@ -33,7 +33,10 @@ hashmap<string,int> STD_CODE (UNKNOWN);
 #define url_type(i) type (i, TYPE_URL)
 #define identifier(i) type (i, TYPE_IDENTIFIER)
 #define graphical(i) type (i, TYPE_GRAPHICAL)
+#define constraint(i) type (i, TYPE_CONSTRAINT)
+#define graphical_id(i) type (i, TYPE_GRAPHICAL_ID)
 #define point_type(i) type (i, TYPE_POINT)
+#define effect(i) type (i, TYPE_EFFECT)
 #define animation(i) type (i, TYPE_ANIMATION)
 #define duration(i) type (i, TYPE_DURATION)
 
@@ -47,6 +50,8 @@ hashmap<string,int> STD_CODE (UNKNOWN);
 #define returns_identifier() type (TYPE_IDENTIFIER)
 #define returns_animation() type (TYPE_ANIMATION)
 #define returns_graphical() type (TYPE_GRAPHICAL)
+#define returns_constraint() type (TYPE_CONSTRAINT)
+#define returns_effect() type (TYPE_EFFECT)
 
 static tag_info
 fixed (int arity, int extra=0, int child_mode= CHILD_UNIFORM) {
@@ -189,6 +194,10 @@ init_std_drd () {
   init (MID, "mid", options (1, 2));
   init (RIGHT, "right", options (1, 2));
   init (BIG, "big", fixed (1));
+  init (LONG_ARROW, "long-arrow",
+        options (2, 1, DETAILED) -> name ("long arrow") ->
+        accessible (1) -> name (1, "superscript") ->
+        accessible (2) -> name (2, "subscript"));
   init (LPRIME, "lprime", fixed (1) -> name ("left prime"));
   init (RPRIME, "rprime", fixed (1) -> name ("right prime"));
   init (BELOW, "below", fixed (2) -> accessible (0));
@@ -217,6 +226,9 @@ init_std_drd () {
         name (1, "accent"));
   init (NEG, "neg", fixed (1) -> accessible (0) -> name ("negation"));
   init (TREE, "tree", repeat (2, 1) -> accessible (0));
+  init (SYNTAX, "syntax", fixed (1, 1, BIFORM) ->        
+        accessible (0) -> name (0, "body") ->
+        regular (1) -> name (1, "syntax"));
 
   init (TFORMAT, "tformat",
 	var_repeat (1, 1, BIFORM) -> inner_border () ->
@@ -386,6 +398,7 @@ init_std_drd () {
   init (PAG_LENGTH, "pag-length", fixed (0) -> returns_length ());
   init (GW_LENGTH, "gw-length", fixed (0) -> returns_length ());
   init (GH_LENGTH, "gh-length", fixed (0) -> returns_length ());
+  init (GU_LENGTH, "gu-length", fixed (0) -> returns_length ());
   init (TMPT_LENGTH, "tmpt-length", fixed (0) -> returns_length ());
   init (PX_LENGTH, "px-length", fixed (0) -> returns_length ());
   init (MSEC_LENGTH, "msec-length", fixed (0) -> returns_length ());
@@ -538,12 +551,29 @@ init_std_drd () {
 	repeat (1, 1) -> accessible (0));
   init (GR_GROUP, "gr-group",
 	repeat (1, 1) -> returns_graphical () -> graphical (0));
-  init (GR_LINEAR_TRANSFORM, "gr-linear-transform",
+  init (GR_TRANSFORM, "gr-transform",
 	fixed (1, 1, BIFORM) -> returns_graphical () ->
-	graphical (0));
+	accessible (0) -> graphical (0));
+  init (GR_EFFECT, "gr-effect",
+	var_repeat (1, 1, BIFORM) -> returns_graphical () ->
+	accessible (0) -> graphical (0) ->
+        effect (1));
+  /*
   init (TEXT_AT, "text-at",
 	fixed (1, 1, BIFORM) -> returns_graphical () ->
 	accessible (0) -> point_type (1));
+  init (MATH_AT, "math-at",
+	fixed (1, 1, BIFORM) -> returns_graphical () ->
+	accessible (0) -> point_type (1) ->
+        locals (0, "mode", "math"));
+  */
+  init (TEXT_AT, "text-at",
+	options (1, 1, BIFORM) -> returns_graphical () ->
+	accessible (0) -> point_type (1));
+  init (MATH_AT, "math-at",
+	options (1, 1, BIFORM) -> returns_graphical () ->
+	accessible (0) -> point_type (1) ->
+        locals (0, "mode", "math"));
   init (_POINT, "point",
 	repeat (1, 1) -> returns_graphical () -> point_type (0));
   init (LINE, "line",
@@ -551,15 +581,15 @@ init_std_drd () {
   init (CLINE, "cline",
 	repeat (3, 1) -> returns_graphical () -> point_type (0));
   init (ARC, "arc",
-	repeat (3, 1) -> returns_graphical () -> point_type (0));
+	fixed (3) -> returns_graphical () -> point_type (0));
   init (CARC, "carc",
-	repeat (3, 1) -> returns_graphical () -> point_type (0));
+	fixed (3) -> returns_graphical () -> point_type (0));
   init (SPLINE, "spline",
 	repeat (2, 1) -> returns_graphical () -> point_type (0));
   init (VAR_SPLINE, "spline*",
 	repeat (2, 1) -> returns_graphical () -> point_type (0));
   init (CSPLINE, "cspline",
-	repeat (2, 1) -> returns_graphical () -> point_type (0));
+	repeat (3, 1) -> returns_graphical () -> point_type (0));
   init (FILL, "fill",
 	repeat (1, 1));                       // Not yet implemented
   init (IMAGE, "image",
@@ -579,8 +609,128 @@ init_std_drd () {
   init (FRAME_INVERSE, "frame-inverse",
 	fixed (1) -> point_type (0));
 
+  init (IS_EQUAL, "is-equal",
+        fixed (2) -> returns_constraint () -> graphical_id (0));
+  init (IS_INTERSECTION, "is-intersection",
+        fixed (3) -> returns_constraint () -> graphical_id (0));
+  init (ON_CURVE, "on-curve",
+        options (2, 1, BIFORM) -> returns_constraint () ->
+        graphical_id (0) -> numeric (1));
+  init (ON_TEXT_BORDER, "on-text-border",
+        fixed (2) -> returns_constraint () -> graphical_id (0));
+  init (ON_GRID, "on-grid",
+        fixed (2) -> returns_constraint () -> graphical_id (0));
+
+  init (TRANSFORM_3D, "transform-3d",
+        fixed (1, 1, BIFORM) -> returns_graphical () ->
+        graphical (0));
+  init (OBJECT_3D, "object-3d",
+        repeat (1, 1) -> returns_graphical () ->
+        graphical (0));
+  init (TRIANGLE_3D, "triangle-3d",
+        fixed (3, 1, BIFORM) -> returns_graphical () ->
+        graphical (0) -> string_type (1));
+  init (LIGHT_3D, "light-3d",
+        fixed (1, 1, BIFORM) -> returns_graphical () ->
+        graphical (0));
+  init (LIGHT_DIFFUSE, "light-diffuse",
+        repeat (1, 1, BIFORM) ->
+        point_type (0));
+  init (LIGHT_SPECULAR, "light-specular",
+        repeat (2, 1, BIFORM) ->
+        point_type (0));
+
+  init (EFF_MOVE, "eff-move",
+        fixed (3, 0, DETAILED) -> returns_effect () ->
+        effect (0) -> name (0, "body") ->
+        length (1) -> name (1, "dx") ->
+        length (2) -> name (2, "dy"));
+  init (EFF_MAGNIFY, "eff-magnify",
+        fixed (3, 0, DETAILED) -> returns_effect () ->
+        effect (0) -> name (0, "body") ->
+        numeric (1) -> name (1, "magx") ->
+        numeric (2) -> name (2, "magy"));
+  init (EFF_BUBBLE, "eff-bubble",
+        fixed (3, 0, DETAILED) -> returns_effect () ->
+        effect (0) -> name (0, "body") ->
+        length (1) -> name (1, "radius") ->
+        numeric (2) -> name (2, "amplitude"));
+  init (EFF_GAUSSIAN, "eff-gaussian",
+        options (1, 2, DETAILED) -> returns_effect () ->
+        length (0) -> name (0, "rx") -> long_name (0, "blur x-radius") ->
+        length (1) -> name (1, "ry") -> long_name (1, "blur y-radius") ->
+        numeric (2) -> name (2, "phi") -> long_name (2, "blur angle"));
+  init (EFF_OVAL, "eff-oval",
+        options (1, 2, DETAILED) -> returns_effect () ->
+        length (0) -> name (0, "rx") -> long_name (0, "x-radius") ->
+        length (1) -> name (1, "ry") -> long_name (1, "y-radius") ->
+        numeric (2) -> name (2, "phi") -> long_name (2, "angle"));
+  init (EFF_RECTANGULAR, "eff-rectangular",
+        options (1, 2, DETAILED) -> returns_effect () ->
+        length (0) -> name (0, "rx") -> long_name (0, "x-radius") ->
+        length (1) -> name (1, "ry") -> long_name (1, "y-radius") ->
+        numeric (2) -> name (2, "phi") -> long_name (2, "angle"));
+  init (EFF_MOTION, "eff-motion",
+        options (2, 0, DETAILED) -> returns_effect () ->
+        length (0) -> name (0, "dx") -> long_name (0, "x-shift") ->
+        length (1) -> name (1, "dy") -> long_name (1, "y-shift"));
+  init (EFF_BLUR, "eff-blur",
+        fixed (2, 0, DETAILED) -> returns_effect () ->
+        effect (0) -> name (0, "body") ->
+        effect (1) -> name (1, "pen"));
+  init (EFF_OUTLINE, "eff-outline",
+        fixed (2, 0, DETAILED) -> returns_effect () ->
+        effect (0) -> name (0, "body") ->
+        effect (1) -> name (1, "pen"));
+  init (EFF_THICKEN, "eff-thicken",
+        fixed (2, 0, DETAILED) -> returns_effect () ->
+        effect (0) -> name (0, "body") ->
+        effect (1) -> name (1, "pen"));
+  init (EFF_ERODE, "eff-erode",
+        fixed (2, 0, DETAILED) -> returns_effect () ->
+        effect (0) -> name (0, "body") ->
+        effect (1) -> name (1, "pen"));
+  init (EFF_SUPERPOSE, "eff-superpose",
+	repeat (1, 1) -> returns_effect () -> effect (0));
+  init (EFF_ADD, "eff-add",
+	repeat (1, 1) -> returns_effect () -> effect (0));
+  init (EFF_SUB, "eff-sub",
+	repeat (1, 1) -> returns_effect () -> effect (0));
+  init (EFF_MUL, "eff-mul",
+	repeat (1, 1) -> returns_effect () -> effect (0));
+  init (EFF_MIN, "eff-min",
+	repeat (1, 1) -> returns_effect () -> effect (0));
+  init (EFF_MAX, "eff-max",
+	repeat (1, 1) -> returns_effect () -> effect (0));
+  init (EFF_MIX, "eff-mix",
+	fixed (4, 0, DETAILED) -> returns_effect () ->
+        effect (0) -> name (0, "eff1") ->
+        numeric (1) -> name (1, "alpha1") ->
+        effect (2) -> name (2, "eff2") ->
+        numeric (3) -> name (3, "alpha2"));
+  init (EFF_NORMALIZE, "eff-normalize",
+        fixed (1) -> returns_effect () ->
+        effect (0) -> name (0, "body"));
+  init (EFF_MONOCHROME, "eff-monochrome",
+        fixed (3, 0, DETAILED) -> returns_effect () ->
+        effect (0) -> name (0, "body") ->
+        string_type (1) -> name (1, "color") ->
+        numeric (2) -> name (2, "opacity"));
+  init (EFF_COLOR_MATRIX, "eff-color-matrix",
+        fixed (1, 1, BIFORM) -> returns_effect () ->
+        effect (0) -> name (0, "body") ->
+	numeric (1));
+  init (EFF_MAKE_TRANSPARENT, "eff-make-transparent",
+        fixed (2, 0, DETAILED) -> returns_effect () ->
+        effect (0) -> name (0, "body") ->
+	string_type (1) -> name (1, "bg") -> name (1, "background color"));
+  init (EFF_MAKE_OPAQUE, "eff-make-opaque",
+        fixed (2, 0, DETAILED) -> returns_effect () ->
+        effect (0) -> name (0, "body") ->
+	string_type (1) -> name (1, "bg") -> name (1, "background color"));
+
   init (CANVAS, "canvas", fixed (6, 1, BIFORM) -> accessible (1));
-  init (ORNAMENT, "ornament", fixed (1) -> accessible (0));
+  init (ORNAMENT, "ornament", options (1, 1) -> accessible (0));
 
   init (FORMAT, "format",
 	repeat (1, 1));
