@@ -19,6 +19,15 @@ tree superfluous_with_correct (tree t);
 tree superfluous_invisible_correct (tree t);
 tree missing_invisible_correct (tree t, int force= -1);
 tree upgrade_brackets (tree t, string mode= "text");
+tree upgrade_big (tree t);
+tree downgrade_brackets (tree t, bool del_miss= false, bool big_dot= true);
+tree downgrade_big (tree t);
+tree move_brackets (tree t);
+
+int  count_math_errors (tree t, int mode= 0);
+void math_status_cumul (tree t);
+void math_status_print ();
+void math_status_reset ();
 
 tree latex_correct (tree t);
 tree automatic_correct (tree t, string version);
