@@ -16,6 +16,7 @@
 #include "analyze.hpp"
 #include "timer.hpp"
 #include "Bridge/impl_typesetter.hpp"
+#include "new_style.hpp"
 #ifdef EXPERIMENTAL
 #include "../../Style/Environment/std_environment.hpp"
 #endif // EXPERIMENTAL
@@ -27,29 +28,44 @@ bool enable_fastenv= false;
 * Contructors, destructors and notification of modifications
 ******************************************************************************/
 
-static bool
-is_aux (url u) {
-  if (!is_atomic (u->t)) return false;
-  string s= u->t->label;
-  return starts (s, "* ") && ends (s, " *");
-}
-
 edit_typeset_rep::edit_typeset_rep ():
   the_style (TUPLE),
   cur (hashmap<string,tree> (UNINIT)),
   pre (UNINIT), init (UNINIT), fin (UNINIT),
-  env (drd, is_aux (buf->name)? buf->extra: buf->name,
-       buf->ref, (buf->prj==NULL? buf->ref: buf->prj->ref),
-       buf->aux, (buf->prj==NULL? buf->aux: buf->prj->aux)),
+  env (drd, buf->buf->master,
+       buf->data->ref, (buf->prj==NULL? buf->data->ref: buf->prj->data->ref),
+       buf->data->aux, (buf->prj==NULL? buf->data->aux: buf->prj->data->aux)),
   ttt (new_typesetter (env, subtree (et, rp), reverse (rp))) {}
 edit_typeset_rep::~edit_typeset_rep () { delete_typesetter (ttt); }
 
+void
+edit_typeset_rep::set_data (new_data data) {
+  set_style (data->style);
+  set_init  (data->init);
+  set_fin   (data->fin);
+  set_att   (data->att);
+  notify_page_change ();
+  add_init (data->init);
+  notify_change (THE_DECORATIONS);
+  typeset_invalidate_env ();
+}
+
+void
+edit_typeset_rep::get_data (new_data& data) {
+  data->style= get_style ();
+  data->init = get_init ();
+  data->fin  = get_fin ();
+  data->att  = get_att ();
+}
+
 typesetter edit_typeset_rep::get_typesetter () { return ttt; }
 tree edit_typeset_rep::get_style () { return the_style; }
 void edit_typeset_rep::set_style (tree t) { the_style= copy (t); }
 hashmap<string,tree> edit_typeset_rep::get_init () { return init; }
 hashmap<string,tree> edit_typeset_rep::get_fin () { return fin; }
+hashmap<string,tree> edit_typeset_rep::get_att () { return buf->data->att; }
 void edit_typeset_rep::set_fin (hashmap<string,tree> H) { fin= H; }
+void edit_typeset_rep::set_att (hashmap<string,tree> H) { buf->data->att= H; }
 
 void
 edit_typeset_rep::set_init (hashmap<string,tree> H) {
@@ -64,15 +80,11 @@ edit_typeset_rep::add_init (hashmap<string,tree> H) {
   notify_change (THE_ENVIRONMENT);
 }
 
-void
-edit_typeset_rep::set_base_name (url name) {
-  env->base_file_name= name;
-}
-
 void
 edit_typeset_rep::clear_local_info () {
-  buf->ref= hashmap<string,tree> ();
-  buf->aux= hashmap<string,tree> ();
+  buf->data->ref= hashmap<string,tree> ();
+  buf->data->aux= hashmap<string,tree> ();
+  buf->data->att= hashmap<string,tree> ();
 }
 
 /******************************************************************************
@@ -115,21 +127,25 @@ use_modules (tree t) {
 
 void
 edit_typeset_rep::typeset_style_use_cache (tree style) {
+  style= preprocess_style (style, buf->buf->master);
+  //cout << "Typesetting style using cache " << style << LF;
   bool ok;
   hashmap<string,tree> H;
   tree t;
-  SERVER (style_get_cache (style, H, t, ok));
+  style_get_cache (style, H, t, ok);
   if (ok) {
     env->patch_env (H);
     ok= drd->set_locals (t);
+    drd->set_environment (H);
   }
   if (!ok) {
+    //cout << "Typeset without cache " << style << LF;
     if (!is_tuple (style)) FAILED ("tuple expected as style");
-    tree t (USE_PACKAGE, A (style));
-    env->exec (t);
-    env->read_env (H);
-    drd->heuristic_init (H);
-    SERVER (style_set_cache (style, H, drd->get_locals ()));
+    H= get_style_env (style);
+    drd= get_style_drd (style);
+    style_set_cache (style, H, drd->get_locals ());
+    env->patch_env (H);
+    drd->set_environment (H);
   }
   use_modules (env->read (THE_MODULES));
 }
@@ -146,7 +162,8 @@ edit_typeset_rep::typeset_preamble () {
 
 void
 edit_typeset_rep::typeset_prepare () {
-  env->read_only= buf->read_only;
+  env->base_file_name= buf->buf->master;
+  env->read_only= buf->buf->read_only;
   env->write_default_env ();
   env->patch_env (pre);
   env->style_init_env ();
@@ -181,6 +198,18 @@ edit_typeset_rep::typeset_invalidate_env () {
   cur= hashmap<path,hashmap<string,tree> > (hashmap<string,tree> (UNINIT));
 }
 
+static void
+restricted_exec (edit_env env, tree t, int end) {
+  if (is_func (t, ASSIGN, 2) && end == 2)
+    env->exec (t);
+  else if (is_document (t) || is_concat (t))
+    for (int i=0; i < min (end, 10); i++)
+      restricted_exec (env, t[i], arity (t[i]));
+  else if (is_compound (t, "hide-preamble", 1) ||
+           is_compound (t, "show-preamble", 1))
+    env->exec (t[0]);
+}
+
 void
 edit_typeset_rep::typeset_exec_until (path p) {
   //time_t t1= texmacs_time ();
@@ -190,19 +219,26 @@ edit_typeset_rep::typeset_exec_until (path p) {
 	cout << "TeXmacs] Warning: resynchronizing for path " << p << "\n";
       // apply_changes ();
     }
+  if (p == tp && inside_graphics (true) && p != closest_inside (et, p)) {
+    //cout << "TeXmacs] Warning: corrected cursor\n";
+    tp= closest_inside (et, tp);
+    p = tp;
+  }
+
   if (N(cur[p])!=0) return;
   if (N(cur)>=25) // avoids out of memory in weird cases
     typeset_invalidate_env ();
   typeset_prepare ();
   if (enable_fastenv) {
+    if (!(rp < p)) {
+      cerr << "TeXmacs] erroneous path " << p << "\n";
+      FAILED ("invalid typesetting path");
+    }
     tree t= subtree (et, rp);
-    if (is_func (t, DOCUMENT) && N(t) > 0)
-      if (is_compound (t[0], "hide-preamble", 1) ||
-	  is_compound (t[0], "show-preamble", 1))
-	env->exec (t[0][0]);
     path q= path_up (p / rp);
     while (!is_nil (q)) {
       int i= q->item;
+      restricted_exec (env, t, i);
       tree w= drd->get_env_child (t, i, tree (ATTR));
       if (w == "") break;
       //cout << "t= " << t << "\n";
@@ -315,6 +351,20 @@ edit_typeset_rep::get_env_language () {
   else return prog_language (get_env_string (PROG_LANGUAGE));
 }
 
+SI
+edit_typeset_rep::get_page_width () {
+  (void) get_env_string (PAGE_WIDTH);
+  return (env->page_user_width + env->page_odd_margin +
+          env->page_right_margin + std_shrinkf - 1) / std_shrinkf;
+}
+
+SI
+edit_typeset_rep::get_page_height () {
+  (void) get_env_string (PAGE_HEIGHT);
+  return (env->page_user_height + env->page_top_margin +
+          env->page_bottom_margin + std_shrinkf - 1) / std_shrinkf;
+}
+
 /******************************************************************************
 * Execution without typesetting
 ******************************************************************************/
@@ -356,7 +406,7 @@ edit_typeset_rep::exec (tree t, hashmap<string,tree> H, bool expand_refs) {
   env->write_env (H);
   t= env->exec (t);
   if (expand_refs)
-    t= expand_references (t, buf->ref);
+    t= expand_references (t, buf->data->ref);
   t= simplify_execed (t);
   t= simplify_correct (t);
   env->write_env (H2);
@@ -374,6 +424,21 @@ edit_typeset_rep::exec_texmacs (tree t) {
   return exec_texmacs (t, rp * 0);
 }
 
+tree
+edit_typeset_rep::exec_verbatim (tree t, path p) {
+  typeset_exec_until (p);
+  hashmap<string,tree> H= copy (cur[p]);
+  H ("TeXmacs")= tree (MACRO, "TeXmacs");
+  H ("LaTeX")= tree (MACRO, "LaTeX");
+  H ("TeX")= tree (MACRO, "TeX");
+  return exec (t, H);
+}
+
+tree
+edit_typeset_rep::exec_verbatim (tree t) {
+  return exec_verbatim (t, rp * 0);
+}
+
 tree
 edit_typeset_rep::exec_html (tree t, path p) {
   if (p == (rp * 0)) typeset_preamble ();
@@ -382,7 +447,20 @@ edit_typeset_rep::exec_html (tree t, path p) {
   tree patch= as_tree (eval ("(stree->tree (tmhtml-env-patch))"));
   hashmap<string,tree> P (UNINIT, patch);
   H->join (P);
-  return exec (t, H);
+  tree w (WITH);
+  if (H->contains ("html-title"))
+    w << string ("html-title") << H["html-title"];
+  if (H->contains ("html-css"))
+    w << string ("html-css") << H["html-css"];
+  if (H->contains ("html-head-javascript"))
+    w << string ("html-head-javascript") << H["html-head-javascript"];
+  if (H->contains ("html-head-javascript-src"))
+    w << string ("html-head-javascript-src") << H["html-head-javascript-src"];
+  if (N(w) == 0) return exec (t, H);
+  else {
+    w << t;
+    return exec (w, H);
+  }
   //tree r= exec (t, H);
   //cout << "In: " << t << "\n";
   //cout << "Out: " << r << "\n";
@@ -487,6 +565,10 @@ void
 edit_typeset_rep::init_env (string var, tree by) {
   if (init (var) == by) return;
   init (var)= by;
+  if (var != PAGE_SCREEN_WIDTH &&
+      var != PAGE_SCREEN_HEIGHT &&
+      var != ZOOM_FACTOR)
+    require_save ();
   notify_change (THE_ENVIRONMENT);
 }
 
@@ -512,6 +594,7 @@ edit_typeset_rep::typeset (SI& x1, SI& y1, SI& x2, SI& y2) {
   bench_end ("typeset");
   //time_t t2= texmacs_time ();
   //if (t2 - t1 >= 10) cout << "typeset took " << t2-t1 << "ms\n";
+  picture_cache_clean ();
 }
 
 void
