@@ -19,12 +19,13 @@
 #include "drd_mode.hpp"
 #include "message.hpp"
 #include "tree_traverse.hpp"
+#include "boot.hpp"
 #ifdef EXPERIMENTAL
 #include "../../Style/Evaluate/evaluate_main.hpp"
 #endif
+#include "gui.hpp" // for gui_interrupted
 
 extern void (*env_next_prog)(void);
-extern void selection_correct (tree t, path i1, path i2, path& o1, path& o2);
 
 /*static*/ string
 MODE_LANGUAGE (string mode) {
@@ -49,10 +50,11 @@ edit_interface_rep::edit_interface_rep ():
   sh_s (""), sh_mark (0), pre_edit_s (""), pre_edit_mark (0),
   popup_win (),
   message_l (""), message_r (""), last_l (""), last_r (""),
-  sfactor (sv->get_default_shrinking_factor ()),
-  pixel (sfactor*PIXEL), copy_always (),
-  last_click (0), last_x (0), last_y (0), dragging (false),
-  made_selection (false), table_selection (false),
+  zoomf (sv->get_default_zoom_factor ()),
+  magf (zoomf / std_shrinkf),
+  pixel ((SI) tm_round ((std_shrinkf * PIXEL) / zoomf)), copy_always (),
+  last_x (0), last_y (0), last_t (0),
+  made_selection (false), table_selection (false), mouse_adjusting(false),
   oc (0, 0), temp_invalid_cursor (false),
   shadow (NULL), stored (NULL),
   cur_sb (2), cur_wb (2)
@@ -62,11 +64,10 @@ edit_interface_rep::edit_interface_rep ():
 }
 
 edit_interface_rep::~edit_interface_rep () {
-  if (is_attached (this)) {
-    renderer ren= get_renderer (this);
-    ren->delete_shadow (shadow);
-    ren->delete_shadow (stored);
-  }
+  if (shadow != NULL) tm_delete (shadow);
+  if (stored != NULL) tm_delete (stored);
+  shadow = NULL;
+  stored = NULL;
 }
 
 edit_interface_rep::operator tree () {
@@ -75,13 +76,16 @@ edit_interface_rep::operator tree () {
 
 void
 edit_interface_rep::suspend () {
+  if (got_focus) {
+    interrupt_shortcut ();
+    set_message ("", "", false);
+  }
   got_focus= false;
   notify_change (THE_FOCUS);
-  if (is_attached (this)) {
-    renderer ren= get_renderer (this);
-    ren->delete_shadow (shadow);
-    ren->delete_shadow (stored);
-  }
+  if (shadow != NULL) tm_delete (shadow);
+  if (stored != NULL) tm_delete (stored);
+  shadow = NULL;
+  stored = NULL;
 }
 
 void
@@ -92,6 +96,8 @@ edit_interface_rep::resume () {
   SERVER (menu_icons (1, "(horizontal (link texmacs-mode-icons))"));
   SERVER (menu_icons (2, "(horizontal (link texmacs-focus-icons))"));
   SERVER (menu_icons (3, "(horizontal (link texmacs-extra-icons))"));
+  if (use_side_tools)
+    { SERVER (side_tools (0, "(vertical (link texmacs-side-tools))")); }
   cur_sb= 2;
   tp= make_cursor_accessible (tp, true);
   notify_change (THE_FOCUS + THE_EXTENTS + THE_CURSOR);
@@ -107,19 +113,16 @@ edit_interface_rep::get_pixel_size () {
 }
 
 void
-edit_interface_rep::set_shrinking_factor (int sf) {
-  if (sfactor != sf) {
-    sfactor= sf;
-    pixel  = sf*PIXEL;
-    init_env (SFACTOR, as_string (sf));
-    notify_change (THE_ENVIRONMENT);
-  }
+edit_interface_rep::set_zoom_factor (double zoom) {
+  zoomf = zoom;
+  magf  = zoomf / std_shrinkf;
+  pixel = (int) tm_round ((std_shrinkf * PIXEL) / zoomf);
 }
 
 void
 edit_interface_rep::invalidate (SI x1, SI y1, SI x2, SI y2) {
-  send_invalidate (this, (x1-sfactor+1)/sfactor, (y1-sfactor+1)/sfactor,
-		         (x2+sfactor-1)/sfactor, (y2+sfactor-1)/sfactor);
+  send_invalidate (this, (SI) floor (x1*magf), (SI) floor (y1*magf),
+                         (SI) ceil  (x2*magf), (SI) ceil  (y2*magf));
 }
 
 void
@@ -134,28 +137,71 @@ edit_interface_rep::invalidate (rectangles rs) {
 void
 edit_interface_rep::update_visible () {
   SERVER (get_visible (vx1, vy1, vx2, vy2));
-  vx1 *= sfactor; vy1 *= sfactor; vx2 *= sfactor; vy2 *= sfactor;
+  vx1= (SI) (vx1 / magf); vy1= (SI) (vy1 / magf);
+  vx2= (SI) (vx2 / magf); vy2= (SI) (vy2 / magf);
+}
+
+SI
+edit_interface_rep::get_visible_width () {
+  update_visible ();
+  return vx2 - vx1;
+}
+
+SI
+edit_interface_rep::get_visible_height () {
+  update_visible ();
+  return vy2 - vy1;
+}
+
+#ifdef QTTEXMACS
+#include <QApplication>
+#include <QStyle>
+static SI
+scrollbar_width () {
+  return (qApp->style()->pixelMetric (QStyle::PM_ScrollBarExtent) + 2) * PIXEL;
+}
+#else
+static SI
+scrollbar_width () {
+  return 20 * PIXEL;
+}
+#endif
+
+SI
+edit_interface_rep::get_window_width () {
+  SI w, h;
+  widget me= ::get_canvas (widget (cvw));
+  ::get_size (me, w, h);
+  bool sb= (get_init_string (SCROLL_BARS) != "false");
+  if (full_screen) {
+    string medium= get_init_string (PAGE_MEDIUM);
+    if (medium == "automatic" || medium == "beamer") sb= false;
+  }
+  if (sb) w -= scrollbar_width ();
+  return w;
 }
 
 SI
 edit_interface_rep::get_window_height () {
-  update_visible ();
-  return vy2 - vy1;
+  SI w, h;
+  widget me= ::get_canvas (widget (cvw));
+  ::get_size (me, w, h);
+  return h;
 }
 
 void
 edit_interface_rep::scroll_to (SI x, SI y) {
   stored_rects= rectangles ();
   copy_always = rectangles ();
-  SERVER (scroll_to (x/sfactor, y/sfactor));
+  SERVER (scroll_to ((SI) (x * magf), ((SI) (y * magf))));
 }
 
 void
 edit_interface_rep::set_extents (SI x1, SI y1, SI x2, SI y2) {
   stored_rects= rectangles ();
   copy_always = rectangles ();
-  SERVER (set_extents ((x1-sfactor+1)/sfactor, (y1-sfactor+1)/sfactor,
-		       (x2+sfactor-1)/sfactor, (y2+sfactor-1)/sfactor));
+  SERVER (set_extents ((SI) floor (x1*magf), (SI) floor (y1*magf),
+                       (SI) ceil  (x2*magf), (SI) ceil  (y2*magf)));
 }
 
 /******************************************************************************
@@ -174,7 +220,7 @@ edit_interface_rep::cursor_visible () {
 	(cu->oy+ cu->y1 <  vy1) ||
 	(cu->oy+ cu->y2 >= vy2))
       {
-	scroll_to (cu->ox- ((vx2-vx1)>>1), cu->oy+ ((vy2-vy1)>>1));
+	scroll_to (cu->ox, cu->oy);
 	send_invalidate_all (this);
       }
   }
@@ -263,20 +309,64 @@ is_graphical (tree t) {
     is_func (t, SPLINE) || is_func (t, CSPLINE);
 }
 
+static void
+correct_adjacent (rectangles& rs1, rectangles& rs2) {
+  if (N(rs1) != 1 || N(rs2) != 1) return;
+  SI bot1= rs1->item->y1;
+  SI top2= rs2->item->y2;
+  if (rs1->item->y1 <= rs2->item->y1) {
+    //cout << "Discard " << rs1->item->y1 << ", " << rs2->item->y1 << "\n";
+    return;
+  }
+  if (rs1->item->y2 <= rs2->item->y2) {
+    //cout << "Discard " << rs1->item->y2 << ", " << rs2->item->y2 << "\n";
+    return;
+  }
+  SI mid= (bot1 + top2) >> 1;
+  rs1->item->y1= mid;
+  rs2->item->y2= mid;
+}
+
 void
 edit_interface_rep::compute_env_rects (path p, rectangles& rs, bool recurse) {
   if (p == rp) return;
   tree st= subtree (et, p);
-  if (is_atomic (st) ||
-      drd->is_child_enforcing (st) ||
-      //is_document (st) || is_concat (st) ||
-      is_func (st, TABLE) || is_func (st, SUBTABLE) ||
-      is_func (st, ROW) || is_func (st, TFORMAT) ||
-      is_graphical (st) ||
-      (is_func (st, WITH) && is_graphical (st[N(st)-1])) ||
-      (is_func (st, WITH) && is_func (st[N(st)-1], TEXT_AT)) ||
-      (is_compound (st, "math", 1) &&
-       is_compound (subtree (et, path_up (p)), "input")))
+  if ((is_func (st, TABLE) || is_func (st, SUBTABLE)) &&
+      recurse && get_preference ("show table cells") == "on") {
+    rectangles rl;
+    for (int i=0; i<N(st); i++) {
+      if (is_func (st[i], ROW))
+        for (int j=0; j<N(st[i]); j++) {
+          selection sel= eb->find_check_selection (p*i*j*0, p*i*j*1);
+          rectangles rsel= copy (thicken (sel->rs, 0, 2 * pixel));
+          if (i > 0 && is_func (st[i-1], ROW) && j < N(st[i-1])) {
+            selection bis= eb->find_check_selection (p*(i-1)*j*0, p*(i-1)*j*1);
+            rectangles rbis= copy (thicken (bis->rs, 0, 2 * pixel));
+            correct_adjacent (rbis, rsel);
+          }
+          if (i+1 < N(st) && is_func (st[i+1], ROW) && j < N(st[i+1])) {
+            selection bis= eb->find_check_selection (p*(i+1)*j*0, p*(i+1)*j*1);
+            rectangles rbis= copy (thicken (bis->rs, 0, 2 * pixel));
+            correct_adjacent (rsel, rbis);
+          }
+          rectangles selp= thicken (rsel,  pixel/2,  pixel/2);
+          rectangles selm= thicken (rsel, -pixel/2, -pixel/2);
+          rl << simplify (::correct (selp - selm));
+        }
+    }
+    rs << simplify (rl);
+    if (recurse) compute_env_rects (path_up (p), rs, recurse);
+  }
+  else if (is_atomic (st) ||
+           drd->is_child_enforcing (st) ||
+           //is_document (st) || is_concat (st) ||
+           is_func (st, TABLE) || is_func (st, SUBTABLE) ||
+           is_func (st, ROW) || is_func (st, TFORMAT) ||
+           is_graphical (st) ||
+           (is_func (st, WITH) && is_graphical (st[N(st)-1])) ||
+           (is_func (st, WITH) && is_graphical_text (st[N(st)-1])) ||
+           (is_compound (st, "math", 1) &&
+            is_compound (subtree (et, path_up (p)), "input")))
     compute_env_rects (path_up (p), rs, recurse);
   else {
     int new_mode= DRD_ACCESS_NORMAL;
@@ -286,18 +376,23 @@ edit_interface_rep::compute_env_rects (path p, rectangles& rs, bool recurse) {
     if (is_accessible_cursor (et, p * right_index (st)) || in_source ()) {
       bool right;
       path p1= p * 0, p2= p * 1, q1, q2;
-      if (is_script (subtree (et, p), right)) {
-	p1= start (et, p * 0);
-	p2= end   (et, p * 0);
-      }
+      if (is_script (subtree (et, p), right) ||
+          is_func (st, TEXT_AT) ||
+          is_func (st, MATH_AT))
+        {
+          p1= start (et, p * 0);
+          p2= end   (et, p * 0);
+        }
       if (is_func (st, CELL)) { q1= p1; q2= p2; }
-      else selection_correct (et, p1, p2, q1, q2);
+      else selection_correct (p1, p2, q1, q2);
       selection sel= eb->find_check_selection (q1, q2);
       if (N(focus_get ()) >= N(p))
-        rs << outline (sel->rs, pixel);
+        if (!recurse || get_preference ("show full context") == "on")
+          rs << outline (sel->rs, pixel);
     }
     set_access_mode (old_mode);
-    if (recurse) compute_env_rects (path_up (p), rs, recurse);
+    if (recurse || N(rs) == 0)
+      compute_env_rects (path_up (p), rs, recurse);
   }
 }
 
@@ -321,7 +416,7 @@ edit_interface_rep::has_changed (int question) {
 int
 edit_interface_rep::idle_time (int event_type) {
   if (env_change == 0 &&
-      get_renderer (this) -> repainted () &&
+      (!query_invalid (this))  &&
       (!check_event (event_type)) &&
       got_focus)
     return texmacs_time () - last_change;
@@ -348,10 +443,19 @@ edit_interface_rep::apply_changes () {
       SERVER (menu_icons (1, "(horizontal (link texmacs-mode-icons))"));
       SERVER (menu_icons (2, "(horizontal (link texmacs-focus-icons))"));
       SERVER (menu_icons (3, "(horizontal (link texmacs-extra-icons))"));
+      if (use_side_tools)
+        { SERVER (side_tools (0, "(vertical (link texmacs-side-tools))")); }
       set_footer ();
-      if (!get_renderer (this) -> interrupted ()) drd_update ();
+      if (has_current_window ()) {
+        if (need_save())
+          concrete_window () -> set_window_name (buf->buf->title * " *");
+        else
+          concrete_window () -> set_window_name (buf->buf->title);
+      }
+      if (!gui_interrupted ()) drd_update ();
       cache_memorize ();
       last_update= last_change;
+      save_user_preferences ();
     }
     return;
   }
@@ -364,25 +468,36 @@ edit_interface_rep::apply_changes () {
   
   // cout << "Handling automatic resizing\n";
   int sb= 1;
-  if (is_attached (this) && get_init_string (PAGE_MEDIUM) == "automatic") {
-    SI wx, wy;
-    ::get_size (get_window (this), wx, wy);
-    if (get_init_string (SCROLL_BARS) == "false") sb= 0;
-    if (get_server () -> in_full_screen_mode ()) sb= 0;
-    if (sb) wx -= 20 * PIXEL;
-    if (wx != cur_wx || wy != cur_wy) {
-      cur_wx= wx; cur_wy= wy;
-      init_env (PAGE_SCREEN_WIDTH, as_string (wx*sfactor) * "tmpt");
-      init_env (PAGE_SCREEN_HEIGHT, as_string (wy*sfactor) * "tmpt");
+  if (is_attached (this) && has_current_window ()) {
+    tree new_zoom= as_string (zoomf);
+    tree old_zoom= get_init_value (ZOOM_FACTOR);
+    if (new_zoom != old_zoom) {
+      init_env (ZOOM_FACTOR, new_zoom);
       notify_change (THE_ENVIRONMENT);
     }
   }
+  if (is_attached (this) &&
+      has_current_window () &&
+      get_init_string (PAGE_MEDIUM) == "automatic")
+    {
+      SI wx, wy;
+      if (cvw == NULL) ::get_size (get_window (this), wx, wy);
+      else ::get_size (widget (cvw), wx, wy);
+      if (get_init_string (SCROLL_BARS) == "false") sb= 0;
+      if (get_server () -> in_full_screen_mode ()) sb= 0;
+      if (sb) wx -= scrollbar_width();
+      if (wx != cur_wx || wy != cur_wy) {
+	cur_wx= wx; cur_wy= wy;
+	init_env (PAGE_SCREEN_WIDTH, as_string ((SI) (wx/magf)) * "tmpt");
+	init_env (PAGE_SCREEN_HEIGHT, as_string ((SI) (wy/magf)) * "tmpt");
+	notify_change (THE_ENVIRONMENT);
+      }
+    }
+  if (get_init_string (PAGE_MEDIUM) == "beamer" && full_screen) sb= 0;
   if (sb != cur_sb) {
     cur_sb= sb;
-    if (get_server() -> has_window()) {
-      tm_window win= get_server () -> get_window ();
-      win -> set_scrollbars (sb);
-    }
+    if (has_current_window ())
+      concrete_window () -> set_scrollbars (sb);
   }
   
   // window decorations (menu bar, icon bars, footer)
@@ -441,8 +556,50 @@ edit_interface_rep::apply_changes () {
 #endif
   
   // cout << "Handling extents\n";
-  if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS))
-    set_extents (eb->x1, eb->y1, eb->x2, eb->y2);
+  if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS)) {
+    string medium= get_init_string (PAGE_MEDIUM);
+    SI ex1= (SI) eb->x1*magf;
+    SI ey1= (SI) eb->y1*magf;
+    SI ex2= (SI) eb->x2*magf;
+    SI ey2= (SI) eb->y2*magf;
+    abs_round (ex1, ey1);
+    abs_round (ex2, ey2);
+    SI w, h;
+    widget me= ::get_canvas (widget (cvw));
+    ::get_size (me, w, h);
+#ifdef X11TEXMACS
+    w -= 2*PIXEL;
+    h -= 2*PIXEL;
+#endif
+    if (cur_sb && ey2 - ey1 > h) w -= scrollbar_width ();
+    if (cur_sb && ex2 - ex1 > w) h -= scrollbar_width ();
+    if (ex2 - ex1 <= w + 2*PIXEL) {
+      if (medium == "automatic" ||
+          (medium == "beamer" && full_screen))
+        ex2= ex1 + w;
+      else {
+#ifdef X11TEXMACS
+        ex1= (ex1 + ex2 - w) / 2;
+        abs_round (ex1);
+        ex2= ex1 + w;
+#endif
+      }
+    }
+    if (ey2 - ey1 <= h + 2*PIXEL) {
+      if (medium == "papyrus" || medium == "automatic" ||
+          (medium == "beamer" && full_screen))
+        ey1= ey2 - h;
+      else {
+#ifdef X11TEXMACS
+        ey1= (ey1 + ey2 - h) / 2;
+        abs_round (ey1);
+        ey2= ey1 + h;
+#endif
+      }
+    }
+    SERVER (set_extents (ex1, ey1, ex2, ey2));
+    //set_extents (eb->x1, eb->y1, eb->x2, eb->y2);
+  }
   
   // cout << "Cursor\n";
   temp_invalid_cursor= false;
@@ -468,40 +625,59 @@ edit_interface_rep::apply_changes () {
     oc= copy (cu);
    
     // set hot spot in the gui
-    send_cursor (this, (cu->ox-sfactor+1)/sfactor, (cu->oy-sfactor+1)/sfactor);
+    send_cursor (this, (SI) floor (cu->ox * magf),
+                       (SI) floor (cu->oy * magf));
 
-    rectangles old_rects= env_rects;
+    path sp= selection_get_cursor_path ();
+    bool semantic_flag= semantic_active (path_up (sp));
+    bool full_context= (get_preference ("show full context") == "on");
+    bool table_cells= (get_preference ("show table cells") == "on");
+    bool show_focus= (get_preference ("show focus") == "on");
+    bool semantic_only= (get_preference ("show only semantic focus") == "on");
+    rectangles old_env_rects= env_rects;
+    rectangles old_foc_rects= foc_rects;
     env_rects= rectangles ();
+    foc_rects= rectangles ();
     path pp= path_up (tp);
     tree pt= subtree (et, pp);
     if (none_accessible (pt));
     else pp= path_up (pp);
-    compute_env_rects (pp, env_rects, true);
-    if (env_rects != old_rects) {
-      invalidate (old_rects);
+    if (full_context || table_cells)
+      compute_env_rects (pp, env_rects, true);
+    if (show_focus && (!semantic_flag || !semantic_only))
+      compute_env_rects (pp, foc_rects, false);
+    if (env_rects != old_env_rects) {
+      invalidate (old_env_rects);
       invalidate (env_rects);
     }
     else if (env_change & THE_FOCUS) invalidate (env_rects);
+    if (foc_rects != old_foc_rects) {
+      invalidate (old_foc_rects);
+      invalidate (foc_rects);
+    }
+    else if (env_change & THE_FOCUS) invalidate (foc_rects);
     
-    old_rects= sem_rects;
-    bool old_correct= sem_correct;
+    rectangles old_sem_rects= sem_rects;
+    bool old_sem_correct= sem_correct;
     sem_rects= rectangles ();
     sem_correct= true;
-    if (semantic_active (path_up (tp))) {
+    if (semantic_flag && show_focus) {
+      path sp= selection_get_cursor_path ();
       path p1= tp, p2= tp;
-      sem_correct= semantic_select (path_up (tp), p1, p2, 2);
+      if (selection_active_any ()) selection_get (p1, p2);
+      sem_correct= semantic_select (path_up (sp), p1, p2, 2);
       if (!sem_correct) {
-        path sr= semantic_root (path_up (tp));
+        path sr= semantic_root (path_up (sp));
         p1= start (et, sr);
         p2= end (et, sr);
       }
       path q1, q2;
-      selection_correct (et, p1, p2, q1, q2);
+      selection_correct (p1, p2, q1, q2);
       selection sel= eb->find_check_selection (q1, q2);
       sem_rects << outline (sel->rs, pixel);
     }
-    if (sem_rects != old_rects || sem_correct != old_correct) {
-      invalidate (old_rects);
+    if (sem_rects != old_sem_rects || sem_correct != old_sem_correct) {
+      invalidate (old_sem_rects);
       invalidate (sem_rects);
     }
     else if (env_change & THE_FOCUS) invalidate (sem_rects);
@@ -515,12 +691,11 @@ edit_interface_rep::apply_changes () {
     if (made_selection) {
       table_selection= selection_active_table ();
       selection sel; selection_get (sel);
-      /*
-       selection_rects=
-       simplify (::correct (thicken (sel->rs, pixel, pixel) - sel->rs));
-       */
-      selection_rects= simplify (::correct (thicken (sel->rs, pixel, 3*pixel) -
-                                            thicken (sel->rs, 0, 2*pixel)));
+      rectangles rs= thicken (sel->rs, pixel, 3*pixel);
+#ifndef QTTEXMACS
+      rs= simplify (::correct (rs - thicken (rs, -pixel, -pixel)));
+#endif
+      selection_rects= rs;
       invalidate (selection_rects);
     }
   }
@@ -589,10 +764,12 @@ edit_interface_rep::animate () {
 void
 edit_interface_rep::full_screen_mode (bool flag) {
   full_screen= flag;
+  send_invalidate_all (this);
 }
 
 void
 edit_interface_rep::before_menu_action () {
+  archive_state ();
   start_editing ();
   set_input_normal ();
 }
@@ -601,12 +778,42 @@ void
 edit_interface_rep::after_menu_action () {
   notify_change (THE_DECORATIONS);
   end_editing ();
+  windows_delayed_refresh (1);
+}
+
+rectangle
+edit_interface_rep::get_window_extents () {
+  SI ox, oy, w, h;
+  widget me= ::get_canvas (widget (cvw));
+  ::get_position (me, ox, oy);
+  ::get_size (me, w, h);
+  SI vx1, vy1, vx2, vy2;
+  SERVER (get_visible (vx1, vy1, vx2, vy2));
+  ox -= vx1; oy -= vy2;
+  return rectangle (ox, oy - h, ox + w, oy);
+}
+
+cursor
+edit_interface_rep::search_cursor (path p) {
+  return eb->find_check_cursor (p);
+}
+
+selection
+edit_interface_rep::search_selection (path start, path end) {
+  selection sel= eb->find_check_selection (start, end);
+  rectangle r= least_upper_bound (sel->rs) / std_shrinkf;
+  return sel;
 }
 
 /******************************************************************************
 * event handlers
 ******************************************************************************/
 
+bool
+edit_interface_rep::is_editor_widget () {
+  return true;
+}
+
 void
 edit_interface_rep::handle_get_size_hint (SI& w, SI& h) {
   gui_root_extents (w, h);
@@ -619,6 +826,6 @@ edit_interface_rep::handle_notify_resize (SI w, SI h) {
 }
 
 void
-edit_interface_rep::handle_set_shrinking_factor (int sf) {
-  set_shrinking_factor (sf);
+edit_interface_rep::handle_set_zoom_factor (double zoom) {
+  set_zoom_factor (zoom);
 }
