@@ -4,10 +4,9 @@
 * DESCRIPTION: the interface for TeXmacs
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef EDIT_INTERFACE_H
@@ -33,29 +32,34 @@ protected:
   time_t        next_animate;  // time for next animation
   bool          full_screen;   // full screen mode ?
   bool          got_focus;     // do we have keyboard focus ?
-  string        sh_s;          // current string for shorthands
-  int           sh_len;        // length of translation
+  string        sh_s;          // current string for shortcuts
+  double        sh_mark;       // 0 or mark for undoing shortcut
+  string        pre_edit_s;    // pre-edit string
+  double        pre_edit_mark; // 0 or mark for undoing pre-edit
   widget        popup_win;     // the current popup window
-  string        message_l;     // a left message to display
-  string        message_r;     // a right message to display
-  string        last_l;        // last displayed left message
-  string        last_r;        // last displayed right message
-  int           sfactor;       // the current shrinking factor
-  SI            pixel;         // sfactor*PIXEL
+  tree          message_l;     // a left message to display
+  tree          message_r;     // a right message to display
+  tree          last_l;        // last displayed left message
+  tree          last_r;        // last displayed right message
+  double        zoomf;         // the current zoom factor
+  double        magf;          // the current magnification factor
+  SI            pixel;         // current size of a pixel on the screen
   rectangles    copy_always;   // for wiping out cursor
   int           input_mode;    // INPUT_NORMAL, INPUT_SEARCH, INPUT_REPLACE
 
 protected:
-  time_t        last_click;    // last click on left mouse button
   SI            last_x, last_y;
-  bool          start_drag, start_right_drag;
-  bool          dragging, right_dragging;
+  time_t        last_t;
   SI            start_x, start_y;
   SI            end_x, end_y;
   bool          made_selection;
   bool          table_selection;
+  int           mouse_adjusting;  // mask with active key modifiers upon click
   rectangles    selection_rects;
   rectangles    env_rects;
+  rectangles    foc_rects;
+  rectangles    sem_rects;
+  bool          sem_correct;
   cursor        oc;
   bool          temp_invalid_cursor;
   array<string> completions;
@@ -81,11 +85,14 @@ public:
 
   /* routines for dealing with shrinked coordinates */
   int  get_pixel_size ();
-  void set_shrinking_factor (int sf);
+  SI   get_visible_width ();
+  SI   get_visible_height ();
+  SI   get_window_width ();
+  SI   get_window_height ();
+  void set_zoom_factor (double zoom);
   void invalidate (SI x1, SI y1, SI x2, SI y2);
   void invalidate (rectangles rs);
   void update_visible ();
-  SI   get_window_height ();
   void scroll_to (SI x, SI y1);
   void set_extents (SI x1, SI y1, SI x2, SI y2);
 
@@ -97,10 +104,10 @@ public:
   void draw_cursor (renderer ren);
   void draw_selection (renderer ren);
   void draw_graphics (renderer ren);
-  void draw_pre (renderer ren, rectangle r);
-  void draw_post (renderer ren, rectangle r);
-  void draw_with_shadow (rectangle r);
-  void draw_with_stored (rectangle r);
+  void draw_pre (renderer win, renderer ren, rectangle r);
+  void draw_post (renderer win, renderer ren, rectangle r);
+  void draw_with_shadow (renderer win, rectangle r);
+  void draw_with_stored (renderer win, rectangle r);
 
   /* handle changes */
   void notify_change (int changed);
@@ -117,6 +124,9 @@ public:
   void full_screen_mode (bool flag);
   void before_menu_action ();
   void after_menu_action ();
+  cursor search_cursor (path p);
+  selection search_selection (path start, path end);
+  rectangle get_window_extents ();
 
   /* keyboard handling */
   int  get_input_mode ();
@@ -127,22 +137,28 @@ public:
   bool in_replace_mode ();
   bool in_spell_mode ();
   bool kbd_get_command (string which, string& help, command& cmd);
+  void interrupt_shortcut ();
   bool try_shortcut (string comb);
+  tree kbd (string s);
+  tree kbd_shortcut (string s);
   void key_press (string key);
   void emulate_keyboard (string keys, string action= "");
   bool complete_try ();
   void complete_message ();
   void complete_start (string prefix, array<string> compls);
   bool complete_keypress (string key);
+  string session_complete_command (tree t);
+  void custom_complete (tree t);
 
   /* mouse handling */
   void mouse_any (string s, SI x, SI y, int mods, time_t t);
   void mouse_click (SI x, SI y);
   bool mouse_extra_click (SI x, SI y);
   void mouse_drag (SI x, SI y);
-  void mouse_select (SI x, SI y, int mods);
+  void mouse_select (SI x, SI y, int mods, bool drag);
   void mouse_paste (SI x, SI y);
   void mouse_adjust (SI x, SI y);
+  void mouse_adjust_selection (SI x, SI y, int mods);
   void mouse_scroll (SI x, SI y, bool up);
   cursor get_cursor ();
   void set_pointer (string name);
@@ -150,33 +166,33 @@ public:
   void update_active_loci ();
 
   /* the footer */
-  string compute_text_footer (tree st);
-  string compute_operation_footer (tree st);
-  string compute_compound_footer (tree t, path p);
-  bool   set_latex_footer (tree st);
-  bool   set_hybrid_footer (tree st);
-  void   set_left_footer (string l);
-  void   append_left_footer (string& s, string env_var);
-  void   set_left_footer ();
-  void   set_right_footer (string r);
-  void   set_right_footer ();
-  void   set_footer ();
-  void   set_message (string l, string r= "", bool temp= false);
-  void   recall_message ();
+  tree compute_text_footer (tree st);
+  tree compute_operation_footer (tree st);
+  tree compute_compound_footer (tree t, path p);
+  bool set_latex_footer (tree st);
+  bool set_hybrid_footer (tree st);
+  void set_left_footer (tree l);
+  void append_left_footer (tree& l, string env_var);
+  void set_left_footer ();
+  void set_right_footer (tree r);
+  void set_right_footer ();
+  void set_footer ();
+  void set_message (tree l, tree r= "", bool temp= false);
+  void recall_message ();
 
   /* event handlers */
+  bool is_editor_widget ();
   void handle_get_size_hint (SI& w, SI& h);
   void handle_notify_resize (SI w, SI h);
   void handle_keypress (string key, time_t t);
   void handle_keyboard_focus (bool has_focus, time_t t);
   void handle_mouse (string kind, SI x, SI y, int mods, time_t t);
-  void handle_set_shrinking_factor (int sf);
-  void handle_clear (SI x1, SI y1, SI x2, SI y2);
-  void handle_repaint (SI x1, SI y1, SI x2, SI y2);
+  void handle_set_zoom_factor (double zoomf);
+  void handle_clear (renderer win, SI x1, SI y1, SI x2, SI y2);
+  void handle_repaint (renderer win, SI x1, SI y1, SI x2, SI y2);
 
   friend class interactive_command_rep;
   friend class tm_window_rep;
-  friend class tm_project_rep;
 };
 
 #endif // defined EDIT_INTERFACE_H
