@@ -4,10 +4,9 @@
 * DESCRIPTION: Editing mathematics
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef EDIT_MATH_H
@@ -19,7 +18,7 @@ public:
   edit_math_rep ();
   ~edit_math_rep ();
 
-  void make_group ();
+  void make_rigid ();
   void make_lprime (string s);
   void make_rprime (string s);
   void make_below ();
@@ -32,9 +31,13 @@ public:
   void make_wide_under (string wide);
   void make_neg ();
   void make_tree ();
+  void back_around (tree t, path p, bool forward);
   void back_prime (tree t, path p, bool forward);
+  void back_in_around (tree t, path p, bool forward);
+  void back_in_long_arrow (tree t, path p, bool forward);
   void back_in_wide (tree t, path p, bool forward);
   void back_in_tree (tree t, path p, bool forward);
+  void pre_remove_around (path p);
 };
 
 #endif // defined EDIT_MATH_H
