@@ -13,7 +13,8 @@
 #include "Interface/edit_interface.hpp"
 #include "convert.hpp"
 #include "packrat.hpp"
-#include "tree_traverse.hpp"
+#include "tree_select.hpp"
+#include "drd_mode.hpp"
 
 /******************************************************************************
 * Internationalization
@@ -24,10 +25,6 @@ selection_encode (string lan, string s) {
   if ((lan == "czech") || (lan == "hungarian") ||
       (lan == "polish") || (lan == "slovene"))
     return cork_to_il2 (s);
-  else if ((lan == "bulgarian") || (lan == "russian"))
-    return koi8_to_iso (s);
-  else if (lan == "ukrainian")
-    return koi8uk_to_iso (s);
   else if (lan == "spanish")
     return spanish_to_ispanish (s);
   else if (lan == "german")
@@ -40,10 +37,6 @@ selection_decode (string lan, string s) {
   if ((lan == "czech") || (lan == "hungarian") ||
       (lan == "polish") || (lan == "slovene"))
     return il2_to_cork (s);
-  else if ((lan == "bulgarian") || (lan == "russian"))
-    return iso_to_koi8 (s);
-  else if (lan == "ukrainian")
-    return iso_to_koi8uk (s);
   else if (lan == "spanish")
     return ispanish_to_spanish (s);
   else if (lan == "german")
@@ -57,7 +50,7 @@ selection_decode (string lan, string s) {
 
 edit_select_rep::edit_select_rep ():
   selecting (false), shift_selecting (false), mid_p (),
-  selection_import ("texmacs"), selection_export ("texmacs"),
+  selection_import ("default"), selection_export ("default"),
   focus_p (), focus_hold (false) {}
 edit_select_rep::~edit_select_rep () {}
 
@@ -67,29 +60,45 @@ edit_select_rep::~edit_select_rep () {}
 
 path
 edit_select_rep::semantic_root (path p) {
-  while (!is_nil (p) && is_script (subtree (et, p)))
-    p= path_up (p);
-  while (!is_nil (p) && is_format (subtree (et, path_up (p))))
+  while (p != rp) {
+    tree st= subtree (et, path_up (p));
+    if (path_up (p) != rp && is_func (st, DOCUMENT, 1))
+      st= subtree (et, path_up (p, 2));
+    if (is_func (st, CELL)) break;
+    if (is_compound (st) && N(st) == 1) {
+      tree env= drd->get_env (L(st), 0);
+      tree mt= drd_env_read (env, MODE);
+      tree pt= drd_env_read (env, "prog-language");
+      if (mt == "math") break;
+      if (mt == "prog" && pt == "minimal") break;
+      if (mt == "text") return rp;
+    }
     p= path_up (p);
+  }
   return p;
 }
 
 bool
-edit_select_rep::semantic_active (path p) {
-  p= semantic_root (p);
+edit_select_rep::semantic_active (path q) {
   if (as_string (eval ("(get-preference \"semantic editing\")")) == "on") {
+    path p= semantic_root (q);
     //cout << subtree (et, p) << ", " << p << " -> " << end (et, p) << "\n";
-    tree mode= get_env_value (MODE, end (et, p));
-    tree plan= get_env_value (PROG_LANGUAGE, end (et, p));
-    return mode == "math" || (mode == "prog" && plan == "minimal");
+    tree mt= get_env_value (MODE, end (et, p));
+    tree lt= get_env_value (MODE_LANGUAGE (mt->label), end (et, p));
+    string lan= (is_atomic (lt)? lt->label: string ("std-math"));
+    if (mt == "math" || (mt == "prog" && lan == "minimal"))
+      return packrat_available_path (lan, subtree (et, p), q / p);
   }
-  else return false;
+  return false;
 }
 
 bool
 edit_select_rep::semantic_select (path p, path& q1, path& q2, int mode) {
   if (!semantic_active (p)) return false;
+  if (mode < 2 && get_preference ("semantic selections") != "on") return false;
   p= semantic_root (p);
+  while (p != rp && !(p <= q1 && p <= q2))
+    p= semantic_root (path_up (p));
   tree mt= get_env_value (MODE, end (et, p));
   tree lt= get_env_value (MODE_LANGUAGE (mt->label), end (et, p));
   string lan= (is_atomic (lt)? lt->label: string ("std-math"));
@@ -110,10 +119,20 @@ edit_select_rep::semantic_select (path p, path& q1, path& q2, int mode) {
 
 void
 edit_select_rep::select (path p1, path p2) {
+  //cout << "Select " << p1 << " -- " << p2 << "\n";
   if (start_p == p1 && end_p == p2) return;
   if (!(rp <= p1 && rp <= p2)) return;
-  if (p1 != p2)
-    (void) semantic_select (common (p1, p2), p1, p2, 0);
+  if (start_p == end_p && p1 == p2) {
+    start_p= copy (p1);
+    end_p  = copy (p2);
+    return;
+  }
+  if (p1 != p2) {
+    path cp= common (p1, p2);
+    tree st= subtree (et, cp);
+    if (!is_func (st, TABLE) && !is_func (st, ROW))
+      (void) semantic_select (cp, p1, p2, 0);
+  }
   if (path_less (p1, p2)) {
     start_p= copy (p1);
     end_p  = copy (p2);
@@ -269,6 +288,7 @@ edit_select_rep::select_enlarge () {
   path pp= sp, p1= start_p, p2= end_p;
   if (start_p == pp * 0 && end_p == pp * right_index (subtree (et, pp)))
     if (!is_nil (pp)) pp= path_up (pp);
+  if (is_func (subtree (et, pp), TFORMAT)) pp= path_up (pp);
   if (semantic_select (pp, p1, p2, 1))
     select (p1, p2);
   else {
@@ -337,14 +357,15 @@ edit_select_rep::selection_active_normal () {
 }
 
 bool
-edit_select_rep::selection_active_table () {
+edit_select_rep::selection_active_table (bool strict) {
   if (!selection_active_any ()) return false;
   path p= common (start_p, end_p);
   if ((p == start_p) || (p == end_p)) p= path_up (p);
   tree t= subtree (et, p);
   return
     is_func (t, TFORMAT) || is_func (t, TABLE) ||
-    is_func (t, ROW) || is_func (t, CELL);
+    is_func (t, ROW) || is_func (t, CELL) ||
+    (!strict && N(t) == 1 && is_func (t[0], TFORMAT));
 }
 
 bool
@@ -419,118 +440,47 @@ table_search_cell (tree t, int row, int col) {
 ******************************************************************************/
 
 void
-selection_correct (tree t, path i1, path i2, path& o1, path& o2) {
-  if (i1 == i2) {
-    o1= i1;
-    o2= i2;
-  }
-  else if (is_atom (i1) || is_atom (i2)) {
-    if (is_atomic (t)) {
-      o1= i1;
-      o2= i2;
-    }
-    else {
-      o1= start (t);
-      o2= end (t);
-    }
-  }
-  else if (i1->item == i2->item) {
-    selection_correct (t[i1->item], i1->next, i2->next, o1, o2);
-    o1= path (i1->item, o1);
-    o2= path (i2->item, o2);
-  }
-  else {
-    tree_label l= L(t);
-    if ((l==DOCUMENT) || (l==PARA) || (l==CONCAT)) {
-      if (is_compound (t[i1->item])) {
-	path mid;
-	selection_correct (t[i1->item], i1->next, end (t[i1->item]), o1, mid);
-	o1= path (i1->item, o1);
-      }
-      else o1= i1;
-      if (is_compound (t[i2->item])) {
-	path mid;
-	selection_correct (t[i2->item], start(t[i2->item]), i2->next, mid, o2);
-	o2= path (i2->item, o2);
-      }
-      else o2= i2;
-    }
-    else {
-      o1= start (t);
-      o2= end (t);
-    }
-  }
-}
-
-static void
-selection_bcorrect (drd_info drd, tree t, path i1, path i2, path& o1, path& o2)
-{
-  o1= i1; o2= i2;
-  if (is_compound (t) && !is_atom (i1) && !is_atom (i2) &&
-      i1->item == i2->item) {
-    path O1, O2;
-    selection_bcorrect (drd, t[i1->item], i1->next, i2->next, O1, O2);
-    if (drd->var_without_border (L(t[i1->item])) && (O1->item != O2->item)) {
-      o1= path (0);
-      o2= path (1);
-    }
-    else {
-      o1= path (i1->item, O1);
-      o2= path (i1->item, O2);
-    }
-  }
-}
-
-tree
-compute_selection (tree t, path start, path end) {
-  int  i1= start->item;
-  int  i2= end->item;
-  path p1= start->next;
-  path p2= end->next;
-
-  if (is_nil (p1) || is_nil (p2)) {
-    if (start == path (right_index (t))) return "";
-    if (end == path (0)) return "";
-    if (start == end) return "";
-    if (is_nil (p1) && is_nil (p2)) {
-      if (is_compound (t)) return copy (t);
-      if (i1>=i2) return "";
-      return t->label (i1, i2);
-    }
-    if (is_compound (t) && (!is_format (t))) return copy (t);
-    if (is_nil (p1)) {
-      i1= 0;
-      p1= (start->item==0? 0: right_index (t[i1]));
-    }
-    if (is_nil (p2)) {
-      i2= N(t)-1;
-      p2= (end->item==0? 0: right_index (t[i2]));
-    }
-  }
-
-  if (i1==i2) return compute_selection (t[i1], p1, p2);
-  if (is_compound (t) && (!is_format (t))) return copy (t);
-
-  int i;
-  tree r (t, i2-i1+1);
-  r[0]     = compute_selection (t[i1], p1, path (right_index (t[i1])));
-  r[N(r)-1]= compute_selection (t[i2], path (0), p2);
-  for (i=1; i<N(r)-1; i++) r[i]= copy (t[i+i1]);
-  return r;
+edit_select_rep::selection_correct (path i1, path i2, path& o1, path& o2) {
+  ASSERT (rp <= i1 && rp <= i2, "paths not inside document");
+  int old_mode= get_access_mode ();
+  if (get_init_string (MODE) == "src")
+    set_access_mode (DRD_ACCESS_SOURCE);
+  ::selection_correct (subtree (et, rp), i1 / rp, i2 / rp, o1, o2);
+  set_access_mode (old_mode);
+  o1= rp * o1; o2= rp * o2;
 }
 
 path
 edit_select_rep::selection_get_subtable (
   int& row1, int& col1, int& row2, int& col2)
 {
-  path fp= ::table_search_format (et, common (start_p, end_p));
-  tree st= subtree (et, fp);
-  table_search_coordinates (st, tail (start_p, N(fp)), row1, col1);
-  table_search_coordinates (st, tail (end_p, N(fp)), row2, col2);
-  if (row1>row2) { int tmp= row1; row1= row2; row2= tmp; }
-  if (col1>col2) { int tmp= col1; col1= col2; col2= tmp; }
-  table_bound (fp, row1, col1, row2, col2);
-  return fp;
+  if (selection_active_table ()) {
+    path fp= ::table_search_format (et, common (start_p, end_p));
+    if (is_nil (fp)) return fp;
+    tree st= subtree (et, fp);
+    table_search_coordinates (st, tail (start_p, N(fp)), row1, col1);
+    table_search_coordinates (st, tail (end_p, N(fp)), row2, col2);
+    if (row1>row2) { int tmp= row1; row1= row2; row2= tmp; }
+    if (col1>col2) { int tmp= col1; col1= col2; col2= tmp; }
+    table_bound (fp, row1, col1, row2, col2);
+    return fp;
+  }
+  else if (selection_active_table (false)) {
+    path fp= ::table_search_format (et, common (start_p, end_p) * 0);
+    if (is_nil (fp)) return fp;
+    path p= fp;
+    tree st;
+    while (true) {
+      st= subtree (et, p);
+      if (is_func (st, TABLE) && N(st) > 0 && is_func (st[0], ROW)) break;
+      if (!is_func (st, TFORMAT)) return path ();
+      p= p * (N(st) - 1);
+    }
+    row1= 0; col1= 0;
+    row2= N(st)-1; col2= N(st[0])-1;
+    return fp;
+  }
+  else return path ();
 }
 
 void
@@ -555,18 +505,29 @@ edit_select_rep::selection_get (selection& sel) {
     sel= selection (rectangles (r), fp * 0, fp * 1);
   }
   else {
-    path aux_start, aux_end, p_start, p_end;
-    selection_bcorrect (drd, et, start_p, end_p, aux_start, aux_end);
-    selection_correct (et, aux_start, aux_end, p_start, p_end);
+    path p_start, p_end;
+    //cout << "Find " << start_p << " -- " << end_p << "\n";
+    selection_correct (start_p, end_p, p_start, p_end);
+    //cout << "Find " << p_start << " -- " << p_end << "\n";
     sel= eb->find_check_selection (p_start, p_end);
+    //cout << "sel= " << sel << "\n";
   }
 }
 
 void
 edit_select_rep::selection_get (path& start, path& end) {
+  if (selection_active_table ()) {
+    int row1, col1, row2, col2;
+    path fp= selection_get_subtable (row1, col1, row2, col2);
+    start= fp * 0;
+    end= fp * 1;
+  }
+  else selection_correct (start_p, end_p, start, end);
+  /*
   selection sel; selection_get (sel);
   start= sel->start;
   end  = sel->end;
+  */
 }
 
 path
@@ -592,7 +553,7 @@ edit_select_rep::selection_get () {
     // cout << "Selecting...\n";
     selection_get (start, end);
     // cout << "Between paths: " << start << " and " << end << "\n";
-    tree t= ::compute_selection (et, start, end);
+    tree t= selection_compute (et, start, end);
     // cout << "Selection : " << t << "\n";
     return simplify_correct (t);
   }
@@ -602,22 +563,36 @@ path
 edit_select_rep::selection_get_path () {
   path start, end;
   selection_get (start, end);
+  if (end == start && end_p != start_p)
+    return path_up (start);
   return common (start, end);
 }
 
+path
+edit_select_rep::selection_get_cursor_path () {
+  if (!selection_active_any ()) return tp;
+  return start (et, selection_get_path ());
+}
+
+tree
+edit_select_rep::selection_get_env_value (string var) {
+  if (!selection_active_any ()) return get_env_value (var);
+  return get_env_value (var, selection_get_cursor_path ());
+}
+
 /******************************************************************************
 * Copy and paste
 ******************************************************************************/
 
 void
 edit_select_rep::selection_raw_set (string key, tree t) {
-  (void) ::set_selection (key, t, "");
+  (void) ::set_selection (key, t, "", "", "", "texmacs");
 }
 
 tree
 edit_select_rep::selection_raw_get (string key) {
   tree t; string s;
-  (void) ::get_selection (key, t, s);
+  (void) ::get_selection (key, t, s, "texmacs");
   return t;
 }
 
@@ -649,23 +624,34 @@ edit_select_rep::selection_set_paths (path start, path end) {
 void
 edit_select_rep::selection_set (string key, tree t, bool persistant) {
   selecting= shift_selecting= false;
-  string mode= get_env_string (MODE);
-  string lan = get_env_string (MODE_LANGUAGE (mode));
+  string mode= as_string (selection_get_env_value (MODE));
+  string lan = as_string (selection_get_env_value (MODE_LANGUAGE (mode)));
   tree sel= tuple ("texmacs", t, mode, lan);
   /* TODO: add mode="graphics" somewhere in the context of the <graphics>
      tag. To be done when implementing the different embeddings for
      nicely copying graphics into text, text into graphics, etc. */
-  string s;
+  string s, sh, sv;
   if (key == "primary" || key == "mouse") {
-    if (selection_export == "verbatim") t= exec_texmacs (t, tp);
+    if (selection_export == "verbatim") t= exec_verbatim (t, tp);
     if (selection_export == "html") t= exec_html (t, tp);
     if (selection_export == "latex") t= exec_latex (t, tp);
     if ((selection_export == "latex") && (mode == "math"))
-      t= tree (WITH, "mode", "math", t);
-    s= tree_to_generic (t, selection_export * "-snippet");
+      t= compound ("math", t);
+    if (selection_export != "default")
+      s= tree_to_generic (t, selection_export * "-snippet");
+    else {
+      s= tree_to_generic (t, "texmacs-snippet");
+#ifdef QTTEXMACS
+      tree tmp;
+      tmp= exec_verbatim (t, tp);
+      sv= tree_to_generic (tmp, "verbatim-snippet");
+      //tmp= exec_html (t, tp);
+      //sh= tree_to_generic (tmp, "html-snippet");
+#endif
+    }
     s= selection_encode (lan, s);
   }
-  if (::set_selection (key, sel, s) && !persistant)
+  if (::set_selection (key, sel, s, sv, sh, selection_export) && !persistant)
     selection_cancel ();
 }
 
@@ -695,7 +681,7 @@ edit_select_rep::selection_copy (string key) {
 void
 edit_select_rep::selection_paste (string key) {
   tree t; string s;
-  (void) ::get_selection (key, t, s);
+  (void) ::get_selection (key, t, s, selection_import);
   if (inside_active_graphics ()) {
     if (is_tuple (t, "texmacs", 3))
       call ("graphics-paste", t[1]);
@@ -704,10 +690,12 @@ edit_select_rep::selection_paste (string key) {
   if (is_tuple (t, "extern", 1)) {
     string mode= get_env_string (MODE);
     string lan = get_env_string (MODE_LANGUAGE (mode));
+    string fm;
     if ((selection_import == "latex") && (mode == "prog")) mode= "verbatim";
     if ((selection_import == "latex") && (mode == "math")) mode= "latex-math";
     if ((selection_import == "html") && (mode == "prog")) mode= "verbatim";
-    string fm= selection_import * "-snippet";
+    if (selection_import == "default") fm= "texmacs-snippet";
+    else fm= selection_import * "-snippet";
     tree doc= generic_to_tree (selection_decode(lan, as_string(t[1])), fm);
     if (is_func (doc, DOCUMENT, 1)) doc= doc[0]; // temporary fix
     insert_tree (doc);
@@ -715,15 +703,27 @@ edit_select_rep::selection_paste (string key) {
   if (is_tuple (t, "texmacs", 3)) {
     string mode= get_env_string (MODE);
     string lan = get_env_string (MODE_LANGUAGE (mode));
-    if ((mode == "prog") && (t[2] == "math")) {
+    if (is_compound (t[1], "text", 1) && mode == "text")
+      t= tuple ("texmacs", t[1][0], "text", lan);
+    if (is_compound (t[1], "math", 1) && mode == "math")
+      t= tuple ("texmacs", t[1][0], "math", lan);
+    if (mode == "math" && t[2] == "text")
+      set_message ("Error: invalid paste of text into a formula", "paste");
+    else if (mode == "prog" && t[2] == "math") {
       tree in= tuple (lan, t[1]);
       tree r= stree_to_tree (call ("plugin-math-input", tree_to_stree (in)));
       insert_tree (r);
     }
     else {
       if ((t[2] != mode) && (t[2] != "src") && (mode != "src") &&
-	  ((t[2] == "math") || (mode == "math")))
-	insert_tree (tree (WITH, copy (MODE), copy (t[2]), ""), path (2, 0));
+	  ((t[2] == "math") || (mode == "math"))) {
+        if (t[2] == "math")
+          insert_tree (compound ("math", ""), path (0, 0));
+        else if (t[2] == "text")
+          insert_tree (compound ("text", ""), path (0, 0));
+        else
+          insert_tree (tree (WITH, copy (MODE), copy (t[2]), ""), path (2, 0));
+      }
       if (is_func (t[1], TFORMAT) || is_func (t[1], TABLE)) {
 	int row, col;
 	path fp= search_format (row, col);
@@ -830,6 +830,19 @@ edit_select_rep::raw_cut (path p1, path p2) {
 
     if (is_func (st, TFORMAT))
       table_del_format (fp, row1+1, col1+1, row2+1, col2+1, "");
+
+    if (fp == search_format ()) {
+      table_correct_block_content ();
+      table_resize_notify ();
+    }
+    else {
+      observer obs= position_new (tp);
+      go_to (start (et, fp * ::table_search_cell (st, row1, col1)));
+      table_correct_block_content ();
+      table_resize_notify ();
+      go_to (position_get (obs));
+      position_delete (obs);
+    }
     return;
   }
 
@@ -885,7 +898,7 @@ edit_select_rep::selection_cut (string key) {
       go_to (p2);
       if (p2 == p1) return;
       if (key != "none") {
-        tree sel= compute_selection (et, p1, p2);
+        tree sel= selection_compute (et, p1, p2);
         selection_set (key, simplify_correct (sel));
       }
     }
@@ -909,6 +922,10 @@ edit_select_rep::selection_move () {
   position_delete (pos);
 }
 
+/******************************************************************************
+* Focus related routines
+******************************************************************************/
+
 path
 edit_select_rep::manual_focus_get () {
   return focus_p;
@@ -929,6 +946,7 @@ edit_select_rep::manual_focus_release () {
 
 path
 edit_select_rep::focus_search (path p, bool skip_flag, bool up_flag) {
+  //cout << "Search focus " << p << ", " << skip_flag << ", " << up_flag << "\n";
   if (!(rp < p)) return rp;
   tree st= subtree (et, p);
   if (!skip_flag) return p;
@@ -950,6 +968,7 @@ edit_select_rep::focus_search (path p, bool skip_flag, bool up_flag) {
 
 path
 edit_select_rep::focus_get (bool skip_flag) {
+  //cout << "Search focus " << focus_p << ", " << skip_flag << "\n";
   if (!is_nil (focus_p))
     return focus_search (focus_p, skip_flag, false);
   if (selection_active_any ())
