@@ -13,22 +13,22 @@
 #define EDITOR_H
 #include "typesetter.hpp"
 #ifdef AQUATEXMACS
-#include "Cocoa/aqua_simple_widget.h"
+#  include "Cocoa/aqua_simple_widget.h"
 #else
-#ifdef QTTEXMACS
-#include "Qt/qt_simple_widget.hpp"
-#else
-#include "Widkit/simple_wk_widget.hpp"
-#endif
+#  ifdef QTTEXMACS
+#    include "Qt/qt_simple_widget.hpp"
+#  else
+#    include "Widkit/simple_wk_widget.hpp"
+#  endif
 #endif
 #include "server.hpp"
-#include "scheme.hpp"
 #include "drd_info.hpp"
 #ifdef EXPERIMENTAL
-#include "../Style/Environment/environment.hpp"
-#include "../Style/Memorizer/memorizer.hpp"
+#  include "../Style/Environment/environment.hpp"
+#  include "../Style/Memorizer/memorizer.hpp"
 #endif
-#define TEXMACS_COPYRIGHT (string("(c) 1999-2006 by Joris van der Hoeven"))
+#include "new_data.hpp"
+#define TEXMACS_COPYRIGHT (string("(c) 1999-2013 by Joris van der Hoeven and others"))
 
 #define THE_CURSOR 1
 #define THE_FOCUS 2
@@ -40,43 +40,48 @@
 #define THE_LOCUS 128
 
 class tm_buffer_rep;
-typedef tm_buffer_rep* tm_buffer;
+class tm_view_rep;
 class server_rep;
+typedef tm_buffer_rep* tm_buffer;
+typedef tm_view_rep* tm_view;
 class modification;
+class editor;
 extern bool enable_fastenv;
 
 class editor_rep: public simple_widget_rep {
 public:
-  server_rep* sv;   // the underlying texmacs server
+  server_rep*  sv;   // the underlying texmacs server
+  widget_rep*  cvw;  // non reference counted canvas widget
+  tm_view_rep* mvw;  // master view
 
 protected:
-  tm_buffer   buf;  // the underlying buffer
-  drd_info    drd;  // the drd for the buffer
-  tree&       et;   // all TeXmacs trees
-  box         eb;   // box translation of tree
-  path        rp;   // path to the root of the document in et
-  path        tp;   // path of cursor in tree
+  tm_buffer    buf;  // the underlying buffer
+  drd_info     drd;  // the drd for the buffer
+  tree&        et;   // all TeXmacs trees
+  box          eb;   // box translation of tree
+  path         rp;   // path to the root of the document in et
+  path         tp;   // path of cursor in tree
 #ifdef EXPERIMENTAL
-  environment ste;  // environment for style rewriting
-  tree        cct;  // clean copy of the document tree
-  memorizer   mem;  // style converted document tree
+  environment  ste;  // environment for style rewriting
+  tree         cct;  // clean copy of the document tree
+  memorizer    mem;  // style converted document tree
 #endif
 
   /* exchanging information with the interface */
-  virtual SI      get_window_height () = 0;
-  virtual void    get_selection (path& start, path& end) = 0;
-  virtual void    set_selection (path start, path end) = 0;
-  virtual cursor& the_cursor () = 0;
-  virtual cursor& the_ghost_cursor () = 0;
+  virtual void      get_selection (path& start, path& end) = 0;
+  virtual void      set_selection (path start, path end) = 0;
+  virtual cursor&   the_cursor () = 0;
+  virtual cursor&   the_ghost_cursor () = 0;
 
   /* exchanging information with the typesetter */
   virtual typesetter           get_typesetter () = 0;
   virtual hashmap<string,tree> get_init () = 0;
   virtual hashmap<string,tree> get_fin () = 0;
+  virtual hashmap<string,tree> get_att () = 0;
   virtual void                 set_init (hashmap<string,tree> H= tree ("?"))=0;
   virtual void                 add_init (hashmap<string,tree> H) = 0;
   virtual void                 set_fin (hashmap<string,tree> H) = 0;
-  virtual void                 set_base_name (url name) = 0;
+  virtual void                 set_att (hashmap<string,tree> H) = 0;
 
   /* exchanging property information */
   virtual void   set_bool_property (string what, bool val) = 0;
@@ -96,6 +101,7 @@ protected:
   virtual void back_around (tree t, path p, bool forward) = 0;
   virtual void back_prime (tree t, path p, bool forward) = 0;
   virtual void back_in_around (tree t, path p, bool forward) = 0;
+  virtual void back_in_long_arrow (tree t, path p, bool forward) = 0;
   virtual void back_in_wide (tree t, path p, bool forward) = 0;
   virtual void back_in_tree (tree t, path p, bool forward) = 0;
   virtual void pre_remove_around (path p) = 0;
@@ -105,6 +111,7 @@ protected:
   virtual void back_general (path p, bool forward) = 0;
   virtual void back_in_with (tree t, path p, bool forward) = 0;
   virtual void back_in_general (tree t, path p, bool forward) = 0;
+  virtual void back_in_text_at (tree t, path p, bool forward) = 0;
 
   /* other protected subroutines */
   virtual tree kbd (string s) = 0;
@@ -112,6 +119,7 @@ protected:
   virtual path tree_path (path sp, SI x, SI y, SI delta) = 0;
   virtual void apply_changes () = 0;
   virtual void animate () = 0;
+  virtual path search_format () = 0;
   virtual path search_format (int& row, int& col) = 0;
   virtual void table_bound (path fp, int& i1, int& j1, int& i2, int& j2) = 0;
   virtual tree table_get_subtable (path p, int i1, int j1, int i2, int j2) = 0;
@@ -128,6 +136,10 @@ public:
   virtual void suspend () = 0;
   virtual void resume () = 0;
   virtual int  get_pixel_size () = 0;
+  virtual SI   get_visible_width () = 0;
+  virtual SI   get_visible_height () = 0;
+  virtual SI   get_window_width () = 0;
+  virtual SI   get_window_height () = 0;
   virtual void invalidate (SI x1, SI y1, SI x2, SI y2) = 0;
   virtual void invalidate (rectangles rs) = 0;
   virtual void notify_change (int changed) = 0;
@@ -137,6 +149,9 @@ public:
   virtual void full_screen_mode (bool flag) = 0;
   virtual void before_menu_action () = 0;
   virtual void after_menu_action () = 0;
+  virtual rectangle get_window_extents () = 0;
+  virtual cursor search_cursor (path p) = 0;
+  virtual selection search_selection (path start, path end) = 0;
   virtual int  get_input_mode () = 0;
   virtual void set_input_mode (int mode) = 0;
   virtual void set_input_normal () = 0;
@@ -157,9 +172,10 @@ public:
   virtual void mouse_click (SI x, SI y) = 0;
   virtual bool mouse_extra_click (SI x, SI y) = 0;
   virtual void mouse_drag (SI x, SI y) = 0;
-  virtual void mouse_select (SI x, SI y, int mods) = 0;
+  virtual void mouse_select (SI x, SI y, int mods, bool drag) = 0;
   virtual void mouse_paste (SI x, SI y) = 0;
   virtual void mouse_adjust (SI x, SI y) = 0;
+  virtual void mouse_adjust_selection (SI x, SI y, int mods) = 0;
   virtual void mouse_scroll (SI x, SI y, bool up) = 0;
   virtual cursor get_cursor () = 0;
   virtual void set_pointer (string name) = 0;
@@ -222,6 +238,8 @@ public:
 
   /* public routines from edit_typeset */
   virtual void     clear_local_info () = 0;
+  virtual void     set_data (new_data data) = 0;
+  virtual void     get_data (new_data& data) = 0;
   virtual SI       as_length (string l) = 0;
   virtual string   add_lengths (string l1, string l2) = 0;
   virtual string   multiply_length (double x, string l) = 0;
@@ -244,8 +262,12 @@ public:
   virtual double   get_env_double (string var_name) = 0;
   virtual double   get_init_double (string var_name) = 0;
   virtual language get_env_language () = 0;
+  virtual SI       get_page_width () = 0;
+  virtual SI       get_page_height () = 0;
   virtual tree     exec_texmacs (tree t, path p) = 0;
   virtual tree     exec_texmacs (tree t) = 0;
+  virtual tree     exec_verbatim (tree t, path p) = 0;
+  virtual tree     exec_verbatim (tree t) = 0;
   virtual tree     exec_html (tree t, path p) = 0;
   virtual tree     exec_html (tree t) = 0;
   virtual tree     exec_latex (tree t, path p) = 0;
@@ -271,9 +293,11 @@ public:
   virtual void notify_assign_node (path p, tree_label op) = 0;
   virtual void notify_insert_node (path p, tree t) = 0;
   virtual void notify_remove_node (path p) = 0;
+  virtual void notify_set_cursor (path p, tree data) = 0;
   virtual void post_notify (path p) = 0;
   virtual void clear_undo_history () = 0;
   virtual double this_author () = 0;
+  virtual void archive_state () = 0;
   virtual void start_editing () = 0;
   virtual void end_editing () = 0;
   virtual void start_slave (double a) = 0;
@@ -347,6 +371,7 @@ public:
   virtual void   table_remove_column (bool forward, bool flag= false) = 0;
   virtual int    table_nr_rows () = 0;
   virtual int    table_nr_columns () = 0;
+  virtual void   table_set_extents (int rows, int cols) = 0;
   virtual int    table_which_row () = 0;
   virtual int    table_which_column () = 0;
   virtual path   table_search_cell (int row, int col) = 0;
@@ -358,6 +383,7 @@ public:
   virtual void   table_row_decoration (bool forward) = 0;
   virtual void   table_column_decoration (bool forward) = 0;
   virtual void   table_correct_block_content () = 0;
+  virtual void   table_resize_notify () = 0;
   virtual void   set_cell_mode (string mode) = 0;
   virtual string get_cell_mode () = 0;
   virtual void   cell_set_format (string var, tree val) = 0;
@@ -416,18 +442,21 @@ public:
 
   virtual bool selection_active_any () = 0;
   virtual bool selection_active_normal () = 0;
-  virtual bool selection_active_table () = 0;
+  virtual bool selection_active_table (bool strict= true) = 0;
   virtual bool selection_active_small () = 0;
   virtual bool selection_active_enlarging () = 0;
 
   virtual void selection_raw_set (string key, tree t) = 0;
   virtual tree selection_raw_get (string key) = 0;
+  virtual void selection_correct (path i1, path i2, path& o1, path& o2) = 0;
   virtual path selection_get_subtable (int& i1, int& j1, int& i2, int& j2) = 0;
   virtual void selection_get (selection& sel) = 0;
   virtual void selection_get (path& start, path& end) = 0;
   virtual path selection_get_start () = 0;
   virtual path selection_get_end () = 0;
   virtual path selection_get_path () = 0;
+  virtual path selection_get_cursor_path () = 0;
+  virtual tree selection_get_env_value (string var) = 0;
   virtual void selection_set (string key, tree t, bool persistant= false) = 0;
   virtual void selection_set (tree t) = 0;
   virtual void selection_set_start (path p= path()) = 0;
@@ -492,6 +521,7 @@ public:
   virtual void export_ps (url ps_name,
 			  string first="1", string last="1000000") = 0;
   virtual array<int> print_snippet (url u, tree t) = 0;
+  virtual bool graphics_file_to_clipboard (url output) = 0;
   virtual void footer_eval (string s) = 0;
   virtual tree the_line () = 0;
   virtual tree the_root () = 0;
@@ -510,11 +540,15 @@ public:
   virtual void edit_test () = 0;
 
   friend class tm_window_rep;
-  friend class tm_data_rep;
   friend class tm_server_rep;
   friend class server_command_rep;
-  friend void  edit_announce (editor_rep* ed, modification mod);
-  friend void  edit_done (editor_rep* ed, modification mod);
+  friend void   edit_announce (editor_rep* ed, modification mod);
+  friend void   edit_done (editor_rep* ed, modification mod);
+  friend string get_editor_status_report ();
+  friend void   tm_failure (const char* msg);
+  friend void   set_buffer_tree (url name, tree doc);
+  friend void   set_current_view (url u);
+  friend void   focus_on_editor (editor ed);
 };
 
 class editor {
@@ -527,11 +561,11 @@ EXTEND_NULL_CODE(widget,editor);
 
 editor new_editor (server_rep* sv, tm_buffer buf);
 
-#define SERVER(cmd) {                    \
-  tm_view temp_vw= sv->get_view (false); \
-  focus_on_this_editor ();               \
-  sv->cmd;                               \
-  sv->set_view (temp_vw);                \
+#define SERVER(cmd) {                 \
+  url temp= get_current_view_safe (); \
+  focus_on_this_editor ();            \
+  sv->cmd;                            \
+  set_current_view (temp);            \
 }
 
 #endif // defined EDITOR_H
