@@ -14,6 +14,7 @@
 #include "font.hpp"
 #include "hashmap.hpp"
 #include "timer.hpp"
+#include "Freetype/tt_file.hpp"
 
 hashmap<string,tree> font_conversion ("rule");
 
@@ -34,6 +35,15 @@ font_rule (tree which, tree by) {
 * Find a font
 ******************************************************************************/
 
+string
+strip_suffix (string name) {
+  if (occurs (".", name)) {
+    int pos= search_backwards (".", name);
+    name= name (0, pos);
+  }
+  return name;
+}
+
 static bool
 matches (tree t, tree which, hashmap<string,tree>& H) {
   int i, n= arity (which);
@@ -75,9 +85,26 @@ find_font_bis (tree t) {
   if (is_tuple (t, "unicode", 3))
     return unicode_font (as_string (t[1]), as_int (t[2]), as_int (t[3]));
 
+  if (is_tuple (t, "unimath", 5)) {
+    font up= find_font (t[1]);
+    font it= find_font (t[2]);
+    font bup= find_font (t[3]);
+    font bit= find_font (t[4]);
+    font rb= find_font (t[5]);
+    if (is_nil (up)) return up;
+    if (is_nil (it)) return it;
+    if (is_nil (bup)) return bup;
+    if (is_nil (bit)) return bit;
+    if (is_nil (rb)) return rb;
+    return unicode_math_font (up, it, bup, bit, rb);
+  }
+
   if (is_tuple (t, "x", 3))
     return x_font (as_string (t[1]), as_int (t[2]), as_int (t[3]));
 
+  if (is_tuple (t, "qt", 3))
+    return qt_font (as_string (t[1]), as_int (t[2]), as_int (t[3]));
+
   if (is_tuple (t, "tex", 3))
     return tex_font (as_string (t[1]), as_int (t[2]), as_int (t[3]));
 
@@ -143,7 +170,22 @@ find_font_bis (tree t) {
     return math_font (t, fn, error_fn);
   }
 
-  if (!font_conversion->contains (t[0]->label)) return font ();
+  if (!font_conversion->contains (t[0]->label)) {
+    font_database_load ();
+    if (is_tuple (t) && N(t) == 6) {
+      string family = as_string (t[0]);
+      string variant= as_string (t[1]);
+      string series = as_string (t[2]);
+      string shape  = as_string (t[3]);
+      array<string> a= font_database_search (family, variant, series, shape);
+      //cout << t << " -> " << a << "\n";
+      for (int i=0; i<N(a); i++)
+        if (tt_font_exists (strip_suffix (a[i])))
+          return unicode_font (strip_suffix (a[i]),
+                               as_int (t[4]), as_int (t[5]));
+    }
+    return font ();
+  }
 
   tree rule= font_conversion [t[0]->label];
   int i, n= N(rule);
@@ -164,23 +206,30 @@ find_font (tree t) {
   return fn;
 }
 
+font
+find_magnified_font (tree t, double zoom) {
+  font fn= find_font (t);
+  if (is_nil (fn)) return fn;
+  return fn->magnify (zoom);
+}
+
 /******************************************************************************
 * User interface
 ******************************************************************************/
 
 font
-find_font (string family, string fn_class,
+find_font (string family, string variant,
 	   string series, string shape, int sz, int dpi)
 {
   string s=
-    family * "-" * fn_class * "-" *
+    family * "-" * variant * "-" *
     series * "-" * shape * "-" *
     as_string (sz) * "-" * as_string (dpi);
   if (font::instances->contains (s)) return font (s);
 
   tree t1 (TUPLE, 6);
   t1[0]= family;
-  t1[1]= fn_class;
+  t1[1]= variant;
   t1[2]= series; t1[3]= shape;
   t1[4]= as_string (sz); t1[5]= as_string (dpi);
   font fn= find_font (t1);
@@ -191,7 +240,7 @@ find_font (string family, string fn_class,
 
   tree t2 (TUPLE, 5);
   t2[0]= family;
-  t2[1]= fn_class; t2[2]= series;
+  t2[1]= variant; t2[2]= series;
   t2[3]= as_string (sz); t2[4]= as_string (dpi);
   fn= find_font (t2);
   if (!is_nil (fn)) {
@@ -201,7 +250,7 @@ find_font (string family, string fn_class,
 
   tree t3 (TUPLE, 4);
   t3[0]= family;
-  t3[1]= fn_class; t3[2]= as_string (sz); t3[3]= as_string (dpi);
+  t3[1]= variant; t3[2]= as_string (sz); t3[3]= as_string (dpi);
   fn= find_font (t3);
   if (!is_nil (fn)) {
     font::instances (s)= (pointer) fn.rep;
