@@ -24,13 +24,13 @@
 enum slot_id {
   SLOT_IDENTIFIER,
   SLOT_WINDOW,
-  SLOT_RENDERER,
   SLOT_VISIBILITY,
   SLOT_FULL_SCREEN,
   SLOT_NAME,
   SLOT_SIZE,
   SLOT_POSITION,
   SLOT_UPDATE,
+  SLOT_REFRESH,
   SLOT_KEYBOARD,
   SLOT_KEYBOARD_FOCUS,
   SLOT_MOUSE,
@@ -38,16 +38,18 @@ enum slot_id {
   SLOT_MOUSE_POINTER,
   SLOT_INVALIDATE,
   SLOT_INVALIDATE_ALL,
+  SLOT_INVALID,
   SLOT_REPAINT,
   SLOT_DELAYED_MESSAGE,
   SLOT_DESTROY,
 
-  SLOT_SHRINKING_FACTOR,
+  SLOT_ZOOM_FACTOR,
   SLOT_EXTENTS,
   SLOT_VISIBLE_PART,
   SLOT_SCROLLBARS_VISIBILITY,
   SLOT_SCROLL_POSITION,
   SLOT_CANVAS,
+  SLOT_SCROLLABLE,
   SLOT_CURSOR,
 
   SLOT_HEADER_VISIBILITY,
@@ -60,6 +62,8 @@ enum slot_id {
   SLOT_FOCUS_ICONS,
   SLOT_USER_ICONS_VISIBILITY,
   SLOT_USER_ICONS,
+  SLOT_SIDE_TOOLS_VISIBILITY,
+  SLOT_SIDE_TOOLS,
   SLOT_FOOTER_VISIBILITY,
   SLOT_LEFT_FOOTER,
   SLOT_RIGHT_FOOTER,
@@ -72,10 +76,9 @@ enum slot_id {
   SLOT_INPUT_TYPE,
   SLOT_INPUT_PROPOSAL,
   SLOT_FILE,
-  SLOT_DIRECTORY
-#ifdef _MBD_EXPERIMENTAL_PRINTER_WIDGET
-  , SLOT_PRINTER_SETTINGS
-#endif
+  SLOT_DIRECTORY,
+  
+  slot_id__LAST // Please leave last and don't assign integer values to members
 };
 
 class slot {
@@ -245,12 +248,6 @@ get_window (widget w) {
   return read (w, SLOT_WINDOW);
 }
 
-inline renderer
-get_renderer (widget w) {
-  // get renderer associated to widget (or NULL if the widget is not attached)
-  return query<renderer> (w, SLOT_RENDERER);
-}
-
 inline void
 set_visibility (widget w, bool flag) {
   // map or unmap a window widget
@@ -315,6 +312,14 @@ send_update (widget w) {
   send (w, SLOT_UPDATE);
 }
 
+inline void
+send_refresh (widget w, string kind) {
+  // this message is issued if the contents of one of the dynamic
+  // subwidgets (as constructed using refresh_widget) of w
+  // may have changed and needs to be recomputed
+  send<string> (w, SLOT_REFRESH, kind);
+}
+
 inline void
 send_keyboard (widget w, string key, time_t t= 0) {
   // send a key press event
@@ -384,11 +389,18 @@ send_invalidate (widget w, SI x1, SI y1, SI x2, SI y2) {
   send<SI,SI,SI,SI> (w, SLOT_INVALIDATE, x1, y1, x2, y2);
 }
 
+inline bool
+query_invalid (widget w) {
+  // does this widget has invalid regions
+  return query<bool> (w, SLOT_INVALID);
+}
+
+
 inline void
-send_repaint (widget w, SI x1, SI y1, SI x2, SI y2) {
+send_repaint (widget w, renderer ren, SI x1, SI y1, SI x2, SI y2) {
   // request widget to repaint a region;
   // the region is specified w.r.t. the top left corner of w
-  send<SI,SI,SI,SI> (w, SLOT_REPAINT, x1, y1, x2, y2);
+  send<renderer,SI,SI,SI,SI> (w, SLOT_REPAINT, ren, x1, y1, x2, y2);
 }
 
 inline void
@@ -408,9 +420,9 @@ send_destroy (widget w) {
 ******************************************************************************/
 
 inline void
-set_shrinking_factor (widget w, int sf) {
-  // set shrinking factor for canvas
-  send<int> (w, SLOT_SHRINKING_FACTOR, sf);
+set_zoom_factor (widget w, double zoom) {
+  // set zoom factor for canvas
+  send<double> (w, SLOT_ZOOM_FACTOR, zoom);
 }
 
 inline void
@@ -450,9 +462,15 @@ get_scroll_position (widget w, SI& x, SI& y) {
 }
 
 inline void
-set_canvas (widget w, widget cv) {
+set_scrollable (widget w, widget cv) {
   // set the scrollable canvas itself
-  write (w, SLOT_CANVAS, cv);
+  write (w, SLOT_SCROLLABLE, cv);
+}
+
+inline widget
+get_canvas (widget w) {
+  // get the scrollable canvas itself
+  return read (w, SLOT_CANVAS);
 }
 
 inline void
@@ -555,6 +573,24 @@ set_user_icons (widget w, widget bar) {
   write (w, SLOT_USER_ICONS, bar);
 }
 
+inline void
+set_side_tools_visibility (widget w, bool visible) {
+  // set visibility of user icons bar
+  send<bool> (w, SLOT_SIDE_TOOLS_VISIBILITY, visible);
+}
+
+inline bool
+get_side_tools_visibility (widget w) {
+  // get visibility of user icons bar
+  return query<bool> (w, SLOT_SIDE_TOOLS_VISIBILITY);
+}
+
+inline void
+set_side_tools (widget w, widget bar) {
+  // set user icons bar
+  write (w, SLOT_SIDE_TOOLS, bar);
+}
+
 inline void
 set_footer_visibility (widget w, bool visible) {
   // set visibility of footer
