@@ -112,8 +112,8 @@ as_tree (matrix<T> m) {
   return t;
 }
 
-TMPL inline ostream&
-operator << (ostream& out, matrix<T> m) {
+TMPL inline tm_ostream&
+operator << (tm_ostream& out, matrix<T> m) {
   return out << as_math_string (as_tree (m));
 }
 
@@ -144,7 +144,7 @@ unary (matrix<T> m) {
   T* a= A(m);
   T* r= tm_new_array<T> (n);
   for (i=0; i<n; i++)
-    r[i]= Op::eval (a[i]);
+    r[i]= Op::op (a[i]);
   return matrix<T> (r, NR(m), NC(m));
 }
 
@@ -164,7 +164,7 @@ binary (matrix<T> m1, matrix<T> m2) {
   T* b= A(m2);
   T* r= tm_new_array<T> (n);
   for (i=0; i<n; i++)
-    r[i]= Op::eval (a[i], b[i]);
+    r[i]= Op::op (a[i], b[i]);
   return matrix<T> (r, NR(m1), NC(m1));
 }
 
@@ -196,7 +196,9 @@ TMPL inline vector<T>
 operator * (matrix<T> m, vector<T> v) {
   int i, j, rows= NR (m), cols= NC (m);
   ASSERT (N (v) == cols, "dimensions don't match");
-  vector<T> prod (T(0), rows);
+  vector<T> prod (rows);
+  for (j=0; j<cols; j++)
+    prod[j]= T(0);
   for (i=0; i<rows; i++)
     for (j=0; j<cols; j++)
       prod [i] += m (i, j) * v[j];
@@ -207,13 +209,36 @@ TMPL inline array<T>
 operator * (matrix<T> m, array<T> v) {
   int i, j, rows= NR (m), cols= NC (m);
   ASSERT (N (v) == cols, "dimensions don't match");
-  array<T> prod (T(0), rows);
+  array<T> prod (rows);
+  for (j=0; j<cols; j++)
+    prod[j]= T(0);
   for (i=0; i<rows; i++)
     for (j=0; j<cols; j++)
       prod [i] += m (i, j) * v[j];
   return prod;
 }
 
+TMPL array<T>
+projective_apply (matrix<T> m, array<T> v) {
+  int n= NR (m);
+  ASSERT (NC (m) == n && N(v) + 1 == n, "dimensions don't match");
+  array<T> pv= copy (v);
+  pv << T (1.0);
+  array<T> pr= m * pv;
+  array<T> r (n-1);
+  for (int i=0; i<n-1; i++)
+    r[i]= pr[i] / pr[n-1];
+  return r;
+}
+
+TMPL array<array<T> >
+projective_apply (matrix<T> m, array<array<T> > v) {
+  array<array<T> > r (N(v));
+  for (int i=0; i<N(v); i++)
+    r[i]= projective_apply (m, v[i]);
+  return r;
+}
+
 #undef TMPL
 #undef BINARY_TMPL
 #undef R
