@@ -19,6 +19,7 @@
 #include "merge_sort.hpp"
 #include "scheme.hpp"
 #include "image_files.hpp"
+#include "link.hpp"
 
 string PS_CLIP_PUSH ("gsave");
 string PS_CLIP_POP ("grestore");
@@ -41,39 +42,46 @@ string PS2 ("z");
 printer_rep::printer_rep (
   url ps_file_name2, int dpi2, int nr_pages2,
   string page_type2, bool landscape2, double paper_w2, double paper_h2):
+    renderer_rep (false),
     ps_file_name (ps_file_name2), dpi (dpi2),
     nr_pages (nr_pages2), page_type (page_type2),
     landscape (landscape2), paper_w (paper_w2), paper_h (paper_h2),
-    linelen (0), fg (-1), bg (-1), ncols (0),
+    use_alpha (get_preference ("experimental alpha") == "on"),
+    linelen (0), fg ((color) (-1)), bg ((color) (-1)), ncols (0),
     lw (-1), nwidths (0), cfn (""), nfonts (0),
     xpos (0), ypos (0), tex_flag (false),
     defs ("?"), tex_chars ("?"), tex_width ("?"),
     tex_fonts ("?"), tex_font_chars (array<int>(0))    
 {
-  type_1    = get_font_type () > 0;
-
   string tex_pro, special_pro, color_pro, texps_pro;
   load_string ("$TEXMACS_PATH/misc/convert/tex.pro", tex_pro, true);
   load_string ("$TEXMACS_PATH/misc/convert/special.pro", special_pro, true);
   load_string ("$TEXMACS_PATH/misc/convert/color.pro", color_pro, true);
   load_string ("$TEXMACS_PATH/misc/convert/texps.pro", texps_pro, true);
   
-  prologue   << "%!PS-Adobe-2.0\n"
+  prologue   << "%!PS-Adobe-2.0";
+  if (suffix (ps_file_name) == "eps")
+    prologue << " EPSF-2.0";
+  prologue   << "\n"
 	     << "%%Creator: TeXmacs-" TEXMACS_VERSION "\n"
 	     << "%%Title: " << as_string (tail (ps_file_name)) << "\n"
 	     << "%%Pages: " << as_string (nr_pages) << "\n"
 	     << "%%PageOrder: Ascend\n";
   if (page_type != "user")
     prologue << "%%DocumentPaperSizes: " << page_type << "\n";
+  if (landscape) {
+    psw= (int) (28.36*paper_h+ 0.5);
+    psh= (int) (28.36*paper_w+ 0.5);
+  }
+  else {
+    psw= (int) (28.36*paper_w+ 0.5);
+    psh= (int) (28.36*paper_h+ 0.5);
+  }
+  prologue << "%%BoundingBox: 0 0 "
+           << as_string (psw) << " "
+           << as_string (psh) << "\n";
   if (landscape)
-    prologue << "%%BoundingBox: 0 0 "
-	     << as_string ((int) (28.36*paper_h+ 0.5)) << " "
-	     << as_string ((int) (28.36*paper_w+ 0.5)) << "\n"
-	     << "%%Orientation: Landscape\n";
-  else
-    prologue << "%%BoundingBox: 0 0 "
-	     << as_string ((int) (28.36*paper_w+ 0.5)) << " "
-	     << as_string ((int) (28.36*paper_h+ 0.5)) << "\n";
+    prologue << "%%Orientation: Landscape\n";
   prologue   << "%%EndComments\n\n"
 	     << tex_pro << "\n"
 	     << special_pro << "\n"
@@ -118,15 +126,24 @@ printer_rep::~printer_rep () {
 
   generate_tex_fonts ();
   prologue << "end\n"
+
+           << "systemdict /pdfmark known{userdict /?pdfmark systemdict /exec get put}{userdict /?pdfmark systemdict /pop get put userdict /pdfmark systemdict /cleartomark get put}ifelse\n"
+
            << "%%EndProlog\n\n"
 	   << "%%BeginSetup\n"
 	   << "%%Feature: *Resolution " << as_string (dpi) << "dpi\n"
 	   << "TeXDict begin\n";
-  if (page_type != "user") {
-    prologue << "%%BeginPaperSize: " << page_type << "\n";
+  prologue << "%%BeginPaperSize: " << page_type << "\n";
+  if (page_type != "user")
     prologue << page_type << "\n";
-    prologue << "%%EndPaperSize\n";
+  else {
+    prologue << "/setpagedevice where\n";
+    prologue << "{ pop << /PageSize ["
+             << as_string (psw) << " " << as_string (psh)
+             << "] >> setpagedevice }\n";
+    prologue << "if\n";
   }
+  prologue << "%%EndPaperSize\n";
   if (landscape)
     prologue << "@landscape\n";
   prologue << "%%EndSetup\n";
@@ -153,8 +170,8 @@ printer_rep::next_page () {
   set_clipping (0, (int) (-(dpi*PIXEL*paper_h)/2.54),
 		(int) ((dpi*PIXEL*paper_w)/2.54), 0);
 
-  fg  = -1;
-  bg  = -1;
+  fg  = (color) (-1);
+  bg  = (color) (-1);
   lw  = -1;
   cfn = "";
   xpos= 0;
@@ -211,11 +228,9 @@ printer_rep::print (string s) {
 
 void
 printer_rep::print (SI x, SI y) {
-  x += ox; y += oy;
-  if (x>=0) x= x/PIXEL; else x= (x-PIXEL+1)/PIXEL;
-  if (y>=0) y= y/PIXEL; else y= (y-PIXEL+1)/PIXEL;
+  decode (x, y);
   print (as_string (x-dpi));
-  print (as_string (-y-dpi));
+  print (as_string (y-dpi));
 }
 
 void
@@ -244,10 +259,18 @@ printer_rep::move_to (SI x, SI y) {
   print ("a");
 }
 
+string
+printer_rep::define_alpha (int a) {
+  string aa= as_string (((double) a) / 255.0);
+  string s= "[ /ca " * aa * " /CA " * aa * " /SetTransparency pdfmark";
+  if (!defs->contains (s)) define ("A" * as_string (a), s);
+  return defs[s];
+}
+
 void
 printer_rep::select_color (color c) {
-  int r, g, b;
-  get_rgb_color (c, r, g, b);
+  int r, g, b, a;
+  get_rgb_color (c, r, g, b, a);
   r= 10000+ ((r*1000)/255);
   g= 10000+ ((g*1000)/255);
   b= 10000+ ((b*1000)/255);
@@ -255,6 +278,7 @@ printer_rep::select_color (color c) {
   string gg= as_string (g); gg= gg(1,2) * "." * gg(2,5);
   string bb= as_string (b); bb= bb(1,2) * "." * bb(2,5);
   string s = rr * " " * gg * " " * bb * " setrgbcolor";
+  if (use_alpha) s= s * " " * define_alpha (a);
   if (!defs->contains (s)) {
     define ("C" * as_string (ncols), s);
     ncols++;
@@ -556,27 +580,38 @@ printer_rep::set_clipping (SI x1, SI y1, SI x2, SI y2, bool restore) {
 * graphical routines
 ******************************************************************************/
 
-color
-printer_rep::get_color () {
-  return fg;
+pencil
+printer_rep::get_pencil () {
+  return pen;
 }
 
-color
+brush
 printer_rep::get_background () {
-  return bg;
+  return bgb;
 }
 
 void
-printer_rep::set_color (color c) {
-  if (fg==c) return;
-  fg= c;
-  select_color (c);
+printer_rep::set_pencil (pencil pen2) {
+  pen= pen2;
+  color c= pen->get_color ();
+  if (c != fg) {
+    fg= c;
+    select_color (fg);
+  }
+  //if (pen->w != lw) {
+  // FIXME: apparently, the line width can be overidden by some of
+  // the graphical constructs (example file: newimpl.tm, in which
+  // the second dag was not printed using the right width)
+  lw= pen->get_width ();
+  select_line_width (lw);
+  //}
 }
 
 void
-printer_rep::set_background (color c) {
-  if (bg==c) return;
-  bg= c;
+printer_rep::set_background (brush b) {
+  //if (bgb==b) return;
+  bgb= b;
+  bg= b->get_color ();
 }
 
 void
@@ -597,15 +632,6 @@ printer_rep::draw (int ch, font_glyphs fn, SI x, SI y) {
   xpos += gl->lwidth;
 }
 
-void
-printer_rep::set_line_style (SI w, int type, bool round) {
-  (void) type;
-  (void) round;
-  if (lw == w) return;
-  lw= w;
-  select_line_width (w);
-}
-
 void
 printer_rep::line (SI x1, SI y1, SI x2, SI y2) {
   print (x1, y1);
@@ -678,12 +704,6 @@ printer_rep::polygon (array<SI> x, array<SI> y, bool convex) {
   print (PS_POL_END);
 }
 
-void
-printer_rep::xpm (url file_name, SI x, SI y) {
-  (void) file_name; (void) x; (void) y;
-  FAILED ("not yet implemented");
-}
-
 /*
 string
 incorporate_postscript (string s) {
@@ -704,15 +724,10 @@ incorporate_postscript (string s) {
 
 void
 printer_rep::image (
-  url u, SI w, SI h, SI x, SI y,
-  double cx1, double cy1, double cx2, double cy2)
+  string name, string eps, SI x1, SI y1, SI x2, SI y2,
+  SI w, SI h, SI x, SI y, int alpha)
 {
-  int bx1, by1, bx2, by2;
-  ps_bounding_box (u, bx1, by1, bx2, by2);
-  int x1= bx1 + (int) (cx1 * (bx2 - bx1) + 0.5);
-  int y1= by1 + (int) (cy1 * (by2 - by1) + 0.5);
-  int x2= bx1 + (int) (cx2 * (bx2 - bx1) + 0.5);
-  int y2= by1 + (int) (cy2 * (by2 - by1) + 0.5);
+  (void) alpha; // FIXME
 
   double sc_x= (72.0/dpi) * ((double) (w/PIXEL)) / ((double) (x2-x1));
   double sc_y= (72.0/dpi) * ((double) (h/PIXEL)) / ((double) (y2-y1));
@@ -755,10 +770,8 @@ printer_rep::image (
   /* @beginspecial 0 @llx 0 @lly 613.291260 @urx 613.291260 @ury 6110 @rwi
      @clip @setspecial */
   
-  string ps_image= ps_load (u);
-  string imtext= is_ramdisc (u)? "inline image": as_string (u);
-  body << "%%BeginDocument: " << imtext  << "\n";
-  body << ps_image; // incorporate_postscript (ps_image);
+  body << "%%BeginDocument: " << name  << "\n";
+  body << eps; // incorporate_postscript (eps);
   body << "%%EndDocument";
   cr ();
 
@@ -819,6 +832,85 @@ printer_rep::apply_shadow (SI x1, SI y1, SI x2, SI y2) {
   (void) x1; (void) y1; (void) x2; (void) y2;
 }
 
+renderer
+printer_rep::shadow (picture& pic, SI x1, SI y1, SI x2, SI y2) {
+  renderer ren= renderer_rep::shadow (pic, x1, y1, x2, y2);
+  ren->set_zoom_factor (1.0);
+  return ren;
+}
+
+void
+printer_rep::draw_picture (picture p, SI x, SI y, int alpha) {
+  (void) alpha; // FIXME
+  int w= p->get_width (), h= p->get_height ();
+  int ox= p->get_origin_x (), oy= p->get_origin_y ();
+  int pixel= 5*PIXEL;
+  string name= "picture";
+  string eps= picture_as_eps (p, 600);
+  int x1= -ox;
+  int y1= -oy;
+  int x2= w - ox;
+  int y2= h - oy;
+  x -= (int) 2.06 * ox * pixel; // FIXME: where does the magic 2.06 come from?
+  y -= (int) 2.06 * oy * pixel;
+  image (name, eps, x1, y1, x2, y2, w * pixel, h * pixel, x, y, 255);
+}
+
+void
+printer_rep::draw_scalable (scalable im, SI x, SI y, int alpha) {
+  if (im->get_type () != scalable_image)
+    renderer_rep::draw_scalable (im, x, y, alpha);
+  else {
+    url u= im->get_name ();
+    rectangle r= im->get_logical_extents ();
+    SI w= r->x2, h= r->y2;
+    string ps_image= ps_load (u);
+    string imtext= is_ramdisc (u)? "inline image": as_string (u);
+    int x1, y1, x2, y2;
+    ps_bounding_box (u, x1, y1, x2, y2);
+    image (imtext, ps_image, x1, y1, x2, y2, w, h, x, y, alpha);
+  }
+}
+
+void
+printer_rep::anchor (string label, SI x, SI y) {
+  string s = "(";
+  s = s << prepare_text (label) << ") cvn";
+  if (linelen>0) cr ();
+  print ("[ /Dest");
+  print (s);
+  print ("/View [/XYZ");
+  print (x, y);
+  print ("null] /DEST pdfmark");
+  cr ();
+}
+
+void
+printer_rep::href (string label, SI x1, SI y1, SI x2, SI y2) {
+  bool preserve= (get_locus_rendering ("locus-on-paper") == "preserve");
+  if (linelen>0) cr ();
+  print ("[");
+  if (starts (label, "#")) {
+    print ("/Dest");
+    print ("(" * prepare_text (label) * ") cvn");
+  }
+  else {
+    print ("/Action");
+    print ("<< /Subtype /URI /URI (" * prepare_text (label) * ") >>");
+  }
+  print ("/Rect [");
+  print (x1 - 5*PIXEL, y1 - 10*PIXEL);
+  print (x2 + 5*PIXEL, y2 + 10*PIXEL);
+  print ("]");
+  if (preserve)
+    print ("/Border [16 16 1 [3 10]] /Color [0.75 0.5 1.0]");
+  else
+    print ("/Border [16 16 0 [3 10]] /Color [0.75 0.5 1.0]");
+  print ("/Subtype /Link");
+  print ("/ANN pdfmark");
+  cr ();
+}
+
 /******************************************************************************
 * user interface
 ******************************************************************************/
@@ -827,6 +919,7 @@ renderer
 printer (url ps_file_name, int dpi, int nr_pages,
 	 string page_type, bool landscape, double paper_w, double paper_h)
 {
+  page_type= as_string (call ("standard-paper-size", object (page_type)));
   return tm_new<printer_rep> (ps_file_name, dpi, nr_pages,
-			  page_type, landscape, paper_w, paper_h);
+                              page_type, landscape, paper_w, paper_h);
 }
