@@ -47,7 +47,7 @@ CONCRETE_CODE(string);
 
 extern inline int N (string a) { return a->n; }
 string   copy (string a);
-ostream& operator << (ostream& out, string a);
+tm_ostream& operator << (tm_ostream& out, string a);
 string&  operator << (string& a, char);
 string&  operator << (string& a, string b);
 string   operator * (const char* a, string b);
@@ -62,6 +62,10 @@ double as_double (string s);
 char*  as_charp  (string s);
 string as_string_bool (bool f);
 string as_string (int i);
+string as_string (unsigned int i);
+string as_string (long int i);
+string as_string (long long int i);
+string as_string (unsigned long int i);
 string as_string (double x);
 string as_string (const char* s);
 bool   is_bool   (string s);
@@ -81,4 +85,37 @@ void  system_wait (string message, string argument= "", int level= 0);
 void  system_warning (string message, string argument= "", int level= 0);
 void  system_error (string message, string argument= "", int level= 0);
 
+/******************************************************************************
+* C-style strings with automatic memory management
+******************************************************************************/
+
+class c_string;
+class c_string_rep: concrete_struct {
+  char* value;
+  
+private:
+  inline c_string_rep (c_string_rep &): concrete_struct () {};
+    // disable copy constructor
+  inline c_string_rep& operator=(c_string_rep&) {};
+    // disable assignment
+  
+public:
+  inline c_string_rep (char* v = NULL): value (v) {}
+  inline ~c_string_rep () { if (value != NULL) tm_delete_array (value); }
+  friend class c_string;
+};
+
+class c_string {
+  CONCRETE(c_string);
+public:
+  inline c_string ():
+    rep (tm_new<c_string_rep> ()) {}
+  inline c_string (int len):
+    rep (tm_new<c_string_rep> (tm_new_array<char> (len))) {}
+  inline c_string (string s):
+    rep (tm_new<c_string_rep> (as_charp (s))) {}
+  inline operator char* () const { return rep->value; }
+};
+CONCRETE_CODE(c_string);
+
 #endif // defined STRING_H
