@@ -10,6 +10,7 @@
 ******************************************************************************/
 
 #include "bibtex_functions.hpp"
+#include "vars.hpp"
 
 /******************************************************************************
 * Helper functions
@@ -42,7 +43,7 @@ bib_parse_char (string s, int& pos, int& depth, bool& special, int& math) {
           r << s[pos];
           pos++;
         }
-	else {
+        else {
           while (pos < N(s) && (is_alpha (s[pos]) || is_digit (s[pos]))) {
             r << s[pos];
             pos++;
@@ -138,7 +139,7 @@ bib_to_latex (string s) {
   while (pos < N(s)) {
     specialsav= special;
     string ch= bib_parse_char (s, pos, depth, special, math);
-//    cerr << ch << " " << as_string (keepcase) << " " << as_string (depth) << "\n";
+    //    cerr << ch << " " << as_string (keepcase) << " " << as_string (depth) << "\n";
     if (ch == "$$") r << "$";
     else if (special || math) r << ch;
     else if (bib_is_space (ch)) {
@@ -150,7 +151,7 @@ bib_to_latex (string s) {
     }
     else if (ch == "%") r << "\\%";
     else if (bib_is_char (ch, '{') && depth > 0 &&
-	     bib_get_char (s, pos) != "\\") {
+             bib_get_char (s, pos) != "\\") {
       if (keepcase == -1) {
         r << "\\keepcase{";
         keepcase= depth;
@@ -164,7 +165,7 @@ bib_to_latex (string s) {
   }
   for (int i= keepcase; i>0; i--) r << "}";
   r << "}";
-//  cerr << r << "\n";
+  //  cerr << r << "\n";
   return r;
 }
 
@@ -231,6 +232,10 @@ bib_first_char (tree t) {
     if (beg < N(s)) return &(s[beg]);
     else return 0;
   }
+  else if (is_compound (t, "verbatim"))
+    return 0;
+  else if (is_func (t, WITH, 3) && t[0] == FONT_FAMILY && t[1] == "tt")
+    return 0;
   else {
     int pos= 0;
     if (L(t) == WITH) pos= N(t)-1;
@@ -283,7 +288,9 @@ bib_change_case (string s, string op) {
 
 void
 bib_change_case (tree& t, string (*change_case) (string)) {
+  //cout << "Change case " << t << "\n";
   if (is_atomic (t) && change_case) t->label= change_case (t->label);
+  else if (is_compound (t, "verbatim"));
   else if (L(t) == WITH) bib_change_case (t[N(t)-1], change_case);
   else if (L(t) == as_tree_label ("keepcase")) t= t[0];
   else if (L(t) == CONCAT || L(t) == DOCUMENT)
@@ -305,7 +312,13 @@ bib_upcase (scheme_tree st) {
 }
 
 scheme_tree
-bib_default (scheme_tree st) {
+bib_default_preserve_case (scheme_tree st) {
+  tree t= simplify_correct (scheme_tree_to_tree (st));
+  return tree_to_scheme_tree (t); 
+}
+
+scheme_tree
+bib_default_upcase_first (scheme_tree st) {
   tree t= simplify_correct (scheme_tree_to_tree (st));
   bib_change_case (t, 0);
   return tree_to_scheme_tree (t); 
@@ -408,7 +421,7 @@ bib_assoc (tree entry, string key) {
       if (bib_is_field (doc[i]) && doc[i][0]->label == key)
 	return doc[i][1];
   }
-  return tree ();
+  return "";
 }
 
 /******************************************************************************
@@ -484,9 +497,9 @@ get_fvl (string sfvl) {
       f= w << " " << f;
     }
   }
-  res << "\\section{}" << bib_to_latex (f);
-  res << "\\section{}" << bib_to_latex (v);
-  res << "\\section{}" << bib_to_latex (l);
+  res << "\\nextbib{}" << bib_to_latex (f);
+  res << "\\nextbib{}" << bib_to_latex (v);
+  res << "\\nextbib{}" << bib_to_latex (l);
   return res;
 }
 
@@ -510,9 +523,9 @@ get_vl_f (string svl, string sf) {
     w << words;
     f= w << " " << f;
   }
-  res << "\\section{}" << bib_to_latex (f);
-  res << "\\section{}" << bib_to_latex (v);
-  res << "\\section{}" << bib_to_latex (l);
+  res << "\\nextbib{}" << bib_to_latex (f);
+  res << "\\nextbib{}" << bib_to_latex (v);
+  res << "\\nextbib{}" << bib_to_latex (l);
   return res;
 }
 
@@ -527,7 +540,7 @@ get_vl_j_f (string svl, string sj, string sf) {
     w << words;
     j= w << " " << j;
   }
-  res << "\\section{}" << bib_to_latex (j);
+  res << "\\nextbib{}" << bib_to_latex (j);
   return res;
 }
 
@@ -554,12 +567,12 @@ get_first_von_last (string s) {
   c= get_until_char (s, pos, "");
   if (a != "" && b == "" && c == "") {
     string res= get_fvl (a);
-    res << "\\section{}";
+    res << "\\nextbib{}";
     return res;
   }
   if (a != "" && b != "" && c == "") {
     string res= get_vl_f (a, b);
-    res << "\\section{}";
+    res << "\\nextbib{}";
     return res;
   }
   if (a != "" && b != "" && c != "") return get_vl_j_f (a, b, c);
@@ -571,7 +584,7 @@ bib_names (string s, int& nbfields) {
   string res;
   int pos= 0;
   nbfields++;
-  res << "\\section{}{\\bibnames}";
+  res << "\\nextbib{}{\\bibnames}";
   while (pos < N(s)) {
     string name;
     int deb= pos;
@@ -579,7 +592,7 @@ bib_names (string s, int& nbfields) {
     int pos2= pos;
     if (pos < N(s)) pos -= 5;
     if (pos <= N(s) && deb < pos) name= s (deb, pos);
-    res << "\\section{}{\\bibname}";
+    res << "\\nextbib{}{\\bibname}";
     res << get_first_von_last (name);
     nbfields += 5;
     pos= pos2;
@@ -599,6 +612,7 @@ bib_purify_tree (tree t, string& res) {
   else if (L(t) == CONCAT || L(t) == DOCUMENT) {
     for (int i= 0; i<N(t); i++) bib_purify_tree (t[i], res);
   }
+  cout << UNINDENT;
 }
 
 string
@@ -695,6 +709,7 @@ bib_get_prefix (tree t, string& pre, int& i) {
 string
 bib_prefix (scheme_tree st, int i) {
   tree t= simplify_correct (scheme_tree_to_tree (st));
+  if (t == "others") return "+";
   string pre;
   int j= i;
   bib_get_prefix (t, pre, j);
@@ -796,16 +811,16 @@ bib_get_fields (tree t, string& latex) {
         tree f= t[i][2][j];
         if (bib_is_field (f)) {
           if ((f[0]->label == "author" || f[0]->label == "editor") &&
-	      is_atomic (f[1])) {
+              is_atomic (f[1])) {
             latex << bib_names (f[1]->label, nbfields);
           }
-	  else if (is_atomic (f[1]) && f[0]->label != "pages") {
+          else if (is_atomic (f[1]) && f[0]->label != "pages") {
             if (!bib_is_blank_string (f[1]->label)) {
-	      latex << "\\section{}" << bib_to_latex (f[1]->label);
+              latex << "\\nextbib{}" << bib_to_latex (f[1]->label);
               nbfields++;;
             }
             else {
-              latex << "\\section{}";
+              latex << "\\nextbib{}";
               nbfields++;
             }
           }
@@ -822,11 +837,13 @@ static void
 bib_set_fields (tree& t, array<tree> latex, int& ind) {
   for (int i= 0; i<N(t); i++) {
     if (bib_is_entry (t[i])) {
+      //cout << "Changing " << t[i] << "\n";
       for (int j= 0; j<N(t[i][2]); j++) {
         tree f= t[i][2][j];
+        //cout << "  Field " << f << "\n";
         if (bib_is_field (f)) {
           if ((f[0]->label == "author" || f[0]->label == "editor") &&
-	      is_atomic (f[1])) {
+              is_atomic (f[1])) {
             tree res= compound ("bib-names"); ind++;
             while (latex[ind] == compound ("bibname")) {
               tree name= compound ("bib-name"); ind++;
@@ -838,9 +855,9 @@ bib_set_fields (tree& t, array<tree> latex, int& ind) {
             }
             f[1]= res;
           }
-	  else if (f[0]->label == "pages" && is_atomic (f[1]))
+          else if (f[0]->label == "pages" && is_atomic (f[1]))
             f[1]= bib_field_pages (f[1]->label);
-	  else {
+          else {
             f[1]= latex[ind];
             ind++;
           }
@@ -854,9 +871,10 @@ bib_set_fields (tree& t, array<tree> latex, int& ind) {
 
 static array<tree>
 bib_latex_array (tree latex) {
+  if (is_func(latex, WITH)) latex= latex[N(latex) - 1];
   int i= 0;
   array<tree> res;
-  while (i < N(latex) && latex[i] == compound ("section", tree ())) {
+  while (i < N(latex) && latex[i] == compound ("nextbib", "")) {
     i++;
     if (i < N(latex)) {
       if (latex[i] == compound ("bibnames")
@@ -866,7 +884,7 @@ bib_latex_array (tree latex) {
       }
       else {
         tree elt (CONCAT);
-        while (i < N(latex) && latex[i] != compound ("section", tree ())) {
+        while (i < N(latex) && latex[i] != compound ("nextbib", "")) {
           elt << latex[i];
           i++;
         }
@@ -882,7 +900,8 @@ bib_parse_fields (tree& t) {
   string fields;
   int i= 0;
   int nb= bib_get_fields (t, fields);
-  array<tree> latex= bib_latex_array (latex_to_tree (parse_latex (fields)));
+  array<tree> latex= bib_latex_array (
+      latex_to_tree (parse_latex (fields, false, true, false, false)));
   if (nb == N(latex)) bib_set_fields (t, latex, i);
 }
 
@@ -900,7 +919,7 @@ bib_field (scheme_tree st, string field) {
         return tree_to_scheme_tree (doc[i][1]);
     }
   }
-  return tree ();
+  return "";
 }
 
 /******************************************************************************
@@ -909,7 +928,7 @@ bib_field (scheme_tree st, string field) {
 
 bool
 bib_empty (scheme_tree st, string f) {
-  return (bib_field (st, f) == tree ());
+  return (bib_field (st, f) == "");
 }
 
 /******************************************************************************
@@ -975,7 +994,7 @@ bib_strings_dict (tree t) {
           }
           dict(key)= sval;
         }
-	else if (is_atomic (val))
+        else if (is_atomic (val))
           dict(key)= val->label;
       }
     }
@@ -1030,7 +1049,7 @@ bib_select_entries (tree t, tree bib_t) {
       r->insert (b);
       entries << h[b];
       tree cr= bib_assoc (h[b], string ("crossref"));
-      if (cr != tree ()) bt << as_string (cr);
+      if (cr != "") bt << as_string (cr);
     }
   }
   return entries;
