@@ -138,8 +138,6 @@ aqua_gui_rep::clear_selection (string key) {
  * Miscellaneous
  ******************************************************************************/
 
-void aqua_gui_rep::image_gc (string name) { (void) name; }
-// FIXME: remove this unused function
 void aqua_gui_rep::set_mouse_pointer (string name) { (void) name; }
 // FIXME: implement this function
 void aqua_gui_rep::set_mouse_pointer (string curs_name, string mask_name)  { (void) curs_name; (void) mask_name; } ;
@@ -429,7 +427,7 @@ set_default_font (string name) {
 }
 
 font
-get_default_font (bool tt, bool mini) {
+get_default_font (bool tt, bool mini, bool bold) {
   (void) tt; (void) mini;
   // get the default font or monospaced font (if tt is true)
 	
@@ -457,14 +455,17 @@ load_system_font (string family, int size, int dpi,
   // Copy a selection 't' with string equivalent 's' to the clipboard 'cb'
   // Returns true on success
 bool
-set_selection (string key, tree t, string s) {
+set_selection (string key, tree t,
+               string s, string sv, string sh, string format) {
+  (void) format;
   return the_gui->set_selection (key, t, s);
 }
 
   // Retrieve the selection 't' with string equivalent 's' from clipboard 'cb'
   // Returns true on success; sets t to (extern s) for external selections
 bool
-get_selection (string key, tree& t, string& s) {
+get_selection (string key, tree& t, string& s, string format) { 
+  (void) format;
   return the_gui->get_selection (key, t, s);
 }
 
@@ -496,12 +497,6 @@ bool check_event (int type)
   // we check for keyboard events while repainting windows
 { return the_gui->check_event(type); }
 
-void image_gc (string name) {
-  // Garbage collect images of a given name (may use wildcards)
-  // This routine only needs to be implemented if you use your own image cache
-  the_aqua_renderer()->image_gc(name); 
-}
-
 void
 show_help_balloon (widget balloon, SI x, SI y) { 
   // Display a help balloon at position (x, y); the help balloon should
@@ -518,10 +513,14 @@ show_wait_indicator (widget base, string message, string argument) {
   the_gui->show_wait_indicator(base,message,argument); 
 }
 
-font x_font (string family, int size, int dpi)
-{
-  (void) family; (void) size; (void) dpi;
-  if (DEBUG_EVENTS) cout << "x_font(): SHOULD NOT BE CALLED\n";
-  return NULL;
+void
+external_event (string type, time_t t) {
+  // External events, such as pushing a button of a remote infrared commander
+#if 0
+  QTMWidget *tm_focus = qobject_cast<QTMWidget*>(qApp->focusWidget());
+  if (tm_focus) {
+    simple_widget_rep *wid = tm_focus->tm_widget();
+    if (wid) the_gui -> process_keypress (wid, type, t);
+  }
+#endif
 }
-
