@@ -64,6 +64,12 @@ aqua_widget_rep::popup_window_widget (string s) {
 }
 
 
+
+TMMenuItem *
+aqua_widget_rep::as_menuitem() { 
+  return [TMMenuItem alloc];  
+}
+
 /******************************************************************************
 * aqua_view_widget_rep
 ******************************************************************************/
@@ -411,7 +417,7 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
       [[sv documentView] setFrameSize: sz];
     }
     break;
-  case SLOT_HEADER_VISIBILITY:
+ case SLOT_HEADER_VISIBILITY:
     {
       TYPE_CHECK (type_box (val) == type_helper<bool>::id);
       bool f= open_box<bool> (val);
@@ -427,7 +433,7 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
       updateVisibility();
     }
     break;
-  case SLOT_CONTEXT_ICONS_VISIBILITY:
+  case SLOT_MODE_ICONS_VISIBILITY:
     {
       TYPE_CHECK (type_box (val) == type_helper<bool>::id);
       bool f= open_box<bool> (val);
@@ -496,14 +502,14 @@ aqua_tm_widget_rep::send (slot s, blackbox val) {
     }
     break;
     
-  case SLOT_SHRINKING_FACTOR:
+  case SLOT_ZOOM_FACTOR:
     {
       TYPE_CHECK (type_box (val) == type_helper<int>::id);
       simple_widget_rep *w = (simple_widget_rep *)[(TMView*)[sv documentView] widget];
       if (w) {
-        int new_sf = open_box<int> (val);
-        if (DEBUG_EVENTS) cout << "New shrinking factor :" << new_sf << LF;
-        w->handle_set_shrinking_factor (new_sf);
+        double new_zoom = open_box<double> (val);
+        if (DEBUG_EVENTS) cout << "New zoom factor :" << new_zoom << LF;
+        w->handle_set_zoom_factor (new_zoom);
       }
       break;
     }
@@ -567,7 +573,7 @@ aqua_tm_widget_rep::query (slot s, int type_id) {
     TYPE_CHECK (type_id == type_helper<bool>::id);
     return close_box<bool> (visibility[3]);
         
-  case SLOT_CONTEXT_ICONS_VISIBILITY:
+  case SLOT_MODE_ICONS_VISIBILITY:
     TYPE_CHECK (type_id == type_helper<bool>::id);
     return close_box<bool> (visibility[2]);
     
@@ -668,9 +674,9 @@ TMMenuHelper *the_menu_helper = nil;
 void
 aqua_tm_widget_rep::write (slot s, blackbox index, widget w) {
   switch (s) {
-  case SLOT_CANVAS: 
+  case SLOT_SCROLLABLE: 
     {
-      check_type_void (index, "SLOT_CANVAS");
+      check_type_void (index, "SLOT_SCROLLABLE");
       NSView *v = ((aqua_view_widget_rep*) w.rep)->view;
       [sv setDocumentView: v];
       [[sv window] makeFirstResponder:v];
@@ -685,14 +691,19 @@ aqua_tm_widget_rep::write (slot s, blackbox index, widget w) {
     [bc setMenu:to_nsmenu(w) forRow:0];
     layout();
     break;
-  case SLOT_CONTEXT_ICONS:
-    check_type_void (index, "SLOT_CONTEXT_ICONS");
+  case SLOT_MODE_ICONS:
+    check_type_void (index, "SLOT_MODE_ICONS");
     [bc setMenu:to_nsmenu(w) forRow:1];
     layout();
     break;
+  case SLOT_FOCUS_ICONS:
+    check_type_void (index, "SLOT_FOCUS_ICONS");
+    [bc setMenu:to_nsmenu(w) forRow:2];
+    layout();
+    break;
   case SLOT_USER_ICONS:
     check_type_void (index, "SLOT_USER_ICONS");
-    [bc setMenu:to_nsmenu(w) forRow:2];
+    [bc setMenu:to_nsmenu(w) forRow:3];
     layout();
     break;
   case SLOT_INTERACTIVE_PROMPT:
@@ -804,6 +815,10 @@ aqua_window_widget_rep::send (slot s, blackbox val) {
     NOT_IMPLEMENTED ;
     // send_update (THIS, val);
     break;
+  case SLOT_REFRESH:
+    NOT_IMPLEMENTED ;
+    // send_refresh (THIS, val);
+    break;
     
   default:
     FAILED ("cannot handle slot type");
@@ -910,8 +925,8 @@ simple_widget_rep::handle_mouse (string kind, SI x, SI y, int mods, time_t t) {
 }
 
 void
-simple_widget_rep::handle_set_shrinking_factor (int sf) {
-  (void) sf;
+simple_widget_rep::handle_set_zoom_factor (double zoom) {
+  (void) zoom;
 }
 
 void
@@ -927,10 +942,51 @@ simple_widget_rep::handle_repaint (SI x1, SI y1, SI x2, SI y2) {
 
 void
 simple_widget_rep::send (slot s, blackbox val) {
-  aqua_view_widget_rep::send(s, val);
+  if (DEBUG_QT) cout << "aqua_simple_widget_rep::send " << slot_name(s) << LF;
+  switch (s) {
+    case SLOT_INVALIDATE:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<coord4>::id);
+      coord4 p= open_box<coord4> (val);
+      if (DEBUG_QT)
+        cout << "Invalidating rect " << rectangle(p.x1,p.x2,p.x3,p.x4) << LF;
+      aqua_renderer_rep* ren = (aqua_renderer_rep*)get_renderer (this);
+      if (ren) {
+        SI x1 = p.x1, y1 = p.x2, x2 = p.x3, y2 = p.x4;    
+        ren->outer_round (x1, y1, x2, y2);
+        ren->decode (x1, y1);
+        ren->decode (x2, y2);
+        //tm_canvas()->invalidate_rect (x1,y2,x2,y1);
+      }
+    }
+      break;
+    case SLOT_INVALIDATE_ALL:
+    {
+      ASSERT (is_nil (val), "type mismatch");
+      if (DEBUG_QT)
+        cout << "Invalidating all"<<  LF;
+      //tm_canvas()->invalidate_all ();
+    }
+      break;
+    case SLOT_CURSOR:
+    {
+      TYPE_CHECK (type_box (val) == type_helper<coord2>::id);
+      coord2 p= open_box<coord2> (val);
+      //QPoint pt = to_qpoint(p);
+      //tm_canvas() -> setCursorPos(pt);
+    }
+      break;
+      
+    default:
+      if (DEBUG_QT) cout << "[aqua_simple_widget_rep] ";
+      aqua_view_widget_rep::send (s, val);
+      //      FAILED ("unhandled slot type");
+  }
+  
 }
 
 
+
 blackbox
 simple_widget_rep::query (slot s, int type_id) {
   return aqua_view_widget_rep::query(s,type_id);
@@ -965,7 +1021,7 @@ simple_widget_rep::write (slot s, blackbox index, widget w) {
 #pragma mark Widget interface
 
 
-widget plain_window_widget (widget w, string s) 
+widget plain_window_widget (widget w, string s, command c)
 // creates a decorated window with name s and contents w
 {
   return concrete(w)->plain_window_widget(s);
@@ -1052,3 +1108,58 @@ wait_widget (SI width, SI height, string message) {
   (void) width; (void) height; (void) message;
   return widget(); 
 }
+
+
+
+/******************************************************************************
+ * TeXmacs interface for the creation of widgets.
+ * See Graphics/Gui/widget.hpp for comments.
+ ******************************************************************************/
+
+//widget horizontal_menu (array<widget> arr) { return widget(); }
+//widget vertical_menu (array<widget> arr) { return widget(); }
+//widget horizontal_list (array<widget> arr) { return widget(); }
+//widget vertical_list (array<widget> arr)  { return widget(); }
+widget aligned_widget (array<widget> lhs, array<widget> rhs, SI hsep, SI vsep, SI lpad, SI rpad)  { return widget(); }
+widget tabs_widget (array<widget> tabs, array<widget> bodies)  { return widget(); }
+widget icon_tabs_widget (array<url> us, array<widget> ts, array<widget> bs)  { return widget(); }
+widget wrapped_widget (widget w, command cmd) { return widget(); }
+//widget tile_menu (array<widget> a, int cols)  { return widget(); }
+//widget minibar_menu (array<widget> arr)  { return widget(); }
+//widget menu_separator (bool vertical)  { return widget(); }
+//widget menu_group (string name, int style)  { return widget(); }
+//widget pulldown_button (widget w, promise<widget> pw)  { return widget(); }
+//widget pullright_button (widget w, promise<widget> pw)  { return widget(); }
+//widget menu_button (widget w, command cmd, string pre, string ks, int style)  { return widget(); }
+//widget balloon_widget (widget w, widget help)  { return widget(); }
+//widget text_widget (string s, int style, color col, bool tsp)  { return widget(); }
+//widget xpm_widget (url file_name)  { return widget(); }
+widget toggle_widget (command cmd, bool on, int style)  { return widget(); }
+widget enum_widget (command cmd, array<string> vals, string val, int style, string width)  { return widget(); }
+widget choice_widget (command cmd, array<string> vals, array<string> chosen) { return widget(); }
+widget choice_widget (command cmd, array<string> vals, string cur) { return widget(); }
+widget choice_widget (command cmd, array<string> vals, string cur, string filter)  { return widget(); }
+widget user_canvas_widget (widget wid, int style)  { return widget(); }
+widget resize_widget (widget w, int style, string w1, string h1,
+                      string w2, string h2, string w3, string h3)  { return widget(); }
+widget hsplit_widget (widget l, widget r)  { return widget(); }
+widget vsplit_widget (widget t, widget b)  { return widget(); }
+widget refresh_widget (string tmwid, string kind)  { return widget(); }
+//widget glue_widget (bool hx, bool vx, SI w, SI h)  { return widget(); }
+//widget glue_widget (tree col, bool hx, bool vx, SI w, SI h)  { return widget(); }
+//widget inputs_list_widget (command call_back, array<string> prompts)  { return widget(); }
+//widget input_text_widget (command call_back, string type, array<string> def,
+//                          int style, string width)  { return widget(); }
+//widget color_picker_widget (command call_back, bool bg, array<tree> proposals)  { return widget(); }
+//widget file_chooser_widget (command cmd, string type, bool save)  { return widget(); }
+//widget printer_widget (command cmd, url ps_pdf_file)  { return widget(); }
+//widget texmacs_widget (int mask, command quit)  { return widget(); }
+widget ink_widget (command cb)  { return widget(); }
+
+//// Widgets which are not strictly required by TeXmacs have void implementations
+
+//widget empty_widget () { NOT_IMPLEMENTED; return widget(); }
+//widget extend (widget w, array<widget> a) { (void) a; return w; }
+//widget wait_widget (SI width, SI height, string message) {
+//  (void) width; (void) height; (void) message; return widget();
+//}
