@@ -1,7 +1,7 @@
 
 /******************************************************************************
 * MODULE     : QTMGuiHelper.hpp
-* DESCRIPTION: QT Gui helper class
+* DESCRIPTION: QT Gui helper class. Infrastructure for delayed menu installation 
 * COPYRIGHT  : (C) 2008 Massimiliano Gubinelli
 *******************************************************************************
 * This software falls under the GNU general public license version 3 or later.
@@ -15,15 +15,33 @@
 #include "qt_gui.hpp"
 #include <QObject>
 
+/*!
+ */
 class QTMGuiHelper : public QObject {
   Q_OBJECT
   qt_gui_rep* gui;
-  
+
 public:
   inline QTMGuiHelper (qt_gui_rep *_gui) : QObject (), gui (_gui) {}
 
+protected:
+  bool eventFilter (QObject *obj, QEvent *event);
+  
 public slots:
   void doUpdate ();
+  void doRefresh ();
+  void doReadSocketNotification (int socket);  
+  void doWriteSocketNotification (int socket);  
+  
+  void aboutToShowMainMenu ();
+  void aboutToHideMainMenu ();
+  void doPopWaitingWidgets ();
+ 
+  void emitTmSlotRefresh (string kind);
+
+signals:
+  void refresh ();
+  void tmSlotRefresh (string); //!< qt_widgets which need to refresh connect here.
 };
 
 #endif // QTMGUIHELPER_HPP
