@@ -9,9 +9,8 @@
 * in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
-#include <QtGui>
-
 #include "QTMWidget.hpp"
+#include "qt_widget.hpp"
 #include "qt_renderer.hpp"
 #include "qt_gui.hpp"
 #include "qt_utilities.hpp"
@@ -29,12 +28,18 @@
 #include <QX11Info>
 extern Drawable qt_x11Handle(const QPaintDevice *pd);
 extern const QX11Info *qt_x11Info(const QPaintDevice *pd);
-#undef KeyPress  // conflict between QEvent::KeyPrees and X11 defintion
+#undef KeyPress  // conflict between QEvent::KeyPress and X11 defintion
 #endif // Q_WS_X11
 
 #endif // USE_CAIRO
 
 #include <QEvent>
+#include <QSet>
+#include <QResizeEvent>
+#include <QKeyEvent>
+#include <QPaintEvent>
+#include <QMouseEvent>
+#include <QFocusEvent>
 
 #define PIXEL 256
 
@@ -150,39 +155,81 @@ initkeymap () {
 }
 
 
-
-QTMWidget::QTMWidget (simple_widget_rep *_wid) 
-  : QTMScrollView (), backingPixmap(), imwidget(NULL) {
-  setObjectName("A QTMWidget");
-  setProperty ("texmacs_widget", QVariant::fromValue ((void*) _wid));
-  QAbstractScrollArea::viewport()->setMouseTracking (true);
+/*! Constructor.
+ 
+  \param _parent The parent QWidget.
+  \param _tmwid the TeXmacs widget who owns this object.
+ */
+QTMWidget::QTMWidget (QWidget* _parent, qt_simple_widget_rep* _tmwid) 
+  : QTMScrollView (_parent), backingPixmap(1,1), imwidget(NULL)
+{
+  setObjectName ("A QTMWidget");
   setFocusPolicy (Qt::StrongFocus);
-  backing_pos = origin;
-  setAttribute(Qt::WA_InputMethodEnabled);
+  setAttribute (Qt::WA_InputMethodEnabled);
+  all_widgets.insert (this);
+
+  backing_pos = origin ();
+  surface()->setMouseTracking (true);
+  if(_tmwid)
+    set_tm_widget (_tmwid);
 }
 
 
 QTMWidget::~QTMWidget () {
-  if (DEBUG_QT) cout << "destroying " << this << LF;
+  if (DEBUG_QT) 
+    cout << "Destroying QTMWidget " << (long)this 
+         << " of widget " << (tm_widget() ? tm_widget()->type_as_string() : "NULL") 
+         << LF;
+  
+    // remove ourselves from the list of QWidgets to be repainted.
+  all_widgets.remove(this);
 }
 
+/*! Sets the underlying qt_simple_widget_rep for this object.
+ 
+ Because construction of QTMWidgets is always done at a stage where either 
+ there is no such object (see qt_tm_embedded_rep's constructor) or it is not yet
+ properly initialized (see qt_simple_widget_rep's constructor), we need this
+ extra method.
+*/
+void
+QTMWidget::set_tm_widget (qt_simple_widget_rep* _tmwid) {
+  tmwid = widget(_tmwid);
+}
+
+qt_simple_widget_rep*
+QTMWidget::tm_widget () const { 
+  return concrete_simple_widget(tmwid); 
+}
+
+
 void 
 QTMWidget::invalidate_rect (int x1, int y1, int x2, int y2) {
-  // larger rectangle is needed to avoid artifacts during cursor movements
-  // see bug #28664
-  rectangle r = rectangle (x1-3, y1-1, x2+4, y2+1);
+#ifdef Q_WS_MAC
+  //HACK: for unknown reasons we need to enlarge the invalid rect to prevent
+  //artifacts while moving the cursor (for example at the end of a formula like
+  // $a+f$. These artifacts seems present only on 64 bit Macs. 
+  rectangle r = rectangle (x1-10, y1-10, x2+10, y2+10);
+#else
+  rectangle r = rectangle (x1, y1, x2, y2);
+#endif
   // cout << "invalidating " << r << LF;
   invalid_regions = invalid_regions | rectangles (r);
 }
 
 void 
 QTMWidget::invalidate_all () {
-  QSize sz = QAbstractScrollArea::viewport()->size();
+  QSize sz = surface()->size();
+ // QPoint pt = QAbstractScrollArea::viewport()->pos();
    //cout << "invalidate all " << LF;
   invalid_regions = rectangles();
   invalidate_rect (0, 0, sz.width(), sz.height());
 }
 
+bool
+QTMWidget::is_invalid () {
+  return !is_nil(invalid_regions);
+}
 
 basic_renderer_rep* 
 QTMWidget::getRenderer() {
@@ -214,27 +261,28 @@ QTMWidget::getRenderer() {
   return ren;
 }
 
+/*
+ This function is called by the qt_gui::update method to keep the backing
+ store in sync and propagate the changes to the surface on screen.
+ First we check that the backing store geometry is right and then we
+ request to the texmacs canvas widget to repaint the regions which were
+ marked invalid. Subsequently, for each succesfully repainted region, we
+ propagate its contents from the backing store to the onscreen surface.
+ If repaint has been interrupted we do not propagate the changes and proceed
+ to mark the region invalid again.
+*/
 void
 QTMWidget::repaint_invalid_regions () {
 
-  // this function is called by the qt_gui::update method to keep the backing
-  // store in sync and propagate the changes to the surface on screen.
-  // first we check that the backing store geometry is right and then we
-  // request to the texmacs canvas widget to repaint the regions which were
-  // marked invalid. Subsequently, for each succesfully repainted region, we
-  // propagate its contents from the backing store to the onscreen surface.
-  // If repaint has been interrupted we do not propagate the changes and proceed
-  // to mark the region invalid again.
-
   QRegion qrgn; 
-  // qrgn is to keep track of the area on the sceen which needs to be updated 
+  // qrgn is to keep track of the area on the screen which needs to be updated 
 
   // update backing store origin wrt. TeXmacs document
-  if ( backing_pos != origin ) {
+  if ( backing_pos != origin() ) {
 
-    int dx =  origin.x() - backing_pos.x();
-    int dy =  origin.y() - backing_pos.y();
-    backing_pos = origin;
+    int dx =  origin().x() - backing_pos.x();
+    int dy =  origin().y() - backing_pos.y();
+    backing_pos = origin();
     
     QPixmap newBackingPixmap (backingPixmap.size());
     QPainter p (&newBackingPixmap);
@@ -255,9 +303,8 @@ QTMWidget::repaint_invalid_regions () {
       //cout << r << " ---> " << q << LF;
       invalid_regions = invalid_regions->next;
     }
-    invalid_regions= invalid & 
-    rectangles(rectangle(0,0,
-                         sz.width(),sz.height())) ;
+    invalid_regions= invalid & rectangles(rectangle(0,0,
+                                                    sz.width(),sz.height()));
     
     if (dy<0) 
       invalidate_rect(0,0,sz.width(),min(sz.height(),-dy));
@@ -269,8 +316,8 @@ QTMWidget::repaint_invalid_regions () {
     else if (dx>0)
       invalidate_rect(max(0,sz.width()-dx),0,sz.width(),sz.height());
     
-    // we cal update now to allow repainint of invalid regions
-    // this cannot be done directly since interpose handler needs
+    // we call update now to allow repainting of invalid regions
+    // this cannot be done directly since interpose_handler needs
     // to be run at least once in some situations
     // (for example when scrolling is initiated by TeXmacs itself)
     //the_gui->update();
@@ -279,19 +326,33 @@ QTMWidget::repaint_invalid_regions () {
     qrgn += QRect(QPoint(0,0),sz);
   }
   
+    //cout << "   repaint QPixmap of size " << backingPixmap.width() << " x " 
+    // << backingPixmap.height() << LF;
   // update backing store size
   {
     QSize _oldSize = backingPixmap.size();
-    QSize _newSize = QAbstractScrollArea::viewport()->size();
+    QSize _newSize = surface()->size();
+    
+      //cout << "      surface size of " << _newSize.width() << " x " 
+      // << _newSize.height() << LF;
+    
+    
     if (_newSize != _oldSize) {
       // cout << "RESIZING BITMAP"<< LF;
       QPixmap newBackingPixmap (_newSize);
-      //QPainter p (&newBackingPixmap);
-      //p.drawPixmap(0,0,backingPixmap);
-      //p.end();
+      QPainter p (&newBackingPixmap);
+      p.drawPixmap(0,0,backingPixmap);
+      //p.fillRect(0, 0, _newSize.width(), _newSize.height(), Qt::red);
+      if (_newSize.width() >= _oldSize.width()) {
+        invalidate_rect(_oldSize.width(), 0, _newSize.width(), _newSize.height());
+        p.fillRect(QRect(_oldSize.width(), 0, _newSize.width()-_oldSize.width(), _newSize.height()), Qt::gray);
+      }
+      if (_newSize.height() >= _oldSize.height()) {
+        invalidate_rect(0,_oldSize.height(), _newSize.width(), _newSize.height());
+        p.fillRect(QRect(0,_oldSize.height(), _newSize.width(), _newSize.height()-_oldSize.height()), Qt::gray);
+      }
+      p.end();
       backingPixmap = newBackingPixmap;
-      invalidate_all();
-      the_gui -> process_resize(tm_widget(), 0, 0); // FIXME
     }
   }
   
@@ -304,27 +365,26 @@ QTMWidget::repaint_invalid_regions () {
         invalid_regions= rectangles (lub);
       
       basic_renderer_rep* ren = getRenderer();
-      tm_widget()->set_current_renderer(ren);
       
-      SI ox = -backing_pos.x()*PIXEL;
+      SI ox = -backing_pos.x()*PIXEL;  // Warning: this is NOT from_qpoint()
       SI oy = backing_pos.y()*PIXEL;
       
       rectangles rects = invalid_regions;
       invalid_regions = rectangles();
-      
+
       while (!is_nil (rects)) {
         rectangle r = copy (rects->item);
         rectangle r0 = rects->item;
         QRect qr = QRect(r0->x1, r0->y1, r0->x2 - r0->x1, r0->y2 - r0->y1);
         //cout << "repainting " << r0 << "\n";
-        ren->set_origin(ox,oy); 
+        ren->set_origin(ox, oy); 
         ren->encode (r->x1, r->y1);
         ren->encode (r->x2, r->y2);
         ren->set_clipping (r->x1, r->y2, r->x2, r->y1);
-        tm_widget()->handle_repaint (r->x1, r->y2, r->x2, r->y1);
-        if (ren->interrupted ()) {
+        tm_widget()->handle_repaint (ren, r->x1, r->y2, r->x2, r->y1);
+        if (gui_interrupted ()) {
           //cout << "interrupted repainting of  " << r0 << "\n";
-          //ren->set_color(green);
+          //ren->set_pencil (green);
           //ren->line(r->x1, r->y1, r->x2, r->y2);
           //ren->line(r->x1, r->y2, r->x2, r->y1);
           invalidate_rect (r0->x1, r0->y1, r0->x2, r0->y2);
@@ -333,61 +393,75 @@ QTMWidget::repaint_invalid_regions () {
         rects = rects->next;
       }
       
-      tm_widget()->set_current_renderer(NULL);
       ren->end();
     } // !is_nil(invalid_regions)
     
   }
 
   // propagate immediatly the changes to the screen  
-  QAbstractScrollArea::viewport()->repaint(qrgn);
+  surface()->repaint(qrgn);
   
 }
 
 void 
 QTMWidget::scrollContentsBy ( int dx, int dy ) {
   QTMScrollView::scrollContentsBy (dx,dy);
-  // the_gui::update needs to be run as soon as possible to refresh the status
-  // of the widget.
-  needs_update(); 
+
+  force_update();
+  // we force an update of the internal state to be in sync with the moving
+  // scrollbars
 }
 
 void 
 QTMWidget::resizeEvent( QResizeEvent* event ) {
-  // cout << "QTMWidget::resizeEvent (" << event->size().width()
-  //      << "," << event->size().height() << ")" << LF;
-  QTMScrollView::resizeEvent (event);
-  // the_gui::update needs to be run as soon as possible to refresh the status
-  // of the widget.
-  needs_update(); 
+  (void) event;
+  // Is this ok?
+  //coord2 s = from_qsize(event->size());
+  //the_gui -> process_resize(tm_widget(), s.x1, s.x2);
+
+  // force_update();
+
+  //FIXME: I would like to have a force_update here but this causes a failed
+  //assertion in TeXmacs since the at the boot not every internal structure is
+  //initialized at this point. It seems not too difficult to fix but I
+  //postpone this to discuss with Joris. 
+  //
+  //Not having a force_update results in some lack of sync of the surface
+  //while the user is actively resizing with the mouse.
 }
 
+void
+QTMWidget::resizeEventBis (QResizeEvent *event) {
+  coord2 s = from_qsize(event->size());
+  the_gui -> process_resize(tm_widget(), s.x1, s.x2);
+}
+
+
+
+/*!
+ In the current implementation repainting takes place during the call to
+ the widget's repaint_invalid_regions() method in the_gui::update. All
+ we have to do is to take the backing store and put it on screen according
+ to the QRegion marked invalid. 
+ CHECK: Maybe just putting onscreen all the region bounding rectangles might 
+ be less expensive.
+*/
 void
 QTMWidget::paintEvent (QPaintEvent* event) {
-  // In the current implementation repainting take place during the call to
-  // the widget's repaint_invalid_regions method in the_gui::update. All
-  // we have to do is to take the backing store and put it on screen according
-  // to the QRegion marked invalid. 
-  // CHECK: Maybe just put onscreen all the region bounding rectangle could not 
-  // be so expensive.
-  
-  
-  if (DEBUG_QT) 
-  {
+  /*
+  if (DEBUG_QT) {
     QRect rect = event->rect ();
     cout << "paintEvent ("<< rect.x() << "," <<  rect.y()
     << "," <<  rect.width() << "," <<  rect.height() << ")" << LF ;
   }
-    
-  {    
-    QPainter p (QAbstractScrollArea::viewport());
-    QVector<QRect> rects = event->region().rects();
-    for (int i=0; i< rects.count(); i++) {
-      QRect qr = rects.at(i);
-      p.drawPixmap(qr,backingPixmap,qr);
-    }
+  */
+
+  QPainter p (surface());
+  QVector<QRect> rects = event->region().rects();
+  for (int i=0; i< rects.count(); i++) {
+    QRect qr = rects.at(i);
+    p.drawPixmap(qr,backingPixmap,qr);
   }
-  
 }
 
 
@@ -403,8 +477,7 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
 
   if (DEBUG_QT)
     cout << "keypressed\n";
-  simple_widget_rep *wid =  tm_widget();
-  if (!wid) return;
+  if (tm_widget()->ref_count == 0) return;
 
   {
     int key = event->key();
@@ -422,12 +495,31 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
     }
 
     string r;
+#ifdef __MINGW32__
+    // denis begin
+    if (event->text().count() == 1) {
+      QChar c= event->text()[0];
+      if (c.isPrint() && event->modifiers() != Qt::MetaModifier) {
+        // not a control character or dead key or modifier
+        char ac=c.toAscii();
+        if (ac && ac != ' ') { // a true ascii printable
+          r= ac;
+          if (DEBUG_QT) cout << "ascii key= " <<r << "\n";	
+          the_gui->process_keypress(tm_widget(), r, texmacs_time());
+          return;
+        }
+      }
+    }
+    // denis end
+#endif
     if (qtkeymap->contains (key)) {
       r = qtkeymap[key];
-    } else if (qtdeadmap->contains (key)) {
+    }
+    else if (qtdeadmap->contains (key)) {
       mods &=~ Qt::ShiftModifier;
       r = qtdeadmap[key];
-    } else {
+    }
+    else {
       QString nss = event->text();
       unsigned short unic= nss.data()[0].unicode();
       if (unic < 32 && key < 128 && key > 0) {
@@ -439,27 +531,38 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
         r= string ((char) key);
       } else {
         switch(unic) {
-        case 96:   r= "`"; 
+          case 96:   r= "`"; 
             // unicode to cork conversion not appropriate for this case...
 #ifdef Q_WS_MAC
-          // CHECKME: are these two MAC exceptions really needed?
-                   if (mods & Qt::AltModifier) r= "grave";
+            // CHECKME: are these two MAC exceptions really needed?
+            if (mods & Qt::AltModifier) r= "grave";
 #endif
-                   break;
-        case 168:  r= "umlaut"; break;
-        case 180:  r= "acute"; break;
-        // the following combining characters should be caught by qtdeadmap
-        case 0x300: r= "grave"; break;
-        case 0x301: r= "acute"; break;
-        case 0x302: r= "hat"; break;
-        case 0x308: r= "umlaut"; break;
-        case 0x33e: r= "tilde"; break;
-        default:
-          QByteArray buf= nss.toUtf8();
-          string rr (buf.constData(), buf.count());
-          r= utf8_to_cork (rr);
-          if (r == "<less>") r= "<";
-          if (r == "<gtr>") r= ">";
+            break;
+          case 168:  r= "umlaut"; break;
+          case 180:  r= "acute"; break;
+            // the following combining characters should be caught by qtdeadmap
+          case 0x300: r= "grave"; break;
+          case 0x301: r= "acute"; break;
+          case 0x302: r= "hat"; break;
+          case 0x308: r= "umlaut"; break;
+          case 0x33e: r= "tilde"; break;
+          default:
+            QByteArray buf= nss.toUtf8();
+            string rr (buf.constData(), buf.count());
+            string tstr= utf8_to_cork (rr);
+            // HACK! The encodings defined in langs/encoding and which
+            // utf8_to_cork uses (via the converters loaded in
+            // converter_rep::load()), enclose the texmacs symbols in "< >", 
+            // but this format is not used for keypresses, so we must remove
+            // them.
+            int len= N(tstr);
+            if (len >= 1 && tstr[0] == '<' && tstr[1] != '#' && tstr[len-1] == '>')
+              r= tstr(1, len-1);
+            else
+              r= tstr;
+            if (r == "less") r= "<";
+            else if (r == "gtr")r= ">";
+
         }
 #ifdef Q_WS_MAC
         // CHECKME: are these two MAC exceptions really needed?
@@ -468,7 +571,7 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
         mods &=~ Qt::ShiftModifier;
       }
     }
-
+    
     if (r == "") return;
 
 #ifdef Q_WS_MAC
@@ -488,14 +591,14 @@ QTMWidget::keyPressEvent (QKeyEvent* event) {
     if (DEBUG_QT)
       cout << "key press: " << r << LF;
     //int start= texmacs_time ();
-    //wid -> handle_keypress (r, texmacs_time());
-    the_gui -> process_keypress (wid, r, texmacs_time());
+    //tm_widget() -> handle_keypress (r, texmacs_time());
+    the_gui -> process_keypress (tm_widget(), r, texmacs_time());
     //int end= texmacs_time ();
     //if (end > start) cout << "Keypress " << end - start << "\n";
-  //  the_gui->update (); // FIXME: remove this line when
+    //the_gui->update (); // FIXME: remove this line when
                         // edit_typeset_rep::get_env_value will be faster
     
-//    needs_update();
+    //needs_update();
   }
 }
 
@@ -512,24 +615,27 @@ mouse_state (QMouseEvent* event, bool flag) {
   if ((bstate & Qt::XButton1       ) != 0) i += 8;
   if ((bstate & Qt::XButton2       ) != 0) i += 16;
 #ifdef Q_WS_MAC
-  if ((kstate & Qt::AltModifier    ) != 0) i = 2;
-  if ((kstate & Qt::MetaModifier   ) != 0) i = 4;
+    // We emulate right and middle clicks with ctrl and option, but we pass the
+    // modifiers anyway: old code continues to work and new one can use them.
+  if ((kstate & Qt::MetaModifier   ) != 0) i = 1024+4; // control key
+  if ((kstate & Qt::AltModifier    ) != 0) i = 2048+2; // option key
   if ((kstate & Qt::ShiftModifier  ) != 0) i += 256;
-  if ((kstate & Qt::ControlModifier) != 0) i += 2048;
+  if ((kstate & Qt::ControlModifier) != 0) i += 4096;   // cmd key
 #else
   if ((kstate & Qt::ShiftModifier  ) != 0) i += 256;
-  if ((kstate & Qt::ControlModifier) != 0) i += 512;
+  if ((kstate & Qt::ControlModifier) != 0) i += 1024;
   if ((kstate & Qt::AltModifier    ) != 0) i += 2048;
-  if ((kstate & Qt::MetaModifier   ) != 0) i += 16384;
+  if ((kstate & Qt::MetaModifier   ) != 0) i += 4096;
 #endif
   return i;
 }
 
 static string
 mouse_decode (unsigned int mstate) {
-  if      (mstate & 1 ) return "left";
-  else if (mstate & 2 ) return "middle";
+  if (mstate & 2 ) return "middle";
   else if (mstate & 4 ) return "right";
+    // we check for left clicks after the others for macos (see ifdef in mouse_state)
+  else if (mstate & 1 ) return "left";
   else if (mstate & 8 ) return "up";
   else if (mstate & 16) return "down";
   return "unknown";
@@ -661,23 +767,46 @@ QTMWidget::inputMethodEvent (QInputMethodEvent* event) {
 #endif
   }
   
-  if (!preedit_string.isEmpty()) {
-    if (DEBUG_QT)
-      cout << "IM preediting :" << preedit_string.toUtf8().data() << LF;
-
-    // find cursor position in the preedit string
-    QList<QInputMethodEvent::Attribute> attrs = event->attributes();
-    int pos = preedit_string.count();
-    for(int i=0; i< attrs.count(); i++) 
-      if (attrs[i].type == QInputMethodEvent::Cursor) 
-        pos = attrs[i].start;
+  if (DEBUG_QT)
+    cout << "IM preediting :" << preedit_string.toUtf8().data() << LF;
+  
+  string r = "pre-edit:";
+  if (!preedit_string.isEmpty())
+  {
     
-    string r = "pre-edit:" * as_string(pos) * ":" 
-                           * from_qstring(preedit_string);
-    simple_widget_rep *wid =  tm_widget();
-    if (wid)
-      the_gui -> process_keypress (wid, r, texmacs_time());
+    // find cursor position in the preedit string
+    QList<QInputMethodEvent::Attribute>  const & attrs = event->attributes();
+    //    int pos = preedit_string.count();
+    int pos = 0;
+    bool visible_cur = false;
+    for(int i=0; i< attrs.count(); i++) 
+      if (attrs[i].type == QInputMethodEvent::Cursor) {
+        pos = attrs[i].start;
+        visible_cur = (attrs[i].length != 0);
+      }
+    
+    // find selection in the preedit string
+    int sel_start = 0;
+    int sel_length = 0;
+    if (pos <  preedit_string.count()) {
+      for(int i=0; i< attrs.count(); i++) 
+        if ((attrs[i].type == QInputMethodEvent::TextFormat) &&
+            (attrs[i].start <= pos) &&
+            (pos < attrs[i].start + attrs[i].length)) {
+          sel_start = attrs[i].start;
+          sel_length =  attrs[i].length;
+          if (!visible_cur) pos += attrs[i].length;
+        }
+    } else {
+      sel_start = pos;
+      sel_length = 0;
+    }
+    (void) sel_start; (void) sel_length;
+    
+    r = r * as_string(pos) * ":" * from_qstring(preedit_string);
   }
+  if (tm_widget()->ref_count != 0)
+    the_gui -> process_keypress (tm_widget(), r, texmacs_time());
   event->accept();
 }  
 
@@ -685,9 +814,9 @@ QVariant
 QTMWidget::inputMethodQuery ( Qt::InputMethodQuery query ) const {
   switch (query) {
     case Qt::ImMicroFocus :
-      return QVariant(QRect(cursor_pos ,QSize(5,5)));
+      return QVariant (QRect (cursor_pos, QSize (5,5)));
     default:
-      return QVariant();
+      return QWidget::inputMethodQuery (query);
   }
 }
 
@@ -698,50 +827,56 @@ QTMWidget::inputMethodQuery ( Qt::InputMethodQuery query ) const {
 
 void
 QTMWidget::mousePressEvent (QMouseEvent* event) {
-  simple_widget_rep *wid= tm_widget ();
-  if (!wid) return;
-  QPoint point = event->pos() + origin;
+  if (tm_widget()->ref_count == 0) return;
+  QPoint point = event->pos() + origin();
   scale (point);
   unsigned int mstate= mouse_state (event, false);
   string s= "press-" * mouse_decode (mstate);
-  the_gui -> process_mouse (wid, s, point.x (), point.y (), 
+  the_gui -> process_mouse (tm_widget(), s, point.x (), point.y (), 
                             mstate, texmacs_time ());
- // wid -> handle_mouse (s, point.x (), point.y (), mstate, texmacs_time ());
+ // tm_widget() -> handle_mouse (s, point.x (), point.y (), mstate, texmacs_time ());
+  /*
   if (DEBUG_QT)
     cout << "mouse event: " << s << " at "
          << point.x () << ", " << point.y () << LF;
+   */
+  event->accept();
 }
 
 void
 QTMWidget::mouseReleaseEvent (QMouseEvent* event) {
-  simple_widget_rep *wid = tm_widget();
-  if (!wid) return;
-  QPoint point = event->pos() + origin;;
+  if (tm_widget()->ref_count == 0) return;
+  QPoint point = event->pos() + origin();
   scale (point);
   unsigned int mstate= mouse_state (event, true);
   string s= "release-" * mouse_decode (mstate);
-  the_gui -> process_mouse (wid, s, point.x (), point.y (), 
+  the_gui -> process_mouse (tm_widget(), s, point.x (), point.y (), 
                             mstate, texmacs_time ());
-//  wid -> handle_mouse (s, point.x (), point.y (), mstate, texmacs_time ());
+//  tm_widget() -> handle_mouse (s, point.x (), point.y (), mstate, texmacs_time ());
+  /*
   if (DEBUG_QT)
     cout << "mouse event: " << s << " at "
          << point.x () << ", " << point.y () << LF;
+   */
+  event->accept();
 }
 
 void
 QTMWidget::mouseMoveEvent (QMouseEvent* event) {
-  simple_widget_rep *wid = tm_widget();
-  if (!wid) return;
-  QPoint point = event->pos() + origin;
+  if (tm_widget()->ref_count == 0) return;
+  QPoint point = event->pos() + origin();
   scale (point);
   unsigned int mstate= mouse_state (event, false);
   string s= "move";
-  the_gui -> process_mouse (wid, s, point.x (), point.y (), 
+  the_gui -> process_mouse (tm_widget(), s, point.x (), point.y (), 
                             mstate, texmacs_time ());
-//  wid -> handle_mouse (s, point.x (), point.y (), mstate, texmacs_time ());
+//  tm_widget() -> handle_mouse (s, point.x (), point.y (), mstate, texmacs_time ());
+  /*
   if (DEBUG_QT)
     cout << "mouse event: " << s << " at "
          << point.x () << ", " << point.y () << LF;
+   */
+  event->accept();
 }
 
 
@@ -751,29 +886,35 @@ QTMWidget::event (QEvent* event) {
     QKeyEvent *ke = static_cast<QKeyEvent*> (event);
     keyPressEvent (ke);
     return true;
-  }
+  } 
   return QTMScrollView::event (event);
 }
 
 
 void
 QTMWidget::focusInEvent ( QFocusEvent * event ) {
-  if (DEBUG_QT) cout << "FOCUSIN" << LF;
-  simple_widget_rep *wid = tm_widget ();
-  if (wid) {
-    the_gui -> process_keyboard_focus (wid, true, texmacs_time());
-    //wid -> handle_keyboard_focus (true, texmacs_time ());
+  if (tm_widget()->ref_count != 0) {
+    if (DEBUG_QT) cout << "FOCUSIN: " << tm_widget()->type_as_string() << LF;
+    the_gui -> process_keyboard_focus (tm_widget(), true, texmacs_time());
+    //tm_widget() -> handle_keyboard_focus (true, texmacs_time ());
   }
   QTMScrollView::focusInEvent (event);
 }
 
 void
 QTMWidget::focusOutEvent ( QFocusEvent * event ) {
-  if (DEBUG_QT)   cout << "FOCUSOUT" << LF;
-  simple_widget_rep *wid = tm_widget ();
-  if (wid) {
-    the_gui -> process_keyboard_focus (wid, false, texmacs_time());
-//    wid -> handle_keyboard_focus (false, texmacs_time ());
+  if (tm_widget()->ref_count != 0) {
+    if (DEBUG_QT) cout << "FOCUSOUT: " << tm_widget()->type_as_string() << LF;
+    the_gui -> process_keyboard_focus (tm_widget(), false, texmacs_time());
+//    tm_widget() -> handle_keyboard_focus (false, texmacs_time ());
   }
   QTMScrollView::focusOutEvent (event);
 }
+
+
+QSize
+QTMWidget::sizeHint () const {
+  SI w,h;
+  tm_widget()->handle_get_size_hint(w,h);
+  return to_qsize(w, h);
+}
