@@ -9,6 +9,7 @@
 * in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
+#include "convert.hpp"
 #include "iterator.hpp"
 #include "dictionary.hpp"
 #include "qt_gui.hpp"
@@ -20,32 +21,45 @@
 #include <QDesktopWidget>
 #include <QClipboard>
 #include <QFileOpenEvent>
+#include <QStackedLayout>
 #include <QLabel>
 #include <QSocketNotifier>
 #include <QSetIterator>
+#include <QTranslator>
+#include <QLocale>
 #include "QTMGuiHelper.hpp"
 #include "QTMWidget.hpp"
 #include "QTMWindow.hpp"
 #include "qt_renderer.hpp" // for the_qt_renderer
+#include "file.hpp" // added for copy_as_graphics
+#include <QMimeData>
+#include <QByteArray>
+#include <QApplication>
+#include <QCoreApplication>
+#include <QImage>
+
+#ifdef MACOSX_EXTENSIONS
+#include "MacOS/mac_utilities.h"
+#endif
 
 #include "tm_link.hpp" // for number_of_servers
-
-#include "Scheme/object.hpp"
-//#include "TeXmacs/server.hpp" // for get_server
-
+#include "scheme.hpp"
+//#include "server.hpp" // for get_server
+#include "tm_window.hpp"
+#include "new_window.hpp"
 #include "qt_simple_widget.hpp"
 #include "qt_window_widget.hpp"
 
-extern window (*get_current_window) (void);
 
 qt_gui_rep* the_gui= NULL;
 int nr_windows = 0; // FIXME: fake variable, referenced in tm_server
 
-time_t time_credit;  // interval to interrupt long redrawings
+time_t time_credit = 100;  // interval to interrupt long redrawings
 time_t timeout_time; // new redraw interruption
 time_t lapse = 0; // optimization for delayed commands
 
 // marshalling flags between update, needs_update and check_event.
+bool disable_check_event = false;
 bool updating = false;
 bool needing_update = false;
 bool wait_for_delayed_commands = true;
@@ -57,48 +71,46 @@ bool wait_for_delayed_commands = true;
 * Constructor and geometry
 ******************************************************************************/
 
+#include <QLibraryInfo>
+
 qt_gui_rep::qt_gui_rep(int &argc, char **argv):
-  interrupted (false)
+  interrupted (false), waitWindow (NULL), popup_wid_time(0), q_translator(0)
 {
   (void) argc; (void) argv;
   // argc= argc2;
   // argv= argv2;
 
-  interrupted   = false;
-  time_credit = 25;
-  timeout_time= texmacs_time () + time_credit;
+  interrupted  = false;
+  time_credit  = 100;
+  timeout_time = texmacs_time () + time_credit;
 
   //waitDialog = NULL;
   
-  set_output_language (get_locale_language ());
   gui_helper = new QTMGuiHelper (this);
-  qApp -> installEventFilter (gui_helper);
-  
+  qApp->installEventFilter (gui_helper);
+
 #ifdef QT_MAC_USE_COCOA
   //HACK: this filter is needed to overcome a bug in Qt/Cocoa
   extern void mac_install_filter(); // defined in src/Plugins/MacOS/mac_app.mm
   mac_install_filter();
 #endif
-  
-  
-  
-  qApp-> installTranslator(new QTMTranslator(qApp));
-  
+
+  set_output_language (get_locale_language ());
+  refresh_language();
+
   updatetimer = new QTimer (gui_helper);
   updatetimer->setSingleShot (true);
-  QObject::connect ( updatetimer, SIGNAL(timeout()), 
-                     gui_helper, SLOT(doUpdate()));
+  QObject::connect (updatetimer, SIGNAL(timeout()),
+                    gui_helper, SLOT(doUpdate()));
 //  (void) default_font ();
 }
 
 /* important routines */
 void
 qt_gui_rep::get_extents (SI& width, SI& height) {
-  QDesktopWidget* d= QApplication::desktop();
-  int w = d->width();  // returns desktop width
-  int h = d->height(); // returns desktop height        
-  width = ((SI) w) * PIXEL;
-  height= ((SI) h) * PIXEL;
+  coord2 size = from_qsize (QApplication::desktop()->size());
+  width  = size.x1;
+  height = size.x2;
 }
 
 void
@@ -112,9 +124,10 @@ qt_gui_rep::~qt_gui_rep()  {
   delete gui_helper;
   
   while (waitDialogs.count()) {
-    delete waitDialogs.last();
+    waitDialogs.last()->deleteLater();
     waitDialogs.removeLast();
-  }
+  }    
+  if (waitWindow) delete waitWindow;
   
  // delete updatetimer; we do not need this given that gui_helper is the 
  // parent of updatetimer
@@ -125,39 +138,86 @@ qt_gui_rep::~qt_gui_rep()  {
 ******************************************************************************/
 
 bool
-qt_gui_rep::get_selection (string key, tree& t, string& s) {
+qt_gui_rep::get_selection (string key, tree& t, string& s, string format) {
   QClipboard *cb= QApplication::clipboard ();
   QClipboard::Mode mode= QClipboard::Clipboard;
-  bool owns= false;
-  if (key == "primary" || (key == "mouse" && cb->supportsSelection ())) {
+  if (key == "primary" || (key == "mouse" && cb->supportsSelection ()))
     if (key == "mouse") mode= QClipboard::Selection;
-    const QMimeData *md= cb->mimeData (mode);
-    if (md) owns= md->hasFormat ("application/x-texmacs-clipboard");  
-  }
-  else owns= true;
-  
+
+  QString originalText= cb->text (mode);
+  const QMimeData *md= cb->mimeData (mode);
+  QByteArray buf;
+  string input_format;
+
   s= "";
   t= "none";
+  // Knowing when we owns (or not) the content is not clear
+  bool owns= (format != "temp" && format != "wrapbuf" && key != "primary") &&
+             !(key == "mouse" && cb->supportsSelection ());
+
+  if (!owns && md->hasFormat ("application/x-texmacs-pid")) {
+    buf= md->data ("application/x-texmacs-pid");
+    if (!(buf.isEmpty())) {
+      owns= string (buf.constData(), buf.size())
+            == as_string (QCoreApplication::applicationPid ());
+    }
+  }
+
   if (owns) {
     if (!selection_t->contains (key)) return false;
     t= copy (selection_t [key]);
     s= copy (selection_s [key]);
     return true;
   }
-  else {
-    QString originalText= cb->text (mode);
-    QByteArray buf= originalText.toAscii ();
-    if (!(buf.isEmpty())) s << string (buf.constData(), buf.size());
-    t= tuple ("extern", s);
-    return true;
+
+  if (format == "default") {
+    if (md->hasFormat ("application/x-texmacs-clipboard")) {
+      buf= md->data ("application/x-texmacs-clipboard");
+      input_format= "texmacs-snippet";
+    }
+    else if (md->hasHtml ()) {
+      buf= md->html().toUtf8 ();
+      input_format= "html-snippet";
+    }
+    else if (md->hasFormat ("text/plain;charset=utf8")) {
+      buf= md->data ("text/plain;charset=utf8");
+      input_format= "verbatim-snippet";
+    }
+    else {
+      buf= md->text().toUtf8 ();
+      input_format= "verbatim-snippet";
+    }
   }
+  else if (format == "verbatim" && (
+        get_preference ("verbatim->texmacs:encoding") == "utf-8" ||
+        get_preference ("verbatim->texmacs:encoding") == "auto"  ))
+    buf= md->text().toUtf8 ();
+  else {
+    if (md->hasFormat ("plain/text")) buf= md->data ("plain/text").data();
+    else buf= md->text().toUtf8 ();
+  }
+  if (!(buf.isEmpty())) s << string (buf.constData(), buf.size());
+  if (input_format == "html-snippet" && seems_buggy_html_paste (s))
+    s= correct_buggy_html_paste (s);
+  if (seems_buggy_paste (s))
+    s= correct_buggy_paste (s);
+  if (input_format != "")
+    s= as_string (call ("convert", s, input_format, "texmacs-snippet"));
+  if (input_format == "html-snippet") {
+    tree t= as_tree (call ("convert", s, "texmacs-snippet", "texmacs-tree"));
+    t= default_with_simplify (t);
+    s= as_string (call ("convert", t, "texmacs-tree", "texmacs-snippet"));
+  }
+  t= tuple ("extern", s);
+  return true;
 }
 
 bool
-qt_gui_rep::set_selection (string key, tree t, string s) {
+qt_gui_rep::set_selection (string key, tree t,
+                           string s, string sv, string sh, string format) {
   selection_t (key)= copy (t);
   selection_s (key)= copy (s);
-        
+
   QClipboard *cb= QApplication::clipboard ();
   QClipboard::Mode mode= QClipboard::Clipboard;
   if (key == "primary");
@@ -166,16 +226,42 @@ qt_gui_rep::set_selection (string key, tree t, string s) {
   else return true;
   cb->clear (mode);
 
-  char *selection = as_charp (s);
-  cb->setText (selection, mode);
-  QByteArray ba (selection);
+  c_string selection (s);
+  cb->setText (QString::fromAscii(selection), mode);
   QMimeData *md= new QMimeData;
-  md->setData ("application/x-texmacs-clipboard", ba);
-  md->setText (selection);
-  cb->setMimeData (md, mode); 
+
+  if (format == "verbatim" || format == "default") {
+    if (format == "default") {
+      md->setData ("application/x-texmacs-clipboard", (char*)selection);
+
+      QString pid_str;
+      pid_str.setNum(QCoreApplication::applicationPid ());
+      md->setData ("application/x-texmacs-pid", pid_str.toAscii());
+
+      (void) sh;
+      //selection= c_string (sh);
+      //md->setHtml (selection);
+      //tm_delete_array(selection);
+
+      selection= c_string (sv);
+    }
+
+    string enc= get_preference ("texmacs->verbatim:encoding");
+    if (enc == "auto")
+      enc= get_locale_charset ();
+
+    if (enc == "utf-8" || enc == "UTF-8")
+      md->setText (QString::fromUtf8 (selection));
+    else if (enc == "iso-8859-1" || enc == "ISO-8859-1")
+      md->setText (QString::fromLatin1 (selection));
+    else
+      md->setText (QString::fromAscii (selection));
+  }
+  else
+    md->setText (QString::fromAscii (selection));
+  cb->setMimeData (md, mode);
   // according to the docs, ownership of mimedata is transferred to clipboard
   // so no memory leak here
-  tm_delete_array (selection);
   return true;
 }
 
@@ -186,14 +272,14 @@ qt_gui_rep::clear_selection (string key) {
   
   QClipboard *cb= QApplication::clipboard();
   QClipboard::Mode mode= QClipboard::Clipboard;
-  bool owns= false;
   if (key == "primary");
   else if (key == "mouse" && cb->supportsSelection())
     mode= QClipboard::Selection;
   else return;
 
+  bool owns= false;
   const QMimeData *md= cb->mimeData (mode);
-  if (md) owns= md->hasFormat ("application/x-texmacs");
+  if (md) owns= md->hasFormat ("application/x-texmacs-clipboard");
   if (owns) cb->clear (mode);
 }
 
@@ -201,8 +287,6 @@ qt_gui_rep::clear_selection (string key) {
 * Miscellaneous
 ******************************************************************************/
 
-void qt_gui_rep::image_gc (string name) { (void) name; }
-// FIXME: remove this unused function
 void qt_gui_rep::set_mouse_pointer (string name) { (void) name; }
 // FIXME: implement this function
 void qt_gui_rep::set_mouse_pointer (string curs_name, string mask_name) 
@@ -214,9 +298,8 @@ void qt_gui_rep::set_mouse_pointer (string curs_name, string mask_name)
 
 void
 qt_gui_rep::show_wait_indicator (widget w, string message, string arg)  {
-  (void) w; (void) message; (void) arg;
-  
-  if (DEBUG_QT)  cout << "show_wait_indicator \"" << message << "\"\"" << arg << "\"" << LF;
+  if (DEBUG_QT)
+    cout << "show_wait_indicator \"" << message << "\"\"" << arg << "\"" << LF;
 
   qt_window_widget_rep *wid = static_cast<qt_window_widget_rep*> (w.rep);
 
@@ -227,33 +310,28 @@ qt_gui_rep::show_wait_indicator (widget w, string message, string arg)  {
   
   //FIXME: we must center the wait widget wrt the current active window
   
+  if (!waitWindow) {
+    waitWindow = new QWidget (wid->qwid->window());
+    waitWindow->setWindowFlags (Qt::Window | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
+    QStackedLayout *layout = new QStackedLayout();
+    layout->setSizeConstraint(QLayout::SetFixedSize);
+    waitWindow->setLayout(layout);
+  }
+  
   if (waitDialogs.count()) {
-    waitDialogs.last()->close();
+    waitWindow->layout()->removeWidget(waitDialogs.last());
   }
   
   if (N(message)) {
-    
     // push a new wait message in the list
     
     if (arg != "") message= message * " " * arg * "...";
     
-    QLabel* lab = new  QLabel(); //(wid->wid->window());
-    lab->setWindowFlags(Qt::Window | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
-    //lab->setWindowModality(Qt::ApplicationModal);
+    QLabel* lab = new  QLabel();
     lab->setFocusPolicy(Qt::NoFocus);
     lab->setMargin(15);
-    lab->setText (to_qstring (tm_var_encode(message)));
-    QSize sz = lab->sizeHint();
-    QRect rect = QRect(QPoint(0,0),sz);
-    //HACK: 
-    // processEvents is needed to let Qt update windows coordinates in the case
-    qApp->processEvents();
-    //ENDHACK
-    QPoint pt = wid->wid->window()->geometry().center();
-    rect.moveCenter(pt);
-    lab->move(rect.topLeft());
+    lab->setText (to_qstring (message));
     waitDialogs << lab;
-    
   } else {
     // pop the next wait message from the list
     if (waitDialogs.count()) {
@@ -263,15 +341,30 @@ qt_gui_rep::show_wait_indicator (widget w, string message, string arg)  {
   }
 
   if (waitDialogs.count()) {
-    waitDialogs.last()->show();
-    //waitDialogs.last()->raise();
+    waitWindow->layout()->addWidget(waitDialogs.last());
+    waitWindow->updateGeometry();
+    {
+      QSize sz = waitWindow->geometry().size();
+      QRect rect = QRect(QPoint(0,0),sz);
+      //HACK: 
+      // processEvents is needed to let Qt update windows coordinates in the case
+      qApp->processEvents();
+      //ENDHACK
+      QPoint pt = wid->qwid->window()->geometry().center();
+      rect.moveCenter(pt);
+      waitWindow->move(rect.topLeft());
+      
+    }
+    waitWindow->show();
     qApp->processEvents();
-    waitDialogs.last()->repaint();
-    qApp->processEvents();
-    QApplication::flush();
-
+    waitWindow->repaint();
+  } else {
+    waitWindow->close();
   }
-
+  qApp->processEvents();
+  QApplication::flush();
+  
+  
   //    wid->wid->setUpdatesEnabled(true);
 
   // next time we do update the dialog will disappear
@@ -285,19 +378,16 @@ void gui_interpose (void (*r) (void)) { the_interpose_handler= r; }
 void
 qt_gui_rep::event_loop () {
   QApplication *app = (QApplication*) QApplication::instance();
-
   update();
   //needs_update();
   app->exec();
 }
 
+
 /******************************************************************************
  * Sockets notifications
  ******************************************************************************/
 
-static hashmap<socket_notifier,pointer> read_notifiers;
-static hashmap<socket_notifier,pointer> write_notifiers;
-
 void 
 qt_gui_rep::add_notifier (socket_notifier sn)
 {
@@ -308,7 +398,6 @@ qt_gui_rep::add_notifier (socket_notifier sn)
   remove_notifier (sn);
 
   if (DEBUG_QT) cout << "ADD NOTIFIER " << sn->fd << LF;
-
   
   // installs both a read and a write notifier 
   // (the texmacs interface does not specify enough its needs)
@@ -321,7 +410,7 @@ qt_gui_rep::add_notifier (socket_notifier sn)
   qsn = new QSocketNotifier(sn->fd, QSocketNotifier::Write, gui_helper);
   write_notifiers (sn) = (pointer) (qsn);
   QObject::connect( qsn, SIGNAL(activated(int)), 
-                    gui_helper, SLOT(doWriteSocketNotification(int)) );  
+                    gui_helper, SLOT(doWriteSocketNotification(int)) );
 }
 
 void 
@@ -360,7 +449,6 @@ qt_gui_rep::enable_notifier (socket_notifier sn, bool flag)
 }
 
 
-
 /******************************************************************************
  * Delayed commands
  ******************************************************************************/
@@ -435,6 +523,10 @@ gui_open (int& argc, char** argv) {
   // start the gui
  // new QApplication (argc,argv); now in texmacs.cpp
   the_gui = tm_new<qt_gui_rep> (argc, argv);
+  
+#ifdef MACOSX_EXTENSIONS
+  mac_begin_remote();
+#endif
 }
 
 void
@@ -449,94 +541,31 @@ gui_close () {
   ASSERT (the_gui != NULL, "gui not yet open");
   tm_delete (the_gui);
   the_gui=NULL;
+
+#ifdef MACOSX_EXTENSIONS
+  mac_end_remote();
+#endif
 }
 
 void
 gui_root_extents (SI& width, SI& height) {
   // get the screen size
   the_gui->get_extents (width, height);
-  if (DEBUG_QT) 
-    cout << "gui_root_extents (" << width << "," << height << ")" << LF;
+    //if (DEBUG_QT) 
+    //cout << "gui_root_extents (" << width << "," << height << ")" << LF;
 }
 
 void
 gui_maximal_extents (SI& width, SI& height) {
   // get the maximal size of a window (can be larger than the screen size)
   the_gui->get_max_size (width, height);
-  if (DEBUG_QT) 
-    cout << "gui_maximal_extents (" << width << "," << height << ")" << LF;
+    //if (DEBUG_QT) 
+    //cout << "gui_maximal_extents (" << width << "," << height << ")" << LF;
 }
 
 void
 gui_refresh () {
-  // called upon change of output language
-  // emit a signal which force every QTMAction to change his text
-  // according to the new language
-
-  the_gui->gui_helper->doRefresh();
-}
-
-
-/******************************************************************************
- * QTMGuiHelper methods
- ******************************************************************************/
-
-void
-QTMGuiHelper::doUpdate () {
-//  cout << "UPDATE " << texmacs_time () << LF;
-  gui->update();
-}
-
-void
-QTMGuiHelper::doRefresh () {
-  emit refresh();
-}
-
-bool
-QTMGuiHelper::eventFilter (QObject *obj, QEvent *event) {
-   if (event->type() == QEvent::FileOpen) {
-     QFileOpenEvent *openEvent = static_cast<QFileOpenEvent *>(event);
-     const char *s = openEvent->file().toAscii().constData();
-     //qDebug ("File Open Event %s", s);
-     call ( "texmacs-load-buffer", object(url_system (s)), 
-            object("generic"), object(1), object(false));
-     return true;
-   } else {
-     // standard event processing
-     return QObject::eventFilter(obj, event);
-   }
-}
-
-void
-QTMGuiHelper::doWriteSocketNotification (int socket) {
-  if (DEBUG_QT) 
-    cout << "WRITE SOCKET NOTIFICATION " << socket << " "
-         << texmacs_time () << LF;
-  iterator<socket_notifier> it = iterate (write_notifiers);
-  while (it->busy ()) {
-    socket_notifier sn= it->next ();
-    if (sn->fd == socket) {
-      //sn->notify();
-      the_gui->process_socket_notification (sn);
-      the_gui->enable_notifier (sn, false);
-    }
-  }
-}
-
-void
-QTMGuiHelper::doReadSocketNotification (int socket) {
-  if (DEBUG_QT) 
-    cout << "READ SOCKET NOTIFICATION " << socket << " "
-         << texmacs_time () << LF;
-  iterator<socket_notifier> it = iterate (read_notifiers);
-  while (it->busy ()) {
-    socket_notifier sn= it->next ();
-    if (sn->fd == socket) {
-      //sn->notify();
-      the_gui->process_socket_notification (sn);
-      the_gui->enable_notifier (sn, false);
-    }
-  }
+  the_gui->refresh_language(); 
 }
 
 
@@ -552,6 +581,7 @@ enum qp_type_id {
   QP_RESIZE,
   QP_SOCKET_NOTIFICATION,
   QP_COMMAND,
+  QP_COMMAND_ARGS,
   QP_DELAYED_COMMANDS
 };
 
@@ -580,9 +610,11 @@ public:
 
 static array<queued_event> waiting_events;
 
+void add_event(const queued_event &ev);
 
 void
 process_event (queued_event ev) {
+  //cout << "<" << (qp_type_id) ev.x1 << LF;
   switch ((qp_type_id) ev.x1) {
     case QP_NULL :
       break;
@@ -598,10 +630,6 @@ process_event (queued_event ev) {
       typedef triple<widget, bool, time_t > T;
       T x = open_box <T> (ev.x2) ;
       concrete_simple_widget(x.x1) -> handle_keyboard_focus (x.x2, x.x3) ;
-      send_invalidate_all(x.x1);
-      //FIXME: the above invalidate is due to incorrect repainting when
-      //       the widget loose the focus. I should investigate better
-      //       the problem.
     }
       break;
     case QP_MOUSE :
@@ -634,6 +662,14 @@ process_event (queued_event ev) {
       cmd->apply();
     }
       break;
+    case QP_COMMAND_ARGS :
+    {
+      typedef pair<command, object> T;
+      T x = open_box <T> (ev.x2);
+      // cout << "QP_COMMAND_ARGS" << LF;
+      x.x1->apply(x.x2);
+    }
+      break;
     case QP_DELAYED_COMMANDS :
     {
       // cout << "QP_DELAYED_COMMANDS" << LF;
@@ -641,47 +677,56 @@ process_event (queued_event ev) {
       wait_for_delayed_commands = true;
     }
       break;
+
     default:   
       FAILED("Unexpected queued event");
   }
+  //cout << ">" << (qp_type_id) ev.x1 << LF;
 }
 
+queued_event 
+next_event() {
+  queued_event ev;
+  if (N(waiting_events)>0) 
+    ev = waiting_events[0];
+  
+  array<queued_event> a;
+
+  for(int i=1; i<N(waiting_events); i++)
+    a << waiting_events[i];
+  waiting_events = a;
+  
+  return ev;
+}
 
 void 
 qt_gui_rep::process_queued_events (int max) {
-  // we process a maximum of max events. There are two kind of events: those
-  // which need a pass on interpose_handler just after and the others. We count
-  // only the first kind of events. In update() we call this function with
-  // max=1 so that only one of these "sensible" events is handled. Otherwise
-  // updating of internal TeXmacs structure become very slow. This can be 
-  // considerer a limitation of the current implementation of interpose_handler
-  // Likewise this function is just an hack to get things working properly.
+    // we process a maximum of max events. There are two kind of events: those
+    // which need a pass on interpose_handler just after and the others. We count
+    // only the first kind of events. In update() we call this function with
+    // max=1 so that only one of these "sensible" events is handled. Otherwise
+    // updating of internal TeXmacs structure become very slow. This can be 
+    // considerer a limitation of the current implementation of interpose_handler
+    // Likewise this function is just an hack to get things working properly.
   
-  array<queued_event> a = waiting_events, b;
-  waiting_events = array<queued_event> ();
-  
-  int i, n = N(a);
   int count = 0;
-  //cout << "(" << n << " events)";
-  for (i=0; i<n; i++) {
-    //cout << (int)a[i].x1 << ",";
-    if ((max < 0) || (count<max))  {
-      process_event(a[i]);
-      switch (a[i].x1) {
-        case QP_COMMAND:
-        case QP_SOCKET_NOTIFICATION:
-        case QP_RESIZE:
-        case QP_DELAYED_COMMANDS:
-          break;
-        default:
-          count++;
-          break;
-      }
-    } else 
-      b << a[i];
+    //cout << "(" << n << " events)"
+  while ((max < 0) || (count<max))  {
+    queued_event ev = next_event();
+    if (ev.x1 == QP_NULL) break;
+    process_event(ev);
+    switch (ev.x1) {
+      case QP_COMMAND:
+      case QP_COMMAND_ARGS:
+      case QP_SOCKET_NOTIFICATION:
+      case QP_RESIZE:
+      case QP_DELAYED_COMMANDS:
+        break;
+      default:
+        count++;
+        break;
+    }
   }
-  b << waiting_events;
-  waiting_events = b;
 }
 
 
@@ -695,7 +740,6 @@ add_event(const queued_event &ev)
   } else {
     waiting_events << ev;
 //    process_event(ev);
-//    the_gui->update();
     needs_update();
     // NOTE: we cannot update now since sometimes this seems to give problems
     // to the update of the window size after a resize. In that situation
@@ -707,38 +751,66 @@ add_event(const queued_event &ev)
 }
 
 void 
-qt_gui_rep::process_keypress (simple_widget_rep *wid, string key, time_t t) {
+qt_gui_rep::process_keypress (qt_simple_widget_rep *wid, string key, time_t t) {
   typedef triple<widget, string, time_t > T;
+  if (wid->type != qt_widget_rep::simple_widget) {
+    if (DEBUG_QT)
+      cout << "WTF! Situation] process_keypress() for non simple_widget. Widget: " 
+           << wid->type_as_string() << LF; 
+
+    return;
+  }
   add_event( queued_event ( QP_KEYPRESS, close_box<T> (T(wid, key, t)))); 
  // wid -> handle_keypress (key, t);
  // needs_update ();
 }
 
 void 
-qt_gui_rep::process_keyboard_focus ( simple_widget_rep *wid, bool has_focus, 
+qt_gui_rep::process_keyboard_focus (qt_simple_widget_rep *wid, bool has_focus, 
                                      time_t t ) {
   typedef triple<widget, bool, time_t > T;
+  if (wid->type != qt_widget_rep::simple_widget) {
+    if (DEBUG_QT)
+      cout << "WTF! Situation] process_keyboard_focus() for non simple_widget. Widget: " 
+           << wid->type_as_string() << LF; 
+
+    return;
+  }
   add_event( 
-    queued_event ( QP_KEYBOARD_FOCUS, close_box<T> (T(wid, has_focus, t)))); 
+    queued_event (QP_KEYBOARD_FOCUS, close_box<T> (T(wid, has_focus, t)))); 
   //wid -> handle_keyboard_focus (has_focus, t);
   //needs_update ();
 }
 
 void 
-qt_gui_rep::process_mouse ( simple_widget_rep *wid, string kind, SI x, SI y, 
+qt_gui_rep::process_mouse (qt_simple_widget_rep *wid, string kind, SI x, SI y, 
                             int mods, time_t t ) {
   typedef quintuple<string, SI, SI, int, time_t > T1;
   typedef pair<widget, T1> T;
+
+  if (wid->type != qt_widget_rep::simple_widget) {
+    if (DEBUG_QT)
+      cout << "WTF! Situation] process_mouse() for non simple_widget. Widget: " 
+           << wid->type_as_string() << LF; 
+    return;
+  }
   add_event ( 
-    queued_event ( QP_MOUSE, close_box<T> ( T (wid, T1 (kind, x, y, mods, t))))); 
+    queued_event (QP_MOUSE, close_box<T> ( T (wid, T1 (kind, x, y, mods, t))))); 
 //  wid -> handle_mouse (kind, x, y, mods, t);
 //  needs_update ();
 }
 
 void 
-qt_gui_rep::process_resize ( simple_widget_rep *wid, SI x, SI y ) {
+qt_gui_rep::process_resize (qt_simple_widget_rep *wid, SI x, SI y ) {
   typedef triple<widget, SI, SI > T;
-  add_event(  queued_event ( QP_RESIZE, close_box<T> (T(wid, x, y)))); 
+  if (wid->type != qt_widget_rep::simple_widget) {
+    if (DEBUG_QT)
+      cout << "WTF! Situation] process_resize() for non simple_widget. Widget: " 
+           << wid->type_as_string() << LF; 
+   
+    return;
+  }
+  add_event( queued_event ( QP_RESIZE, close_box<T> (T(wid, x, y)))); 
 //  wid -> handle_notify_resize (x, y);
 //  needs_update ();
 }
@@ -758,6 +830,13 @@ qt_gui_rep::process_command (command _cmd) {
              queued_event (QP_COMMAND, close_box< command > (_cmd)));
 }
 
+void 
+qt_gui_rep::process_command (command _cmd, object _args) {
+  typedef pair<command, object > T;
+  add_event ( 
+             queued_event (QP_COMMAND_ARGS, close_box< T > (T(_cmd,_args))));
+}
+
 void 
 qt_gui_rep::process_delayed_commands () {
   add_event ( 
@@ -782,7 +861,7 @@ qt_gui_rep::check_event (int type) {
   
   // do not interrupt while not in update
   // (for example while painting the icons in the menus)
-  if (!updating) return false;
+  if (!updating || disable_check_event) return false;
   
   switch (type) {
     case INTERRUPT_EVENT:
@@ -790,11 +869,8 @@ qt_gui_rep::check_event (int type) {
       else {
         time_t now= texmacs_time ();
         if (now - timeout_time < 0) return false;
-        //fill_event_queue();
-//        timeout_time= now + (100 / (N(waiting_events) + 1));
-        timeout_time= now + 1000;
-//        interrupted= (N(waiting_events) > 0);
-        interrupted= true;
+        timeout_time= now + time_credit;
+        interrupted= (N(waiting_events) > 0);
         //if (interrupted) cout << "INTERRUPT " 
         //  << now << "------------------" << LF;
         return interrupted;
@@ -822,7 +898,6 @@ qt_gui_rep::update () {
 
   // cout << "<" << texmacs_time() << " " << N(delayed_queue) << " ";
   
-  time_credit = 100;
   updatetimer->stop();
   updating = true;
   
@@ -832,18 +907,27 @@ qt_gui_rep::update () {
   time_t now = texmacs_time();
   needing_update = false;
 
+  time_credit = 100 / (N(waiting_events)+1);
+
   
   // 1.
   // check if a wait dialog is active and in the case remove it.
   // if we are here then the long operation is finished.
   
-  while (waitDialogs.count()) {
-    waitDialogs.last()->close();
-    waitDialogs.last()->deleteLater();
-    waitDialogs.removeLast();
+  if (waitDialogs.count()) {
+    waitWindow->layout()->removeWidget(waitDialogs.last());
+    waitWindow->close();
+    while (waitDialogs.count()) {
+      waitDialogs.last()->deleteLater();
+      waitDialogs.removeLast();
+    }
   }
   
-  
+  if (popup_wid_time > 0 && now > popup_wid_time) {
+    popup_wid_time = 0;
+    _popup_wid->send(SLOT_VISIBILITY, close_box<bool>(true));
+  }
+
   // 2.
   // manage delayed commands
   
@@ -858,38 +942,37 @@ qt_gui_rep::update () {
     // if there are not waiting events call at least once
     // the interpose handler
     if (the_interpose_handler) the_interpose_handler();
+  } else while ((N(waiting_events) > 0) && (count_events < max_proc_events)) {
+    // cout << "e";
+    //cout << "PROCESS QUEUED EVENTS START..."; cout.flush();
+    process_queued_events (1);
+    //cout << "AND END"  << LF;
+    count_events++;
+    
+    //cout << "TYPESET START..."; cout.flush();
+    if (the_interpose_handler) the_interpose_handler();
+    //cout << "AND END" << LF;
   }
-  do {
-    if ((N(waiting_events) > 0) && (count_events < max_proc_events)) {
-        // cout << "e";
-        //cout << "PROCESS QUEUED EVENTS START..."; cout.flush();
-        process_queued_events (1);
-        //cout << "AND END"  << LF;
-        count_events++;
-        
-        //cout << "TYPESET START..."; cout.flush();
-        if (the_interpose_handler) the_interpose_handler();
-        //cout << "AND END" << LF;
-      } else {
-      // redrawing
-      // cout << "r";
-      count_events = 0;
-      
-      // repaint invalid regions  
-      interrupted = false;
-      timeout_time = texmacs_time() + time_credit/(N(waiting_events)+1);
-      
-      //cout << "REPAINT START..."; cout.flush();
-      
-      QSetIterator<QTMWidget*> i(QTMWidget::all_widgets);
-      while (i.hasNext()) {
-        QTMWidget *w = i.next();
-        w->repaint_invalid_regions();
-      }
-      //cout << "AND END" << LF;
-    }
-  } while (N(waiting_events)>0);
   
+  {
+    // redrawing
+    // cout << "r";
+    count_events = 0;
+    
+    // repaint invalid regions  
+    interrupted = false;
+    timeout_time = texmacs_time() + time_credit;
+    
+    //cout << "REPAINT START..."; cout.flush();
+    
+    QSetIterator<QTMWidget*> i(QTMWidget::all_widgets);
+    while (i.hasNext()) {
+      QTMWidget *w = i.next();
+      if (w->isVisible())
+        w->repaint_invalid_regions();
+    }
+    //cout << "AND END" << LF;
+  }
   
   if (N(waiting_events) > 0) needing_update = true;
   if (interrupted) needing_update = true;
@@ -917,22 +1000,68 @@ qt_gui_rep::update () {
   //        The interval cannot be too small to keep CPU usage low in idle state
 } 
 
+/*! Called upon change of output language.
+ 
+ We currently emit a signal which forces every QTMAction to change his text
+ according to the new language, but the preferred Qt way seems to use
+ LanguageChange events (these are triggered upon installation of QTranslators)
+ */ 
+void
+qt_gui_rep::refresh_language() {
+  QTranslator* qtr = new QTranslator();
+  if (qtr->load("qt_" + 
+        QLocale (to_qstring (language_to_locale (get_output_language()))).name(),
+        QLibraryInfo::location(QLibraryInfo::TranslationsPath))) {
+    if (q_translator) 
+      qApp->removeTranslator(q_translator);
+    qApp->installTranslator(qtr);
+    q_translator = qtr;
+  } else {
+    delete qtr;
+  }
+  gui_helper->doRefresh();
+}
 
 
+
+/*! Display a popup help balloon (i.e. a tooltip) at window coordinates x, y 
+ 
+ We use a dedicated wrapper QWidget which handles mouse events: as soon as the 
+ mouse is moved out of we hide it.
+ Problem: the widget need not appear below the mouse pointer, thus making it
+ impossible to access links or widgets inside it.
+ Solution?? as soon as the mouse moves (out of the widget), start a timer, 
+ giving enough time to the user to move (back) into the widget, then abort the
+ close operation if he gets there.
+*/
+void
+qt_gui_rep::show_help_balloon (widget wid, SI x, SI y) {
+  if (popup_wid_time > 0) return;
+
+  _popup_wid = popup_window_widget(wid, "Balloon");
+  SI winx, winy;
+    // HACK around wrong? reporting of window widget for embedded texmacs-inputs:
+    // call get_window on the current window (concrete_window()->win is set to
+    // the texmacs-input widget whenever there is one)
+  get_position (get_window (concrete_window()->win), winx, winy);
+  set_position (_popup_wid, x+winx, y+winy);
+  popup_wid_time = texmacs_time() + 666;  // update() will eventually show the widget
+}
+
 /******************************************************************************
 * Font support
 ******************************************************************************/
 
 void
 set_default_font (string name) {
-        (void) name;
+  (void) name;
   // set the name of the default font
   // this is ignored since Qt handles fonts for the widgets
 }
 
 font
-get_default_font (bool tt, bool mini) {
-  (void) tt; (void) mini;
+get_default_font (bool tt, bool mini, bool bold) {
+  (void) tt; (void) mini; (void) bold;
   // get the default font or monospaced font (if tt is true)
         
   // return a null font since this function is not called in the Qt port.
@@ -948,8 +1077,8 @@ void
 load_system_font (string family, int size, int dpi,
                   font_metric& fnm, font_glyphs& fng)
 {
-        (void) family; (void) size; (void) dpi; (void) fnm; (void) fng;
-        if (DEBUG_QT) cout << "load_system_font(): SHOULD NOT BE CALLED\n";
+  (void) family; (void) size; (void) dpi; (void) fnm; (void) fng;
+  if (DEBUG_QT) cout << "load_system_font(): SHOULD NOT BE CALLED\n";
 }
 
 /******************************************************************************
@@ -957,17 +1086,19 @@ load_system_font (string family, int size, int dpi,
 ******************************************************************************/
 
 bool
-set_selection (string key, tree t, string s) {
+set_selection (string key, tree t, 
+               string s, string sv, string sh, string format) {
   // Copy a selection 't' with string equivalent 's' to the clipboard 'cb'
+  // and possibly the variants 'sv' and 'sh' for verbatim and html
   // Returns true on success
-  return the_gui->set_selection (key, t, s);
+  return the_gui->set_selection (key, t, s, sv, sh, format);
 }
 
 bool
-get_selection (string key, tree& t, string& s) {
+get_selection (string key, tree& t, string& s, string format) {
   // Retrieve the selection 't' with string equivalent 's' from clipboard 'cb'
   // Returns true on success; sets t to (extern s) for external selections
-  return the_gui->get_selection (key, t, s);
+  return the_gui->get_selection (key, t, s, format);
 }
 
 void
@@ -976,6 +1107,41 @@ clear_selection (string key) {
   the_gui->clear_selection (key);
 }
 
+bool
+qt_gui_rep::put_graphics_on_clipboard (url file) {
+   string extension= suffix (file) ;
+   
+  // for bitmaps this works :
+  if ((extension == "bmp") || (extension == "png") ||
+      (extension == "jpg") || (extension == "jpeg")) {
+    QClipboard *clipboard = QApplication::clipboard();
+    c_string tmp (concretize (file));
+    clipboard->setImage (QImage (tmp));
+  }
+  else {
+    // vector formats
+    // Are there applications receiving eps, pdf,... through the clipboard?
+    // I have not experimented with EMF/WMF (windows) or SVM (Ooo)
+    QString mime="image/*"; // generic image format;
+    if (extension == "eps") mime= "application/postscript";
+    if (extension == "pdf") mime= "application/pdf";
+    if (extension == "svg") mime= "image/svg+xml"; //this works with Inskcape version >= 0.47
+   
+    string filecontent;
+    load_string (as_string (file), filecontent, true);
+    
+    c_string tmp (filecontent);
+    QByteArray rawdata (tmp);
+
+    QMimeData *mymimeData = new QMimeData;
+    mymimeData->setData(mime, rawdata);
+
+    QClipboard *clipboard = QApplication::clipboard();
+    clipboard->setMimeData(mymimeData);// default mode= QClipboard::Clipboard 
+  }
+  return true;
+}
+
 /******************************************************************************
 * Miscellaneous
 ******************************************************************************/
@@ -987,6 +1153,16 @@ beep () {
   QApplication::beep();
 }
 
+void
+force_update() {
+  if (updating) {
+    needing_update = true;
+  }
+  else {
+    the_gui->update();
+  }
+}
+
 void
 needs_update () {
   // 0 ms - call immediately when all other events have 
@@ -1007,17 +1183,11 @@ check_event (int type) {
   return the_gui->check_event(type);
 }
 
-void image_gc (string name) {
-// Garbage collect images of a given name (may use wildcards)
-// This routine only needs to be implemented if you use your own image cache
- the_qt_renderer()->image_gc(name);
-}
-
 void
 show_help_balloon (widget balloon, SI x, SI y) {
   // Display a help balloon at position (x, y); the help balloon should
   // disappear as soon as the user presses a key or moves the mouse
-  (void) balloon; (void) x; (void) y;
+  the_gui->show_help_balloon(balloon, x, y);
 }
 
 void
@@ -1026,25 +1196,15 @@ show_wait_indicator (widget base, string message, string argument) {
   // The indicator might for instance be displayed at the center of
   // the base widget which triggered the lengthy operation;
   // the indicator should be removed if the message is empty
-  the_gui->show_wait_indicator(base,message,argument);
+  the_gui->show_wait_indicator(base, message, argument);
 }
 
-font x_font (string family, int size, int dpi)
-{
-  (void) family; (void) size; (void) dpi;
-  if (DEBUG_QT) cout << "x_font(): SHOULD NOT BE CALLED\n";
-  return NULL;
-}
-
-
-QString 
-QTMTranslator::translate ( const char * context, const char * sourceText, 
-                           const char * disambiguation ) const 
-{
-  (void) disambiguation;  (void) context;
-  if (DEBUG_QT) {
-    cout << "Translating: " << sourceText << LF;
-    cout << "Translation: " << qt_translate (sourceText) << LF;
+void
+external_event (string type, time_t t) {
+  // External events, such as pushing a button of a remote infrared commander
+  QTMWidget *tm_focus = qobject_cast<QTMWidget*>(qApp->focusWidget());
+  if (tm_focus) {
+    simple_widget_rep* wid = tm_focus->tm_widget();
+    if (wid) the_gui -> process_keypress (wid, type, t);
   }
-  return QString (to_qstring (qt_translate (sourceText)));
 }
