@@ -15,6 +15,8 @@
 #include "qt_utilities.hpp"
 #include "file.hpp"
 #include "image_files.hpp"
+#include "scheme.hpp"
+#include "frame.hpp"
 
 #include <QObject>
 #include <QWidget>
@@ -81,28 +83,29 @@ static hashmap<string,qt_pixmap> images;
 ******************************************************************************/
 
 qt_renderer_rep::qt_renderer_rep (QPainter *_painter, int w2, int h2):
-  basic_renderer_rep(w2, h2), painter(_painter) {}
+  basic_renderer_rep (true, w2, h2), painter(_painter) {}
 
 qt_renderer_rep::~qt_renderer_rep () {}
 
+void*
+qt_renderer_rep::get_handle () {
+  return (void*) this;
+}
+
 void
 qt_renderer_rep::begin (void* handle) {
-  QPaintDevice *device = (QPaintDevice*)handle;
-  painter->begin (device);
+  QPaintDevice *device = static_cast<QPaintDevice*>(handle);
+  if (!painter->begin (device) && DEBUG_QT)
+    cout << "qt_renderer_rep::begin(): uninitialized QPixmap of size "
+         << ((QPixmap*)handle)->width() << " x " << ((QPixmap*)handle)->height()
+         << LF;
+    
   w = painter->device()->width();
   h = painter->device()->height();
 }
 
 void qt_renderer_rep::end () { painter->end (); }
 
-QColor
-qt_color(color c)
-{
-  int r, g, b;
-  get_rgb_color (c,r,g,b);
-  return QColor(r, g, b);
-}
-
 void 
 qt_renderer_rep::get_extents (int& w2, int& h2) {  
   if (painter->device()) {
@@ -113,8 +116,40 @@ qt_renderer_rep::get_extents (int& w2, int& h2) {
 }
 
 /******************************************************************************
- * Clipping
- ******************************************************************************/
+* Transformations
+******************************************************************************/
+
+void
+qt_renderer_rep::set_transformation (frame fr) {
+  ASSERT (fr->linear, "only linear transformations have been implemented");
+
+  SI cx1, cy1, cx2, cy2;
+  get_clipping (cx1, cy1, cx2, cy2);
+  rectangle oclip (cx1, cy1, cx2, cy2);
+
+  frame cv= scaling (point (pixel, -pixel), point (-ox, -oy));
+  frame tr= invert (cv) * fr * cv;
+  point o = tr (point (0.0, 0.0));
+  point ux= tr (point (1.0, 0.0)) - o;
+  point uy= tr (point (0.0, 1.0)) - o;
+  //cout << "Set transformation " << o << ", " << ux << ", " << uy << "\n";
+  QTransform qtr (ux[0], ux[1], uy[0], uy[1], o[0], o[1]);
+  painter->save ();
+  painter->setTransform (qtr, true);
+
+  rectangle nclip= fr [oclip];
+  clip (nclip->x1, nclip->y1, nclip->x2, nclip->y2);
+}
+
+void
+qt_renderer_rep::reset_transformation () {
+  unclip ();
+  painter->restore ();
+}
+
+/******************************************************************************
+* Clipping
+******************************************************************************/
 
 void
 qt_renderer_rep::set_clipping (SI x1, SI y1, SI x2, SI y2, bool restore)
@@ -127,35 +162,95 @@ qt_renderer_rep::set_clipping (SI x1, SI y1, SI x2, SI y2, bool restore)
   if ((x1<x2) && (y2<y1)) {
     QRect r(x1,y2,x2-x1,y1-y2);
     painter->setClipRect(r);
+  } else {
+    painter->setClipRect(QRect());
   }
 }
 
-
-
 /******************************************************************************
 * Drawing 
 ******************************************************************************/
 
+bool is_percentage (tree t, string s= "%");
+double as_percentage (tree t);
+
+static QImage*
+get_pattern_image (brush br, SI pixel) {
+  tree pattern= br->get_pattern ();
+  url u= as_string (pattern[0]);
+  int imw_pt, imh_pt;
+  image_size (u, imw_pt, imh_pt);
+  double pt= ((double) 600*PIXEL) / 72.0;
+  SI imw= (SI) (((double) imw_pt) * pt);
+  SI imh= (SI) (((double) imh_pt) * pt);
+
+  SI w= imw, h= imh;
+  if (is_int (pattern[1])) w= as_int (pattern[1]);
+  else if (is_percentage (pattern[1]))
+    w= (SI) (as_percentage (pattern[1]) * ((double) w));
+  else if (is_percentage (pattern[1], "@"))
+    w= (SI) (as_percentage (pattern[1]) * ((double) h));
+  if (is_int (pattern[2])) h= as_int (pattern[2]);
+  else if (is_percentage (pattern[2]))
+    h= (SI) (as_percentage (pattern[2]) * ((double) h));
+  else if (is_percentage (pattern[2], "@"))
+    h= (SI) (as_percentage (pattern[2]) * ((double) w));
+  w= ((w + pixel - 1) / pixel);
+  h= ((h + pixel - 1) / pixel);
+  QImage* pm= get_image (u, w, h);
+  return pm;
+}
+
 void
-qt_renderer_rep::set_color (color c) {
-  basic_renderer_rep::set_color(c);
+qt_renderer_rep::set_pencil (pencil np) {
+  basic_renderer_rep::set_pencil (np);
   QPen p (painter->pen ());
   QBrush b (painter->brush ());
-  p.setColor (qt_color(cur_fg));
-  b.setColor (qt_color(cur_fg));
+  QColor qc= to_qcolor (pen->get_color ());
+  SI pw= 0;
+  if (pen->get_width () > pixel)
+    pw= (pen->get_width () + thicken) / (1.0*pixel);
+  p.setColor (qc);
+  b.setColor (qc);
+  p.setWidth (pw);
+  if (np->get_type () == pencil_brush) {
+    brush br= np->get_brush ();
+    QImage* pm= get_pattern_image (br, pixel);
+    int pattern_alpha= br->get_alpha ();
+    painter->setOpacity (qreal (pattern_alpha) / qreal (255));
+    if (pm != NULL) {
+      b= QBrush (*pm);
+      p= QPen (b, pw);
+    }
+  }
+  p.setCapStyle (pen->get_cap () == cap_round? Qt::RoundCap: Qt::SquareCap);
+  p.setJoinStyle (Qt::RoundJoin);
   painter->setPen (p);
   painter->setBrush (b);
 }
 
 void
-qt_renderer_rep::set_line_style (SI lw, int type, bool round) {
-  (void) type;
-  QPen p (painter->pen ());
-  if (lw <= pixel) p.setWidth (0);
-  else p.setWidth ((lw+thicken) / (1.0*pixel));
-  p.setCapStyle (round? Qt::RoundCap: Qt::SquareCap);
-  p.setJoinStyle (Qt::RoundJoin);
-  painter->setPen (p);
+qt_renderer_rep::set_brush (brush br) {
+  basic_renderer_rep::set_brush (br);
+  if (br->get_type () == brush_none) {
+    painter->setPen (QPen (Qt::NoPen));
+    painter->setBrush (QBrush (Qt::NoBrush));
+  }
+  else {
+    QPen p (painter->pen ());
+    QBrush b (painter->brush ());
+    QColor col= to_qcolor (pen->get_color ());
+    p.setColor (col);
+    b.setColor (col);
+    painter->setPen (p);
+    painter->setBrush (b);
+  }
+  if (br->get_type () == brush_pattern) {
+    QImage* pm= get_pattern_image (br, pixel);
+    int pattern_alpha= br->get_alpha ();
+    painter->setOpacity (qreal (pattern_alpha) / qreal (255));
+    if (pm != NULL) painter->setBrush (QBrush (*pm));
+  }
 }
 
 void
@@ -194,9 +289,9 @@ qt_renderer_rep::clear (SI x1, SI y1, SI x2, SI y2) {
   decode (x1, y1);
   decode (x2, y2);
   if ((x1>=x2) || (y1<=y2)) return;
-  QBrush brush (qt_color(cur_bg));
+  QBrush br (to_qcolor (bg_brush->get_color ()));
   painter->setRenderHints (0);
-  painter->fillRect (x1, y2, x2-x1, y1-y2, brush);       
+  painter->fillRect (x1, y2, x2-x1, y1-y2, br);       
 }
 
 void
@@ -220,14 +315,13 @@ qt_renderer_rep::fill (SI x1, SI y1, SI x2, SI y2) {
   decode (x1, y1);
   decode (x2, y2);
 
-  QBrush brush (qt_color(cur_fg));
+  QBrush br (to_qcolor (pen->get_color ()));
   painter->setRenderHints (0);
-  painter->fillRect (x1, y2, x2-x1, y1-y2, brush);       
+  painter->fillRect (x1, y2, x2-x1, y1-y2, br);       
 }
 
 void
 qt_renderer_rep::arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta) {
-  (void) alpha; (void) delta;
   if ((x1>=x2) || (y1>=y2)) return;
   decode (x1, y1);
   decode (x2, y2);
@@ -237,18 +331,17 @@ qt_renderer_rep::arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta) {
 
 void
 qt_renderer_rep::fill_arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta) {
-  (void) alpha; (void) delta;
   if ((x1>=x2) || (y1>=y2)) return;
   decode (x1, y1);
   decode (x2, y2);
-  QBrush brush(qt_color(cur_fg));
+  QBrush br(to_qcolor (pen->get_color ()));
   QPainterPath pp;
   pp.arcMoveTo (x1, y2, x2-x1, y1-y2, alpha / 64);
   pp.arcTo (x1, y2, x2-x1, y1-y2, alpha / 64, delta / 64);
   pp.closeSubpath ();
   pp.setFillRule (Qt::WindingFill);
   painter->setRenderHints (QPainter::Antialiasing);
-  painter->fillPath (pp, brush);
+  painter->fillPath (pp, br);
 }
 
 void
@@ -261,143 +354,51 @@ qt_renderer_rep::polygon (array<SI> x, array<SI> y, bool convex) {
     decode (xx, yy);
     poly[i] = QPointF (xx, yy);
   }
-  QBrush brush(qt_color(cur_fg));
+  QBrush br= painter->brush ();
+  if (is_nil (fg_brush) || fg_brush->get_type () != brush_pattern)
+    // FIXME: is this really necessary?
+    // The brush should have been set at the moment of set_pencil or set_brush
+    br= QBrush (to_qcolor (pen->get_color ()));
   QPainterPath pp;
   pp.addPolygon (poly);
   pp.closeSubpath ();
   pp.setFillRule (convex? Qt::OddEvenFill: Qt::WindingFill);
   painter->setRenderHints (QPainter::Antialiasing);
-  painter->fillPath (pp, brush);
+  painter->fillPath (pp, br);
 }
 
+void
+qt_renderer_rep::draw_triangle (SI x1, SI y1, SI x2, SI y2, SI x3, SI y3) {
+  array<SI> x (3), y (3);
+  x[0]= x1; y[0]= y1;
+  x[1]= x2; y[1]= y2;
+  x[2]= x3; y[2]= y3;
+
+  int i, n= N(x);
+  if ((N(y) != n) || (n<1)) return;
+  QPolygonF poly(n);
+  for (i=0; i<n; i++) {
+    SI xx= x[i], yy= y[i];
+    decode (xx, yy);
+    poly[i] = QPointF (xx, yy);
+  }
+  QBrush br= painter->brush ();
+  if (is_nil (fg_brush) || fg_brush->get_type () != brush_pattern)
+    // FIXME: is this really necessary?
+    // The brush should have been set at the moment of set_pencil or set_brush
+    br= QBrush (to_qcolor (pen->get_color ()));
+  QPainterPath pp;
+  pp.addPolygon (poly);
+  pp.closeSubpath ();
+  pp.setFillRule (Qt::OddEvenFill);
+  painter->setRenderHints (QPainter::Antialiasing, false);
+  painter->fillPath (pp, br);
+}
 
 /******************************************************************************
 * Image rendering
 ******************************************************************************/
 
-struct qt_cache_image_rep: cache_image_element_rep {
-  qt_cache_image_rep (int w2, int h2, time_t time2, QImage *ptr2):
-    cache_image_element_rep (w2, h2, time2, ptr2) {}
-  virtual ~qt_cache_image_rep () {
-    delete static_cast<QImage*> (ptr); }
-};
-
-void
-qt_renderer_rep::image (url u, SI w, SI h, SI x, SI y,
-                        double cx1, double cy1, double cx2, double cy2)
-{
-  // Given an image of original size (W, H),
-  // we display the part (cx1 * W, xy1 * H, cx2 * W, cy2 * H)
-  // at position (x, y) in a rectangle of size (w, h)
-
-  if(cx2<=cx1 || cy2<=cy1) return;
-
-  w= w/pixel; h= h/pixel;
-  decode (x, y);
-
-  // safety check
-  url ru = resolve(u);
-  u = is_none (ru) ? "$TEXMACS_PATH/misc/pixmaps/unknown.ps" : ru;
-  
-  QImage *pm = NULL;
-  tree lookup= tuple (u->t);
-  lookup << as_string (w ) << as_string (h )
-         << as_string (cx1) << as_string (cy1)
-         << as_string (cx2) << as_string (cy2) << "qt-image" ;
-  cache_image_element ci = get_image_cache(lookup);
-  if (!is_nil(ci)) {
-    pm= static_cast<QImage*> (ci->ptr);
-  } else {
-    // rendering
-    bool needs_crop= false;
-    if (qt_supports (u)) {
-      pm= new QImage (utf8_to_qstring (concretize (u)));
-      needs_crop= true;
-    } else if (suffix (u) == "ps" ||
-             suffix (u) == "eps" ||
-             suffix (u) == "pdf") {
-      url temp= url_temp (".png");
-      image_to_png (u, temp, w, h);
-      needs_crop= true;
-/*
-      string idstr= eval_system ("identify",u);
-      int i=0;
-      int a=0,b=0;
-      while(i<N(idstr)) {
-        b=0;
-        if(idstr[i]==' ') {
-          i++;
-          a=i;
-          while(i<N(idstr) && idstr[i]>'0' && idstr[i]<'9')
-            i++;
-          if(i>=N(idstr))
-            break;
-          if(idstr[i] != 'x')
-            continue;
-          i++;
-          b=i;
-          while(i<N(idstr) && idstr[i]>'0' && idstr[i]<'9')
-            i++;
-          if(i<N(idstr) && idstr[i]==' ')
-            break;
-        }
-        i++;
-      }
-      int iw,ih;
-      if(b>0) {
-        iw=as_int(idstr(a,b-1));
-        ih=as_int(idstr(b,i));
-      } else {
-        int bbx1,bby1,bbx2,bby2;
-        ps_bounding_box(u,bbx1,bby1,bbx2,bby2);
-        iw=bbx2-bbx1;
-        ih=bby2-bby1;
-      }
-
-//      float resx = 72*w/((bbx2-bbx1)*(cx2-cx1));
-//      float resy = 72*h/((bby2-bby1)*(cy2-cy1));
-      float resx = 144*w/(iw*(cx2-cx1));
-      float resy = 144*h/(ih*(cy2-cy1));
-
-      url temp= url_temp (".png");
-      system ("convert -density " * as_string(resx) * "x" * as_string(resy)
-             * " -scale 50% -crop " * as_string(w) * "x" * as_string(h)
-             * "+" * as_string (cx1*w/(cx2-cx1))
-             * "+" * as_string ((1-cy2)*h/(cy2-cy1))
-             * "! -background white -flatten", u, temp);
-*/
-      pm= new QImage (to_qstring (as_string (temp)));
-      remove (temp);
-    }
-    if (pm == NULL || pm->isNull ()) {
-      cout << "TeXmacs] warning: cannot render " << concretize (u) << "\n";
-      if (pm != NULL) delete pm;
-      return;
-    }
-
-    if(needs_crop) {
-      int iw= pm->width ();
-      int ih= pm->height ();
-      int x1= as_int (cx1 * iw);
-      int y1= as_int (cy1 * ih);
-      int x2= as_int (cx2 * iw);
-      int y2= as_int (cy2 * ih);
-      int ww= x2 - x1;
-      int hh= y2 - y1;
-
-      (*pm)= pm->copy(QRect (x1, hh-y2, ww, hh));
-      (*pm)= pm->scaled(w,h);
-    }
-
-    ci = tm_new<qt_cache_image_rep> (w,h, texmacs_time(), pm);
-    set_image_cache(lookup, ci);
-    (ci->nr)++;
-  }
-
-  painter->drawImage (x, y-h, *pm);
-};
-
-
 void
 qt_renderer_rep::draw_clipped (QImage *im, int w, int h, SI x, SI y) {
   (void) w; (void) h;
@@ -420,36 +421,83 @@ qt_renderer_rep::draw_clipped (QPixmap *im, int w, int h, SI x, SI y) {
   painter->drawPixmap (x, y, w, h, *im);
 }
 
+void
+qt_renderer_rep::draw_bis (int c, font_glyphs fng, SI x, SI y) {
+  // draw with background pattern
+  SI xo, yo;
+  glyph pre_gl= fng->get (c); if (is_nil (pre_gl)) return;
+  glyph gl= shrink (pre_gl, std_shrinkf, std_shrinkf, xo, yo);
+  int w= gl->width, h= gl->height;
+  QImage *im= new QImage (w, h, QImage::Format_ARGB32);
+  im->fill (Qt::transparent);
 
+  {
+    brush br= pen->get_brush ();
+    QImage* pm= get_pattern_image (br, brushpx==-1? pixel: brushpx);
+    int pattern_alpha= br->get_alpha ();
+    QPainter glim (im);
+    glim.setOpacity (qreal (pattern_alpha) / qreal (255));
+    if (pm != NULL) {
+      SI tx= x- xo*std_shrinkf, ty= y+ yo*std_shrinkf;
+      decode (tx, ty); ty--;
+      QBrush qbr (*pm);
+      QTransform qtf= painter->transform ();
+      qbr.setTransform (qtf.translate (-tx, -ty));
+      glim.setBrush (qbr);
+    }
+    glim.setPen (Qt::NoPen);
+    glim.drawRect (0, 0, w, h);
+
+    int nr_cols= std_shrinkf*std_shrinkf;
+    if (nr_cols >= 64) nr_cols= 64;
+    for (int j=0; j<h; j++)
+      for (int i=0; i<w; i++) {
+        color patcol= im->pixel (i, j);
+        int r, g, b, a;
+        get_rgb (patcol, r, g, b, a);
+        int col = gl->get_x (i, j);
+        im->setPixel (i, j, qRgba (r, g, b, (a*col)/nr_cols));
+      }
+  }
+
+  draw_clipped (im, w, h, x- xo*std_shrinkf, y+ yo*std_shrinkf);
+  delete im;
+}
 
 void
 qt_renderer_rep::draw (int c, font_glyphs fng, SI x, SI y) {
+  if (pen->get_type () == pencil_brush) {
+    draw_bis (c, fng, x, y);
+    return;
+  }
+
   // get the pixmap
-  basic_character xc (c, fng, sfactor, cur_fg, 0);
+  color fgc= pen->get_color ();
+  basic_character xc (c, fng, std_shrinkf, fgc, 0);
   qt_image mi = character_image [xc];
   if (is_nil(mi)) {
-    int r, g, b;
-    get_rgb (cur_fg, r, g, b);
+    int r, g, b, a;
+    get_rgb (fgc, r, g, b, a);
     SI xo, yo;
     glyph pre_gl= fng->get (c); if (is_nil (pre_gl)) return;
-    glyph gl= shrink (pre_gl, sfactor, sfactor, xo, yo);
+    glyph gl= shrink (pre_gl, std_shrinkf, std_shrinkf, xo, yo);
     int i, j, w= gl->width, h= gl->height;
 #ifdef QTMPIXMAPS
     QTMImage *im = new QPixmap(w,h);
     {
-      int nr_cols= sfactor*sfactor;
+      int nr_cols= std_shrinkf*std_shrinkf;
       if (nr_cols >= 64) nr_cols= 64;
 
       im->fill (Qt::transparent);
       QPainter pp(im);
       QPen pen(painter->pen());
-      QBrush brush(pen.color());
+      QBrush br(pen.color());
       pp.setPen(Qt::NoPen);
       for (j=0; j<h; j++)
         for (i=0; i<w; i++) {
           int col = gl->get_x (i, j);
-          brush.setColor (QColor (r, g, b, (255*col)/nr_cols));
-          pp.fillRect (i, j, 1, 1, brush);
+          br.setColor (QColor (r, g, b, (a*col)/nr_cols));
+          pp.fillRect (i, j, 1, 1, br);
         }
       pp.end();
     }
@@ -457,7 +505,7 @@ qt_renderer_rep::draw (int c, font_glyphs fng, SI x, SI y) {
     QTMImage *im= new QImage (w, h, QImage::Format_ARGB32);
     //QTMImage *im= new QImage (w, h, QImage::Format_ARGB32_Premultiplied);
     {
-      int nr_cols= sfactor*sfactor;
+      int nr_cols= std_shrinkf*std_shrinkf;
       if (nr_cols >= 64) nr_cols= 64;
 
       // the following line is disabled because
@@ -467,7 +515,7 @@ qt_renderer_rep::draw (int c, font_glyphs fng, SI x, SI y) {
       for (j=0; j<h; j++)
         for (i=0; i<w; i++) {
           int col = gl->get_x (i, j);
-          im->setPixel (i, j, qRgba (r, g, b, (255*col)/nr_cols));
+          im->setPixel (i, j, qRgba (r, g, b, (a*col)/nr_cols));
         }
     }
 #endif
@@ -480,53 +528,70 @@ qt_renderer_rep::draw (int c, font_glyphs fng, SI x, SI y) {
   }
 
   // draw the character
-  draw_clipped (mi->img, mi->w, mi->h, x- mi->xo*sfactor, y+ mi->yo*sfactor);
-}
-
-/******************************************************************************
-* Setting up and displaying xpm pixmaps
-******************************************************************************/
-
-extern int char_clip;
-
-QPixmap*
-qt_renderer_rep::xpm_image (url file_name) {
-  QPixmap *pxm= NULL;
-  qt_pixmap mi= images [as_string (file_name)];
-  if (is_nil (mi)) {
-    string sss;
-    load_string ("$TEXMACS_PIXMAP_PATH" * file_name, sss, false);
-    if (sss == "")
-      load_string ("$TEXMACS_PATH/misc/pixmaps/TeXmacs.xpm", sss, true);
-    uchar *buf= (uchar*) as_charp (sss);
-    pxm= new QPixmap();
-    pxm->loadFromData (buf, N(sss));
-    tm_delete_array ((char*) buf);
-    //out << sss;
-    //cout << "pxm: " << file_name << "(" << pxm->size().width()
-    //     << "," <<  pxm->size().height() << ")\n";
-    qt_pixmap mi2 (pxm, 0, 0, pxm->width(), pxm->height());
-    mi= mi2;
-    images (as_string (file_name))= mi2;
-  }
-  else pxm=  mi->img ;
-  return pxm;
+  //cout << (char)c << ": " << cx1/256 << ","  << cy1/256 << ","  
+  //<< cx2/256 << ","  << cy2/256 << LF; 
+  draw_clipped (mi->img, mi->w, mi->h, x- mi->xo*std_shrinkf, y+ mi->yo*std_shrinkf);
 }
 
 void
-qt_renderer_rep::xpm (url file_name, SI x, SI y) {
-  y -= pixel; // counter balance shift in draw_clipped
-  QPixmap* image = xpm_image (file_name);
-  ASSERT (sfactor == 1, "shrinking factor should be 1");
-  int w, h;
-  w = image->width ();
-  h = image->height ();
-  int old_clip= char_clip;
-  char_clip= true;
-  draw_clipped (image, w, h, x, y);
-  char_clip=old_clip;
+qt_renderer_rep::draw (const QFont& qfn, const QString& qs,
+                       SI x, SI y, double zoom) {
+  decode (x, y);
+  painter->setFont (qfn);
+  painter->translate (x, y);
+  painter->scale (zoom, zoom);
+  painter->drawText (0, 0, qs);
+  painter->resetTransform ();
 }
 
+/******************************************************************************
+ * main qt renderer
+ ******************************************************************************/
+
+
+qt_renderer_rep*
+the_qt_renderer () {
+  static QPainter *the_painter = NULL;
+  static qt_renderer_rep* the_renderer= NULL;
+  if (!the_renderer) {
+    the_painter = new QPainter();
+    the_renderer= tm_new<qt_renderer_rep> (the_painter);
+  }
+  return the_renderer;
+}
+
+
+/******************************************************************************
+ * Shadow management methods 
+ ******************************************************************************/
+
+/* Shadows are auxiliary renderers which allow double buffering and caching of
+ * graphics. TeXmacs has explicit double buffering from the X11 port. Maybe
+ * it would be better to design a better API abstracting from the low level 
+ * details but for the moment the following code and the qt_proxy_renderer_rep
+ * and qt_shadow_renderer_rep classes are designed to solve two problems:
+ * 
+ * 1) Qt has already double buffering.
+ * 2) in Qt we are not easily allowed to read onscreen pixels (we can only ask a
+ *    widget to redraw himself on a pixmap or read the screen pixels -- this has
+ *    the drawback that if our widget is under another one we won't read the 
+ *    right pixels)
+ * 
+ * qt_proxy_renderer_rep solves the double buffering problem: when texmacs asks
+ * a qt_renderer_rep for a shadow it is given a proxy of the original renderer
+ * texmacs uses this shadow for double buffering and the proxy will simply
+ * forward the drawing operations to the original surface and neglect all the
+ * syncronization operations
+ *
+ * to solve the second problem we do not draw directly on screen in QTMWidget.
+ * Instead we maintain an internal pixmap which represents the state of the pixels
+ * according to texmacs. When we are asked to initialize a qt_shadow_renderer_rep
+ * we simply read the pixels form this backing store. At the Qt level then
+ * (in QTMWidget) we make sure that the state of the backing store is in sync
+ * with the screen via paintEvent/repaint mechanism.
+ *
+ */
+
 
 void
 qt_renderer_rep::new_shadow (renderer& ren) {
@@ -553,7 +618,6 @@ qt_renderer_rep::delete_shadow (renderer& ren)  {
   }
 }
 
-
 void 
 qt_renderer_rep::get_shadow (renderer ren, SI x1, SI y1, SI x2, SI y2) {
   // FIXME: we should use the routine fetch later
@@ -567,23 +631,25 @@ qt_renderer_rep::get_shadow (renderer ren, SI x1, SI y1, SI x2, SI y2) {
   y2= min (y2, cy2- oy);
   shadow->ox= ox;
   shadow->oy= oy;
+  shadow->master= this;
   shadow->cx1= x1+ ox;
   shadow->cy1= y1+ oy;
   shadow->cx2= x2+ ox;
   shadow->cy2= y2+ oy;
-  shadow->master= this;
-#if 0
+  
   decode (x1, y1);
   decode (x2, y2);
   if (x1<x2 && y2<y1) {
     QRect rect = QRect(x1, y2, x2-x1, y1-y2);
-    //    shadow->painter->setCompositionMode(QPainter::CompositionMode_Source);   
-    shadow->painter->drawPixmap (rect, px, rect);
+    //    shadow->painter->setCompositionMode(QPainter::CompositionMode_Source);  
+    shadow->painter->setClipRect(rect);
+//    shadow->painter->drawPixmap (rect, px, rect);
     //    cout << "qt_shadow_renderer_rep::get_shadow " 
     //         << rectangle(x1,y2,x2,y1) << LF;
     //  XCopyArea (dpy, win, shadow->win, gc, x1, y2, x2-x1, y1-y2, x1, y2);
+  } else {
+    shadow->painter->setClipRect(QRect());
   }
-#endif
 }
 
 void 
@@ -625,26 +691,8 @@ qt_renderer_rep::apply_shadow (SI x1, SI y1, SI x2, SI y2)  {
 
 
 /******************************************************************************
- * main qt renderer
- ******************************************************************************/
-
-
-qt_renderer_rep*
-the_qt_renderer () {
-  static QPainter *the_painter = NULL;
-  static qt_renderer_rep* the_renderer= NULL;
-  if (!the_renderer) {
-    the_painter = new QPainter();
-    the_renderer= tm_new<qt_renderer_rep> (the_painter);
-  }
-  return the_renderer;
-}
-
-/******************************************************************************
- * proxy qt renderer
- ******************************************************************************/
-
-
+* proxy qt renderer
+******************************************************************************/
 
 void 
 qt_proxy_renderer_rep::new_shadow (renderer& ren) {
@@ -668,7 +716,6 @@ qt_proxy_renderer_rep::new_shadow (renderer& ren) {
           &(static_cast<qt_shadow_renderer_rep*>(ren)->px));
 }
 
-
 void 
 qt_proxy_renderer_rep::get_shadow (renderer ren, SI x1, SI y1, SI x2, SI y2) {
   // FIXME: we should use the routine fetch later
@@ -691,6 +738,9 @@ qt_proxy_renderer_rep::get_shadow (renderer ren, SI x1, SI y1, SI x2, SI y2) {
   decode (x2, y2);
   if (x1<x2 && y2<y1) {
     QRect rect = QRect(x1, y2, x2-x1, y1-y2);
+
+    shadow->painter->setClipRect(rect);
+
     //    shadow->painter->setCompositionMode(QPainter::CompositionMode_Source);
     QPixmap *_pixmap = static_cast<QPixmap*>(painter->device()); 
     if (_pixmap) {
@@ -699,7 +749,10 @@ qt_proxy_renderer_rep::get_shadow (renderer ren, SI x1, SI y1, SI x2, SI y2) {
     //    cout << "qt_shadow_renderer_rep::get_shadow " 
     //         << rectangle(x1,y2,x2,y1) << LF;
     //  XCopyArea (dpy, win, shadow->win, gc, x1, y2, x2-x1, y1-y2, x1, y2);
+  } else {
+    shadow->painter->setClipRect(QRect());
   }
+
 }
 
 
@@ -744,12 +797,14 @@ qt_shadow_renderer_rep::get_shadow (renderer ren, SI x1, SI y1, SI x2, SI y2) {
   decode (x2, y2);
   if (x1<x2 && y2<y1) {
     QRect rect = QRect(x1, y2, x2-x1, y1-y2);
+    shadow->painter->setClipRect(rect);
+
 //    shadow->painter->setCompositionMode(QPainter::CompositionMode_Source);   
     shadow->painter->drawPixmap (rect, px, rect);
 //    cout << "qt_shadow_renderer_rep::get_shadow " 
 //         << rectangle(x1,y2,x2,y1) << LF;
 //  XCopyArea (dpy, win, shadow->win, gc, x1, y2, x2-x1, y1-y2, x1, y2);
+  } else {
+    shadow->painter->setClipRect(QRect());
   }
 }
-
-
