@@ -4,10 +4,9 @@
 * DESCRIPTION: A file_chooser widget with horizontal and vertical scrollbars.
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Widkit/basic_widget.hpp"
@@ -21,6 +20,7 @@
 #include "image_files.hpp"
 #include "analyze.hpp"
 #include "scheme.hpp"
+#include "dictionary.hpp"
 
 #ifdef OS_WIN32
 #include <X11/Xlib.h>
@@ -39,19 +39,17 @@
 #define BUTTON_CANCEL    6
 #define IMAGE_HSIZE      7
 #define IMAGE_VSIZE      8
-#define IMAGE_CLIP_X1    9
-#define IMAGE_CLIP_Y1   10
-#define IMAGE_CLIP_X2   11
-#define IMAGE_CLIP_Y2   12
-#define CHANGE_SUFFIXES 13
+#define IMAGE_XPOS       9
+#define IMAGE_YPOS      10
+#define CHANGE_SUFFIXES 11
 
 class file_chooser_command_rep: public command_rep {
   wk_widget_rep* fch;
-  int                type;
+  int type;
 public:
   file_chooser_command_rep (wk_widget w, int t): fch(w.rep), type(t) {}
   void apply ();
-  ostream& print (ostream& out) {
+  tm_ostream& print (tm_ostream& out) {
     return out << "File chooser command (" << type << ")"; }
 };
 
@@ -113,10 +111,9 @@ file_chooser_command_rep::apply () {
       string which, s;
       if (type == IMAGE_HSIZE) which= "hsize";
       else if (type == IMAGE_VSIZE) which= "vsize";
-      else if (type == IMAGE_CLIP_X1) which= "clip-x1";
-      else if (type == IMAGE_CLIP_Y1) which= "clip-y1";
-      else if (type == IMAGE_CLIP_X2) which= "clip-x2";
-      else which= "clip-y2";
+      else if (type == IMAGE_XPOS) which= "xpos";
+      else if (type == IMAGE_YPOS) which= "ypos";
+      else break;
       wk_widget inp= fch_wid[0]["image"]["parameters"][which]["input"];
       inp << get_string ("input", s);
       if (s == "#f") fch_wid << set_string ("return", "#f");
@@ -128,7 +125,7 @@ file_chooser_command_rep::apply () {
 
 command
 file_chooser_command (wk_widget fch, int type) {
-  return new file_chooser_command_rep (fch, type);
+  return tm_new<file_chooser_command_rep> (fch, type);
 }
 
 /******************************************************************************
@@ -212,22 +209,23 @@ file_list_widget_rep::handle_get_size (get_size_event ev) {
 
 void
 file_list_widget_rep::handle_repaint (repaint_event ev) { (void) ev;
+  renderer ren= ev->win;
   int i; 
   metric ex;
-  win->set_background (white);
-  win->clear (0, -h, w, 0);
+  ren->set_background (white);
+  ren->clear (0, -h, w, 0);
   font fn= get_default_font ();
-  win->set_shrinking_factor (3);
+  ren->set_shrinking_factor (3);
   SI y= 0;
   for (i=0; i<N(names); i++)
     if (lids[i]) {
-      win->set_color (black);
-      if (hilight == i) win->set_color (red);
+      ren->set_pencil (black);
+      if (hilight == i) ren->set_pencil (red);
       fn->var_get_extents (names[i], ex);
-      fn ->draw (win, names[i], 9*PIXEL, y-fn->y2-6*PIXEL);
+      fn ->draw (ren, names[i], 9*PIXEL, y-fn->y2-6*PIXEL);
       y += fn->y1- fn->y2- 12*PIXEL;
     }
-  win->set_shrinking_factor (1);
+  ren->set_shrinking_factor (1);
 }
 
 void
@@ -322,9 +320,10 @@ image_widget_rep::handle_get_size (get_size_event ev) {
 
 void
 image_widget_rep::handle_repaint (repaint_event ev) { (void) ev;
-  win->set_background (white);
-  win->clear (0, 0, w, h);
-  layout_dark_outline (win, 0, 0, w, h);
+  renderer ren= ev->win;
+  ren->set_background (white);
+  ren->clear (0, 0, w, h);
+  layout_dark_outline (ren, 0, 0, w, h);
   if (file_name != "") {
     SI iw, ih;
     image_size (url_system (file_name), iw, ih);
@@ -336,8 +335,9 @@ image_widget_rep::handle_repaint (repaint_event ev) { (void) ev;
       else ww= (hh * iw) / ih;
     }
 
-    win->image (url_system (file_name),
-		ww, hh, PIXEL, PIXEL, 0.0, 0.0, 1.0, 1.0);
+    SI px= ren->pixel;
+    scalable im= load_scalable_image (url_system (file_name), ww, hh, px);
+    ren->draw_scalable (im, PIXEL, PIXEL);
   }
 }
 
@@ -358,13 +358,12 @@ class file_chooser_widget_rep: public attribute_widget_rep {
   command       cmd;
   string        type;
   array<string> suffix;
-  string        magn;
 
 public:
-  file_chooser_widget_rep (command cmd, string type, string magn);
+  file_chooser_widget_rep (command cmd, string type);
   operator tree ();
 
-  wk_widget input_widget (string what, int type);
+  wk_widget input_widget (string what, string ref, int type);
   wk_widget button_widget (string what, int type);
 
   void handle_get_size (get_size_event ev);
@@ -397,25 +396,35 @@ public:
 ******************************************************************************/
 
 wk_widget
-file_chooser_widget_rep::input_widget (string what, int type) {
-  array<wk_widget> ww (2);
-  array<string> nn (2);
-  ww[0]= text_wk_widget (what, false, "english");
-  ww[1]= input_text_wk_widget (file_chooser_command (this, type));
-  nn[1]= "input";
-  if (type == CHANGE_DIR) ww[1] << set_string ("type", "directory");
+file_chooser_widget_rep::input_widget (string what, string ref, int type) {
+  SI ww1= 1280*PIXEL, hh1= 64*PIXEL, ww2= 1280*PIXEL, hh2= 64*PIXEL;
+  wk_widget ref_w = text_wk_widget (translate (ref));
+  wk_widget what_w= text_wk_widget (translate (what));
+  ref_w  << get_size (ww1, hh1, 0);
+  what_w << get_size (ww2, hh2, 0);
+  SI offset= max (ww1 - ww2, 0);
+
+  array<wk_widget> ww (4);
+  array<string> nn (4);
+  ww[0]= glue_wk_widget (false, false, offset);
+  ww[1]= what_w;
+  ww[2]= input_text_wk_widget (file_chooser_command (this, type));
+  nn[2]= "input";
+  ww[3]= glue_wk_widget (false, false, 3*PIXEL);
+  if (type == CHANGE_DIR) ww[2] << set_string ("type", "directory");
   return horizontal_list (ww, nn);
 }
 
 wk_widget
 file_chooser_widget_rep::button_widget (string what, int type) {
-  return command_button (text_wk_widget (what, false, "english"),
-			 file_chooser_command (this, type), true);
+  return command_button (text_wk_widget (translate (what)),
+			 file_chooser_command (this, type),
+			 WIDGET_STYLE_BUTTON);
 }
 
 file_chooser_widget_rep::file_chooser_widget_rep (
-  command cmd2, string type2, string magn2):
-  attribute_widget_rep (1), cmd (cmd2), type (type2), magn (magn2)
+  command cmd2, string type2):
+  attribute_widget_rep (1), cmd (cmd2), type (type2)
 {
   ref_count++;
 
@@ -423,7 +432,7 @@ file_chooser_widget_rep::file_chooser_widget_rep (
   int i, n= N(t);
   for (i=0; i<n; i++)
     suffix << ("." * as_string (t[i]));
-  if (n == 0) suffix << "";
+  if (n == 0) suffix << string ("");
 
   SI sep= 3*PIXEL;
   int cw2n= 5;
@@ -431,13 +440,11 @@ file_chooser_widget_rep::file_chooser_widget_rep (
   array<wk_widget> cw2 (cw2n);
   array<string> cn2 (cw2n);
   cw2[0]= glue_wk_widget (false, true, sep);
-  cw2[1]=
-    canvas_widget (wk_widget (new file_list_widget_rep (this, suffix, true)));
+  cw2[1]= canvas_widget (wk_widget (tm_new<file_list_widget_rep> (this, suffix, true)));
   cn2[1]= "directories";
   cw2[2]= glue_wk_widget (false, true, sep);
   if (type != "directory") {
-    cw2[3]=
-      canvas_widget (wk_widget (new file_list_widget_rep (this,suffix,false)));
+    cw2[3]= canvas_widget (wk_widget (tm_new<file_list_widget_rep> (this, suffix, false)));
     cn2[3]= "files";
     cw2[4]= glue_wk_widget (false, true, sep-PIXEL);
   }
@@ -451,11 +458,11 @@ file_chooser_widget_rep::file_chooser_widget_rep (
       driveString[0] = x;
       drive_menu << emit_insert (driveString,
 	command_button (text_wk_widget (driveString),
-			new drive_menu_command_rep (this, driveString)));
+			tm_new<drive_menu_command_rep> (this, driveString)));
     }
   array<wk_widget> drw (2);
-  drw[0] = pullright_button (text_wk_widget ("Drive"), drive_menu);
-  drw[1] = text_wk_widget("");
+  drw[0] = pullright_button (text_wk_widget (translate ("Drive")), drive_menu);
+  drw[1] = text_wk_widget ("");
   // drw[1]= glue_wk_widget (false, true, sep);
 #endif
 
@@ -465,7 +472,8 @@ file_chooser_widget_rep::file_chooser_widget_rep (
 #ifdef OS_WIN32
   array<wk_widget> cw3 (11);
   cw3[0]= glue_wk_widget (false, false, sep);
-  cw3[1]= pulldown_button (text_wk_widget ("Drive"), drive_menu, true);
+  cw3[1]= pulldown_button (text_wk_widget (translate ("Drive")),
+			   drive_menu, true);
   cw3[2]= glue_wk_widget (false, false, sep);
   cw3[3]= button_widget ("Home", BUTTON_HOME);
   cw3[4]= glue_wk_widget (false, false, sep);
@@ -485,7 +493,7 @@ file_chooser_widget_rep::file_chooser_widget_rep (
   cw3[5]= button_widget ("Ok", BUTTON_OK);
   cw3[6]= glue_wk_widget (false, false, sep);
   cw3[7]= button_widget ("Cancel", BUTTON_CANCEL);
-#ifdef OS_DARWIN
+#ifdef OS_MACOS
   cw3[8]= glue_wk_widget (false, false, sep + 14*PIXEL);
 #else
   cw3[8]= glue_wk_widget (false, false, sep);
@@ -493,12 +501,12 @@ file_chooser_widget_rep::file_chooser_widget_rep (
 #endif
 
   int cwn= 11;
-  if (type == "image") cwn= 17;
+  if (type == "image") cwn= 16;
   if (type == "directory") cwn= 7;
   array<wk_widget> cw (cwn);
   array<string> cn (cwn);
   cw[0]= glue_wk_widget (true, false, 0, sep);
-  cw[1]= input_widget ("Directory:", CHANGE_DIR);
+  cw[1]= input_widget ("Directory:", "Directory:", CHANGE_DIR);
   cn[1]= "directory";
   cw[2]= glue_wk_widget (true, false, 0, sep);
 
@@ -508,10 +516,10 @@ file_chooser_widget_rep::file_chooser_widget_rep (
   }
 
   if (type != "directory") {
-    cw[3]= input_widget ("File:", CHANGE_FILE);
+    cw[3]= input_widget ("File:", "Directory:", CHANGE_FILE);
     cn[3]= "file";
     cw[4]= glue_wk_widget (true, false, 0, sep);
-    cw[5]= input_widget ("Suffixes:", CHANGE_SUFFIXES);
+    cw[5]= input_widget ("Suffixes:", "Directory:", CHANGE_SUFFIXES);
     cn[5]= "suffixes";
     cw[6]= glue_wk_widget (true, false, 0, sep);
     cw[7]= horizontal_list (cw2, cn2);
@@ -519,25 +527,19 @@ file_chooser_widget_rep::file_chooser_widget_rep (
   }
 
   if (type == "image") {
-    array<wk_widget> imw (11);
-    array<string> ims (11);
-    imw[ 0]= input_widget ("width:", IMAGE_HSIZE);
+    array<wk_widget> imw (7);
+    array<string> ims (7);
+    imw[ 0]= input_widget ("width:", "y-position:", IMAGE_HSIZE);
     ims[ 0]= "hsize";
     imw[ 1]= glue_wk_widget (true, false, 0, sep);
-    imw[ 2]= input_widget ("height:", IMAGE_VSIZE);
+    imw[ 2]= input_widget ("height:", "y-position:", IMAGE_VSIZE);
     ims[ 2]= "vsize";
     imw[ 3]= glue_wk_widget (true, false, 0, sep);
-    imw[ 4]= input_widget ("left border:", IMAGE_CLIP_X1);
-    ims[ 4]= "clip-x1";
+    imw[ 4]= input_widget ("x-position:", "y-position:", IMAGE_XPOS);
+    ims[ 4]= "xpos";
     imw[ 5]= glue_wk_widget (true, false, 0, sep);
-    imw[ 6]= input_widget ("lower border:", IMAGE_CLIP_Y1);
-    ims[ 6]= "clip-y1";
-    imw[ 7]= glue_wk_widget (true, false, 0, sep);
-    imw[ 8]= input_widget ("right border:", IMAGE_CLIP_X2);
-    ims[ 8]= "clip-x2";
-    imw[ 9]= glue_wk_widget (true, false, 0, sep);
-    imw[10]= input_widget ("upper border:", IMAGE_CLIP_Y2);
-    ims[10]= "clip-y2";
+    imw[ 6]= input_widget ("y-position:", "y-position:", IMAGE_YPOS);
+    ims[ 6]= "ypos";
 
     array<wk_widget> cw4 (5);
     array<string> cn4 (5);
@@ -545,17 +547,18 @@ file_chooser_widget_rep::file_chooser_widget_rep (
     cw4[1] = vertical_list (imw, ims);
     cn4[1] = "parameters";
     cw4[2] = glue_wk_widget (false, false, sep);
-    cw4[3] = new image_widget_rep ();
+    cw4[3] = tm_new<image_widget_rep> ();
     cn4[3] = "image";
     cw4[4] = glue_wk_widget (false, false, sep);
 
-    cw[ 8] = separator_wk_widget ();
-    cw[ 9] = glue_wk_widget (true, false, 0, sep);
-    cw[10] = horizontal_list (cw4, cn4);
-    cn[10] = "image";
-    cw[11] = glue_wk_widget (true, false, 0, sep);
-    cw[12] = separator_wk_widget ();
-    cw[13] = glue_wk_widget (true, false, 0, sep);
+    //cw[ 8] = glue_wk_widget (true, false, 0, sep);
+    //cw[ 9] = separator_wk_widget ();
+    cw[ 8] = glue_wk_widget (true, false, 0, sep);
+    cw[ 9] = horizontal_list (cw4, cn4);
+    cn[ 9] = "image";
+    cw[10] = glue_wk_widget (true, false, 0, sep);
+    cw[11] = separator_wk_widget ();
+    cw[12] = glue_wk_widget (true, false, 0, sep);
   }
 
   cw[cwn-3]= glue_wk_widget (true, false, 0, sep);
@@ -611,16 +614,8 @@ file_chooser_widget_rep::handle_set_string (set_string_event ev) {
       if (name != "") name= as_string (url_system (scm_unquote (dir), name));
       a[0]["image"]["image"] << set_string ("name", name);
       array<string> ps_suffix;
-      ps_suffix << ".ps" << ".eps";
+      ps_suffix << string (".ps") << string (".eps");
       wk_widget par_wid= a[0]["image"]["parameters"];
-      if (has_suffix (name, ps_suffix)) {
-	par_wid["hsize"]["input"] << set_string ("input", "");
-	par_wid["vsize"]["input"] << set_string ("input", "");
-      }
-      else {
-	par_wid["hsize"]["input"] << set_string ("input", magn);
-	par_wid["vsize"]["input"] << set_string ("input", magn);
-      }
     }
   }
   else if (ev->which == "return") {
@@ -655,7 +650,7 @@ file_chooser_widget_rep::handle_set_string (set_string_event ev) {
       }
       i=j+1;
     }
-    if (!any) suffix << "";
+    if (!any) suffix << string ("");
     // Force a refresh:
     string dir;
     a[0]["directory"]["input"] << get_string ("input", dir);
@@ -674,26 +669,23 @@ file_chooser_widget_rep::handle_get_string (get_string_event ev) {
       a[0]["directory"]["input"] << get_string ("input", name);
       if (name == "#f") { ev->s= "#f"; return; }
       url u= url_system (scm_unquote (dir));
-      ev->s= "(url-system " * scm_quote (as_string (u)) * ")";
+      ev->s= "(system->url " * scm_quote (as_string (u)) * ")";
     }
     else {
       a[0]["file"]["input"] << get_string ("input", name);
       if (name == "#f") { ev->s= "#f"; return; }
       url u= url_system (scm_unquote (dir)) * url_system (scm_unquote (name));
-      ev->s= "(url-system " * scm_quote (as_string (u)) * ")";
+      ev->s= "(system->url " * scm_quote (as_string (u)) * ")";
     }
     if (type == "image") {
-      string hsize, vsize, cx1, cy1, cx2, cy2;
+      string hsize, vsize, xpos, ypos;
       wk_widget par= a[0]["image"]["parameters"];
       par["hsize"]["input"] << get_string ("input", hsize);
       par["vsize"]["input"] << get_string ("input", vsize);
-      par["clip-x1"]["input"] << get_string ("input", cx1);
-      par["clip-y1"]["input"] << get_string ("input", cy1);
-      par["clip-x2"]["input"] << get_string ("input", cx2);
-      par["clip-y2"]["input"] << get_string ("input", cy2);
-      ev->s=
-	"(list " * ev->s * " " * hsize * " " * vsize * " "
-	         * cx1 * " " * cy1 * " " * cx2 * " " * cy2 * ")";
+      par["xpos"]["input"] << get_string ("input", xpos);
+      par["ypos"]["input"] << get_string ("input", ypos);
+      ev->s= "(list " * ev->s * " " * hsize * " " * vsize * " " *
+	                xpos * " " * ypos * ")";
     }
   }
   else attribute_widget_rep::handle_get_string (ev);
@@ -710,6 +702,6 @@ file_chooser_widget_rep::handle_destroy (destroy_event ev) {
 ******************************************************************************/
 
 wk_widget
-file_chooser_wk_widget (command cmd, string type, string magn) {
-  return new file_chooser_widget_rep (cmd, type, magn);
+file_chooser_wk_widget (command cmd, string type) {
+  return tm_new<file_chooser_widget_rep> (cmd, type);
 }
