@@ -17,6 +17,7 @@
 #include "image_files.hpp"
 #include <X11/cursorfont.h>
 #include "message.hpp"
+#include "x_picture.hpp"
 
 extern hashmap<Window,pointer> Window_to_window;
 
@@ -56,9 +57,14 @@ x_gui_rep::prepare_color (int sf, color fg, color bg) {
   x_character col_entry (0, font_glyphs (), sf, fg, bg);
   color* cols= (color*) color_scale [col_entry];
   if (cols == NULL) {
-    int fR, fG, fB, bR, bG, bB, j;
-    get_rgb_color (fg, fR, fG, fB);
-    get_rgb_color (bg, bR, bG, bB);
+    int fR, fG, fB, fA, bR, bG, bB, bA, j;
+    get_rgb_color (fg, fR, fG, fB, fA);
+    get_rgb_color (bg, bR, bG, bB, bA);
+    if (fA != 255) {
+      fR= (bR * (255 - fA) + fR * fA) / 255;
+      fG= (bG * (255 - fA) + fG * fA) / 255;
+      fB= (bB * (255 - fA) + fB * fA) / 255;
+    }
     cols= tm_new_array<color> (nr_cols+1);
     for (j=0; j<=nr_cols; j++)
       cols [nr_cols-j]= rgb_color ((bR*j + fR*(nr_cols-j)) / nr_cols,
@@ -287,12 +293,15 @@ x_gui_rep::clear_selection (string key) {
 }
 
 bool
-set_selection (string key, tree t, string s) {
+set_selection (string key, tree t,
+               string s, string sv, string sh, string format) {
+  (void) format;
   return the_gui->set_selection (key, t, s);
 }
 
 bool
-get_selection (string key, tree& t, string& s) {
+get_selection (string key, tree& t, string& s, string format) {
+  (void) format;
   return the_gui->get_selection (key, t, s);
 }
 
@@ -446,12 +455,8 @@ void
 x_gui_rep::set_mouse_pointer (widget w, string name, string mask_name) {
   static hashmap<string,tree> xpm_cache ("");
   if (mask_name=="") mask_name= name;
-  x_drawable_rep* dra= tm_new<x_drawable_rep> (this, 1, 1);
-  dra->xpm_initialize (name);
-  if (mask_name!=name) dra->xpm_initialize (mask_name);
-  tm_delete (dra);
-  Pixmap curs= (Pixmap) xpm_bitmap [name];
-  Pixmap mask= (Pixmap) xpm_bitmap [mask_name];
+  Pixmap curs= retrieve_bitmap (load_xpm (name));
+  Pixmap mask= retrieve_bitmap (load_xpm (mask_name));
 
   if (!xpm_cache->contains (name))
     xpm_cache (name)= xpm_load (name);
@@ -463,12 +468,10 @@ x_gui_rep::set_mouse_pointer (widget w, string name, string mask_name) {
   array<SI> hotspot= xpm_hotspot (xpm_cache[name]);
   ASSERT (N(hotspot) != 0, "missing hotspot");
   array<string> cnames_mask= xpm_colors (xpm_cache[mask_name]);
-  char* bgcolor= as_charp (N(cnames_mask)>1 ? cnames_mask[1] :
-					      string ("white"));
-  char* fgcolor= as_charp (N(cnames_curs)>1 ? cnames_curs[1] :
-					      string ("black"));
-  if (!strcmp (bgcolor, "none")) bgcolor= as_charp (string ("white"));
-  if (!strcmp (fgcolor, "none")) fgcolor= as_charp (string ("white"));
+  c_string bgcolor (N(cnames_mask)>1 ? cnames_mask[1]: string ("white"));
+  c_string fgcolor (N(cnames_curs)>1 ? cnames_curs[1]: string ("black"));
+  if (!strcmp (bgcolor, "none")) bgcolor= c_string (string ("white"));
+  if (!strcmp (fgcolor, "none")) fgcolor= c_string (string ("white"));
 
   XColor *bg= NULL, *fg= NULL;
   XColor exact1, closest1;
@@ -483,9 +486,6 @@ x_gui_rep::set_mouse_pointer (widget w, string name, string mask_name) {
   else if (XAllocColor (dpy, cols, &closest2)) bg= &closest2;
   else FAILED ("unable to allocate bgcolor");
 
-  tm_delete_array (bgcolor);
-  tm_delete_array (fgcolor);
-
   SI x= hotspot[0], y= hotspot[1];
   Cursor cursor=XCreatePixmapCursor (dpy, curs, mask, fg, bg, x, y);
   x_window win= get_x_window (w);
@@ -535,7 +535,7 @@ x_gui_rep::show_wait_indicator (widget w, string message, string arg) {
   // to return to the main loop to map and redraw it.
   x_window ww= get_x_window (w);
   if (ww == NULL || message == "") return;
-  if (arg != "") message= message * "#" * arg * "...";
+  if (arg != "") message= message * " " * arg * "...";
   SI width= 400*PIXEL, height= 160*PIXEL;
   widget wait_wid= wait_widget (width, height, message);
   SI mid_x= (ww->win_w>>1)*PIXEL, mid_y= -(ww->win_h>>1)*PIXEL + height;
@@ -551,6 +551,16 @@ x_gui_rep::show_wait_indicator (widget w, string message, string arg) {
   send_invalidate_all (old_wid);
 }
 
+void
+x_gui_rep::external_event (string type, time_t t) {
+  (void) t;
+  if (!is_nil (windows_l)) {
+    Window win= windows_l->item;
+    x_window x_win= (x_window) Window_to_window[win];
+    x_win->key_event (type);
+  }
+}
+
 bool
 x_gui_rep::check_event (int type) {
   bool status;
@@ -605,6 +615,11 @@ show_wait_indicator (widget w, string message, string arg) {
   the_gui->show_wait_indicator (w, message, arg);
 }
 
+void
+external_event (string type, time_t t) {
+  the_gui->external_event (type, t);
+}
+
 void
 needs_update () {
 }
@@ -613,3 +628,10 @@ bool
 check_event (int type) {
   return the_gui->check_event (type);
 }
+
+bool
+gui_interrupted (bool check) {
+  return check_event (check? INTERRUPT_EVENT: INTERRUPTED_EVENT);
+}
+
+
