@@ -4,10 +4,9 @@
 * DESCRIPTION: Initialization of the X11 window manager
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "X11/x_window.hpp"
@@ -17,6 +16,7 @@
 #include "dictionary.hpp"
 #include "iterator.hpp"
 #include "message.hpp"
+#include "sys_utils.hpp"
 #include <locale.h>
 
 x_gui_rep* the_gui= NULL;
@@ -26,6 +26,7 @@ extern hashmap<Window,pointer> Window_to_window;
 * Set up colors
 ******************************************************************************/
 
+bool true_color= false;
 bool reverse_colors= false;
 
 color black, white, red, green, blue;
@@ -37,8 +38,31 @@ static int CFACTOR= 5;
 static int GREYS  = 16;
 static int CTOTAL = (CFACTOR*CFACTOR*CFACTOR+GREYS+1);
 
+static void
+reverse (int& r, int& g, int& b) {
+  int m= min (r, min (g, b));
+  int M= max (r, max (g, b));
+  int t= (r + g + b) / 3;
+  int tt= 255 - t;
+  double mu= 1.0;
+  // tt= 6 * tt / 7;
+  if (M != m) {
+    double lambda1= max (((double) (t - m)) / t,
+			 ((double) (M - t)) / (255 - t));
+    double lambda2= max (((double) (t - m)) / tt,
+			 ((double) (M - t)) / (255 - tt));
+    mu= lambda1 / lambda2;
+  }
+  r= (int) (tt + mu * (r - t) + 0.5);
+  g= (int) (tt + mu * (g - t) + 0.5);
+  b= (int) (tt + mu * (b - t) + 0.5);
+}
+
 int
 x_alloc_color (int r, int g, int b) {
+  if (true_color)
+    return ((r >> 8) << 16) + ((g >> 8) << 8) + (b >> 8);
+
   if (reverse_colors) {
     int m= min (r, min (g, b));
     int M= max (r, max (g, b));
@@ -69,9 +93,10 @@ x_alloc_color (int r, int g, int b) {
 
 void
 x_init_color_map () {
-  int i, r, g, b;
+  if (true_color) return;
 
-  the_gui->cmap= new color [CTOTAL];
+  int i, r, g, b;
+  the_gui->cmap= tm_new_array<color> (CTOTAL);
 
   for (i=0; i<=GREYS; i++)
     the_gui->cmap[i]=
@@ -88,48 +113,80 @@ x_init_color_map () {
 }
 
 color
-rgb_color (int r, int g, int b) {
-  if ((r==g) && (g==b)) return (r*GREYS+ 128)/255;
+rgb_color (int r, int g, int b, int a) {
+  if (true_color) {
+    if (reverse_colors) reverse (r, g, b);
+    return (a << 24) + (r << 16) + (g << 8) + b;
+  }
+  else if ((r==g) && (g==b))
+    return (a << 24) + ((r*GREYS+ 128)/255);
   else {
     r= (r*CSCALES+ 128)/255;
     g= (g*CSCALES+ 128)/255;
     b= (b*CSCALES+ 128)/255;
-    return r*CFACTOR*CFACTOR+ g*CFACTOR+ b+ GREYS+ 1;
+    return (a << 24) + r*CFACTOR*CFACTOR + g*CFACTOR + b + GREYS + 1;
   }
 }
 
 void
-get_rgb_color (color col, int& r, int& g, int& b) {
-  if (col <= GREYS) {
-    r= (col*255)/GREYS;
-    g= (col*255)/GREYS;
-    b= (col*255)/GREYS;
+get_rgb_color (color col, int& r, int& g, int& b, int& a) {
+  if (true_color) {
+    a= (col >> 24) & 255;
+    r= (col >> 16) & 255;
+    g= (col >> 8 ) & 255;
+    b=  col        & 255;
+    if (reverse_colors) reverse (r, g, b);
   }
   else {
-    int rr, gg, bb;
-    col-= (GREYS+1);
-    bb  = col % CFACTOR;
-    gg  = (col/CFACTOR) % CFACTOR;
-    rr  = (col/(CFACTOR*CFACTOR)) % CFACTOR;
-    r   = (rr*255)/CSCALES;
-    g   = (gg*255)/CSCALES;
-    b   = (bb*255)/CSCALES;
+    a= (col >> 24) & 255;
+    col= col & 0xffffff;
+    if (col <= ((color) GREYS)) {
+      r= (col*255)/GREYS;
+      g= (col*255)/GREYS;
+      b= (col*255)/GREYS;
+    }
+    else {
+      int rr, gg, bb;
+      col-= (GREYS+1);
+      bb  = col % CFACTOR;
+      gg  = (col/CFACTOR) % CFACTOR;
+      rr  = (col/(CFACTOR*CFACTOR)) % CFACTOR;
+      r   = (rr*255)/CSCALES;
+      g   = (gg*255)/CSCALES;
+      b   = (bb*255)/CSCALES;
+    }
   }
 }
 
-color
-named_color (string s) {
-  if ((N(s) == 4) && (s[0]=='#')) {
-    int r= 17 * from_hexadecimal (s (1, 2));
-    int g= 17 * from_hexadecimal (s (2, 3));
-    int b= 17 * from_hexadecimal (s (3, 4));
-    return rgb_color (r, g, b);
-  }
-  if ((N(s) == 7) && (s[0]=='#')) {
-    int r= from_hexadecimal (s (1, 3));
-    int g= from_hexadecimal (s (3, 5));
-    int b= from_hexadecimal (s (5, 7));
-    return rgb_color (r, g, b);
+static color
+named_color_bis (string s) {
+  if (N(s) > 0 && s[0] == '#') {
+    if (N(s) == 4) {
+      int r= 17 * from_hexadecimal (s (1, 2));
+      int g= 17 * from_hexadecimal (s (2, 3));
+      int b= 17 * from_hexadecimal (s (3, 4));
+      return rgb_color (r, g, b);
+    }
+    else if (N(s) == 5) {
+      int r= 17 * from_hexadecimal (s (1, 2));
+      int g= 17 * from_hexadecimal (s (2, 3));
+      int b= 17 * from_hexadecimal (s (3, 4));
+      int a= 17 * from_hexadecimal (s (4, 5));
+      return rgb_color (r, g, b, a);
+    }
+    else if (N(s) == 7) {
+      int r= from_hexadecimal (s (1, 3));
+      int g= from_hexadecimal (s (3, 5));
+      int b= from_hexadecimal (s (5, 7));
+      return rgb_color (r, g, b);
+    }
+    else if (N(s) == 9) {
+      int r= from_hexadecimal (s (1, 3));
+      int g= from_hexadecimal (s (3, 5));
+      int b= from_hexadecimal (s (5, 7));
+      int a= from_hexadecimal (s (7, 9));
+      return rgb_color (r, g, b, a);
+    }
   }
   int pastel= (the_gui->depth>=16? 223: 191);
   if (s == "black")          return black;
@@ -146,6 +203,7 @@ named_color (string s) {
   if (s == "pink")           return pink;
   if (s == "broken white")   return rgb_color (255, 255, pastel);
   if (s == "light grey")     return light_grey;
+  if (s == "darker grey")    return rgb_color (64, 64, 64);
   if (s == "dark grey")      return dark_grey;
   if (s == "dark red")       return rgb_color (128, 0, 0);
   if (s == "dark blue")      return rgb_color (0, 0, 128);
@@ -167,14 +225,40 @@ named_color (string s) {
   return black;
 }
 
+color
+named_color (string s, int a) {
+  color c= named_color_bis (s);
+  a= (a * (c >> 24)) / 255;
+  return (a << 24) + (c & 0xffffff);
+}
+
 string
 get_named_color (color c) {
-  SI r, g, b;
-  get_rgb_color (c, r, g, b);
-  return "#" *
-    as_hexadecimal (r, 2) *
-    as_hexadecimal (g, 2) *
-    as_hexadecimal (b, 2);
+  int r, g, b, a;
+  get_rgb_color (c, r, g, b, a);
+  if (a == 255)
+    return "#" *
+      as_hexadecimal (r, 2) *
+      as_hexadecimal (g, 2) *
+      as_hexadecimal (b, 2);
+  else
+    return "#" *
+      as_hexadecimal (r, 2) *
+      as_hexadecimal (g, 2) *
+      as_hexadecimal (b, 2) *
+      as_hexadecimal (a, 2);
+}
+
+color
+blend (color fg, color bg) {
+  if (((fg >> 24) & 255) == 255) return fg;
+  int fR, fG, fB, fA, bR, bG, bB, bA;
+  get_rgb_color (fg, fR, fG, fB, fA);
+  get_rgb_color (bg, bR, bG, bB, bA);
+  fR= (bR * (255 - fA) + fR * fA) / 255;
+  fG= (bG * (255 - fA) + fG * fA) / 255;
+  fB= (bB * (255 - fA) + fB * fA) / 255;
+  return rgb_color (fR, fG, fB);
 }
 
 void
@@ -409,9 +493,9 @@ x_gui_rep::initialize_keyboard_pointer () {
   Map (XK_Cyrillic_E,   "\xdd");
   Map (XK_Cyrillic_YU,  "\xde");
   Map (XK_Cyrillic_YA,  "\xdf");
-  
+
   //Ukrainian letters in T2A encoding
-  Map (XK_Ukrainian_i,   "i"); // Fall back! 
+  Map (XK_Ukrainian_i,   "i"); // Fall back!
   Map (XK_Ukrainian_I,   "I"); // Fall back!
   Map (XK_Ukrainian_yi,   "\xa8");
   Map (XK_Ukrainian_YI,   "\x88");
@@ -675,6 +759,14 @@ x_gui_rep::initialize_keyboard_pointer () {
   map (XK_F10, "F10");
   map (XK_F11, "F11");
   map (XK_F12, "F12");
+  map (XK_F13, "F13");
+  map (XK_F14, "F14");
+  map (XK_F15, "F15");
+  map (XK_F16, "F16");
+  map (XK_F17, "F17");
+  map (XK_F18, "F18");
+  map (XK_F19, "F19");
+  map (XK_F20, "F20");
   // map (XK_Mode_switch, "modeswitch");
 
   // Keypad keys
@@ -761,19 +853,19 @@ x_gui_rep::get_max_size (SI& width, SI& height) {
 * Main initialization
 ******************************************************************************/
 
-x_gui_rep::x_gui_rep (int argc2, char** argv2):
+x_gui_rep::x_gui_rep (int& argc2, char** argv2):
   color_scale ((void*) NULL),
   character_bitmap (NULL), character_pixmap ((pointer) 0),
-  xpm_bitmap (0), xpm_pixmap (0),
   lower_key (""), upper_key (""),
-  selection (NULL), selection_t ("none"), selection_s ("")
+  selection_t ("none"), selection_s (""), selection_w ((Window) 0)
 {
   the_gui= this;
-  if ((dpy= XOpenDisplay (NULL)) == NULL)
-    fatal_error ("I failed to connect to Xserver", "x_gui_rep::x_gui_rep");
+  ASSERT ((dpy= XOpenDisplay (NULL)) != NULL,
+	  "failure to connect to Xserver");
   // XSynchronize (dpy, true);
 
   XGCValues values;
+  XVisualInfo visual;
 
   scr                = DefaultScreen (dpy);
   root               = RootWindow (dpy, scr);
@@ -791,8 +883,49 @@ x_gui_rep::x_gui_rep (int argc2, char** argv2):
   interrupted        = false;
   interrupt_time     = texmacs_time ();
 
+  XA_CLIPBOARD = XInternAtom (dpy, "CLIPBOARD", false);
+  XA_TARGETS = XInternAtom (dpy, "TARGETS", false);
+
+  if (XMatchVisualInfo (dpy, scr, depth, TrueColor, &visual) != 0) {
+    if (visual.red_mask   == (255 << 16) &&
+	visual.green_mask == (255 << 8) &&
+	visual.blue_mask  == 255)
+      true_color= true;
+  }
+
   XSetGraphicsExposures (dpy, gc, true);
 
+  int start= 0;
+  string xmm= eval_system ("xmodmap");
+  for (int i=0; i<=N(xmm); i++)
+    if (i == N(xmm) || xmm[i] == '\n') {
+      string s= xmm (start, i);
+      if (starts (s, "mod") && N(s)>3 && s[3] >= '1' && s[3] <= '5') {
+	int nr= ((int) (s[3] - '0'));
+	int mask= 4 << nr;
+	if (occurs ("Alt_L", s) || occurs ("Alt_R", s)) {
+	  //cout << "alt_mask= " << mask << "\n";
+	  alt_mask= mask;
+	}
+	else if (alt_mask == 0 && occurs ("Mode_switch", s)) {
+	  //cout << "alt_mask= " << mask << "\n";
+	  alt_mask= mask;
+	}
+	else if (occurs ("Meta_L", s) || occurs ("Meta_R", s)) {
+	  //cout << "meta_mask= " << mask << "\n";
+	  meta_mask= mask;
+	}
+	else if (meta_mask == 0 &&
+		 (occurs ("Super_L", s) || occurs ("Super_R", s))) {
+	  //cout << "meta_mask= " << mask << "\n";
+	  meta_mask= mask;
+	}
+      }
+      start= i+1;
+    }
+  if (alt_mask == 0) alt_mask= 8;
+  if (meta_mask == 0) meta_mask= 16;
+
   //get_xmodmap ();
   x_initialize_colors ();
   initialize_input_method ();
@@ -806,14 +939,13 @@ x_gui_rep::~x_gui_rep () {
   clear_selection ("primary");
   XFreeGC (dpy, gc);
   XCloseDisplay (dpy);
-  delete[] cmap;
+  if (!true_color) tm_delete_array (cmap);
 }
 
 void
-gui_open (int argc2, char** argv2) {
-  if (the_gui != NULL)
-    fatal_error ("gui already open", "gui_open");
-  the_gui= new x_gui_rep (argc2, argv2);
+gui_open (int& argc2, char** argv2) {
+  ASSERT (the_gui == NULL, "gui already open");
+  the_gui= tm_new<x_gui_rep> (argc2, argv2);
 }
 
 void
@@ -823,9 +955,8 @@ gui_start_loop () {
 
 void
 gui_close () {
-  if (the_gui == NULL)
-    fatal_error ("gui not yet open", "gui_close");
-  delete the_gui;
+  ASSERT (the_gui != NULL, "gui not yet open");
+  tm_delete (the_gui);
   the_gui= NULL;
 }
 
