@@ -19,6 +19,12 @@
 
   (texmacs-version-release texmacs_version (string string))
   (version-before? version_inf (bool string string))
+  (updater-supported? updater_supported (bool))
+  (updater-check-background updater_check_background (bool))
+  (updater-check-foreground updater_check_foreground (bool))
+  (updater-last-check updater_last_check (long))
+  (updater-set-interval updater_set_interval (bool int))
+  (updater-set-automatic updater_set_automatic (bool bool))
   (os-win32? os_win32 (bool))
   (os-mingw? os_mingw (bool))
   (os-macos? os_macos (bool))
@@ -42,6 +48,7 @@
   (eval-system eval_system (string string))
   (var-eval-system var_eval_system (string string))
   (get-locale-language get_locale_language (string))
+  (get-locale-charset get_locale_charset (string))
   (locale-to-language locale_to_language (string string))
   (language-to-locale language_to_locale (string string))
   (texmacs-time texmacs_time (int))
@@ -49,6 +56,7 @@
   (bench-print bench_print (void string))
   (bench-print-all bench_print (void))
   (system-wait system_wait (void string string))
+  (set-latex-command set_latex_command (void string))
   (set-bibtex-command set_bibtex_command (void string))
   (math-symbol-group math_symbol_group (string string))
   (math-group-members math_group_members (array_string string))
@@ -83,33 +91,58 @@
   (mark-new new_marker (double))
   (glyph-register register_glyph (void string array_array_array_double))
   (glyph-recognize recognize_glyph (string array_array_array_double))
+  (set-new-fonts set_new_fonts (void bool))
+  (new-fonts? get_new_fonts (bool))
+  (tmtm-eqnumber->nonumber eqnumber_to_nonumber (tree tree))
 
   ;; routines for the font database
+  (tt-exists? tt_font_exists (bool string))
   (tt-dump tt_dump (void url))
   (tt-font-name tt_font_name (scheme_tree url))
+  (tt-analyze tt_analyze (array_string string))
   (font-database-build font_database_build (void url))
   (font-database-build-local font_database_build_local (void))
   (font-database-build-global font_database_build_global (void))
+  (font-database-build-characteristics
+   font_database_build_characteristics (void bool))
   (font-database-insert-global font_database_build_global (void url))
+  (font-database-save-local-delta font_database_save_local_delta (void))
   (font-database-load font_database_load (void))
   (font-database-save font_database_save (void))
   (font-database-filter font_database_filter (void))
   (font-database-families font_database_families (array_string))
+  (font-database-delta-families font_database_delta_families (array_string))
   (font-database-styles font_database_styles (array_string string))
   (font-database-search font_database_search (array_string string string))
+  (font-database-characteristics
+   font_database_characteristics (array_string string string))
   (font-family->master family_to_master (string string))
   (font-master->families master_to_families (array_string string))
   (font-master-features master_features (array_string string))
   (font-family-features family_features (array_string string))
   (font-family-strict-features family_strict_features (array_string string))
-  (font-subfamily-features subfamily_features (array_string string))
+  (font-style-features style_features (array_string string))
+  (font-guessed-features guessed_features (array_string string string))
+  (font-guessed-distance guessed_distance (double string string string string))
+  (font-master-guessed-distance guessed_distance (double string string))
+  (font-family-guessed-features guessed_features (array_string string bool))
+  (characteristic-distance
+   characteristic_distance (double array_string array_string))
+  (trace-distance trace_distance (double string string double))
   (logical-font-public logical_font (array_string string string))
-  (logical-font-private logical_font (array_string string string string string))
+  (logical-font-exact logical_font_exact (array_string string string))
+  (logical-font-private
+   logical_font (array_string string string string string))
   (logical-font-family get_family (string array_string))
   (logical-font-variant get_variant (string array_string))
   (logical-font-series get_series (string array_string))
   (logical-font-shape get_shape (string array_string))
-  (logical-font-search search_font (array_string array_string bool))
+  (logical-font-search search_font (array_string array_string))
+  (logical-font-search-exact search_font_exact (array_string array_string))
+  (search-font-families search_font_families (array_string array_string))
+  (search-font-styles search_font_styles (array_string string array_string))
+  (logical-font-patch patch_font (array_string array_string array_string))
+  (font-family-main main_family (string string))
 
   ;; routines for images
   (image->psdoc image_to_psdoc (string url))
@@ -259,9 +292,13 @@
   (escape-shell escape_sh (string string))
   (escape-to-ascii cork_to_ascii (string string))
   (unescape-guile unescape_guile (string string))
+  (string-quote scm_quote (string string))
+  (string-unquote scm_unquote (string string))
   (string-convert convert (string string string string))
   (utf8->cork utf8_to_cork (string string))
   (cork->utf8 cork_to_utf8 (string string))
+  (utf8->t2a utf8_to_t2a (string string))
+  (t2a->utf8 t2a_to_utf8 (string string))
   (utf8->html utf8_to_html (string string))
   (guess-wencoding guess_wencoding (string string))
   (tm->xml-name tm_to_xml_name (string string))
@@ -308,7 +345,7 @@
   (parse-latex parse_latex (tree string))
   (parse-latex-document parse_latex_document (tree string))
   (latex->texmacs latex_to_tree (tree tree))
-  (latex-document->texmacs latex_document_to_tree (tree string))
+  (cpp-latex-document->texmacs latex_document_to_tree (tree string bool bool))
   (latex-class-document->texmacs latex_class_document_to_tree (tree string))
   (parse-xml parse_xml (scheme_tree string))
   (parse-html parse_html (scheme_tree string))
@@ -321,6 +358,7 @@
   (root->url url_root (url string))
   (string->url url (url string))
   (url->string as_string (string url))
+  (url->stree as_tree (scheme_tree url))
   (system->url url_system (url string))
   (url->system as_system_string (string url))
   (unix->url url_unix (url string))
@@ -384,56 +422,15 @@
   (system-1 system (void string url))
   (system-2 system (void string url url))
 
+  ;; Persistent data
   (persistent-set persistent_set (void url string string))
   (persistent-remove persistent_reset (void url string))
   (persistent-has? persistent_contains (bool url string))
   (persistent-get persistent_get (string url string))
   (persistent-file-name persistent_file_name (url url string))
-  
-  ;; TeXmacs file system
-  (tmfs-set tmfs_set (void string collection))
-  (tmfs-reset tmfs_reset (void string collection))
-  (tmfs-get tmfs_get (collection string))
-  (tmfs-new-save tmfs_save (void string string))
-  (tmfs-new-remove tmfs_remove (void string))
-  (tmfs-new-load tmfs_load (string string))
 
-  (tmfs-create-ressource tmfs_create_ressource (string))
-  (tmfs-ressource-head tmfs_get_head (string string))
-  (tmfs-ressource-versions tmfs_get_versions (collection string))
-  (tmfs-save-ressource tmfs_save_ressource (void string string properties))
-  (tmfs-load-ressource-file tmfs_load_ressource_file (string string))
-  (tmfs-load-ressource-properties tmfs_load_ressource_properties
-				  (properties string))
-
-  (tmfs-create-user tmfs_create_user (string string))
-  (tmfs-search-user tmfs_search_user (collection string))
-  (tmfs-set-user tmfs_set_user (void string))
-  (tmfs-get-user tmfs_get_user (string))
-  (tmfs-allows? tmfs_allows (bool string string))
-  (tmfs-set-attributes tmfs_set_attributes (void string properties))
-  (tmfs-get-attributes tmfs_get_attributes (properties string))
-  (tmfs-add-attributes tmfs_add_attributes (void string properties))
-  (tmfs-remove-attributes tmfs_remove_attributes (void string properties))
-  (tmfs-change-attributes tmfs_change_attributes (void string properties))
-  (tmfs-query tmfs_query (solutions properties))
-  (solutions->collection as_collection (collection solutions string))
-
-  (tmfs-create-file tmfs_create_file (string string string))
-  (tmfs-create-file-in tmfs_create_file (string string string string))
-  (tmfs-search-file tmfs_search_file (collection string))
-  (tmfs-save-file tmfs_save_file (void string string))
-  (tmfs-load-file tmfs_load_file (string string))
-  (tmfs-create-project tmfs_create_project (string string))
-  (tmfs-search-project tmfs_search_project (collection string))
-  (tmfs-get-file-projects tmfs_get_file_projects (collection string))
-  (tmfs-get-project-files tmfs_get_project_files (collection string))
-  (tmfs-create-branch tmfs_create_branch (string string string))
-
-  (tmfs-set-root tmfs_set_root (void string url))
-  (tmfs-get-root tmfs_get_root (url string))
-  (tmfs-import tmfs_import (void url))
-  (tmfs-export tmfs_export (void url))
+  ;; SQL interface
+  (sql-exec sql_exec (scheme_tree url string))
 
   ;; TeXmacs servers and clients
   (server-start server_start (void))
@@ -499,13 +496,14 @@
   (widget-texmacs-input texmacs_input_widget (widget content content 
                                                      command bool))
   (widget-ink ink_widget (widget command))
-  (widget-refresh refresh_widget (widget string))
+  (widget-refresh refresh_widget (widget string string))
   (object->promise-widget as_promise_widget (promise_widget object))
   (tree-bounding-rectangle get_bounding_rectangle (array_int tree))
   (show-balloon show_help_balloon (void widget int int))
   (get-style-menu get_style_menu (object))
   (get-add-package-menu get_add_package_menu (object))
   (get-remove-package-menu get_remove_package_menu (object))
+  (refresh-now windows_refresh (void string))
 
   ;; buffers
   (buffer-list get_all_buffers (array_url))
@@ -534,6 +532,7 @@
   (buffer-load buffer_load (bool url))
   (buffer-export buffer_export (bool url url string))
   (buffer-save buffer_save (bool url))
+  (tree-import-loaded import_loaded_tree (tree string url string))
   (tree-import import_tree (tree url string))
   (tree-export export_tree (bool tree url string))
 
