@@ -13,6 +13,7 @@
 #include "vars.hpp"
 #include "analyze.hpp"
 #include "font.hpp"
+#include "gui.hpp"
 
 /******************************************************************************
 * Length arithmetic
@@ -21,7 +22,7 @@
 bool
 is_length (string s) {
   int i;
-  for (i=0; (i<N(s)) && ((s[i]<'a') || (s[i]>'z')); i++);
+  for (i=0; (i<N(s)) && ((s[i]<'a') || (s[i]>'z')); i++) {}
   return is_double (s (0, i)) && is_locase_alpha (s (i, N(s)));
 }
 
@@ -87,7 +88,7 @@ as_tmlen (tree t) {
     string s= t->label;
     int start= 0, i, n=N(s);
     while ((start+1<n) && (s[start]=='-') && (s[start+1]=='-')) start += 2;
-    for (i=start; (i<n) && ((s[i]<'a') || (s[i]>'z')); i++);
+    for (i=start; (i<n) && ((s[i]<'a') || (s[i]>'z')); i++) {}
     string s1= s (start, i);
     string s2= s (i, n);
     if (!(is_double (s1) && is_locase_alpha (s2))) return tree (TMLEN, "0");
@@ -119,8 +120,8 @@ as_length (tree t) {
 
 inline SI std_inch () {
   return (SI) ((double) as_int (std_env [DPI]) * PIXEL); }
-inline int std_sfactor () {
-  return as_int (std_env [SFACTOR]); }
+inline double std_zoom () {
+  return as_double (std_env [ZOOM_FACTOR]); }
 inline int std_dpi () {
   return as_int (std_env [DPI]); }
 inline double std_magnification () {
@@ -145,31 +146,36 @@ std_fn () {
   switch (std_mode ()) {
   case 0:
   case 1:
-    return find_font (current_display (),
-		      as_string (std_env [FONT]),
-		      as_string (std_env [FONT_FAMILY]),
-		      as_string (std_env [FONT_SERIES]),
-		      as_string (std_env [FONT_SHAPE]),
-		      script (fs, std_math_level ()),
-		      (int) (std_magnification () * std_dpi ()));
+    return smart_font (as_string (std_env [FONT]),
+                       as_string (std_env [FONT_FAMILY]),
+                       as_string (std_env [FONT_SERIES]),
+                       as_string (std_env [FONT_SHAPE]),
+                       script (fs, std_math_level ()),
+                       (int) (std_magnification () * std_dpi ()));
   case 2:
-    return find_font (current_display (),
-		      as_string (std_env [MATH_FONT]),
-		      as_string (std_env [MATH_FONT_FAMILY]),
-		      as_string (std_env [MATH_FONT_SERIES]),
-		      as_string (std_env [MATH_FONT_SHAPE]),
-		      script (fs, std_math_level ()),
-		      (int) (std_magnification () * std_dpi ()));
+    return smart_font (as_string (std_env [MATH_FONT]),
+                       as_string (std_env [MATH_FONT_FAMILY]),
+                       as_string (std_env [MATH_FONT_SERIES]),
+                       as_string (std_env [MATH_FONT_SHAPE]),
+                       as_string (std_env [FONT]),
+                       as_string (std_env [FONT_FAMILY]),
+                       as_string (std_env [FONT_SERIES]),
+                       "mathitalic",
+                       script (fs, std_math_level ()),
+                       (int) (std_magnification () * std_dpi ()));
   case 3:
-    return find_font (current_display (),
-		      as_string (std_env [PROG_FONT]),
-		      as_string (std_env [PROG_FONT_FAMILY]),
-		      as_string (std_env [PROG_FONT_SERIES]),
-		      as_string (std_env [PROG_FONT_SHAPE]),
-		      script (fs, std_math_level ()),
-		      (int) (std_magnification () * std_dpi ()));
+    return smart_font (as_string (std_env [PROG_FONT]),
+                       as_string (std_env [PROG_FONT_FAMILY]),
+                       as_string (std_env [PROG_FONT_SERIES]),
+                       as_string (std_env [PROG_FONT_SHAPE]),
+                       as_string (std_env [FONT]),
+                       as_string (std_env [FONT_FAMILY]) * "-tt",
+                       as_string (std_env [FONT_SERIES]),
+                       as_string (std_env [FONT_SHAPE]),
+                       script (fs, std_math_level ()),
+                       (int) (std_magnification () * std_dpi ()));
   default:
-    return current_display ()->default_font ();
+    return get_default_font ();
   }
 }
 
@@ -261,6 +267,16 @@ evaluate_bls_length () {
 		     tree (as_vspace (std_env [PAR_SEP])));
 }
 
+tree
+evaluate_fnbot_length () {
+  return tree (TMLEN, as_string (std_fn()->y1));
+}
+
+tree
+evaluate_fntop_length () {
+  return tree (TMLEN, as_string (std_fn()->y2));
+}
+
 tree
 evaluate_spc_length () {
   space spc= std_fn()->spc;
@@ -303,7 +319,8 @@ evaluate_pag_length () {
 tree evaluate_tmpt_length () {
   return tree (TMLEN, "1"); }
 tree evaluate_px_length () {
-  return tree (TMLEN, as_string (std_sfactor () * PIXEL)); }
+  int px= (int) tm_round ((std_shrinkf * PIXEL) / std_zoom ());
+  return tree (TMLEN, as_string (px)); }
 
 tree
 evaluate_gw_length () {
@@ -317,7 +334,16 @@ evaluate_gh_length () {
   return tree (TMLEN, as_string (6 * std_inch() / 2.54)); // 6cm
 }
 
+tree
+evaluate_gu_length () {
+  // FIXME: not yet implemented
+  return evaluate_cm_length ();
+  //point p0= fr (point (0.0, 0.0));
+  //point p1= fr (point (1.0, 0.0));
+  //return tree (TMLEN, as_string (norm (p1 - p0)));
+}
+
 tree evaluate_msec_length () { return tree (TMLEN, "1"); }
 tree evaluate_sec_length () { return tree (TMLEN, "1000"); }
 tree evaluate_min_length () { return tree (TMLEN, "60000"); }
-tree evaluate_h_length () { return tree (TMLEN, "3600000"); }
+tree evaluate_hr_length () { return tree (TMLEN, "3600000"); }
