@@ -17,7 +17,7 @@
 ******************************************************************************/
 
 #include "connect.hpp"
-#include "pipe_link.hpp"
+#include "socket_notifier.hpp"
 #include "iterator.hpp"
 #include "convert.hpp"
 #include "scheme.hpp"
@@ -38,6 +38,7 @@ struct connection_rep: rep<connection> {
   tm_link ln;            // the underlying link
   int     status;        // status of the connection
   int     prev_status;   // last notified status
+  bool    forced_eval;   // forced input evaluation without call backs
   texmacs_input tm_in;   // texmacs input handler for data from child
   texmacs_input tm_err;  // texmacs input handler for errors from child
 
@@ -48,6 +49,7 @@ public:
   void   read (int channel);
   void   stop ();
   void   interrupt ();
+  void   listen ();
 };
 RESOURCE_CODE(connection);
 
@@ -55,17 +57,27 @@ RESOURCE_CODE(connection);
 * Routines for connections
 ******************************************************************************/
 
+static void 
+connection_callback (void *obj, void* info) {
+  (void) info;
+  //cout << "connection callback " << obj << LF;
+  connection_rep *con = (connection_rep*) obj;
+  con->listen ();
+}
+
+
 connection_rep::connection_rep (string name2, string session2, tm_link ln2):
   rep<connection> (name2 * "-" * session2),
   name (name2), session (session2), ln (ln2),
   status (CONNECTION_DEAD), prev_status (CONNECTION_DEAD),
+  forced_eval (false),
   tm_in ("output"), tm_err ("error") {}
 
 string
 connection_rep::start (bool again) {
   string message;
   if (ln->alive) {
-    message= "Continuation of#" * name * "#session";
+    message= "Continuation of '" * name * "' session";
     status = WAITING_FOR_INPUT;
   }
   else {
@@ -80,6 +92,7 @@ connection_rep::start (bool again) {
   }
   tm_in ->bof ();
   tm_err->bof ();
+  ln->set_command (command (connection_callback, this));
   return message;
 }
 
@@ -162,30 +175,26 @@ connection_notify_status (connection con) {
 }
 
 void
-listen_to_connections () {
-  iterator<string> it= iterate (connection::instances);
-  while (it->busy()) {
-    string name= it->next ();
-    connection con (name);
-    connection_notify_status (con);
-    if (con->status != CONNECTION_DEAD) {
-      con->read (LINK_ERR);
-      connection_notify (con, "error", con->tm_err->get ("error"));
-      con->read (LINK_OUT);
-      connection_notify (con, "output", con->tm_in->get ("output"));
-      connection_notify (con, "prompt", con->tm_in->get ("prompt"));
-      connection_notify (con, "input", con->tm_in->get ("input"));
-      tree t= connection_handlers (con->name);
-      int i, n= N(t);
-      for (i=0; i<n; i++) {
-	tree doc= con->tm_in->get (t[i][0]->label);
-	if (doc != "") call (t[i][1]->label, doc);
-	doc= con->tm_err->get (t[i][0]->label);
-	if (doc != "") call (t[i][1]->label, doc);
-      }
+connection_rep::listen () {
+  if (forced_eval) return;
+  connection_notify_status (this);
+  if (status != CONNECTION_DEAD) {
+    read (LINK_ERR);
+    connection_notify (this, "error", tm_err->get ("error"));
+    read (LINK_OUT);
+    connection_notify (this, "output", tm_in->get ("output"));
+    connection_notify (this, "prompt", tm_in->get ("prompt"));
+    connection_notify (this, "input", tm_in->get ("input"));
+    tree t= connection_handlers (name);
+    int i, n= N(t);
+    for (i=0; i<n; i++) {
+      tree doc= tm_in->get (t[i][0]->label);
+      if (doc != "") call (t[i][1]->label, doc);
+      doc= tm_err->get (t[i][0]->label);
+      if (doc != "") call (t[i][1]->label, doc);
     }
-    connection_notify_status (con);
   }
+  connection_notify_status (this);  
 }
 
 /******************************************************************************
@@ -279,6 +288,7 @@ connection_interrupt (string name, string session) {
   connection con= connection (name * "-" * session);
   if (is_nil (con)) return;
   con->interrupt ();
+  con->listen ();
 }
 
 void
@@ -287,6 +297,7 @@ connection_stop (string name, string session) {
   connection con= connection (name * "-" * session);
   if (is_nil (con)) return;
   con->stop ();
+  con->listen ();
 }
 
 int
@@ -321,7 +332,9 @@ connection_retrieve (string name, string session) {
   if (is_nil (con)) return "";
   tree doc (DOCUMENT);
   while (true) {
-    listen_to_pipes ();
+    con->forced_eval= true;
+    perform_select ();
+    con->forced_eval= false;
     tree next= connection_read (name, session);
     if (next == "");
     else if (is_document (next)) doc << A (next);
