@@ -13,12 +13,15 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <locale.h> // for setlocale
+#include <signal.h>
 
 #include "boot.hpp"
 #include "file.hpp"
 #include "server.hpp"
 #include "timer.hpp"
 #include "data_cache.hpp"
+#include "tm_window.hpp"
 #ifdef AQUATEXMACS
 void mac_fix_paths ();
 #endif
@@ -26,6 +29,7 @@ void mac_fix_paths ();
 #ifdef QTTEXMACS
 #include "Qt/qt_utilities.hpp"
 #include <QApplication>
+#include <QDir>
 #endif
 
 #ifdef MACOSX_EXTENSIONS
@@ -49,6 +53,8 @@ extern int geometry_x, geometry_y;
 extern tree the_et;
 extern bool texmacs_started;
 
+bool disable_error_recovery= false;
+
 tm_ostream& cout= tm_ostream::cout;
 tm_ostream& cerr= tm_ostream::cerr;
 
@@ -65,6 +71,15 @@ test_routines () {
 }
 #endif
 
+/******************************************************************************
+* Clean exit on segmentation faults
+******************************************************************************/
+
+void 
+clean_exit_on_segfault (int sig_num) {
+  FAILED ("segmentation fault");
+}
+
 /******************************************************************************
 * Texmacs paths
 ******************************************************************************/
@@ -82,9 +97,17 @@ TeXmacs_init_paths (int& argc, char** argv) {
 #endif
 
 #if (defined(QTTEXMACS) && defined(Q_WS_MAC)) 
-  // inibith external plugin loading
-  QCoreApplication::setLibraryPaths(QStringList());
-  {
+  // the following line can inibith external plugin loading
+  // QCoreApplication::setLibraryPaths(QStringList());
+  // ideally we would like to control the external plugins
+  // and add the most useful (gif, jpeg, svg converters)
+  // to the bundle package. I still do not have a reliable solution
+  // so just allow everything that is reachable.
+	
+  // plugins need to be installed in TeXmacs.app/Contents/Plugins	
+	QCoreApplication::addLibraryPath( QDir::cleanPath(QCoreApplication::applicationDirPath().append("/../Plugins")) );
+  // cout << from_qstring ( QCoreApplication::libraryPaths () .join("\n") ) << LF;
+	{
     // ensure that private versions of the Qt frameworks have priority on
     // other instances.
     // in the event that we load qt plugins which could possibly link to
@@ -108,12 +131,12 @@ TeXmacs_init_paths (int& argc, char** argv) {
   //cout << get_env("PATH") * ":" * as_string(url("$PWD") * argv[0]
   // * "../../Resources/share/TeXmacs/bin") << LF;
   if (exists("/bin/bash")) {
-    string shell_env =  eval_system("PATH='' /bin/bash -l -c 'echo $PATH'");
+    string shell_env = var_eval_system ("PATH='' /bin/bash -l -c 'echo $PATH'");
     set_env ("PATH", get_env("PATH") * ":" * shell_env * ":" *
-           as_string (exedir * "../Resources/share/TeXmacs/bin"));
+             as_string (exedir * "../Resources/share/TeXmacs/bin"));
   } else {
     set_env ("PATH", get_env("PATH") * ":" *
-               as_string (exedir * "../Resources/share/TeXmacs/bin"));
+             as_string (exedir * "../Resources/share/TeXmacs/bin"));
   }
   // system("set");
 #endif
@@ -167,6 +190,8 @@ TeXmacs_main (int argc, char** argv) {
       else if (s == "-debug-qt") debug (DEBUG_FLAG_QT, true);
       else if (s == "-debug-keyboard") debug (DEBUG_FLAG_KEYBOARD, true);
       else if (s == "-debug-packrat") debug (DEBUG_FLAG_PACKRAT, true);
+      else if (s == "-debug-flatten") debug (DEBUG_FLAG_FLATTEN, true);
+      else if (s == "-debug-correct") debug (DEBUG_FLAG_CORRECT, true);
       else if (s == "-debug-all") {
 	debug (DEBUG_FLAG_EVENTS, true);
 	debug (DEBUG_FLAG_STD, true);
@@ -175,6 +200,7 @@ TeXmacs_main (int argc, char** argv) {
 	debug (DEBUG_FLAG_BENCH, true);
 	debug (DEBUG_FLAG_QT, true);
       }
+      else if (s == "-disable-error-recovery") disable_error_recovery= true;
       else if ((s == "-fn") || (s == "-font")) {
 	i++;
 	if (i<argc) the_default_font= argv[i];
@@ -238,8 +264,7 @@ TeXmacs_main (int argc, char** argv) {
 	  url in  ("$PWD", argv[i-1]);
 	  url out ("$PWD", argv[ i ]);
 	  my_init_cmds= my_init_cmds * " " *
-	    "(texmacs-load-buffer " * scm_quote (as_string (in)) *
-	    " \"generic\" 0 #f) " *
+	    "(load-buffer " * scm_quote (as_string (in)) * " :strict) " *
 	    "(export-buffer " * scm_quote (as_string (out)) * ")";
 	}
       }
@@ -253,7 +278,7 @@ TeXmacs_main (int argc, char** argv) {
       else if ((s == "-S") || (s == "-setup") ||
 	       (s == "-delete-cache") || (s == "-delete-font-cache") ||
 	       (s == "-delete-style-cache") || (s == "-delete-file-cache") ||
-	       (s == "-delete-doc-cache"));
+	       (s == "-delete-doc-cache") || (s == "-delete-plugin-cache"));
       else if (starts (s, "-psn"));
       else {
 	cout << "\n";
@@ -298,35 +323,53 @@ TeXmacs_main (int argc, char** argv) {
   { // opening scope for server sv
   server sv;
 
+  // HACK:
+  // Qt and Guile want to change the locale. 
+  // We need to force it to C to parse correctly the configuration files
+  // (see as_double() in string.cpp)
+  setlocale(LC_NUMERIC, "C");    
+    
+  string where= "";
   for (i=1; i<argc; i++) {
     if (argv[i] == NULL) break;
     string s= argv[i];
     if ((N(s)>=2) && (s(0,2)=="--")) s= s (1, N(s));
     if ((s[0] != '-') && (s[0] != '+')) {
       if (DEBUG_STD) cout << "TeXmacs] Loading " << s << "...\n";
-      sv->load_buffer (url_system (s), "generic", 1);
+      url u= url_system (s);
+      if (!is_rooted (u)) u= resolve (url_pwd (), "") * u;
+      string b= scm_quote (as_string (u));
+      string cmd= "(load-buffer " * b * " " * where * ")";
+      where= " :new-window";
+      exec_delayed (scheme_cmd (cmd));
     }
-    if ((s == "-b") || (s == "-initialize-buffer") ||
-	(s == "-c") || (s == "-convert") ||
-	(s == "-fn") || (s == "-font") ||
-	(s == "-i") || (s == "-initialize") ||
-	(s == "-g") || (s == "-geometry") ||
-	(s == "-x") || (s == "-execute") ||
-	(s == "-log-file")) i++;
+    if      ((s == "-c") || (s == "-convert")) i+=2;
+    else if ((s == "-b") || (s == "-initialize-buffer") ||
+             (s == "-fn") || (s == "-font") ||
+             (s == "-i") || (s == "-initialize") ||
+             (s == "-g") || (s == "-geometry") ||
+             (s == "-x") || (s == "-execute") ||
+             (s == "-log-file")) i++;
   }
   if (install_status == 1) {
     if (DEBUG_STD) cout << "TeXmacs] Loading welcome message...\n";
-    sv->load_buffer (
-      "$TEXMACS_PATH/doc/about/welcome/first.en.tm", "help", 1);
+    url u= "tmfs://help/plain/tm/doc/about/welcome/first.en.tm";
+    string b= scm_quote (as_string (u));
+    string cmd= "(load-buffer " * b * " " * where * ")";
+    where= " :new-window";
+    exec_delayed (scheme_cmd (cmd));
   }
   else if (install_status == 2) {
     if (DEBUG_STD) cout << "TeXmacs] Loading upgrade message...\n";
-    sv->load_buffer (
-      "$TEXMACS_HOME_PATH/doc/about/changes/changes-recent.en.tm", "help", 1);
+    url u= "tmfs://help/plain/tm/doc/about/changes/changes-recent.en.tm";
+    string b= scm_quote (as_string (u));
+    string cmd= "(load-buffer " * b * " " * where * ")";
+    where= " :new-window";
+    exec_delayed (scheme_cmd (cmd));
   }
-  if (sv->no_bufs ()) {
+  if (number_buffers () == 0) {
     if (DEBUG_STD) cout << "TeXmacs] Creating 'no name' buffer...\n";
-    sv->open_window ();
+    open_window ();
   }
 
   bench_print ();
@@ -336,6 +379,7 @@ TeXmacs_main (int argc, char** argv) {
 
   if (DEBUG_STD) cout << "TeXmacs] Starting event loop...\n";
   texmacs_started= true;
+  if (!disable_error_recovery) signal (SIGSEGV, clean_exit_on_segfault);
   gui_start_loop ();
 
   if (DEBUG_STD) cout << "TeXmacs] Stopping server...\n";
@@ -355,6 +399,32 @@ TeXmacs_main (int argc, char** argv) {
 * Main program
 ******************************************************************************/
 
+#ifdef OS_MACOS
+#include <sys/resource.h>
+#endif
+
+void
+boot_hacks () {
+#ifdef OS_MACOS
+// NOTE: under MACOS, there is a limited number of open file descriptors,
+// by default 256.  Any open file descriptor can actually count several times
+// whenever the files is stored in various chunks on disk.  Hence, the limit
+// is easily exceeded, although this situation cannot easily be debugged.
+// Our current hack is to allow for at least 4096 open file descriptors.
+  rlimit lims;
+  getrlimit (RLIMIT_NOFILE, &lims);
+  lims.rlim_cur= max (lims.rlim_cur, 4096);
+  setrlimit (RLIMIT_NOFILE, &lims);
+  //getrlimit (RLIMIT_NOFILE, &lims);
+  //printf ("cur: %i\n", lims.rlim_cur);
+  //printf ("max: %i\n", lims.rlim_max);
+#endif
+}
+
+/******************************************************************************
+* Main program
+******************************************************************************/
+
 void
 immediate_options (int argc, char** argv) {
   if (get_env ("TEXMACS_HOME_PATH") == "")
@@ -371,14 +441,22 @@ immediate_options (int argc, char** argv) {
       remove (url ("$TEXMACS_HOME_PATH/system/settings.scm"));
       remove (url ("$TEXMACS_HOME_PATH/system/setup.scm"));
       remove (url ("$TEXMACS_HOME_PATH/system/cache") * url_wildcard ("*"));
+      remove (url ("$TEXMACS_HOME_PATH/fonts/font-database.scm"));
+      remove (url ("$TEXMACS_HOME_PATH/fonts/font-features.scm"));
+      remove (url ("$TEXMACS_HOME_PATH/fonts/font-characteristics.scm"));
       remove (url ("$TEXMACS_HOME_PATH/fonts/error") * url_wildcard ("*"));
     }
     else if (s == "-delete-cache")
       remove (url ("$TEXMACS_HOME_PATH/system/cache") * url_wildcard ("*"));
     else if (s == "-delete-style-cache")
       remove (url ("$TEXMACS_HOME_PATH/system/cache") * url_wildcard ("__*"));
-    else if (s == "-delete-font-cache")
+    else if (s == "-delete-font-cache") {
       remove (url ("$TEXMACS_HOME_PATH/system/cache/font_cache.scm"));
+      remove (url ("$TEXMACS_HOME_PATH/fonts/font-database.scm"));
+      remove (url ("$TEXMACS_HOME_PATH/fonts/font-features.scm"));
+      remove (url ("$TEXMACS_HOME_PATH/fonts/font-characteristics.scm"));
+      remove (url ("$TEXMACS_HOME_PATH/fonts/error") * url_wildcard ("*"));
+    }
     else if (s == "-delete-doc-cache") {
       remove (url ("$TEXMACS_HOME_PATH/system/cache/doc_cache"));
       remove (url ("$TEXMACS_HOME_PATH/system/cache/dir_cache.scm"));
@@ -390,6 +468,9 @@ immediate_options (int argc, char** argv) {
       remove (url ("$TEXMACS_HOME_PATH/system/cache/dir_cache.scm"));
       remove (url ("$TEXMACS_HOME_PATH/system/cache/stat_cache.scm"));
     }
+    else if (s == "-delete-plugin-cache") {
+      remove (url ("$TEXMACS_HOME_PATH/system/cache/plugin_cache.scm"));
+    }
     else if (s == "-log-file" && i + 1 < argc) {
       i++;
       char* log_file = argv[i];
@@ -404,6 +485,8 @@ immediate_options (int argc, char** argv) {
 
 int
 main (int argc, char** argv) {
+  boot_hacks ();
+  windows_delayed_refresh (1000000000);
   immediate_options (argc, argv);
   set_env ("LC_NUMERIC", "POSIX");
 #ifdef MACOSX_EXTENSIONS
@@ -434,7 +517,7 @@ main (int argc, char** argv) {
 //#ifdef EXPERIMENTAL
 //  test_environments ();
 //#endif
-  start_guile (argc, argv, TeXmacs_main);
+  start_scheme (argc, argv, TeXmacs_main);
   cout.close ();
   cerr.close ();
   return 0;
