@@ -4,10 +4,9 @@
 * DESCRIPTION: Text widgets for output only
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "boxes.hpp"
@@ -25,95 +24,6 @@
 #endif
 #endif
 
-/******************************************************************************
-* Text widgets
-******************************************************************************/
-
-class box_widget_rep: public simple_widget_rep {
-  box  b;
-  bool transparent;
-  SI   X1, Y1, X2, Y2;
-  int  dw, dh;
-
-public:
-  box_widget_rep (box b, bool trans, int dw, int dh);
-  operator tree ();
-
-  void handle_get_size_hint (SI& w, SI& h);
-  void handle_repaint (SI x1, SI y1, SI x2, SI y2);
-};
-
-box_widget_rep::box_widget_rep (
-  box b2, bool trans2, int dw2, int dh2):
-    simple_widget_rep (), b (b2),
-    transparent (trans2), dw (dw2+2*PIXEL), dh (dh2+2*PIXEL) {}
-
-box_widget_rep::operator tree () {
-  return tree (TUPLE, "box", (tree) b);
-}
-
-#define SHRINK 6
-
-void
-box_widget_rep::handle_get_size_hint (SI& w, SI& h) {
-  X1= b->x1; Y1= b->y1;
-  X2= b->x2; Y2= b->y2;
-  w = ((X2- X1+ SHRINK- 1)/SHRINK)+ 2*dw;
-  h = ((Y2- Y1+ SHRINK- 1)/SHRINK)+ 2*dh;
-  abs_round (w, h);
-}
-
-void
-box_widget_rep::handle_repaint (SI x1, SI y1, SI x2, SI y2) {
-  (void) x1; (void) y1; (void) x2; (void) y2;
-  renderer win= get_renderer (this);
-#if defined (QTTEXMACS) || defined(AQUATEXMACS)
-  SI w,h;
-  handle_get_size_hint (w,h);
-#endif
-  if (!transparent) {
-    win->set_background (light_grey);
-    win->set_color (light_grey);
-    win->fill (0, -h, w, 0);
-  }
-  win->set_shrinking_factor (SHRINK);
-  rectangles l (rectangle (0, 0, w, h));
-  SI x= ((SHRINK*w-b->w())>>1) - b->x1;
-  SI y= ((SHRINK*h-b->h())>>1) - b->y1 - SHRINK*h;
-  b->redraw (win, path(), l, x, y);
-  win->set_shrinking_factor (1);
-}
-
-/******************************************************************************
-* Interface
-******************************************************************************/
-
-widget
-box_widget (box b, bool tr) {
-  return widget (new box_widget_rep (b, tr, 3*PIXEL, 3*PIXEL));
-}
-
-widget
-box_widget (scheme_tree p, string s, color col, bool trans, bool ink) {
-  string family  = "roman";
-  string fn_class= "mr";
-  string series  = "medium";
-  string shape   = "normal";
-  int    sz      = 10;
-  int    dpi     = 600;
-  int    n       = arity (p);
-  if ((n >= 1) && is_atomic (p[0])) family  = as_string (p[0]);
-  if ((n >= 2) && is_atomic (p[1])) fn_class= as_string (p[1]);
-  if ((n >= 3) && is_atomic (p[2])) series  = as_string (p[2]);
-  if ((n >= 4) && is_atomic (p[3])) shape   = as_string (p[3]);
-  if ((n >= 5) && is_atomic (p[4])) sz      = as_int (p[4]);
-  if ((n >= 6) && is_atomic (p[5])) dpi     = as_int (p[5]);
-  font fn= find_font (family, fn_class, series, shape, sz, dpi);
-  box  b = text_box (decorate (), 0, s, fn, col);
-  if (ink) b= resize_box (decorate (), b, b->x3, b->y3, b->x4, b->y4, true);
-  return box_widget (b, trans);
-}
-
 /******************************************************************************
 * Getting extents of a typesetted tree
 * Application: get window size for widget tree
@@ -125,6 +35,7 @@ box_widget (scheme_tree p, string s, color col, bool trans, bool ink) {
 #include "convert.hpp"
 #include "formatter.hpp"
 #include "Format/format.hpp"
+#include "new_style.hpp"
 
 void use_modules (tree t);
 
@@ -136,19 +47,19 @@ get_init_environment (tree doc, drd_info& drd) {
   bool ok;
   tree t, style= extract (doc, "style");
   hashmap<string,tree> H;
-  get_server () -> style_get_cache (style, H, t, ok);
+  style_get_cache (style, H, t, ok);
   if (ok) {
     env->patch_env (H);
     ok= drd->set_locals (t);
+    drd->set_environment (H);
   }
   if (!ok) {
-    if (!is_tuple (style))
-      fatal_error ("tuple expected as style",
-		   "edit_interface_rep::typeset_style_using_cache");
-    tree t (USE_PACKAGE, A (style));
-    env->exec (t);
-    env->read_env (H);
-    drd->heuristic_init (H);
+    if (!is_tuple (style)) FAILED ("tuple expected as style");
+    H= get_style_env (style);
+    drd= get_style_drd (style);
+    style_set_cache (style, H, drd->get_locals ());
+    env->patch_env (H);
+    drd->set_environment (H);
   }
   use_modules (env->read (THE_MODULES));
   // FIXME: extract (doc, "init")
@@ -174,3 +85,114 @@ tree_extents (tree doc) {
   h += env->get_length (PAGE_SCREEN_BOT);
   return tuple (as_tree ((w / (5*PIXEL)) + 1), as_tree ((h / (5*PIXEL)) + 1));
 }
+
+/******************************************************************************
+* Typesetted boxes as widgets
+******************************************************************************/
+
+class box_widget_rep: public simple_widget_rep {
+  box    b;
+  color  bg;
+  bool   transparent;
+  double zoomf;
+  double magf;
+  int    dw, dh;
+
+public:
+  box_widget_rep (box b, color bg, bool trans, double zoom, int dw, int dh);
+  operator tree ();
+
+  void handle_get_size_hint (SI& w, SI& h);
+  void handle_repaint (renderer ren, SI x1, SI y1, SI x2, SI y2);
+};
+
+box_widget_rep::box_widget_rep
+  (box b2, color bg2, bool trans2, double zoom, int dw2, int dh2):
+    simple_widget_rep (), b (b2),
+    bg (bg2), transparent (trans2),
+    zoomf (zoom), magf (zoom / std_shrinkf),
+    dw (dw2+2*PIXEL), dh (dh2+2*PIXEL) {}
+
+box_widget_rep::operator tree () {
+  return tree (TUPLE, "box", (tree) b);
+}
+
+void
+box_widget_rep::handle_get_size_hint (SI& w, SI& h) {
+  SI X1= b->x1, Y1= b->y1;
+  SI X2= b->x2, Y2= b->y2;
+  w = ((SI) ceil ((X2- X1) * magf)) + 2*dw;
+  h = ((SI) ceil ((Y2- Y1) * magf)) + 2*dh;
+  abs_round (w, h);
+}
+
+void
+box_widget_rep::handle_repaint (renderer ren, SI x1, SI y1, SI x2, SI y2) {
+  SI w, h;
+  handle_get_size_hint (w, h);
+  if (!transparent) {
+    ren->set_background (bg);
+    ren->set_pencil (bg);
+    ren->fill (x1, y1, x2, y2);
+  }
+  ren->set_zoom_factor (zoomf);
+  rectangles l (rectangle (0, 0, w, h));
+  SI x= ((((SI) (w / magf)) - b->w()) >> 1) - b->x1;
+  SI y= ((((SI) (h / magf)) - b->h()) >> 1) - b->y1 - ((SI) (h / magf));
+  b->redraw (ren, path(), l, x, y);
+  ren->reset_zoom_factor ();
+}
+
+/******************************************************************************
+* Interface
+******************************************************************************/
+
+widget
+box_widget (box b, bool tr) {
+  color col= light_grey;
+  return widget (tm_new<box_widget_rep> (b, col, tr, 5/6.0, 3*PIXEL, 3*PIXEL));
+}
+
+widget
+box_widget (scheme_tree p, string s, color col, bool trans, bool ink) {
+  string family  = "roman";
+  string fn_class= "mr";
+  string series  = "medium";
+  string shape   = "normal";
+  int    sz      = 10;
+  int    dpi     = 600;
+  int    n       = arity (p);
+  if ((n >= 1) && is_atomic (p[0])) family  = as_string (p[0]);
+  if ((n >= 2) && is_atomic (p[1])) fn_class= as_string (p[1]);
+  if ((n >= 3) && is_atomic (p[2])) series  = as_string (p[2]);
+  if ((n >= 4) && is_atomic (p[3])) shape   = as_string (p[3]);
+  if ((n >= 5) && is_atomic (p[4])) sz      = as_int (p[4]);
+  if ((n >= 6) && is_atomic (p[5])) dpi     = as_int (p[5]);
+  font fn= find_font (family, fn_class, series, shape, sz, dpi);
+  box  b = text_box (decorate (), 0, s, fn, col);
+  if (ink) b= resize_box (decorate (), b, b->x3, b->y3, b->x4, b->y4, true);
+  return box_widget (b, trans);
+}
+
+tree enrich_embedded_document (tree body, tree style);
+
+widget
+texmacs_output_widget (tree doc, tree style) {
+  doc= enrich_embedded_document (doc, style);
+  drd_info drd ("none", std_drd);
+  edit_env env= get_init_environment (doc, drd);
+  tree t= extract (doc, "body");
+  lazy lz= make_lazy (env, t, path ());
+  format vf= make_query_vstream_width (array<line_item>(), array<line_item>());
+  format rf= lz->query (LAZY_BOX, vf);
+  SI w= ((format_vstream) rf)->width;
+  box b= (box) lz->produce (LAZY_BOX, make_format_width (w));
+  //cout << (b->w()>>8) << ", " << (b->h()>>8) << "\n";
+  //SI dw1= env->get_length (PAGE_SCREEN_LEFT);
+  //SI dw2= env->get_length (PAGE_SCREEN_RIGHT);
+  //SI dh1= env->get_length (PAGE_SCREEN_BOT);
+  //SI dh2= env->get_length (PAGE_SCREEN_TOP);
+  color col= light_grey;
+  
+  return widget (tm_new<box_widget_rep> (b, col, false, 1.0, 0, 0));
+}
