@@ -11,8 +11,7 @@
 
 #include "boxes.hpp"
 #include "formatter.hpp"
-#include "Graphics/point.hpp"
-#include "timer.hpp"
+#include "point.hpp"
 #include "printer.hpp"
 #include "file.hpp"
 #include "merge_sort.hpp"
@@ -351,6 +350,7 @@ box_rep::graphical_select (SI x, SI y, SI dist) {
   gr_selections res;
   if (graphical_distance (x, y) <= dist) {
     gr_selection gs;
+    gs->type= "box";
     gs->dist= graphical_distance (x, y);
     gs->cp << find_tree_path (x, y, dist);
     // FIXME: check whether this is correct: I do not remember whether
@@ -366,6 +366,7 @@ box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
   gr_selections res;
   if (in_rectangle (x1, y1, x2, y2)) {
     gr_selection gs;
+    gs->type= "box";
     gs->dist= graphical_distance (x1, y1);
     SI dist= (SI)norm (point (x2-x1, y2-y1));
     gs->cp << find_tree_path (x1, y1, dist);
@@ -420,11 +421,11 @@ box_rep::get_leaf_font () {
   return font ();
 }
 
-color
-box_rep::get_leaf_color () {
+pencil
+box_rep::get_leaf_pencil () {
   cerr << "\nTeXmacs] the box is " << box (this) << "\n";
   FAILED ("this box is not textual");
-  return 0;
+  return pencil (false);
 }
 
 language
@@ -486,33 +487,33 @@ box_rep::reindex (int i, int item, int n) {
 
 void
 box_rep::redraw (renderer ren, path p, rectangles& l) {
-  if (((nr_painted&15) == 15) && ren->interrupted (true)) return;
+  if (((nr_painted&15) == 15) && gui_interrupted (true)) return;
   ren->move_origin (x0, y0);
   SI delta= ren->pixel; // adjust visibility to compensate truncation
   if (ren->is_visible (x3- delta, y3- delta, x4+ delta, y4+ delta)) {
     rectangles ll;
     l= rectangles();
     pre_display (ren);
-
+    
     int i, item=-1, n=subnr (), i1= n, i2= -1;
     if (!is_nil(p)) i1= i2= item= p->item;
     for (i=0; i<n; i++) {
       int k= reindex (i, item, n-1);
       if (is_nil(p)) subbox (k)->redraw (ren, path (), ll);
       else if (i!=0) {
-	if (k > item) subbox(k)->redraw (ren, path (0), ll);
-	else subbox(k)->redraw (ren, path (subbox(k)->subnr()-1), ll);
+        if (k > item) subbox(k)->redraw (ren, path (0), ll);
+        else subbox(k)->redraw (ren, path (subbox(k)->subnr()-1), ll);
       }
       else subbox(k)->redraw (ren, p->next, ll);
       if (!is_nil(ll)) {
-	i1= min (i1, k);
-	i2= max (i2, k);
-	l = ll * l;
-	ll= rectangles ();
+        i1= min (i1, k);
+        i2= max (i2, k);
+        l = ll * l;
+        ll= rectangles ();
       }
     }
-
-    if (((nr_painted&15) == 15) && ren->interrupted ()) {
+    
+    if (((nr_painted&15) == 15) && gui_interrupted ()) {
       l= translate (l, -ren->ox, -ren->oy);
       clear_incomplete (l, ren->pixel, item, i1, i2);
       l= translate (l, ren->ox, ren->oy);
@@ -583,8 +584,8 @@ operator != (cursor cu1, cursor cu2) {
   return ! (cu1 == cu2);
 }
 
-ostream&
-operator << (ostream& out, cursor cu) {
+tm_ostream&
+operator << (tm_ostream& out, cursor cu) {
   out << "cursor (" << (cu->ox>>8) << ", " << (cu->oy>>8) << ": "
       << cu->delta << ": "
       << (cu->y1>>8) << ", " << (cu->y2>>8) << ": "
@@ -617,8 +618,8 @@ operator != (selection sel1, selection sel2) {
   return !(sel1 == sel2);
 }
 
-ostream&
-operator << (ostream& out, selection sel) {
+tm_ostream&
+operator << (tm_ostream& out, selection sel) {
   return out << "selection (" << sel->start << ", " << sel->end << ")";
 }
 
@@ -633,9 +634,10 @@ gr_selection::gr_selection (array<path> cp, SI dist):
   rep->dist= dist;
 }
 
-ostream&
-operator << (ostream& out, gr_selection sel) {
-  return out << "gr_selection (" << sel->dist << ", " << sel->cp << ")";
+tm_ostream&
+operator << (tm_ostream& out, gr_selection sel) {
+  return out << "gr_selection (" << sel->type << ", "
+	     << sel->dist << ", " << sel->cp << ")";
 }
 
 struct less_eq_gr_selection {
@@ -643,8 +645,14 @@ struct less_eq_gr_selection {
     return a->dist <= b->dist; }
 };
 
-tree as_tree (gr_selections sels) {
+void
+sort (gr_selections& sels) {
   merge_sort_leq <gr_selection, less_eq_gr_selection> (sels);
+}
+
+tree
+as_tree (gr_selections sels) {
+  sort (sels);
   int i, n= N(sels);
   array<array<path> > res (n);
   for (i=0; i<n; i++)
@@ -652,6 +660,7 @@ tree as_tree (gr_selections sels) {
   return (tree) res;
 }
 
+
 /******************************************************************************
 * Animations
 ******************************************************************************/
@@ -778,7 +787,7 @@ bool box::operator == (box b2) { return rep==b2.rep; }
 bool box::operator != (box b2) { return rep!=b2.rep; }
 
 box::operator tree () { return tree (*rep); }
-ostream& operator << (ostream& out, box b) { return out << ((tree) b); }
+tm_ostream& operator << (tm_ostream& out, box b) { return out << ((tree) b); }
 
 path
 descend_decode (path ip, int side) {
@@ -824,8 +833,8 @@ make_eps (url name, box b, int dpi) {
   b->x0= -b->x3;
   b->y0= -b->y4;
   renderer ren= printer (name, dpi, 1, "user", false, w/cm, h/cm);
-  ren->set_color (black);
   ren->set_background (white);
+  ren->set_pencil (black);
   rectangles rs;
   b->redraw (ren, path (0), rs);
   tm_delete (ren);
