@@ -4,16 +4,16 @@
 * DESCRIPTION: Boxes for graphics
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "env.hpp"
 #include "Boxes/graphics.hpp"
 #include "Boxes/composite.hpp"
-#include "Graphics/math_util.hpp"
+#include "Boxes/construct.hpp"
+#include "math_util.hpp"
 
 /******************************************************************************
 * Graphics boxes
@@ -34,6 +34,7 @@ struct graphics_box_rep: public composite_box_rep {
   void post_display (renderer &ren);
   int reindex (int i, int item, int n);
   virtual int find_child (SI x, SI y, SI delta, bool force);
+  gr_selections graphical_select (SI x, SI y, SI dist);
   gr_selections graphical_select (SI x1, SI y1, SI x2, SI y2);
 };
 
@@ -83,6 +84,7 @@ graphics_box_rep::post_display (renderer &ren) {
 
 int
 graphics_box_rep::reindex (int i, int item, int n) {
+  (void) item; (void) n;
   return i;
 }
 
@@ -94,31 +96,40 @@ graphics_box_rep::find_child (SI x, SI y, SI delta, bool force) {
   for (i=0; i<n; i++)
     if (distance (i, x, y, delta)==0) {
       tree ty= (tree)bs[i];
-      if ((bs[i]->accessible () || force)
-       && is_tuple (ty) && ty[0]=="textat")
+      if ((bs[i]->accessible () || force) && is_tuple (ty) && ty[0]=="text-at")
         return i;
     }
   return m;
 }
 
-gr_selections
-graphics_box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
-  gr_selections res;
-  int i, n= subnr();
-  for (i=0; i<n; i++)
-    res << bs[i]->graphical_select (x1- sx(i), y1- sy(i),
-				    x2- sx(i), y2- sy(i));
-  return res;
-}
 /*NOTE: It seems that the dimensions of the boxes that inherit from
   composite_box are not calculated correctly (namely : one can find
   points inside the box that are outside the rectangle (x1, y1, x2, y2)
   that defines the border of the box). As a consequence, we use a traversal
   routine that doesn't tests contains_rectangle(). When this problem
   will have been corrected, the method of composite_box should work,
-  and consequently, its more specific implementation above should be
+  and consequently, its more specific implementation below should be
   removed (this is the same in concat_boxes and stack_boxes). */
 
+gr_selections
+graphics_box_rep::graphical_select (SI x, SI y, SI dist) {
+  gr_selections res;
+  int i, n= subnr();
+  for (i=n-1; i>=0; i--)
+    res << bs[i]->graphical_select (x- sx(i), y- sy(i), dist);
+  return res;
+}
+
+gr_selections
+graphics_box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
+  gr_selections res;
+  int i, n= subnr();
+  for (i=n-1; i>=0; i--)
+    res << bs[i]->graphical_select (x1- sx(i), y1- sy(i),
+				    x2- sx(i), y2- sy(i));
+  return res;
+}
+
 /******************************************************************************
 * Group boxes
 ******************************************************************************/
@@ -140,6 +151,7 @@ graphics_group_box_rep::graphical_select (SI x, SI y, SI dist) {
   gr_selections res;
   if (graphical_distance (x, y) <= dist) {
     gr_selection gs;
+    gs->type= "group";
     gs->dist= graphical_distance (x, y);
   //gs->p= point (x, y); // The cursor moves freely inside the box
     gs->cp << reverse (path (0, ip));
@@ -155,6 +167,7 @@ graphics_group_box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
   gr_selections res;
   if (in_rectangle (x1, y1, x2, y2)) {
     gr_selection gs;
+    gs->type= "group";
     gs->dist= graphical_distance (x1, y1);
     gs->cp << reverse (path (0, ip));
     gs->pts= array<point> (0);
@@ -166,6 +179,7 @@ graphics_group_box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
 
 int
 graphics_group_box_rep::reindex (int i, int item, int n) {
+  (void) item; (void) n;
   return i;
 }
 
@@ -176,24 +190,21 @@ graphics_group_box_rep::reindex (int i, int item, int n) {
 struct point_box_rep: public box_rep {
   point p;
   SI r;
-  color col;
-  int fill;
-  color fill_col;
+  pencil pen;
+  brush br;
   string style;
-  point_box_rep (
-    path ip, point p, SI radius, color col,
-    int fill, color fill_col, string style);
-  SI graphical_distance (SI x, SI y) { return (SI)norm (p - point (x, y)); }
+  point_box_rep (path ip, point p, SI radius, pencil pen,
+		 brush br, string style);
+  SI graphical_distance (SI x, SI y) { return (SI) norm (p - point (x, y)); }
   gr_selections graphical_select (SI x, SI y, SI dist);
   void display (renderer ren);
   operator tree () { return "point"; }
 };
 
-point_box_rep::point_box_rep (
-  path ip2, point p2, SI r2, color col2,
-  int fill2, color fill_col2, string style2):
-    box_rep (ip2), p (p2), r (r2), col (col2),
-    fill (fill2), fill_col (fill_col2), style (style2)
+point_box_rep::point_box_rep (path ip2, point p2, SI r2, pencil pen2,
+			      brush br2, string style2):
+    box_rep (ip2), p (p2), r (r2), pen (pen2),
+    br (br2), style (style2)
 {
   x1= x3= ((SI) p[0]) - r;
   y1= y3= ((SI) p[1]) - r;
@@ -206,6 +217,7 @@ point_box_rep::graphical_select (SI x, SI y, SI dist) {
   gr_selections res;
   if (graphical_distance (x, y) <= dist) {
     gr_selection gs;
+    gs->type= "point";
     gs->dist= graphical_distance (x, y);
     gs->p= p;
     gs->cp << reverse (path (0, ip));
@@ -227,18 +239,20 @@ point_box_rep::display (renderer ren) {
   y[2]= ((SI) p[1]) + r;
   x[3]= ((SI) p[0]) + r;
   y[3]= ((SI) p[1]) - r;
-  ren->set_line_style (PIXEL);
-  if (style == "square") {
-    if (fill == FILL_MODE_INSIDE || fill == FILL_MODE_BOTH) {
-      ren->set_color (fill_col);
+  if (style == "none");
+  else if (style == "square") {
+    if (pen->get_type () != pencil_none &&
+	br->get_type () != brush_none) {
+      ren->set_pencil (pen->set_width (ren->pixel));
+      ren->set_brush (br);
       ren->line (x[0], y[0], x[1], y[1]);
       ren->line (x[1], y[1], x[2], y[2]);
       ren->line (x[2], y[2], x[3], y[3]);
       ren->line (x[3], y[3], x[0], y[0]);
       ren->polygon (x, y, false);
     }
-    if (fill == FILL_MODE_NONE || fill == FILL_MODE_BOTH) {
-      ren->set_color (col);
+    if (pen->get_type () != pencil_none) {
+      ren->set_pencil (pen->set_width (ren->pixel));
       ren->line (x[0], y[0], x[1], y[1]);
       ren->line (x[1], y[1], x[2], y[2]);
       ren->line (x[2], y[2], x[3], y[3]);
@@ -246,14 +260,13 @@ point_box_rep::display (renderer ren) {
     }
   }
   else {
-    if (style == "disk"
-     || fill == FILL_MODE_INSIDE || fill == FILL_MODE_BOTH) {
-      ren->set_color (style == "disk" ? col : fill_col);
+    if (style == "disk" || br->get_type () != brush_none) {
+      ren->set_brush (style == "disk" ? pen->get_brush () : br);
       ren->arc (x[0], y[0]+ren->pixel, x[2], y[2]+ren->pixel, 0, 64*360);
       ren->fill_arc (x[0], y[0]+ren->pixel, x[2], y[2]+ren->pixel, 0, 64*360);
     }
-    if (fill == FILL_MODE_NONE || fill == FILL_MODE_BOTH) {
-      ren->set_color (col);
+    if (pen->get_type () != pencil_none) {
+      ren->set_pencil (pen->set_width (ren->pixel));
       ren->arc (x[0], y[0]+ren->pixel, x[2], y[2]+ren->pixel, 0, 64*360);
     }
   }
@@ -265,18 +278,16 @@ point_box_rep::display (renderer ren) {
 
 struct curve_box_rep: public box_rep {
   array<point> a;
-  SI width;
-  color col;
+  pencil pen;
   curve c;
   array<bool> style;
   SI style_unit;
   array<SI> styled_n;
-  int fill;
-  color fill_col;
+  brush fill_br;
   array<box> arrows;
-  curve_box_rep (path ip, curve c, SI width, color col,
+  curve_box_rep (path ip, curve c, pencil pen,
 		 array<bool> style, SI style_unit,
-		 int fill, color fill_col,
+		 brush fill_br,
 		 array<box> arrows);
   box transform (frame fr);
   SI graphical_distance (SI x, SI y);
@@ -288,13 +299,13 @@ struct curve_box_rep: public box_rep {
   void apply_style ();
 };
 
-curve_box_rep::curve_box_rep (path ip2, curve c2, SI W, color C,
-  array<bool> style2, SI style_unit2, int fill2, color fill_col2,
+curve_box_rep::curve_box_rep (path ip2, curve c2, pencil pen2,
+  array<bool> style2, SI style_unit2, brush fill_br2,
   array<box> arrows2)
   :
-  box_rep (ip2), width (W), col (C), c (c2),
+  box_rep (ip2), pen (pen2), c (c2),
   style (style2), style_unit (style_unit2),
-  fill (fill2), fill_col (fill_col2)
+  fill_br (fill_br2)
 {
   a= c->rectify (PIXEL);
   int i, n= N(a);
@@ -310,13 +321,13 @@ curve_box_rep::curve_box_rep (path ip2, curve c2, SI W, color C,
   arrows= array<box>(2);
   point p1, p2;
   bool error;
-  if (N(arrows2)>0 && !nil (arrows2[0])) {
+  if (N(arrows2)>0 && !is_nil (arrows2[0])) {
     point tg= c->grad (0.0, error);
     if (!error) {
       frame fr= scaling (1.0, a[0]) *
 		rotation_2D (point (0.0, 0.0), arg (tg));
       arrows[0]= arrows2[0]->transform (fr);
-      if (!nil (arrows[0])) {
+      if (!is_nil (arrows[0])) {
 	x1= min (x1, arrows[0]->x1);
 	y1= min (y1, arrows[0]->y1);
 	x2= max (x2, arrows[0]->x2);
@@ -324,13 +335,13 @@ curve_box_rep::curve_box_rep (path ip2, curve c2, SI W, color C,
       }
     }
   }
-  if (N(arrows2)>1 && !nil (arrows2[1])) {
+  if (N(arrows2)>1 && !is_nil (arrows2[1])) {
     point tg= c->grad (1.0, error);
     if (!error) {
       frame fr= scaling (1.0, a[N(a)-1]) *
 		rotation_2D (point (0.0, 0.0), arg (tg));
       arrows[1]= arrows2[1]->transform (fr);
-      if (!nil (arrows[1])) {
+      if (!is_nil (arrows[1])) {
 	x1= min (x1, arrows[1]->x1);
 	y1= min (y1, arrows[1]->y1);
 	x2= max (x2, arrows[1]->x2);
@@ -338,14 +349,14 @@ curve_box_rep::curve_box_rep (path ip2, curve c2, SI W, color C,
       }
     }
   }
+  SI width= pen->get_width ();
   x3= x1 - (width>>1); y3= y1 - (width>>1); 
   x4= x2 + (width>>1); y4= y2 + (width>>1);
 }
 
 box
 curve_box_rep::transform (frame fr) {
-  return curve_box (ip, fr (c), width, col,
-    style, style_unit, fill, fill_col, arrows);
+  return curve_box (ip, fr (c), pen, style, style_unit, fill_br, arrows);
 }
 
 SI
@@ -376,6 +387,7 @@ curve_box_rep::graphical_select (SI x, SI y, SI dist) {
       SI n= (SI)norm (p - pts[i]);
       if (n <= dist) {
 	gr_selection gs;
+	gs->type= "curve-handle";
 	gs->dist= n;
 	gs->p= pts[i];
 	gs->cp << reverse (paths[i]);
@@ -396,6 +408,7 @@ curve_box_rep::graphical_select (SI x, SI y, SI dist) {
 	SI n= (SI)norm (p - p2);
 	if (n <= dist) {
 	  gr_selection gs;
+	  gs->type= "curve-point";
 	  gs->dist= n;
 	  gs->p= p2;
 	  gs->cp << reverse (paths[i]);
@@ -416,6 +429,7 @@ curve_box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
   gr_selections res;
   if (in_rectangle (x1, y1, x2, y2)) {
     gr_selection gs;
+    gs->type= "curve";
     gs->dist= graphical_distance (x1, y1);
     gs->cp << reverse (path (0, ip));
     gs->pts= array<point> (0);
@@ -428,8 +442,8 @@ curve_box_rep::graphical_select (SI x1, SI y1, SI x2, SI y2) {
 void
 curve_box_rep::display (renderer ren) {
   int i, n;
-  if (fill == FILL_MODE_INSIDE || fill == FILL_MODE_BOTH) {
-    ren->set_color (fill_col);
+  if (fill_br->get_type () != brush_none) {
+    ren->set_brush (fill_br);
     n= N(a);
     array<SI> x (n), y (n);
     for (i=0; i<n; i++) {
@@ -438,9 +452,8 @@ curve_box_rep::display (renderer ren) {
     }
     ren->polygon (x, y, false);
   }
-  if (fill == FILL_MODE_NONE || fill == FILL_MODE_BOTH) {
-    ren->set_color (col);
-    ren->set_line_style (width, 0, false);
+  if (pen->get_type () != pencil_none) {
+    ren->set_pencil (pen->set_cap (cap_flat));
  // TODO: Add options for handling round/nonround joins & line ends
     if (N (style) == 0) {
       n= N(a);
@@ -504,8 +517,8 @@ curve_box_rep::display (renderer ren) {
   }
 
   rectangles ll;
-  if (!nil (arrows[0])) arrows[0]->redraw (ren, path (), ll);
-  if (!nil (arrows[1])) arrows[1]->redraw (ren, path (), ll);
+  if (!is_nil (arrows[0])) arrows[0]->redraw (ren, path (), ll);
+  if (!is_nil (arrows[1])) arrows[1]->redraw (ren, path (), ll);
 }
 
 SI
@@ -588,6 +601,32 @@ curve_box_rep::apply_style () {
   if (style[n2-1]) styled_n[N(styled_n)-1]= nbu;
 }
 
+/******************************************************************************
+* 3D graphics
+******************************************************************************/
+
+struct spacial_box_rep: public box_rep {
+  spacial obj;
+  spacial_box_rep (path ip, spacial obj2);
+  void display (renderer ren);
+  operator tree () { return "spacial"; }
+};
+
+spacial_box_rep::spacial_box_rep (path ip, spacial obj2):
+  box_rep (ip), obj (obj2)
+{
+  rectangle r= obj->get_extents ();
+  x1= x3= r->x1;
+  y1= y3= r->y1;
+  x2= x4= r->x2;
+  y2= y4= r->y2;
+}
+
+void
+spacial_box_rep::display (renderer ren) {
+  ren->draw_spacial (obj);
+}
+
 /******************************************************************************
 * User interface
 ******************************************************************************/
@@ -596,26 +635,30 @@ box
 graphics_box (
   path ip, array<box> bs, frame f, grid g, point lim1, point lim2)
 {
-  return new graphics_box_rep (ip, bs, f, g, lim1, lim2);
+  box r= tm_new<graphics_box_rep> (ip, bs, f, g, lim1, lim2);
+  if (r->x1 != 0) r= move_box (ip, r, -r->x1, 0);
+  return r;
 }
 
 box
 graphics_group_box (path ip, array<box> bs) {
-  return new graphics_group_box_rep (ip, bs);
+  return tm_new<graphics_group_box_rep> (ip, bs);
 }
 
 box
-point_box (
-  path ip, point p, SI r, color col, int fill, color fill_col, string style) {
-  return new point_box_rep (ip, p, r, col, fill, fill_col, style);
+point_box (path ip, point p, SI r, pencil pen, brush br, string style) {
+  return tm_new<point_box_rep> (ip, p, r, pen, br, style);
 }
 
 box
-curve_box (path ip, curve c, SI width, color col,
-  array<bool> style, SI style_unit,
-  int fill, color fill_col,
-  array<box> arrows)
+curve_box (path ip, curve c, pencil pen,
+           array<bool> style, SI style_unit,
+           brush fill_br, array<box> arrows)
 {
-  return new curve_box_rep (ip, c, width, col,
-			    style, style_unit, fill, fill_col, arrows);
+  return tm_new<curve_box_rep> (ip, c, pen, style, style_unit, fill_br, arrows);
+}
+
+box
+spacial_box (path ip, spacial obj) {
+  return tm_new<spacial_box_rep> (ip, obj);
 }
