@@ -4,77 +4,77 @@
 * DESCRIPTION: Boxes whose behaviour is changed
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
-#include "Boxes/composite.hpp"
+#include "Boxes/change.hpp"
 #include "Boxes/construct.hpp"
 #include "scheme.hpp"
+#include "gui.hpp"
+#include "effect.hpp"
 
 /******************************************************************************
 * changing the behaviour of a box
 ******************************************************************************/
 
-struct change_box_rep: public composite_box_rep {
-  bool child_flag, big_flag;
-  change_box_rep (path ip, bool child_flag, bool big_flag= false);
-  operator tree () { return tree (TUPLE, "change", (tree) bs[0]); }
-  int find_child (SI x, SI y, SI delta, bool force);
-  path find_left_box_path () {
-    return child_flag?
-      composite_box_rep::find_left_box_path ():
-      path (0, bs[0]->find_left_box_path ()); }
-  path find_right_box_path () {
-    return child_flag?
-      composite_box_rep::find_right_box_path ():
-      path (0, bs[0]->find_right_box_path ()); }
-
-  double left_slope () {
-    return big_flag? bs[0]->left_slope(): box_rep::left_slope(); }
-  double right_slope () { 
-    return big_flag? bs[0]->right_slope(): box_rep::right_slope(); }
-  SI left_correction () {
-    return big_flag? bs[0]->left_correction():box_rep::left_correction(); }
-  SI right_correction () {
-    return big_flag? bs[0]->right_correction():box_rep::right_correction(); }
-  SI lsub_correction () {
-    return big_flag? bs[0]->lsub_correction(): box_rep::lsub_correction(); }
-  SI lsup_correction () {
-    return big_flag? bs[0]->lsup_correction(): box_rep::lsup_correction(); }
-  SI rsub_correction () {
-    return big_flag? bs[0]->rsub_correction(): box_rep::rsub_correction(); }
-  SI rsup_correction () {
-    return big_flag? bs[0]->rsup_correction(): box_rep::rsup_correction(); }
-  SI sub_lo_base (int l) {
-    return big_flag? bs[0]->sub_lo_base (l): box_rep::sub_lo_base (l); }
-  SI sub_hi_lim  (int l) {
-    return big_flag? bs[0]->sub_hi_lim (l): box_rep::sub_hi_lim (l); }
-  SI sup_lo_lim  (int l) {
-    return big_flag? bs[0]->sup_lo_lim (l): box_rep::sup_lo_lim (l); }
-  SI sup_lo_base (int l) {
-    return big_flag? bs[0]->sup_lo_base (l): box_rep::sup_lo_base (l); }
-  SI sup_hi_lim  (int l) {
-    return big_flag? bs[0]->sup_hi_lim (l): box_rep::sup_hi_lim (l); }
-
-  SI get_leaf_offset (string search) {
-    return sx1(0) + bs[0]->get_leaf_offset (search); }
-
-  gr_selections graphical_select (SI x, SI y, SI dist);
-  gr_selections graphical_select (SI x1, SI y1, SI x2, SI y2);
-};
-
 change_box_rep::change_box_rep (path ip, bool fl1, bool fl2):
   composite_box_rep (ip), child_flag (fl1), big_flag (fl2) {}
 
+change_box_rep::operator tree () {
+  return tree (TUPLE, "change", (tree) bs[0]); }
+
 int
 change_box_rep::find_child (SI x, SI y, SI delta, bool force) {
   if (child_flag) return composite_box_rep::find_child (x, y, delta, force);
   return 0;
 }
 
+path
+change_box_rep::find_left_box_path () {
+  return child_flag?
+    composite_box_rep::find_left_box_path ():
+    path (0, bs[0]->find_left_box_path ());
+}
+
+path
+change_box_rep::find_right_box_path () {
+  return child_flag?
+    composite_box_rep::find_right_box_path ():
+    path (0, bs[0]->find_right_box_path ());
+}
+
+double change_box_rep::left_slope () {
+  return big_flag? bs[0]->left_slope(): box_rep::left_slope(); }
+double change_box_rep::right_slope () { 
+  return big_flag? bs[0]->right_slope(): box_rep::right_slope(); }
+SI change_box_rep::left_correction () {
+  return big_flag? bs[0]->left_correction():box_rep::left_correction(); }
+SI change_box_rep::right_correction () {
+  return big_flag? bs[0]->right_correction():box_rep::right_correction(); }
+SI change_box_rep::lsub_correction () {
+  return big_flag? bs[0]->lsub_correction(): box_rep::lsub_correction(); }
+SI change_box_rep::lsup_correction () {
+  return big_flag? bs[0]->lsup_correction(): box_rep::lsup_correction(); }
+SI change_box_rep::rsub_correction () {
+  return big_flag? bs[0]->rsub_correction(): box_rep::rsub_correction(); }
+SI change_box_rep::rsup_correction () {
+  return big_flag? bs[0]->rsup_correction(): box_rep::rsup_correction(); }
+SI change_box_rep::sub_lo_base (int l) {
+  return big_flag? bs[0]->sub_lo_base (l): box_rep::sub_lo_base (l); }
+SI change_box_rep::sub_hi_lim  (int l) {
+  return big_flag? bs[0]->sub_hi_lim (l): box_rep::sub_hi_lim (l); }
+SI change_box_rep::sup_lo_lim  (int l) {
+  return big_flag? bs[0]->sup_lo_lim (l): box_rep::sup_lo_lim (l); }
+SI change_box_rep::sup_lo_base (int l) {
+  return big_flag? bs[0]->sup_lo_base (l): box_rep::sup_lo_base (l); }
+SI change_box_rep::sup_hi_lim  (int l) {
+  return big_flag? bs[0]->sup_hi_lim (l): box_rep::sup_hi_lim (l); }
+
+SI change_box_rep::get_leaf_offset (string search) {
+  return sx1(0) + bs[0]->get_leaf_offset (search); }
+
 gr_selections
 change_box_rep::graphical_select (SI x, SI y, SI dist) { 
 //TODO : Check if it is correct
@@ -112,6 +112,22 @@ move_box_rep::move_box_rep (path ip, box b, SI x, SI y, bool fl1, bool fl2):
   finalize ();
 }
 
+struct shift_box_rep: public change_box_rep {
+  shift_box_rep (path ip, box b, SI x, SI y, bool fl1, bool fl2);
+  int get_type () { return MOVE_BOX; }
+  operator tree () { return tree (TUPLE, "shift", (tree) bs[0]); }
+};
+
+shift_box_rep::shift_box_rep (path ip, box b, SI x, SI y, bool fl1, bool fl2):
+  change_box_rep (ip, fl1, fl2)
+{
+  insert (b, x, y);
+  position ();
+  x1 -= x; y1 -= y;
+  x2 -= x; y2 -= y;
+  finalize ();
+}
+
 /******************************************************************************
 * Resizing boxes
 ******************************************************************************/
@@ -149,6 +165,116 @@ vcorrect_box_rep::vcorrect_box_rep (path ip, box b, SI top_cor, SI bot_cor):
   finalize ();
 }
 
+/******************************************************************************
+* Transformed boxes
+******************************************************************************/
+
+// FIXME: several things are not yet OK
+// 1) The clip region is not tight anymore under transformations,
+// so it should only be considered as a hint for rejecting redraw operations
+// for which we know for sure that no work has to be done.
+// The actual clipping for the painting itself should be ensured
+// using a different mechanism.
+// 2) Cursor movement and selections inside transformed text are
+// not implemented.  We might determine the "underlying transformation"
+// of a cursor position or selection and draw the cursor/selection
+// using that transformation.
+
+struct transformed_box_rep: public change_box_rep {
+  frame fr;
+public:
+  transformed_box_rep (path ip, box b, frame fr);
+  operator tree () { return tree (TUPLE, "transform", (tree) bs[0]); }
+  void pre_display (renderer &ren);
+  void post_display (renderer &ren);
+  //cursor find_cursor (path bp);
+  //selection find_selection (path lbp, path rbp);
+};
+
+transformed_box_rep::transformed_box_rep (path ip, box b, frame fr2):
+  change_box_rep (ip, true), fr (fr2)
+{
+  insert (b, 0, 0);
+  position ();
+  finalize ();
+  rectangle r1= fr (rectangle (x1, y1, x2, y2));
+  rectangle r2= fr (rectangle (x3, y3, x4, y4));
+  x1= r1->x1; y1= r1->y1;
+  x2= r1->x2; y2= r1->y2;
+  x3= r2->x1; y3= r2->y1;
+  x4= r2->x2; y4= r2->y2;
+}
+
+void
+transformed_box_rep::pre_display (renderer &ren) {
+  ren->set_transformation (fr);
+}
+
+void
+transformed_box_rep::post_display (renderer &ren) {
+  ren->reset_transformation ();
+}
+
+/******************************************************************************
+* Effects applied on boxes
+******************************************************************************/
+
+struct effect_box_rep: public change_box_rep {
+  tree   eff_t;
+  effect eff;
+public:
+  effect_box_rep (path ip, array<box> bs, tree eff);
+  operator tree () { return tree (TUPLE, "effect", eff_t); }
+  void redraw (renderer ren, path p, rectangles& l);
+};
+
+effect_box_rep::effect_box_rep (path ip, array<box> bs, tree eff2):
+  change_box_rep (ip, true), eff_t (eff2), eff (build_effect (eff2))
+{
+  for (int i=0; i<N(bs); i++)
+    insert (bs[i], 0, 0);
+  array<rectangle> rs (N(bs));
+  for (int i=0; i<N(bs); i++)
+    rs[i]= rectangle (bs[i]->x1, bs[i]->y1, bs[i]->x2, bs[i]->y2);
+  rectangle r= eff->get_logical_extents (rs);
+  x1= r->x1;
+  y1= r->y1;
+  x2= r->x2;
+  y2= r->y2;
+  rs= array<rectangle> (N(bs));
+  for (int i=0; i<N(bs); i++)
+    rs[i]= rectangle (bs[i]->x3, bs[i]->y3, bs[i]->x4, bs[i]->y4);
+  r = eff->get_extents (rs);
+  x3= r->x1;
+  y3= r->y1;
+  x4= r->x2;
+  y4= r->y2;
+  finalize ();
+}
+
+extern int nr_painted;
+
+void
+effect_box_rep::redraw (renderer ren, path p, rectangles& l) {
+  if (((nr_painted&15) == 15) && gui_interrupted (true)) return;
+  ren->move_origin (x0, y0);
+  array<picture> pics (subnr ());
+  SI shad_pixel= ren->pixel;
+  for (int i=0; i<subnr(); i++) {
+    renderer shad= ren->shadow (pics[i], sx3(i), sy3(i), sx4(i), sy4(i));
+    shad_pixel= shad->pixel;
+    rectangles rs;
+    subbox (i)->redraw (shad, path (), rs);
+    delete_renderer (shad);
+  }
+  if (((nr_painted&15) == 15) && gui_interrupted (true));
+  else {
+    picture result_pic= eff->apply (pics, shad_pixel);
+    ren->draw_picture (result_pic, 0, 0);
+  }
+  ren->move_origin (-x0, -y0);
+}
+
 /******************************************************************************
 * Clipped boxes
 ******************************************************************************/
@@ -242,19 +368,17 @@ repeat_box_rep::repeat_box_rep (path ip, box b, box repeat, SI xoff):
 
 struct cell_box_rep: public change_box_rep {
   SI    bl, br, bb, bt;
-  color fg;
-  tree  bg, old_bg;
+  brush fg, bg, old_bg;
   cell_box_rep (path ip, box b, SI x0, SI y0, SI x1, SI y1, SI x2, SI y2,
-		SI bl, SI br, SI bb, SI bt, color fg, tree bg);
+		SI bl, SI br, SI bb, SI bt, brush fg, brush bg);
   operator tree () { return tree (TUPLE, "cell", (tree) bs[0]); }
   void pre_display (renderer &ren);
   void post_display (renderer &ren);
-  void display (renderer ren);
 };
 
 cell_box_rep::cell_box_rep (
   path ip, box b, SI X0, SI Y0, SI X1, SI Y1, SI X2, SI Y2,
-  SI Bl, SI Br, SI Bb, SI Bt, color Fg, tree Bg):
+  SI Bl, SI Br, SI Bb, SI Bt, brush Fg, brush Bg):
   change_box_rep (ip, false),
   bl (Bl<<1), br (Br<<1), bb (Bb<<1), bt (Bt<<1),
   fg (Fg), bg (Bg)
@@ -263,7 +387,7 @@ cell_box_rep::cell_box_rep (
   position ();
   x1= X1; y1= Y1;
   x2= X2; y2= Y2;
-  if ((bg != "") || (bl>0) || (br>0) || (bb>0) || (bt>0)) {
+  if (bg->get_type () != brush_none || (bl>0) || (br>0) || (bb>0) || (bt>0)) {
     // the 4*PIXEL extra space is sufficient for (shrinking factor) <= 8
     x3= min (x3, x1 - (bl>>1) - (bl>0? PIXEL<<2: 0));
     y3= min (y3, y1 - (bb>>1) - (bb>0? PIXEL<<2: 0));
@@ -275,37 +399,41 @@ cell_box_rep::cell_box_rep (
 
 void
 cell_box_rep::pre_display (renderer& ren) {
-  if (bg == "") return;
-  old_bg= ren->get_background_pattern ();
-  ren->set_background_pattern (bg);
-  ren->clear_pattern (x1, y1, x2, y2);
-}
-
-void
-cell_box_rep::post_display (renderer &ren) {
-  if (bg == "") return;
-  ren->set_background_pattern (old_bg);
-}
-
-void
-cell_box_rep::display (renderer ren) {
-  if ((bl>0) || (br>0) || (bb>0) || (bt>0)) {
-    SI l= bl, r= br, b= bb, t= bt;
-    if (ren->sfactor > 1) { // correction for screen display only
-      SI  pixel= ren->pixel;
-      l= ((l + (pixel - 1)) / pixel) * pixel;
-      r= ((r + (pixel - 1)) / pixel) * pixel;
-      b= ((b + (pixel - 1)) / pixel) * pixel;
-      t= ((t + (pixel - 1)) / pixel) * pixel;
-    }
-    SI X1= x1-(l>>1), X2= x2+(r>>1);
-    SI Y1= y1-(b>>1), Y2= y2+(t>>1);
-    ren->set_color (fg);
-    ren->fill (X1  , Y1  , X1+l, Y2  );
-    ren->fill (X2-r, Y1  , X2  , Y2  );
-    ren->fill (X1  , Y1  , X2  , Y1+b);
-    ren->fill (X1  , Y2-t, X2  , Y2  );
+  SI l= bl, r= br, b= bb, t= bt;
+  SI lx1, rx1, by1, ty1;
+  SI lx2, rx2, by2, ty2;
+  if (ren->is_screen) { // correction for screen display only
+    SI  pixel= ren->pixel;
+    l= ((l + (pixel - 1)) / pixel) * pixel;
+    r= ((r + (pixel - 1)) / pixel) * pixel;
+    b= ((b + (pixel - 1)) / pixel) * pixel;
+    t= ((t + (pixel - 1)) / pixel) * pixel;
   }
+
+  lx1= x1 - (l>>1); lx2= lx1 + l;
+  by1= y1 - (b>>1); by2= by1 + b;
+  rx2= x2 + (r>>1); rx1= rx2 - r;
+  ty2= y2 + (t>>1); ty1= ty2 - t;
+
+  if (bg->get_type () != brush_none) {
+    old_bg= ren->get_background ();
+    ren->set_background (bg);
+    ren->clear_pattern (lx2, by2, rx1, ty1);
+  }
+
+  if ((l>0) || (r>0) || (b>0) || (t>0)) {
+    ren->set_pencil (fg);
+    ren->fill (lx1, by1, lx2, ty2);
+    ren->fill (rx1, by1, rx2, ty2);
+    ren->fill (lx1, by1, rx2, by2);
+    ren->fill (lx1, ty1, rx2, ty2);
+  }
+}
+
+void
+cell_box_rep::post_display (renderer& ren) {
+  if (bg->get_type () != brush_none)
+    ren->set_background (old_bg);
 }
 
 /******************************************************************************
@@ -345,74 +473,6 @@ public:
   }
 };
 
-/******************************************************************************
-* Highlight boxes
-******************************************************************************/
-
-struct highlight_box_rep: public change_box_rep {
-  SI w, xpad, ypad;
-  tree bg, old_bg;
-  color sun, shad;
-  highlight_box_rep (path ip, box b, SI w, SI xpad, SI ypad,
-		     tree bg, color sun, color shad);
-  operator tree () { return tree (TUPLE, "highlight", (tree) bs[0]); }
-  void pre_display (renderer &ren);
-  void post_display (renderer &ren);
-  void display (renderer ren);
-};
-
-highlight_box_rep::highlight_box_rep (
-  path ip, box b, SI w2, SI xp2, SI yp2, tree bg2, color sun2, color shad2):
-  change_box_rep (ip, true), w (w2), xpad (xp2), ypad (yp2),
-  bg (bg2), sun (sun2), shad (shad2)
-{
-  insert (b, w + xpad, 0);
-  position ();
-  x1= b->x1;
-  y1= b->y1 - w - ypad;
-  x2= b->x2 + 2 * (w + xpad);
-  y2= b->y2 + w + ypad;
-  x3= min (x1, b->x3 + w + xpad);
-  y3= min (y1, b->y3);
-  x4= max (x2, b->x4 + w + xpad);
-  y4= max (y2, b->y4);
-  finalize ();
-}
-
-void
-highlight_box_rep::pre_display (renderer& ren) {
-  old_bg= ren->get_background_pattern ();
-  ren->set_background_pattern (bg);
-  SI W= w;
-  if (!ren->is_printer ()) {
-    SI pixel= ren->pixel;
-    W= ((w + pixel - 1) / pixel) * pixel;
-  }
-  ren->clear_pattern (x1+W, y1+W, x2-W, y2-W);
-}
-
-void
-highlight_box_rep::post_display (renderer &ren) {
-  ren->set_background_pattern (old_bg);
-}
-
-void
-highlight_box_rep::display (renderer ren) {
-  SI W= w;
-  if (!ren->is_printer ()) {
-    SI pixel= ren->pixel;
-    W= ((w + pixel - 1) / pixel) * pixel;
-  }
-  ren->set_color (sun);
-  ren->fill (x1  , y2-W, x2  , y2  );
-  ren->fill (x1  , y1  , x1+W, y2  );
-  ren->set_color (shad);
-  ren->fill (x1+W, y1  , x2  , y1+W);
-  ren->fill (x2-W, y1  , x2  , y2-W);
-  ren->triangle (x1, y1, x1+W, y1, x1+W, y1+W);
-  ren->triangle (x2, y2, x2, y2-W, x2-W, y2-W);
-}
-
 /******************************************************************************
 * action boxes
 ******************************************************************************/
@@ -454,20 +514,39 @@ action_box_rep::action (tree t, SI x, SI y, SI delta) {
 struct locus_box_rep: public change_box_rep {
   list<string> ids;
   SI pixel;
+  string ref;
+  string anchor;
   locus_box_rep (path ip, box b, list<string> ids, SI pixel);
+  locus_box_rep (path ip, box b, list<string> ids, SI pixel, string _rep, string _anchor);
   operator tree () { return tree (TUPLE, "locus"); }
   void loci (SI x, SI y, SI delta, list<string>& ids2, rectangles& rs);
+  void post_display (renderer &ren);
+
 };
 
 locus_box_rep::locus_box_rep (path ip, box b, list<string> ids2, SI pixel2):
   change_box_rep (ip, true), ids (ids2), pixel (pixel2)
 {
+  ref = "";
+  anchor = "";
   insert (b, 0, 0);
   position ();
   left_justify ();
   finalize ();
 }
 
+locus_box_rep::locus_box_rep (path ip, box b, list<string> ids2, SI pixel2, string _ref, string _anchor):
+  change_box_rep (ip, true), ids (ids2), pixel (pixel2)
+{
+  ref = _ref;
+  anchor = _anchor;
+  insert (b, 0, 0);
+  position ();
+  left_justify ();
+  finalize ();
+}
+
+
 void
 locus_box_rep::loci (SI x, SI y, SI delta, list<string>& l, rectangles& rs) {
   bs[0]->loci (x, y, delta, l, rs);
@@ -475,6 +554,12 @@ locus_box_rep::loci (SI x, SI y, SI delta, list<string>& l, rectangles& rs) {
   rs= rs * outline (rectangles (rectangle (x1, y1, x2, y2)), pixel);
 }
 
+void
+locus_box_rep::post_display (renderer &ren) {
+  if (ref!="") ren->href(ref, x1, y1, x2, y2);
+  if (anchor!="") ren->anchor(anchor, x1, y1);
+}
+
 /******************************************************************************
 * tag boxes
 ******************************************************************************/
@@ -513,28 +598,94 @@ tag_box_rep::find_tag (string search) {
 }
 
 /******************************************************************************
-* textat boxes
+* text_at boxes
 ******************************************************************************/
 
-struct textat_box_rep: public move_box_rep {
-  textat_box_rep (path ip, box b, SI x, SI y):
-    move_box_rep (ip, b, x, y, false, false) {}
+struct text_at_box_rep: public move_box_rep {
+  SI axis;
+  SI pad;
+  text_at_box_rep (path ip, box b, SI x, SI y, SI axis, SI pad);
   gr_selections graphical_select (SI x, SI y, SI dist);
-  operator tree () { return tree (TUPLE, "textat", (tree) bs[0]); }
+  operator tree () { return tree (TUPLE, "text-at", (tree) bs[0]); }
+  /*
+  void pre_display (renderer &ren) {
+    array<SI> xs, ys;
+    xs << x1 - pad << x2 + pad << x2 + pad << x1 - pad << x1 - pad;
+    ys << y1 - pad << y1 - pad << y2 + pad << y2 + pad << y1 - pad;
+    ren->set_pencil (pencil (rgb_color (255, 255, 224), ren->pixel));
+    ren->polygon (xs, ys);
+  }
+  */
 };
 
+text_at_box_rep::text_at_box_rep (path ip, box b, SI x, SI y, SI a2, SI p2):
+  move_box_rep (ip, b, x, y, false, false), axis (a2), pad (p2) {}
+
 gr_selections
-textat_box_rep::graphical_select (SI x, SI y, SI dist) {
+text_at_box_rep::graphical_select (SI x, SI y, SI dist) {
+  array<point> special;
+  special << point (x1 - pad, y1 - pad)
+	  << point (x1 - pad, y2 + pad)
+	  << point (x2 + pad, y2 + pad)
+	  << point (x2 + pad, y1 - pad)
+	  << point ((x1 + x2) >> 1, y1 - pad)
+	  << point ((x1 + x2) >> 1, y2 + pad)
+	  << point (x1 - pad, y1 + axis)
+	  << point (x2 + pad, y1 + axis);
+  array<point> ps;
+  ps << point (x1 - pad, y1 - pad) << point (x2 + pad, y1 - pad)
+     << point (x2 + pad, y2 + pad) << point (x1 - pad, y2 + pad)
+     << point (x1 - pad, y1 - pad);
+  array<curve> cs;
+  for (int i=0; i<N(ps)-1; i++)
+    cs << segment (ps[i], ps[i+1]);
+
   gr_selections res;
-  if (graphical_distance (x, y) <= dist) {
+  point p= point (x, y);
+  if (norm (p - point (sx(0), sy(0))) <= dist) {
     gr_selection gs;
+    gs->type= "text-handle";
+    gs->dist= (SI) norm (p - point (sx(0), sy(0)));
+    gs->p= point (sx(0), sy(0));
+    gs->cp << reverse (path (0, path (1, ip)));
+    gs->pts << gs->p;
+    gs->c= curve ();
+    res << gs;
+  }
+  else if (graphical_distance (x, y) == 0) {
+    gr_selection gs;
+    gs->type= "text";
     gs->dist= graphical_distance (x, y);
-    gs->p= point (x, y);
+    gs->p= p;
     gs->cp << box_rep::find_tree_path (x, y, dist);
     gs->pts << gs->p;
     gs->c= curve ();
     res << gs;
   }
+
+  for (int i=0; i<N(special); i++)
+    if (norm (special[i] - p) <= dist) {
+      gr_selection gs;
+      gs->type= "text-border-point";
+      gs->p= special[i];
+      gs->dist= (SI) norm (gs->p - p);
+      gs->cp << box_rep::find_tree_path (x, y, dist);
+      gs->pts << gs->p;
+      gs->c= curve ();
+      res << gs;
+    }
+  for (int i=0; i<N(cs); i++) {
+    if (N(res) == 0 && norm (closest (cs[i], p) - p) <= dist) {
+      gr_selection gs;
+      gs->type= "text-border";
+      gs->p= closest (cs[i], p);
+      gs->dist= (SI) norm (gs->p - p);
+      gs->cp << box_rep::find_tree_path (x, y, dist);
+      gs->pts << gs->p;
+      gs->c= cs[i];
+      res << gs;    
+    }
+  }
   return res;
 }
 
@@ -544,77 +695,91 @@ textat_box_rep::graphical_select (SI x, SI y, SI dist) {
 
 box
 move_box (path ip, box b, SI x, SI y, bool child_flag, bool big_flag) {
-  return new move_box_rep (ip, b, x, y, child_flag, big_flag);
+  return tm_new<move_box_rep> (ip, b, x, y, child_flag, big_flag);
+}
+
+box
+shift_box (path ip, box b, SI x, SI y, bool child_flag, bool big_flag) {
+  return tm_new<shift_box_rep> (ip, b, x, y, child_flag, big_flag);
 }
 
 box
 resize_box (path ip, box b, SI x1, SI y1, SI x2, SI y2,
 	    bool child_flag, bool adjust) {
-  return new resize_box_rep (ip, b, x1, y1, x2, y2, child_flag, adjust);
+  return tm_new<resize_box_rep> (ip, b, x1, y1, x2, y2, child_flag, adjust);
+}
+
+box
+transformed_box (path ip, box b, frame fr) {
+  return tm_new<transformed_box_rep> (ip, b, fr);
+}
+
+box
+effect_box (path ip, array<box> bs, tree eff) {
+  return tm_new<effect_box_rep> (ip, bs, eff);
 }
 
 box
 clip_box (path ip, box b, SI x1, SI y1, SI x2, SI y2) {
-  return new clip_box_rep (ip, b, x1, y1, x2, y2, UNINIT, UNINIT, 0, 0);
+  return tm_new<clip_box_rep> (ip, b, x1, y1, x2, y2, UNINIT, UNINIT, 0, 0);
 }
 
 box
 clip_box (path ip, box b, SI x1, SI y1, SI x2, SI y2,
 	  tree xt, tree yt, SI scx, SI scy) {
-  return new clip_box_rep (ip, b, x1, y1, x2, y2, xt, yt, scx, scy);
+  return tm_new<clip_box_rep> (ip, b, x1, y1, x2, y2, xt, yt, scx, scy);
 }
 
 box
 vcorrect_box (path ip, box b, SI top_cor, SI bot_cor) {
-  return new vcorrect_box_rep (ip, b, top_cor, bot_cor);
+  return tm_new<vcorrect_box_rep> (ip, b, top_cor, bot_cor);
 }
 
 box
 repeat_box (path ip, box ref, box repeat, SI xoff) {
-  return new repeat_box_rep (ip, ref, repeat, xoff);
+  return tm_new<repeat_box_rep> (ip, ref, repeat, xoff);
 }
 
 box
 cell_box (path ip, box b, SI x0, SI y0, SI x1, SI y1, SI x2, SI y2,
-	  SI bl, SI br, SI bb, SI bt, color fg, tree bg)
+	  SI bl, SI br, SI bb, SI bt, brush fg, brush bg)
 {
-  box cb= new cell_box_rep (ip, b, x0, y0, x1, y1, x2, y2,
-			    bl, br, bb, bt, fg, bg);
+  box cb= tm_new<cell_box_rep> (ip, b, x0, y0, x1, y1, x2, y2,
+                                bl, br, bb, bt, fg, bg);
   return cb;
 }
 
 box
 remember_box (path ip, box b) {
-  return new remember_box_rep (ip, b);
-}
-
-box
-highlight_box (path ip, box b, SI w, SI xpad, SI ypad,
-	       tree bg, color sun, color shad) {
-  return new highlight_box_rep (ip, b, w, xpad, ypad, bg, sun, shad);
+  return tm_new<remember_box_rep> (ip, b);
 }
 
 box
 action_box (path ip, box b, tree filter, command cmd, bool ch, path vip) {
-  return new action_box_rep (ip, b, filter, cmd, ch, vip);
+  return tm_new<action_box_rep> (ip, b, filter, cmd, ch, vip);
 }
 
 box
 action_box (path ip, box b, tree filter, command cmd, bool ch) {
-  return new action_box_rep (ip, b, filter, cmd, ch, decorate ());
+  return tm_new<action_box_rep> (ip, b, filter, cmd, ch, decorate ());
 }
 
 box
 locus_box (path ip, box b, list<string> ids, SI pixel) {
-  return new locus_box_rep (ip, b, ids, pixel);
+  return tm_new<locus_box_rep> (ip, b, ids, pixel);
+}
+
+box
+locus_box (path ip, box b, list<string> ids, SI pixel, string ref, string anchor) {
+  return tm_new<locus_box_rep> (ip, b, ids, pixel, ref, anchor);
 }
 
 box
 tag_box (path ip, box b, tree keys) {
-  return new tag_box_rep (ip, b, keys);
+  return tm_new<tag_box_rep> (ip, b, keys);
 }
 
 box
-textat_box (path ip, box b, SI x, SI y) {
-  return new textat_box_rep (ip, b, x, y);
+text_at_box (path ip, box b, SI x, SI y, SI axis, SI pad) {
+  return tm_new<text_at_box_rep> (ip, b, x, y, axis, pad);
 }
