@@ -32,12 +32,18 @@ initialize_default_env () {
 		       tuple ("1", "#c0c0c0"),
 		       tuple ("10", "#e0e0ff"));
 
+  tree gr_transf (TUPLE,
+                  tuple ("1.0", "0.0", "0.0", "0.0"),
+                  tuple ("0.0", "1.0", "0.0", "0.0"),
+                  tuple ("0.0", "0.0", "1.0", "0.0"),
+                  tuple ("0.0", "0.0", "0.0", "1.0"));         
+
   env (DPI)              = "600";       // resolution in dots per inch
-  env (SFACTOR)          = "5";         // shrinking factor on screen
+  env (ZOOM_FACTOR)      = "1";         // zoom factor on screen
   env (PREAMBLE)         = "false";     // preamble mode ?
   env (SAVE_AUX)         = "true";      // save auxiliary data on disk ?
   env (MODE)             = "text";      // typesetting mode
-  env (INFO_FLAG)        = "short";     // information about labels, etc.
+  env (INFO_FLAG)        = "minimal";   // information about labels, etc.
   env (WINDOW_BARS)      = "auto";      // override menu/icon bar settings
   env (SCROLL_BARS)      = "true";      // allow scroll bars around canvas?
   env (IDENTITY)         = identity_m;  // identity macro
@@ -54,6 +60,7 @@ initialize_default_env () {
   env (FONT_BASE_SIZE)   = "10";        // the font base size
   env (MAGNIFICATION)    = "1";         // magnification (slides for instance)
   env (COLOR)            = "black";     // the color
+  env (OPACITY)          = "100%";      // the opacity
   env (BG_COLOR)         = "white";     // the background color
   env (LOCUS_COLOR)      = "global";    // the color of loci
   env (VISITED_COLOR)    = "global";    // the color of visited loci
@@ -85,7 +92,8 @@ initialize_default_env () {
   env (PROG_SESSION)     = "default";   // computer algebra session name
 
   env (PAR_MODE)         = "justify";   // outline method
-  env (PAR_HYPHEN)       = "normal";    // quality of hyphenation
+  env (PAR_FLEXIBILITY)  = "1000.0";    // threshold for switching to ragged
+  env (PAR_HYPHEN)       = "professional"; // quality of hyphenation
   env (PAR_WIDTH)        = "auto";      // width of paragraph
   env (PAR_LEFT)         = "0cm";       // left indentation
   env (PAR_RIGHT)        = "0cm";       // right indentation
@@ -192,34 +200,55 @@ initialize_default_env () {
   env (CELL_ROW_NR)      = "1";         // row coordinate of cell
   env (CELL_COL_NR)      = "1";         // column coordinate of cell
 
-  env (POINT_STYLE)      = "disk";      // point style (square, circle...)
-
-  env (LINE_WIDTH)       = "1ln";       // line width in graphics
-  env (DASH_STYLE)       = "none";      // dash style
-  env (DASH_STYLE_UNIT)  = "5ln";       // dash style unit
-  env (LINE_ARROWS)      = "none";      // arrows at end of lines
-  env (LINE_CAPS)        = "normal";    // junctions in multilines
-  env (FILL_COLOR)       = "none";      // fill color
-  env (FILL_STYLE)       = "plain";     // fill style
-  env (TEXTAT_HALIGN)    = "left";      // horizontal text-at alignment
-  env (TEXTAT_VALIGN)    = "base";      // vertical text-at alignment
-
-  env (GR_GEOMETRY)      = gr_geometry; // geometry of graphics
-  env (GR_FRAME)         = gr_frame;    // coordinate frame for graphics
-  env (GR_MODE)          = "line";      // graphical mode
-  env (GR_COLOR)         = "default";   // color of new objects
-  env (GR_POINT_STYLE)   = "default";   // point style of new objects
-  env (GR_LINE_WIDTH)    = "default";   // line width for new objects
-  env (GR_DASH_STYLE)      = "default"; // dash style for new objects
-  env (GR_DASH_STYLE_UNIT) = "default"; // dash style unit for new objects
-  env (GR_LINE_ARROWS)   = "default";   // line arrows for new objects
-  env (GR_FILL_COLOR)    = "default";   // fill color for new objects
-  env (GR_TEXTAT_HALIGN) = "default";   // horiz. alignment for new text-ats
-  env (GR_TEXTAT_VALIGN) = "default";   // vert. alignment for new text-ats
+  env (GR_GEOMETRY)         = gr_geometry;    // geometry of graphics
+  env (GR_FRAME)            = gr_frame;       // coordinate frame for graphics
+  env (GR_MODE)             = "line";         // graphical mode
+  env (GR_AUTO_CROP)        = "false";        // auto crop graphics
+  env (GR_CROP_PADDING)     = "1spc";         // padding when auto cropping
   env (GR_GRID)             = gr_grid;        // grid for graphics
   env (GR_GRID_ASPECT)      = gr_grid_aspect; // grid aspect
   env (GR_EDIT_GRID)        = gr_edit_grid;   // edit grid
   env (GR_EDIT_GRID_ASPECT) = gr_grid_aspect; // edit grid (subdivisions)
+  env (GR_TRANSFORMATION)   = gr_transf;      // 3D transformation
+
+  env (GR_MAGNIFY)       = "default";   // magnify of new objects
+  env (GR_OPACITY)       = "default";   // opacity of new objects
+  env (GR_COLOR)         = "default";   // color of new objects
+  env (GR_POINT_STYLE)   = "default";   // point style of new objects
+  env (GR_LINE_WIDTH)    = "default";   // line width for new objects
+  env (GR_LINE_JOIN)     = "default";   // line join for new objects
+  env (GR_LINE_CAPS)     = "default";   // line caps for new objects
+  env (GR_LINE_EFFECTS)    = "default";   // line effects for new objects
+  env (GR_DASH_STYLE)      = "default";   // dash style for new objects
+  env (GR_DASH_STYLE_UNIT) = "default";   // dash style unit for new objects
+  env (GR_ARROW_BEGIN)     = "default";   // arrow begin for new objects
+  env (GR_ARROW_END)       = "default";   // arrow end for new objects
+  env (GR_ARROW_LENGTH)    = "default";   // arrow length for new objects
+  env (GR_ARROW_HEIGHT)    = "default";   // arrow height for new objects
+  env (GR_FILL_COLOR)      = "default";   // fill color for new objects
+  env (GR_FILL_STYLE)      = "default";   // fill style for new objects
+  env (GR_TEXT_AT_HALIGN)  = "default";   // horiz. alignment for new text-ats
+  env (GR_TEXT_AT_VALIGN)  = "default";   // vert. alignment for new text-ats
+  env (GR_TEXT_AT_MARGIN)  = "default";   // margins for new text-ats
+
+  env (GID)              = "default";   // graphical identifier
+  env (MAGNIFY)          = "1";         // magnification for graphical objects
+  env (POINT_STYLE)      = "disk";      // point style (square, circle...)
+  env (LINE_WIDTH)       = "1ln";       // line width in graphics
+  env (LINE_JOIN)        = "normal";    // junctions in multilines
+  env (LINE_CAPS)        = "normal";    // caps at ends
+  env (LINE_EFFECTS)     = "none";      // effects to be applied on line
+  env (DASH_STYLE)       = "none";      // dash style
+  env (DASH_STYLE_UNIT)  = "5ln";       // dash style unit
+  env (ARROW_BEGIN)      = "none";      // arrow at beginning of line
+  env (ARROW_END)        = "none";      // arrow at end of line
+  env (ARROW_LENGTH)     = "5ln";       // longitudal length of arrow
+  env (ARROW_HEIGHT)     = "5ln";       // transverse height of arrow
+  env (FILL_COLOR)       = "none";      // fill color
+  env (FILL_STYLE)       = "plain";     // fill style
+  env (TEXT_AT_HALIGN)   = "left";      // horizontal text-at alignment
+  env (TEXT_AT_VALIGN)   = "base";      // vertical text-at alignment
+  env (TEXT_AT_MARGIN)   = "base";      // margin for smart guides
 
   env (SRC_STYLE)        = "angular";   // style for "source" tags
   env (SRC_SPECIAL)      = "normal";    // special treatment of certain tags
@@ -233,11 +262,12 @@ initialize_default_env () {
   env (CANVAS_BAR_WIDTH)      = "1em";          // width of scroll bars
   env (CANVAS_BAR_PADDING)    = "0.25em";       // distance of scrollbars
   env (CANVAS_BAR_COLOR)      = "grey";         // color of bar
-  env (ORNAMENT_SHAPE)        = "rectangular";  // shape of the ornament
+  env (ORNAMENT_SHAPE)        = "classic";      // shape of the ornament
   env (ORNAMENT_BORDER)       = "1ln";          // border width
   env (ORNAMENT_HPADDING)     = "0px";          // horizontal padding of body
   env (ORNAMENT_VPADDING)     = "0px";          // vertical padding of body
   env (ORNAMENT_COLOR)        = "light grey";   // background color
+  env (ORNAMENT_EXTRA_COLOR)  = "light grey";   // background color for titles
   env (ORNAMENT_SUNNY_COLOR)  = "white";        // sunny color
   env (ORNAMENT_SHADOW_COLOR) = "dark grey";    // shadow color
 
@@ -252,9 +282,11 @@ initialize_default_env () {
   tree dest_url (URL, tree (ARG, "destination"));
   tree dest_script (SCRIPT, tree (ARG, "destination"), tree (ARG, "where"));
   tree dest_ref (URL, tree (MERGE, "#", tree (ARG, "Id")));
+  tree anchor (ID, tree (MERGE, "#", tree (ARG, "Id")));
   tree ln1 (LINK, "hyperlink", copy (src_id), copy (dest_url));
   tree ln2 (LINK, "action", copy (src_id), copy (dest_script));
   tree ln3 (LINK, "hyperlink", copy (ref_id), copy (dest_ref));
+  tree ln4 (LINK, "anchor", anchor);
   tree labflag (FLAG, tree (ARG, "Id"), "blue", "Id");
   tree labtxt (SET_BINDING, tree (ARG, "Id"), tree (VALUE, THE_LABEL));
   tree merged (MERGE, tree (VALUE, THE_TAGS), tuple (tree (ARG, "Id")));
@@ -267,7 +299,8 @@ initialize_default_env () {
   env ("action")= tree (MACRO, "body", "destination", "where",
 			tree (LOCUS, copy (src_id), ln2,
                               tree (ARG, "body")));
-  env ("label")= tree (MACRO, "Id", tree (CONCAT, labflag, labtxt));
+  env ("label")= tree (MACRO, "Id", 
+		       tree (LOCUS, copy (ref_id), ln4, tree (CONCAT, labflag, labtxt)));
   env ("tag")= tree (MACRO, "Id", "body",
 		     tree (WITH, "the-tags", merged,
 			   tree (SURROUND, tagflag, "",
