@@ -46,15 +46,14 @@ edit_env_rep::rewrite (tree t) {
   case EXTERN:
     {
       int i, n= N(t);
-      if (n == 0) return tree (ERROR, "invalid extern");
+      if (n < 1) return tree (ERROR, "invalid extern");
+      string fun= tm_decode(exec_string (t[0]));
       tree r (TUPLE, n);
-      for (i=0; i<n; i++)
+      for (i=1; i<n; i++)
 	r[i]= exec (t[i]);
       object expr= null_object ();
       for (i=n-1; i>0; i--)
 	expr= cons (object (r[i]), expr);
-      if (N(t) < 1) return tree (ERROR, "invalid extern");
-      string fun= exec_string (t[0]);
       expr= cons (string_to_object (fun), expr);
       if (!secure && script_status < 2) {
 	if (!as_bool (call ("secure?", expr)))
@@ -385,6 +384,8 @@ edit_env_rep::exec (tree t) {
     return exec_gw_length ();
   case GH_LENGTH:
     return exec_gh_length ();
+  case GU_LENGTH:
+    return exec_gu_length ();
   case TMPT_LENGTH:
     return exec_tmpt_length ();
   case PX_LENGTH:
@@ -430,6 +431,20 @@ edit_env_rep::exec (tree t) {
 
   case _POINT:
     return exec_point (t);
+
+  case EFF_MOVE:
+    return exec_eff_move (t);
+  case EFF_BUBBLE:
+    return exec_eff_bubble (t);
+  case EFF_GAUSSIAN:
+    return exec_eff_gaussian (t);
+  case EFF_OVAL:
+    return exec_eff_oval (t);
+  case EFF_RECTANGULAR:
+    return exec_eff_rectangular (t);
+  case EFF_MOTION:
+    return exec_eff_motion (t);
+
   case BOX_INFO:
     return exec_box_info (t);
   case FRAME_DIRECT:
@@ -587,6 +602,9 @@ edit_env_rep::exec_drd_props (tree t) {
 	if (is_tuple (val, "repeat", 2))
 	  drd->set_arity (l, as_int (val [1]), as_int (val [2]),
 			  ARITY_REPEAT, CHILD_BIFORM);
+	else if (is_tuple (val, "options", 2))
+	  drd->set_arity (l, as_int (val [1]), as_int (val [2]),
+			  ARITY_OPTIONS, CHILD_BIFORM);
 	else
 	  drd->set_arity (l, as_int (val), 0,
 			  ARITY_NORMAL, CHILD_DETAILED);
@@ -596,10 +614,8 @@ edit_env_rep::exec_drd_props (tree t) {
 	if (is_atomic (val))
 	  drd->set_attribute (l, prop, val->label);
       }
-      else if (prop == "class") {
-	if (is_atomic (val))
-	  drd->set_attribute (l, prop, val->label);
-      }
+      else if (prop == "syntax")
+        drd->set_syntax (l, val);
       else if (prop == "border") {
 	if (val == "yes") drd->set_border (l, BORDER_YES);
 	if (val == "inner") drd->set_border (l, BORDER_INNER);
@@ -714,23 +730,25 @@ edit_env_rep::exec_arg (tree t) {
   if (is_compound (r))
     return tree (ERROR, "bad arg");
   if (is_nil (macro_arg) || (!macro_arg->item->contains (r->label)))
-    return tree (ERROR, "argument " * r->label);
+    return tree (ERROR, "arg " * r->label);
   r= macro_arg->item [r->label];
   list<hashmap<string,tree> > old_var= macro_arg;
   list<hashmap<string,path> > old_src= macro_src;
   if (!is_nil (macro_arg)) macro_arg= macro_arg->next;
   if (!is_nil (macro_src)) macro_src= macro_src->next;
+  bool err= false;
   if (N(t) > 1) {
     int i, n= N(t);
     for (i=1; i<n; i++) {
       tree u= exec (t[i]);
-      if (!is_int (u)) break;
+      if (!is_int (u)) { err= true; break; }
       int nr= as_int (u);
-      if ((!is_compound (r)) || (nr<0) || (nr>=N(r))) break;
+      if ((!is_compound (r)) || (nr<0) || (nr>=N(r))) { err= true; break; }
       r= r[nr];
     }
   }
-  r= exec (r);
+  if (err) r= tree (ERROR, "arg " * r->label);
+  else r= exec (r);
   macro_arg= old_var;
   macro_src= old_src;
   return r;
@@ -784,9 +802,9 @@ edit_env_rep::exec_get_arity (tree t) {
 tree
 edit_env_rep::exec_eval_args (tree t) {
   if (N(t)<1) return tree (ERROR, "bad eval-args");
+  if(is_nil(macro_arg)) return tree(ERROR, "nil argument");
   tree v= macro_arg->item [as_string (t[0])];
-  if (is_atomic (v) || is_nil (macro_arg))
-    return tree (ERROR, "eval arguments " * t[0]->label);
+  if (is_atomic (v)) return tree (ERROR, "eval arguments " * t[0]->label);
   list<hashmap<string,tree> > old_var= macro_arg;
   list<hashmap<string,path> > old_src= macro_src;
   if (!is_nil (macro_arg)) macro_arg= macro_arg->next;
@@ -911,14 +929,17 @@ tree
 edit_env_rep::exec_use_package (tree t) {
   int i, n= N(t);
   for (i=0; i<n; i++) {
+    //cout << "Package " << as_string (t[i]) << "\n";
+    url name= url_none ();
     url styp= "$TEXMACS_STYLE_PATH";
-    url name= as_string (t[i]) * string (".ts");
-    //cout << "Package " << name << "\n";
-    if (is_rooted_web (base_file_name))
-      styp= styp | head (base_file_name);
-    else styp= head (base_file_name) | styp;
+    //if (is_rooted_web (base_file_name)) styp= styp | head (base_file_name);
+    //else styp= ::expand (head (base_file_name) * url_ancestor ()) | styp;
+    if (ends (as_string (t[i]), ".ts")) name= as_string (t[i]);
+    else name= styp * (as_string (t[i]) * string (".ts"));
+    name= resolve (name);
+    //cout << as_string (t[i]) << " -> " << name << "\n";
     string doc_s;
-    if (!load_string (styp * name, doc_s, false)) {
+    if (!load_string (name, doc_s, false)) {
       tree doc= texmacs_document_to_tree (doc_s);
       if (is_compound (doc))
 	exec (filter_style (extract (doc, "body")));
@@ -1118,6 +1139,13 @@ edit_env_rep::exec_divide (tree t) {
     if (den == 0) return tree (ERROR, "division by zero");
     return as_string (as_int (t1->label) / den);
   }
+  if (is_double (t1->label) && (is_double (t2->label))) {
+    double den= as_double (t2->label);
+    if (den == 0) return tree (ERROR, "division by zero");
+    return as_string (floor (as_double (t1->label) / den));
+  }
+  if (is_anylen (t1->label) && (is_anylen (t2->label)))
+    return as_string (tmlen_div (as_tmlen (t1), as_tmlen (t2)));
   return tree (ERROR, "bad divide");
 }
 
@@ -1133,6 +1161,15 @@ edit_env_rep::exec_modulo (tree t) {
     if (den == 0) return tree (ERROR, "modulo zero");
     return as_string (as_int (t1->label) % den);
   }
+  if (is_double (t1->label) && (is_double (t2->label))) {
+    double num= as_double (t1->label);
+    double den= as_double (t2->label);
+    if (den == 0) return tree (ERROR, "modulo zero");
+    double div= floor (num / den);
+    return as_string (num - div * den);
+  }
+  if (is_anylen (t1->label) && (is_anylen (t2->label)))
+    return tmlen_mod (as_tmlen (t1), as_tmlen (t2));
   return tree (ERROR, "bad modulo");
 }
 
@@ -1280,6 +1317,7 @@ edit_env_rep::exec_number (tree t) {
   string s1= t1->label;
   string s2= t2->label;
   int nr= as_int (s1);
+  if (s2 == "arabic") return as_string (nr);
   if (s2 == "roman") return roman_nr (nr);
   if (s2 == "Roman") return Roman_nr (nr);
   if (s2 == "alpha") return alpha_nr (nr);
@@ -1489,15 +1527,21 @@ edit_env_rep::exec_greatereq (tree t) {
 
 tree
 edit_env_rep::exec_hard_id (tree t) {
-  if (N(t) == 0) {
-    pointer ptr= (pointer) this;
+  pointer ptr= (pointer) this;
+  if (N(t) == 0)
     return "%" * as_hexadecimal (ptr);
-  }
   else {
     t= expand (t[0], true);
-    pointer ptr1= (pointer) this;
-    pointer ptr2= (pointer) t.operator -> ();
-    return "%" * as_hexadecimal (ptr1) * "-" * as_hexadecimal (ptr2);
+    pointer tptr= (pointer) t.operator -> ();
+    if (is_accessible (obtain_ip (t)))
+      return "%" * as_hexadecimal (ptr) *
+             "-" * as_hexadecimal (tptr);
+    else {
+      int h= hash (t);
+      return "%" * as_hexadecimal (ptr) *
+             "-" * as_hexadecimal (tptr) *
+             "-" * as_hexadecimal (h);
+    }
   }
 }
 
@@ -1568,7 +1612,7 @@ edit_env_rep::exec_set_binding (tree t) {
       }
     }
   }
-  return tree (HIDDEN, "", keys);
+  return tree (HIDDEN, keys);
 }
 
 tree
@@ -1645,6 +1689,62 @@ edit_env_rep::exec_point (tree t) {
   return as_tree (as_point (u));
 }
 
+tree
+edit_env_rep::exec_eff_move (tree t) {
+  if (N(t) < 3) return tree (ERROR, "bad eff-move");
+  tree body= exec (t[0]);
+  tree dx  = as_tree (as_length (exec (t[1])));
+  tree dy  = as_tree (as_length (exec (t[2])));
+  return tree (EFF_MOVE, body, dx, dy);
+}
+
+tree
+edit_env_rep::exec_eff_bubble (tree t) {
+  if (N(t) < 3) return tree (ERROR, "bad eff-bubble");
+  tree body= exec (t[0]);
+  tree r   = as_tree (as_length (exec (t[1])));
+  tree a   = exec (t[2]);
+  return tree (EFF_BUBBLE, body, r, a);
+}
+
+tree
+edit_env_rep::exec_eff_gaussian (tree t) {
+  if (N(t) < 1) return tree (ERROR, "bad eff-gaussian");
+  tree rx= as_tree (as_length (exec (t[0])));
+  if (N(t) == 1) return tree (EFF_GAUSSIAN, as_tree (rx));
+  if (N(t) < 3) return tree (ERROR, "bad eff-gaussian");
+  tree ry= as_tree (as_length (exec (t[1])));
+  return tree (EFF_GAUSSIAN, rx, ry, exec (t[2]));
+}
+
+tree
+edit_env_rep::exec_eff_oval (tree t) {
+  if (N(t) < 1) return tree (ERROR, "bad eff-oval");
+  tree rx= as_tree (as_length (exec (t[0])));
+  if (N(t) == 1) return tree (EFF_OVAL, as_tree (rx));
+  if (N(t) < 3) return tree (ERROR, "bad eff-oval");
+  tree ry= as_tree (as_length (exec (t[1])));
+  return tree (EFF_OVAL, rx, ry, exec (t[2]));
+}
+
+tree
+edit_env_rep::exec_eff_rectangular (tree t) {
+  if (N(t) < 1) return tree (ERROR, "bad eff-rectangular");
+  tree rx= as_tree (as_length (exec (t[0])));
+  if (N(t) == 1) return tree (EFF_RECTANGULAR, as_tree (rx));
+  if (N(t) < 3) return tree (ERROR, "bad eff-rectangular");
+  tree ry= as_tree (as_length (exec (t[1])));
+  return tree (EFF_RECTANGULAR, rx, ry, exec (t[2]));
+}
+
+tree
+edit_env_rep::exec_eff_motion (tree t) {
+  if (N(t) < 2) return tree (ERROR, "bad eff-motion");
+  tree dx= as_tree (as_length (exec (t[0])));
+  tree dy= as_tree (as_length (exec (t[1])));
+  return tree (EFF_MOTION, dx, dy);
+}
+
 tree
 edit_env_rep::exec_box_info (tree t) {
   if (N(t)<2) return tree (ERROR, "bad box-info");
