@@ -25,10 +25,11 @@ initialize_default_var_type () {
   hashmap<string,int>& var_type= default_var_type;
 
   var_type (DPI)               = Env_Fixed;
-  var_type (SFACTOR)           = Env_Fixed;
+  var_type (ZOOM_FACTOR)       = Env_Zoom;
   var_type (PREAMBLE)          = Env_Preamble;
   var_type (SAVE_AUX)          = Env_Fixed;
   var_type (MODE)              = Env_Mode;
+  var_type (INFO_FLAG)         = Env_Info_Level;
 
   var_type (FONT)              = Env_Font;
   var_type (FONT_FAMILY)       = Env_Font;
@@ -37,7 +38,9 @@ initialize_default_var_type () {
   var_type (FONT_SIZE)         = Env_Font_Size;
   var_type (FONT_BASE_SIZE)    = Env_Font_Size;
   var_type (MAGNIFICATION)     = Env_Magnification;
+  var_type (MAGNIFY)           = Env_Magnify;
   var_type (COLOR)             = Env_Color;
+  var_type (OPACITY)           = Env_Color;
   var_type (LANGUAGE)          = Env_Language;
 
   var_type (MATH_LANGUAGE)     = Env_Language;
@@ -107,9 +110,12 @@ initialize_default_var_type () {
   var_type (DASH_STYLE)        = Env_Dash_Style;
   var_type (DASH_STYLE_UNIT)   = Env_Dash_Style_Unit;
   var_type (FILL_COLOR)        = Env_Fill_Color;
-  var_type (LINE_ARROWS)       = Env_Line_Arrows;
-  var_type (TEXTAT_HALIGN)     = Env_Textat_Halign;
-  var_type (TEXTAT_VALIGN)     = Env_Textat_Valign;
+  var_type (ARROW_BEGIN)       = Env_Line_Arrows;
+  var_type (ARROW_END)         = Env_Line_Arrows;
+  var_type (ARROW_LENGTH)      = Env_Line_Arrows;
+  var_type (ARROW_HEIGHT)      = Env_Line_Arrows;
+  var_type (TEXT_AT_HALIGN)    = Env_Text_At_Halign;
+  var_type (TEXT_AT_VALIGN)    = Env_Text_At_Valign;
   var_type (GR_FRAME)          = Env_Frame;
   var_type (GR_GEOMETRY)       = Env_Geometry;
   var_type (GR_GRID)           = Env_Grid;
@@ -231,9 +237,9 @@ edit_env_rep::get_page_pars (SI& w, SI& h, SI& width, SI& height,
   cout << "height= " << (height/PIXEL) << "\n";
   cout << "odd   = " << (odd/PIXEL) << "\n";
   cout << "even  = " << (even/PIXEL) << "\n";
-  cout << "right = " << (right/PIXEL) << "\n";
   cout << "top   = " << (top/PIXEL) << "\n";
   cout << "bot   = " << (bot/PIXEL) << "\n";
+  cout << "cols  = " << nr_cols << "\n";
   */
 }
 
@@ -243,42 +249,59 @@ edit_env_rep::get_page_pars (SI& w, SI& h, SI& width, SI& height,
 
 void
 edit_env_rep::update_font () {
-  fn_size= (int) ((((double) get_int (FONT_BASE_SIZE))+0.5)*
-		  get_double (FONT_SIZE));
+  fn_size= (int) (((double) get_int (FONT_BASE_SIZE)) *
+		  get_double (FONT_SIZE) + 0.5);
   switch (mode) {
   case 0:
   case 1:
-    fn= find_font (get_string (FONT), get_string (FONT_FAMILY),
-		   get_string (FONT_SERIES), get_string (FONT_SHAPE),
-		   script (fn_size, index_level), (int) (magn*dpi));
+    fn= smart_font (get_string (FONT), get_string (FONT_FAMILY),
+                    get_string (FONT_SERIES), get_string (FONT_SHAPE),
+                    script (fn_size, index_level), (int) (magn*dpi));
     break;
   case 2:
-    fn= find_font (get_string (MATH_FONT), get_string (MATH_FONT_FAMILY),
-		   get_string (MATH_FONT_SERIES), get_string (MATH_FONT_SHAPE),
-		   script (fn_size, index_level), (int) (magn*dpi));
+    fn= smart_font (get_string (MATH_FONT), get_string (MATH_FONT_FAMILY),
+                    get_string (MATH_FONT_SERIES), get_string (MATH_FONT_SHAPE),
+                    get_string (FONT), get_string (FONT_FAMILY),
+                    get_string (FONT_SERIES), "mathitalic",
+                    script (fn_size, index_level), (int) (magn*dpi));
     break;
   case 3:
-    fn= find_font (get_string (PROG_FONT), get_string (PROG_FONT_FAMILY),
-		   get_string (PROG_FONT_SERIES), get_string (PROG_FONT_SHAPE),
-		   script (fn_size, index_level), (int) (magn*dpi));
+    fn= smart_font (get_string (PROG_FONT), get_string (PROG_FONT_FAMILY),
+                    get_string (PROG_FONT_SERIES), get_string (PROG_FONT_SHAPE),
+                    get_string (FONT), get_string (FONT_FAMILY) * "-tt",
+                    get_string (FONT_SERIES), get_string (FONT_SHAPE),
+                    script (fn_size, index_level), (int) (magn*dpi));
     break;
   }
 }
 
+int
+decode_alpha (string s) {
+  if (N(s) == 0) return 255;
+  else if (s[N(s)-1] == '%') {
+    // mg: be careful to avoid rounding problems for the conversion from double to int : (int)(2.55*100.0)=254
+    double p = as_double (s (0, N(s)-1));
+    if (p<0.0) p = 0.0;
+    if (p>100.0) p = 100.0;
+    return ((int) (255.0 * p))/100;
+  }
+  else {
+    double p = as_double (s);
+    if (p<0.0) p = 0.0;
+    if (p>1.0) p = 1.0;
+    return ((int) (255.0 * p));
+  }
+}
+
 void
 edit_env_rep::update_color () {
-  string c= get_string (COLOR);
-  string fc= get_string (FILL_COLOR);
-  if (c == "none") {
-    if (fc == "none") fill_mode= FILL_MODE_NOTHING;
-    else fill_mode= FILL_MODE_INSIDE;
-  }
-  else {
-    if (fc == "none") fill_mode= FILL_MODE_NONE;
-    else fill_mode= FILL_MODE_BOTH;
-  }
-  col= named_color (c);
-  fill_color= named_color (fc);
+  alpha= decode_alpha (get_string (OPACITY));
+  tree pc= env [COLOR];
+  tree fc= env [FILL_COLOR];
+  if (pc == "none") pen= pencil (false);
+  else pen= pencil (pc, alpha, get_length (LINE_WIDTH));
+  if (fc == "none") fill_brush= brush (false);
+  else fill_brush= brush (fc, alpha);
 }
 
 void
@@ -288,6 +311,19 @@ edit_env_rep::update_mode () {
   else if (s == "math") mode=2;
   else if (s == "prog") mode=3;
   else mode=0;
+  if (mode == 2) mode_op= OP_SYMBOL;
+  else mode_op= OP_TEXT;
+}
+
+void
+edit_env_rep::update_info_level () {
+  string s= get_string (INFO_FLAG);
+  if (s == "none") info_level= INFO_NONE;
+  else if (s == "minimal") info_level= INFO_MINIMAL;
+  else if (s == "short") info_level= INFO_SHORT;
+  else if (s == "detailed") info_level= INFO_DETAILED;
+  else if (s == "paper") info_level= INFO_PAPER;
+  else info_level= INFO_MINIMAL;
 }
 
 void
@@ -327,12 +363,12 @@ edit_env_rep::update_frame () {
   tree t= env [GR_FRAME];
   SI yinc= gvalign == "top"    ? - gh
 	 : gvalign == "bottom" ? 0
+         : gvalign == "axis" ? - (gh/2) + as_length ("1yfrac")
 	 : - gh / 2;
   if (is_tuple (t, "scale", 2) && is_func (t[2], TUPLE, 2)) {
     SI magn= as_length (t[1]);
     SI x   = as_length (t[2][0]);
     SI y   = as_length (t[2][1]);
-    if (gvalign == "top") yinc += as_length ("1ex");
     fr= scaling (magn, point (x, y + yinc));
   }
   else {
@@ -343,14 +379,17 @@ edit_env_rep::update_frame () {
   }
   point p0= fr (as_point (tuple ("0par", "0par")));
   if (gvalign == "top") {
-    clip_lim1= fr [point (p0[0], p0[1] - gh + as_length ("1ex"))];
-    clip_lim2= fr [point (p0[0] + gw, p0[1] + as_length ("1ex"))];
+    clip_lim1= fr [point (p0[0], p0[1] - gh)];
+    clip_lim2= fr [point (p0[0] + gw, p0[1])];
   }
-  else
-  if (gvalign == "bottom") {
+  else if (gvalign == "bottom") {
     clip_lim1= fr [point (p0[0], p0[1])];
     clip_lim2= fr [point (p0[0] + gw, p0[1] + gh)];
   }
+  else if (gvalign == "axis") {
+    clip_lim1= fr [point (p0[0], p0[1] - (gh/2) + as_length ("1yfrac"))];
+    clip_lim2= fr [point (p0[0] + gw, p0[1] + (gh/2) + as_length ("1yfrac"))];
+  }
   else {
     clip_lim1= fr [point (p0[0], p0[1] - gh/2)];
     clip_lim2= fr [point (p0[0] + gw, p0[1] + gh/2)];
@@ -397,54 +436,120 @@ edit_env_rep::update_src_close () {
 void
 edit_env_rep::update_dash_style () {
   tree t= env [DASH_STYLE];
-  dash_style= array<bool>(0);
+  dash_style= array<bool> (0);
   if (is_string (t)) {
     string s= as_string (t);
-    if (s == "none") {}
-  }
-  else
-  if (is_tuple (t)) {
-    int i, n= N(t);
-    dash_style= array<bool> (n);
-    for (i=0; i<n; i++) {
-      dash_style[i]= true;
-      if (t[i] == "0") dash_style[i]= false;
+    if (N(s) > 0 && (s[0] == '0' || s[0] == '1')) {
+      int i, n= N(s);
+      dash_style= array<bool> (n);
+      for (i=0; i<n; i++)
+        dash_style[i]= (s[i] != '0');
     }
   }
 }
 
-/*FIXME: Currently, the line-arrows property is evaluated
-  only in the context of the variables which appear before
-  it in the <with>. For example :
+void
+decompose_length (string s, double& x, string& un) {
+  int i;
+  for (i=0; i<N(s); i++)
+    if ((s[i]>='a') && (s[i]<='z')) break;
+  x = as_double (s (0, i));
+  un= s (i, N(s));
+}
 
-  - <with|color|blue|<with|color|green|line-arrows|<line|...>|...>
-    draws green objects with green line arrows;
+tree
+edit_env_rep::decode_arrow (tree t, string l, string h) {
+  if (is_string (t)) {
+    string s= t->label;
+    if (s == "" || s == "none") return "";
+    double lx, hx;
+    string lun, hun;
+    decompose_length (l, lx, lun);
+    decompose_length (h, hx, hun);
+    if (s == "<less>")
+      return tree (LINE,
+                   tree (_POINT, l, h),
+                   tree (_POINT, "0" * lun, "0" * hun),
+                   tree (_POINT, l, as_string (-hx) * hun));
+    if (s == "<gtr>")
+      return tree (LINE,
+                   tree (_POINT, as_string (-lx) * lun, h),
+                   tree (_POINT, "0" * lun, "0" * hun),
+                   tree (_POINT, as_string (-lx) * lun,
+                                 as_string (-hx) * hun));
+    if (s == "<less>|")
+      return tree (WITH, FILL_COLOR, tree (VALUE, COLOR),
+                   LINE_WIDTH, "0ln",
+                   tree (CLINE,
+                         tree (_POINT, l, h),
+                         tree (_POINT, "0" * lun, "0" * hun),
+                         tree (_POINT, l, as_string (-hx) * hun)));
+    if (s == "|<gtr>")
+      return tree (WITH, FILL_COLOR, tree (VALUE, COLOR),
+                   LINE_WIDTH, "0ln",
+                   tree (CLINE,
+                         tree (_POINT, as_string (-lx) * lun, h),
+                         tree (_POINT, "0" * lun, "0" * hun),
+                         tree (_POINT, as_string (-lx) * lun,
+                               as_string (-hx) * hun)));
+    if (s == "<gtr>")
+      return tree (LINE,
+                   tree (_POINT, as_string (-lx) * lun, h),
+                   tree (_POINT, "0" * lun, "0" * hun),
+                   tree (_POINT, as_string (-lx) * lun,
+                                 as_string (-hx) * hun));
+    if (s == "<less><less>")
+      return tree (GR_GROUP,
+                   tree (LINE,
+                         tree (_POINT, l, h),
+                         tree (_POINT, "0" * lun, "0" * hun),
+                         tree (_POINT, l, as_string (-hx) * hun)),
+                   tree (LINE,
+                         tree (_POINT, as_string (0 * lx) * lun, h),
+                         tree (_POINT, as_string (-lx) * lun, "0" * hun),
+                         tree (_POINT, as_string (0 * lx) * lun,
+                               as_string (-hx) * hun)));
+    if (s == "<gtr><gtr>")
+      return tree (GR_GROUP,
+                   tree (LINE,
+                         tree (_POINT, as_string (-lx) * lun, h),
+                         tree (_POINT, "0" * lun, "0" * hun),
+                         tree (_POINT, as_string (-lx) * lun,
+                               as_string (-hx) * hun)),
+                   tree (LINE,
+                         tree (_POINT, as_string (0 * lx) * lun, h),
+                         tree (_POINT, as_string (lx) * lun, "0" * hun),
+                         tree (_POINT, as_string (0 * lx) * lun,
+                               as_string (-hx) * hun)));                   
+    if (s == "|")
+      return tree (LINE,
+                   tree (_POINT, "0" * lun, h),
+                   tree (_POINT, "0" * lun, as_string (-hx) * hun));
+    if (s == "o")
+      return tree (WITH, FILL_COLOR, tree (VALUE, COLOR),
+                   LINE_WIDTH, "0ln",
+                   tree (CARC,
+                         tree (_POINT, "0" * lun, h),
+                         tree (_POINT, l, "0" * hun),
+                         tree (_POINT, "0" * lun, as_string (-hx) * hun)));
+    return "";
+  }
+  else return t;
+}
 
-  - while <with|color|blue|<with|line-arrows|<line|...>|color|green|...>
-    draws green objects with blue line arrows.
- */
 void
 edit_env_rep::update_line_arrows () {
-  tree t= env [LINE_ARROWS];
-  line_arrows= array<box>(2);
-  if (is_string (t)) {
-    string s= as_string (t);
-    if (s == "none") {}
-  }
-  else
-  if (is_tuple (t) && N(t)<=2 && N(t)>0) {
-    array<box> b (2);
-    b[1]= t[0]=="" ? box () : typeset_as_box (this, t[0], path(0));
-    if (N(t)>=2) {
-      b[0]= b[1];
-      b[1]= t[1]=="" ? box () : typeset_as_box (this, t[1], path (0));
-    }
-    line_arrows= b;
-  }
+  line_arrows= array<tree> (2);
+  string l= get_string (ARROW_LENGTH);
+  string h= get_string (ARROW_HEIGHT);
+  line_arrows[0]= decode_arrow (env [ARROW_BEGIN], l, h);
+  line_arrows[1]= decode_arrow (env [ARROW_END], l, h);
 }
 
 void
 edit_env_rep::update () {
+  zoomf          = normal_zoom (get_double (ZOOM_FACTOR));
+  pixel          = (SI) tm_round ((std_shrinkf * PIXEL) / zoomf);
   magn           = get_double (MAGNIFICATION);
   index_level    = get_int (MATH_LEVEL);
   display_style  = get_bool (MATH_DISPLAY);
@@ -452,20 +557,20 @@ edit_env_rep::update () {
   vert_pos       = get_int (MATH_VPOS);
   preamble       = get_bool (PREAMBLE);
 
-  update_color ();
   update_mode ();
+  update_info_level ();
   update_language ();
   update_font ();
 
   update_geometry ();
   update_frame ();
   point_style= get_string (POINT_STYLE);
-  lw= get_length (LINE_WIDTH);
+  update_color ();
   update_dash_style ();
   dash_style_unit= get_length (DASH_STYLE_UNIT);
   update_line_arrows ();
-  textat_halign= get_string (TEXTAT_HALIGN);
-  textat_valign= get_string (TEXTAT_VALIGN);
+  text_at_halign= get_string (TEXT_AT_HALIGN);
+  text_at_valign= get_string (TEXT_AT_VALIGN);
 
   update_src_style ();
   update_src_special ();
@@ -484,10 +589,17 @@ edit_env_rep::update (string s) {
     break;
   case Env_Fixed:
     break;
+  case Env_Zoom:
+    zoomf= normal_zoom (get_double (ZOOM_FACTOR));
+    pixel= (SI) tm_round ((std_shrinkf * PIXEL) / zoomf);
+    break;
   case Env_Magnification:
     magn= get_double (MAGNIFICATION);
     update_font ();
-    lw= get_length (LINE_WIDTH);
+    update_color ();
+    break;
+  case Env_Magnify:
+    mgfy= get_double (MAGNIFY);
     break;
   case Env_Language:
     update_language ();
@@ -539,7 +651,7 @@ edit_env_rep::update (string s) {
     point_style= get_string (POINT_STYLE);
     break;
   case Env_Line_Width:
-    lw= get_length (LINE_WIDTH);
+    update_color ();
     break;
   case Env_Dash_Style:
     update_dash_style();
@@ -553,11 +665,11 @@ edit_env_rep::update (string s) {
   case Env_Line_Arrows:
     update_line_arrows();
     break;
-  case Env_Textat_Halign:
-    textat_halign= get_string (TEXTAT_HALIGN);
+  case Env_Text_At_Halign:
+    text_at_halign= get_string (TEXT_AT_HALIGN);
     break;
-  case Env_Textat_Valign:
-    textat_valign= get_string (TEXTAT_VALIGN);
+  case Env_Text_At_Valign:
+    text_at_valign= get_string (TEXT_AT_VALIGN);
     break;
   case Env_Src_Style:
     update_src_style ();
