@@ -5,10 +5,9 @@
 *              and formatting routines
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef BOXES_H
@@ -17,14 +16,12 @@
 #include "rectangles.hpp"
 #include "path.hpp"
 #include "renderer.hpp"
+#include "timer.hpp"
 #include "font.hpp"
 #include "language.hpp"
 #include "hashmap.hpp"
-#include "Graphics/frame.hpp"
-#include "Graphics/grid.hpp"
-
-#define MAX_SI 0x7fffffff
-#define MIN_SI 0x80000000
+#include "frame.hpp"
+#include "grid.hpp"
 
 #define STD_BOX       0
 #define STACK_BOX     1
@@ -55,7 +52,7 @@ CONCRETE_CODE(cursor);
 cursor copy (cursor cu);
 bool operator == (cursor cu1, cursor cu2);
 bool operator != (cursor cu1, cursor cu2);
-ostream& operator << (ostream& out, cursor cu);
+tm_ostream& operator << (tm_ostream& out, cursor cu);
 
 /******************************************************************************
 * The selection class
@@ -78,7 +75,7 @@ CONCRETE_CODE(selection);
 
 bool operator == (selection sel1, selection sel2);
 bool operator != (selection sel1, selection sel2);
-ostream& operator << (ostream& out, selection sel);
+tm_ostream& operator << (tm_ostream& out, selection sel);
 
 /******************************************************************************
 * The graphical selection class
@@ -90,9 +87,10 @@ struct gr_selection {
   gr_selection (array<path> cp= array<path> (), SI dist= 0);
 };
 
-ostream& operator << (ostream& out, gr_selection sel);
+tm_ostream& operator << (tm_ostream& out, gr_selection sel);
 
 typedef array<gr_selection> gr_selections;
+void sort (gr_selections& sels);
 tree as_tree (gr_selections sels);
 
 /******************************************************************************
@@ -146,7 +144,7 @@ public:
   virtual path      find_tag (string name);
 
   virtual int  reindex (int i, int item, int n);
-  void redraw (renderer ren, path p, rectangles& l);
+  virtual void redraw (renderer ren, path p, rectangles& l);
   void redraw (renderer ren, path p, rectangles& l, SI x, SI y);
 
   /*************************** positioning routines **************************/
@@ -226,7 +224,7 @@ public:
   virtual int       get_leaf_right_pos ();
   virtual string    get_leaf_string ();
   virtual font      get_leaf_font ();
-  virtual color     get_leaf_color ();
+  virtual pencil    get_leaf_pencil ();
   virtual language  get_leaf_language ();
   virtual tree      get_leaf_tree ();
   virtual lazy      get_leaf_lazy ();
@@ -249,6 +247,7 @@ public:
   friend struct lazy_paragraph_rep;
   friend class  phrase_box_rep;
   friend class  remember_box_rep;
+  friend class  effect_box_rep;
   friend void make_eps (url dest, box b, int dpi= 600);
 };
 ABSTRACT_NULL_CODE(box);
@@ -256,8 +255,8 @@ ABSTRACT_NULL_CODE(box);
 extern int box_count;
 inline box_rep::box_rep (path ip2):
   x0(0), y0(0), x1(0), y1(0), x2(0), y2(0), x3(0), y3(0), x4(0), y4(0),
-  ip (ip2) { DEBUG(box_count++); }
-inline box_rep::~box_rep () { DEBUG(box_count--); }
+  ip (ip2) { TM_DEBUG(box_count++); }
+inline box_rep::~box_rep () { TM_DEBUG(box_count--); }
 inline bool box_rep::test_in (SI x, SI y) {
   return (x>=x1) && (x<x2) && (y>=y1) && (y<y2); }
 inline SI box_rep::w () { return x2-x1; }
@@ -275,7 +274,7 @@ inline SI box_rep::sy4 (int i) { box b= subbox(i); return b->y0+ b->y4; }
 
 inline box box::operator [] (int i) { return rep->subbox(i); }
 inline int N (box b) { return b.rep->subnr(); }
-ostream& operator << (ostream& out, box b);
+tm_ostream& operator << (tm_ostream& out, box b);
 SI   get_delta (SI x, SI x1, SI x2);
 bool outside (SI x, SI delta, SI x1, SI x2);
 void make_eps (url dest, box b, int dpi);
@@ -293,20 +292,20 @@ void          refresh_at (time_t t);
 #define DECORATION_MIDDLE (-3)
 #define DECORATION_RIGHT  (-4)
 #define DETACHED          (-5)
-#define is_accessible(p) ((nil (p)) || ((p)->item >= 0))
-#define is_decoration(p) ((!nil (p)) && ((p)->item < 0))
+#define is_accessible(p) ((is_nil (p)) || ((p)->item >= 0))
+#define is_decoration(p) ((!is_nil (p)) && ((p)->item < 0))
 inline path descend (path ip, int i) {
-  return (nil (ip) || (ip->item >= 0))? path (i, ip): ip; }
+  return (is_nil (ip) || (ip->item >= 0))? path (i, ip): ip; }
 inline path decorate () {
   return path (DECORATION); }
 inline path decorate (path ip) {
-  return (nil (ip) || (ip->item >= 0))? path (DECORATION, ip): ip; }
+  return (is_nil (ip) || (ip->item >= 0))? path (DECORATION, ip): ip; }
 inline path decorate_left (path ip) {
-  return (nil (ip) || (ip->item >= 0))? path (DECORATION_LEFT, ip): ip; }
+  return (is_nil (ip) || (ip->item >= 0))? path (DECORATION_LEFT, ip): ip; }
 inline path decorate_middle (path ip) {
-  return (nil (ip) || (ip->item >= 0))? path (DECORATION_MIDDLE, ip): ip; }
+  return (is_nil (ip) || (ip->item >= 0))? path (DECORATION_MIDDLE, ip): ip; }
 inline path decorate_right (path ip) {
-  return (nil (ip) || (ip->item >= 0))? path (DECORATION_RIGHT, ip): ip; }
+  return (is_nil (ip) || (ip->item >= 0))? path (DECORATION_RIGHT, ip): ip; }
 path descend_decode (path ip, int side);
 
 inline bool box_rep::accessible () { return is_accessible (find_lip ()); }
@@ -325,6 +324,7 @@ tree attach_dip (tree ref, path ip);
 ******************************************************************************/
 
 struct gr_selection_rep: concrete_struct {
+  string type;
   array<path> cp;
   array<point> pts;
   point p;
